/*   EXTRAITS DE LA LICENCE
	Copyright CEA, contributeurs : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)
  
	Adresse ml :
	BILLARD, non joignable par ml ;
	CALISTE, damien P caliste AT cea P fr.

	Ce logiciel est un programme informatique servant  visualiser des
	structures atomiques dans un rendu pseudo-3D. 

	Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
	respectant les principes de diffusion des logiciels libres. Vous pouvez
	utiliser, modifier et/ou redistribuer ce programme sous les conditions
	de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
	sur le site "http://www.cecill.info".

	Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
	pris connaissance de la licence CeCILL, et que vous en avez accept les
	termes (cf. le fichier Documentation/licence.fr.txt fourni avec ce logiciel).
*/

/*   LICENCE SUM UP
	Copyright CEA, contributors : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)

	E-mail address:
	BILLARD, not reachable any more ;
	CALISTE, damien P caliste AT cea P fr.

	This software is a computer program whose purpose is to visualize atomic
	configurations in 3D.

	This software is governed by the CeCILL  license under French law and
	abiding by the rules of distribution of free software.  You can  use, 
	modify and/ or redistribute the software under the terms of the CeCILL
	license as circulated by CEA, CNRS and INRIA at the following URL
	"http://www.cecill.info". 

	The fact that you are presently reading this means that you have had
	knowledge of the CeCILL license and that you accept its terms. You can
	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
*/
#include "visu_OSInterface.h"

#include <stdio.h>
#include <stdlib.h>

#include <X11/Xlib.h>
#include <GL/glx.h>

#include <visu_tools.h>
#include "visu_X11.h"

static GLXContext ctxDirect;
static GLXContext ctxPixmap;

static GLXDrawable storedDrawable;
static GLXContext storedContext;

gboolean drawableIsPixmap;

struct DumpImage_struct
{
  GLXPixmap glxPixmap;
  Pixmap pixmap;
  int width, height;
};

int screenId;

int setScreen(int screen)
{
  screenId = screen;
  return 1;
}

int initOpenGLContext()
{
  int stereo;

  int list[] = {
    GLX_RGBA,
    GLX_RED_SIZE, 1, GLX_GREEN_SIZE, 1, GLX_BLUE_SIZE, 1,
    GLX_DEPTH_SIZE, 1,
    GLX_DOUBLEBUFFER,
    GLX_STEREO,
    None
  };

  DBG_fprintf(stderr, "Visu GLX : init OpenGl context.\n");

  if (!glXQueryExtension(dpy, 0, 0))
    {
      (void)fprintf(stderr, _("FATAL ERROR! No GLX extension.\nYour X server"
			      " does not support OpenGL extension. Please contact your"
			      " system administrator to ask him to add the 'glx'"
			      " extension to your X server.\n"));
      return 0;
    }

  if (!screenId)
    screenId = DefaultScreen(dpy);
  stereo = 1;
  if ( (vinfo = glXChooseVisual(dpy, screenId, list)) == NULL )
    {
      stereo = 0;
      list[10] = None; 
      if ( (vinfo = glXChooseVisual(dpy, screenId, list)) == NULL )
	{
	  (void)fprintf(stderr, _("FATAL ERROR! Cannot find a visual.\n"
				  "Have you emough right access on the X server?"));
	  return 0;
	}
    }
  if ( (ctxDirect = glXCreateContext(dpy, vinfo, 0, GL_TRUE)) == 0 )
    {
      (void)fprintf(stderr, _("FATAL ERROR! Cannot create direct GLX context.\n"));
      return 0;
    }
  
  storedDrawable = None;
  storedContext = NULL;
  
  drawableIsPixmap = FALSE;

  DBG_fprintf(stderr, "Visu GLX : OK.\n");

  return 1;
}

gboolean OpenGLIsPixmap()
{
  return drawableIsPixmap;
}

int attachOpenGlContext(gboolean isPixmap, gpointer windowDescriptor)
{
  int res;
 
  XFlush(dpy); 
  res = 0;
  if (isPixmap)
    {
      if (windowDescriptor)
	res = glXMakeCurrent(dpy, *(GLXDrawable*)windowDescriptor, ctxPixmap);
      else
	{
	  fprintf(stderr, "INTERNAL ERROR! 'attachOpenGlContext' has been "
		  "called with the flag 'isPixmap' set to TRUE but no parameter"
		  " 'windowDescriptor' has been defined.\n");
	  exit(1);
	}
    }
  else
    {
      if (window)
	res = glXMakeCurrent(dpy, *window, ctxDirect);
      else
	{
	  fprintf(stderr, "INTERNAL ERROR! 'attachOpenGlContext' has been "
		  "called with 'windowDescriptor' paramater set to NULL but"
		  " no default handler has been defined.\n");
	  exit(1);
	}
    }

  drawableIsPixmap = isPixmap;

  if (!res)
    {
      (void)fprintf(stderr, _("INTERNAL ERROR! Cannot attach GLX context to X window\n"));
      return 0;
    }
  DBG_fprintf(stderr, "Visu GLX : Set the current context to pixmap(1)"
	      " or to window(0) : %d.\n", isPixmap);
  return 1;
}

DumpImage* dumpImageNew(int width, int height)
{
  DumpImage *image;
  int d_depth;

  DBG_fprintf(stderr, "Visu GLX : creating a off-screen buffer (%dx%d).\n", width, height);
  image = malloc(sizeof(DumpImage));
  if (!image)
    {
      allocationProblems();
      exit(1);
    }
  d_depth = DefaultDepth(dpy, DefaultScreen(dpy));
  image->width = width;
  image->height = height;
  image->pixmap = XCreatePixmap(dpy, RootWindow(dpy, vinfo->screen), width, height, d_depth);
  if (!image->pixmap)
    {
      (void)fprintf(stderr, _("WARNING! Cannot allocate a Pixmap for the indirect rendering:\n"
			      "Window dump cancelled\n"));
      return (DumpImage*)0;
    }
  image->glxPixmap = glXCreateGLXPixmap(dpy, vinfo, image->pixmap);
  if (!image->glxPixmap)
    {
      (void)fprintf(stderr, _("WARNING! Cannot allocate a GLXPixmap for the indirect rendering:\n"
			      "Window dump cancelled\n"));
      return (DumpImage*)0;
    }
  if ( (ctxPixmap = glXCreateContext(dpy, vinfo, 0, GL_FALSE)) == 0 )
    {
      (void)fprintf(stderr, _("FATAL ERROR! Cannot create indirect GLX context.\n"));
      return 0;
    }

  DBG_fprintf(stderr, "Visu GLX : Creating a Pixmap and a GLX"
	      " pixmap (%dx%d).\n", width, height);
  return image;
}
void dumpImageFree(DumpImage *image)
{
  if (image)
    {
      if (image->pixmap)
	XFreePixmap(dpy, image->pixmap);
      if (image->glxPixmap)
	glXDestroyGLXPixmap(dpy, image->glxPixmap);
      glXDestroyContext(dpy, ctxPixmap);
      ctxPixmap = (GLXContext)0;
      free(image);
    }
}
gpointer dumpImageGet_buffer(DumpImage *image)
{
  if (image)
    return (gpointer)&image->glxPixmap;
  else
    return (gpointer)0;
}

void callSwapGLBuffer()
{
  if (!drawableIsPixmap)
    glXSwapBuffers(dpy, *window);
}

int initOSinterface()
{
  window = (Window*)0;
  screenId = 0;
  return 1;
}

int setDisplay(gpointer display)
{
  dpy = (Display*)display;
  return 1;
}
 
int initDisplay()
{
  if ( (dpy = XOpenDisplay(0)) == 0 )
    {
      (void)fprintf(stderr, _("FATAL ERROR! Cannot connect to %s.\n"
			      "Is there an X server running and do you have"
			      " access on it?\n"), XDisplayName(0));
      return 0;
    }
  return 1;
}

/* Return the display used by OpenGl. It's useful
   if extensions need to ask X for differents things. */
Display* getDisplay()
{
  return dpy;
}

void stopBackingStore()
{
  XSetWindowAttributes wattrs;

  wattrs.backing_store = 0;
  XChangeWindowAttributes(dpy, *window, CWBackingStore, &wattrs);
}

void startBackingStore()
{
  XSetWindowAttributes wattrs;

  wattrs.backing_store = Always;
  XChangeWindowAttributes(dpy, *window, CWBackingStore, &wattrs);
}

void storeOpenGLContext()
{
  storedDrawable = glXGetCurrentDrawable();
  storedContext = glXGetCurrentContext();
}
void restoreOpenGLContext()
{
  int res;

  res = glXMakeCurrent(dpy, storedDrawable, storedContext);
  if (storedDrawable == *window)
    drawableIsPixmap = FALSE;
  if (!res)
    (void)fprintf(stderr, _("INTERNAL ERROR! Cannot restore GLX context to previous drawable.\n"));
}

int setWindowHandler(gpointer windowHandler)
{
  window = (Window*)windowHandler;
  return 1;
}

GLuint initOpenGLFontList()
{
  XFontStruct *fontInfo;
  unsigned int first, last;
  char name[] = 
    "-adobe-helvetica-medium-r-normal--18-180-75-75-p-98-iso8859-1";
  GLuint BASE;
       
  DBG_fprintf(stderr, "Visu GLX : initialise fonts.\n");
  if ((fontInfo = XLoadQueryFont(dpy, name)) == NULL)
    {
      (void)fprintf(stderr, _("WARNING! Specified font not available in gl_font_init\n"
			      "Trying to use fixed font\n"));
      if ((fontInfo = XLoadQueryFont(dpy, "fixed")) == NULL)
	{
	  (void)fprintf(stderr, _("INTERNAL ERROR! Fixed font not available.\n"));
	  exit(1);
	}
    }

  first = fontInfo->min_char_or_byte2;
  last = fontInfo->max_char_or_byte2;

  /* this creates display lists 1 to 256 (not checked) */
  if ( (BASE = glGenLists(last+1)) == 0 )
    {
      (void)fprintf(stderr, _("INTERNAL ERROR! Cannot build font display lists.\n"));
      exit(1);
    }
   
  glXUseXFont(fontInfo->fid, first, last-first+1, BASE+first);

  return BASE;
}
