/*   EXTRAITS DE LA LICENCE
	Copyright CEA, contributeurs : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)
  
	Adresse ml :
	BILLARD, non joignable par ml ;
	CALISTE, damien P caliste AT cea P fr.

	Ce logiciel est un programme informatique servant  visualiser des
	structures atomiques dans un rendu pseudo-3D. 

	Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
	respectant les principes de diffusion des logiciels libres. Vous pouvez
	utiliser, modifier et/ou redistribuer ce programme sous les conditions
	de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
	sur le site "http://www.cecill.info".

	Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
	pris connaissance de la licence CeCILL, et que vous en avez accept les
	termes (cf. le fichier Documentation/licence.fr.txt fourni avec ce logiciel).
*/

/*   LICENCE SUM UP
	Copyright CEA, contributors : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)

	E-mail address:
	BILLARD, not reachable any more ;
	CALISTE, damien P caliste AT cea P fr.

	This software is a computer program whose purpose is to visualize atomic
	configurations in 3D.

	This software is governed by the CeCILL  license under French law and
	abiding by the rules of distribution of free software.  You can  use, 
	modify and/ or redistribute the software under the terms of the CeCILL
	license as circulated by CEA, CNRS and INRIA at the following URL
	"http://www.cecill.info". 

	The fact that you are presently reading this means that you have had
	knowledge of the CeCILL license and that you accept its terms. You can
	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
*/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h> /* For the access markers R_OK, W_OK ... */

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include "visu_tools.h"
#include "visu_data.h"
#include "visu_basic.h"
#include "visu_rendering.h"
#include "visu_object.h"
#include "visu_configFile.h"
#include "visu_pickMesure.h"
#include "opengl.h"
#include "renderingBackend/visu_windowInterface.h"
#include "gtk_main.h"
#include "gtk_observePick.h"
#include "gtk_pairs.h"
#include "gtk_about.h"

GtkProgressBar *dumpBar;
int flagDisableChangePickObserve;

void comboChanged(GtkEditable *entry, gpointer data)
{
  char *label;
  int index;
  GList *element;

  label = (char*)gtk_entry_get_text(GTK_ENTRY(entry));
  if (!label || label[0] == '\0')
    return;

  /* look for the number of the selected element */
  element = g_list_find_custom(listeNomsSubPanel,
			       (gconstpointer)label,
			       lookForNameInComboList);
  index = g_list_position(listeNomsSubPanel, element);

  if (currentSubPanel->onLeave)
    g_idle_add(currentSubPanel->onLeave, (gpointer)0);

  currentSubPanel = (subPanel*)g_list_nth_data(listeSubPanel, index);

  /* disable the changed signal of the notebook
     before changing it. */
  g_signal_handler_block((gpointer)tabSubPanel,
			 tabSubPanelSignalChanged);
  gtk_notebook_set_current_page(GTK_NOTEBOOK(tabSubPanel),
				index);
  g_signal_handler_unblock((gpointer)tabSubPanel,
			   tabSubPanelSignalChanged);

  if (currentSubPanel->onEnter)
    g_idle_add(currentSubPanel->onEnter, (gpointer)0);

  DBG_fprintf(stderr, "Main Gtk : changed combo '%s' -> %d\n",
	      gtk_entry_get_text(GTK_ENTRY(entry)), index);
}
void subPanelChanged(GtkNotebook *book, GtkNotebookPage *page,
		     guint num, gpointer data)
{
  char *label;

  /* get the label of the newly selected tab. */
  label = (char*)g_list_nth_data(listeNomsSubPanel, num);
  
  /* One can't call the function directly for obscur reasons
     dealing with the gtk working : it has to leave this
     callback before doing any other action. */
  if (currentSubPanel->onLeave)
    g_idle_add(currentSubPanel->onLeave, (gpointer)0);

  currentSubPanel = (subPanel*)g_list_nth_data(listeSubPanel, num);

  /* disable the changed signal of the combobox
     before changing it. */
  g_signal_handler_block((gpointer)
			 GTK_COMBO(comboListeSubPanel)->entry,
			 comboListeSubPanelSignalChanged);
  gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(comboListeSubPanel)->entry),
		      label);
  g_signal_handler_unblock((gpointer)
			   GTK_COMBO(comboListeSubPanel)->entry,
			   comboListeSubPanelSignalChanged);

  if (currentSubPanel->onEnter)
    g_idle_add(currentSubPanel->onEnter, (gpointer)0);
  
  DBG_fprintf(stderr, "Main Gtk : changed tab '%s' -> %d\n", label, (int)num);
}

void filterChanged(GtkEditable *entry, gpointer data)
{
  char *label;
  
  label = (char*)gtk_entry_get_text(GTK_ENTRY(entry));
  if (!label || label[0] == '\0')
    return;
  gtk_file_selection_complete(GTK_FILE_SELECTION(data), label);
}

void on_loadButton_clicked(GtkButton *button, gpointer user_data)
{
  GtkFileChooser *fileChooser;
  GtkWidget *dialog;
  int res;
  createGtkLoadWidgetFunc loadAction;
  gchar *directory;
  RenderingMethod* method;
  VisuData *newData;
  
  method = getRenderingMethodInUse();
  if (!method)
    return;
  loadAction = gtkMainGet_renderingSpecificOpen(method);
  if (!loadAction)
    return;

  newData = visuDataNew();
  if (!newData)
    return;

  res = loadAction(newData, &fileChooser, &dialog);
  
  directory = gtk_file_chooser_get_current_folder(fileChooser);
  setLastOpenDirectory((char*)directory);
  g_free(directory);

  if (res)
    {
      gtk_widget_destroy(dialog);
      visuWaitFunction();
      g_idle_add(loadFileWithGtk, (gpointer)newData);
    }
  else
    {
      gtk_widget_destroy(dialog);
      visuDataFree(newData);
    }
}

void openFileinSaveDialog(GtkButton *button, gpointer data)
{
  GtkWidget *entry;

  if (!data)
    {
      fprintf(stderr, "INTERNAL ERROR! openFileinSaveDialog has been called without"
	      "parameter pointing to the entry.\n");
      return;
    }

  entry = (GtkWidget*)data;
  gtk_entry_set_text(GTK_ENTRY(entry), "coucou");
}

void on_saveButton_clicked(GtkButton *button, gpointer user_data)
{
  GtkWidget *entry;
  GtkWidget *chk;
  GtkWidget *label;
  GtkWidget *wd;
  char *file;
  GList *resList, *parList;
  GList *comboLabel;
  GString *helpText;

  saveDialog = create_saveDialog();
  gtk_widget_set_name(saveDialog, "message");

  label = lookup_widget(saveDialog, "labelSaveDialog");
  gtk_widget_set_name(label, "message_title");

  wd = lookup_widget(saveDialog, "checkButtonParameters");
  gtk_widget_set_name(wd, "message_tog");
  wd = lookup_widget(saveDialog, "checkButtonResources");
  gtk_widget_set_name(wd, "message_tig");

  wd = lookup_widget(saveDialog, "labelHelpSave");
  helpText = g_string_new("");
  g_string_append_printf(helpText, _("If you just specify a directory 'dir/',"
				     " it will be save to 'dir/[%s][%s]' by default, "
				     "ortherwise speficy a full path.\n<u>Tips</u>: think to"
				     " create a ~/.v_sim directory, it is scanned at startup."),
			 visuConfigFileGet_defaultFileName(VISU_CONFIGFILE_PARAMETER),
			 visuConfigFileGet_defaultFileName(VISU_CONFIGFILE_RESOURCE));
  gtk_label_set_markup(GTK_LABEL(wd), helpText->str);
  gtk_label_set_line_wrap(GTK_LABEL(wd), TRUE);
  g_string_free(helpText, TRUE);
  gtk_widget_set_name(wd, "label_info");

  entry = lookup_widget(saveDialog, "entryParameters");
  gtk_widget_set_name(entry, "message_entry");
/*   file = visuConfigFileGet_validPath(VISU_CONFIGFILE_PARAMETER, W_OK, 1); */
/*   if (file) */
/*     gtk_entry_set_text(GTK_ENTRY(entry), file); */

  resList = visuConfigFileGet_pathList(VISU_CONFIGFILE_PARAMETER);
  if (resList)
    {
      comboLabel = (GList*)0;
      do
	{
	  file = visuConfigFileGet_nextValidPath(VISU_CONFIGFILE_PARAMETER, W_OK, &resList, 1);
	  if (file)
	    comboLabel = g_list_append(comboLabel, file);
	}
      while(file);
      wd = lookup_widget(saveDialog, "comboParameters");
      gtk_combo_set_value_in_list(GTK_COMBO(wd), FALSE, FALSE);
      gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(wd)->entry),
			     TRUE);
      gtk_combo_set_popdown_strings(GTK_COMBO(wd), comboLabel);
      while(comboLabel)
	{
	  g_free(comboLabel->data);
	  comboLabel = g_list_next(comboLabel);
	}
      g_list_free(comboLabel);
    }

  wd = lookup_widget(saveDialog, "buttonOpenParameters");
  g_signal_connect(G_OBJECT(wd), "clicked",
		   G_CALLBACK(openFileinSaveDialog), (gpointer)entry);


  entry = lookup_widget(saveDialog, "entryResources");
  gtk_widget_set_name(entry, "message_entry");

  resList = visuConfigFileGet_pathList(VISU_CONFIGFILE_RESOURCE);
  if (resList)
    {
      comboLabel = (GList*)0;
      do
	{
	  file = visuConfigFileGet_nextValidPath(VISU_CONFIGFILE_RESOURCE, W_OK, &resList, 1);
	  if (file)
	    comboLabel = g_list_append(comboLabel, file);
	}
      while(file);
      wd = lookup_widget(saveDialog, "comboResources");
      gtk_combo_set_value_in_list(GTK_COMBO(wd), FALSE, FALSE);
      gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(wd)->entry),
			     TRUE);
      gtk_combo_set_popdown_strings(GTK_COMBO(wd), comboLabel);
      while(comboLabel)
	{
	  g_free(comboLabel->data);
	  comboLabel = g_list_next(comboLabel);
	}
      g_list_free(comboLabel);
    }

  chk = lookup_widget(saveDialog, "checkButtonParameters");
  gtk_widget_set_name(chk, "message_tog");
  chk = lookup_widget(saveDialog, "checkButtonResources");
  gtk_widget_set_name(chk, "message_tog");

  gtk_widget_show(saveDialog);
}


void on_okButtonSave_clicked(GtkButton *button, gpointer user_data)
{
  GtkWidget *check, *entry;
  int res;
  gchar* file;
  gchar* tmpFile;
  gsize ecrit, lu;
  gboolean resOk;
  GString *errorMessage;

  entry = lookup_widget(saveDialog, "entryParameters");
  check = lookup_widget(saveDialog, "checkButtonParameters");
  if (gtk_toggle_button_get_active
      (GTK_TOGGLE_BUTTON(check)))
    {
      if (!getFileSystemInUTF8())
	file = g_filename_from_utf8(gtk_entry_get_text
				    (GTK_ENTRY(entry)), -1, &lu, &ecrit, NULL);
      else
	file = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
      if (g_file_test(file, G_FILE_TEST_IS_DIR))
	{
	  tmpFile =
	    g_build_filename(file,
			     visuConfigFileGet_defaultFileName(VISU_CONFIGFILE_PARAMETER),
			     NULL);
	  g_free(file);
	  file = tmpFile;
	}

      errorMessage = (GString*)0;
      resOk = visuConfigFileSave(VISU_CONFIGFILE_PARAMETER,
				 file, &errorMessage);
      if (!resOk)
	{
	  if (errorMessage)
	    raiseAlertDialogWithScrollView(errorMessage->str);
	}
      if (errorMessage)
	{
	  DBG_fprintf(stderr, "Gtk Callbacks : trace from exporting resources ...\n%s",
		      errorMessage->str);
	  g_string_free(errorMessage, TRUE);
	}
      g_free(file);
    }

  entry = lookup_widget(saveDialog, "entryResources");
  check = lookup_widget(saveDialog, "checkButtonResources");
  if (gtk_toggle_button_get_active
      (GTK_TOGGLE_BUTTON(check)))
    {
      if (!getFileSystemInUTF8())
	file = g_filename_from_utf8(gtk_entry_get_text
				    (GTK_ENTRY(entry)), -1, &lu, &ecrit, NULL);
      else
	file = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
      if (g_file_test(file, G_FILE_TEST_IS_DIR))
	{
	  tmpFile=
	    g_build_filename(file,
			     visuConfigFileGet_defaultFileName(VISU_CONFIGFILE_RESOURCE),
			     NULL);
	  g_free(file);
	  file = tmpFile;
	}

      errorMessage = (GString*)0;
      resOk = visuConfigFileSave(VISU_CONFIGFILE_RESOURCE,
				 file, &errorMessage);
      if (!resOk)
	{
	  if (errorMessage)
	    raiseAlertDialogWithScrollView(errorMessage->str);
	}
      if (errorMessage)
	{
	  DBG_fprintf(stderr, "Gtk Callbacks : trace from exporting resources ...\n%s",
		      errorMessage->str);
	  g_string_free(errorMessage, TRUE);
	}
      g_free(file);
    }
  
  gtk_widget_destroy(saveDialog);
}


void on_checkbuttonParameter_toggled(GtkToggleButton *togglebutton,
				     gpointer user_data)
{
  GtkWidget *entry;
  
  entry = lookup_widget(saveDialog, "entryParameters");
  gtk_widget_set_sensitive(entry,
			   gtk_toggle_button_get_active
			   (GTK_TOGGLE_BUTTON(togglebutton)));
}


void on_checkButtonResources_toggled(GtkToggleButton *togglebutton,
				     gpointer user_data)
{
  GtkWidget *entry;
  
  entry = lookup_widget(saveDialog, "entryResources");
  gtk_widget_set_sensitive(entry,
			   gtk_toggle_button_get_active
			   (GTK_TOGGLE_BUTTON(togglebutton)));
}


void on_cancelButtonSave_clicked(GtkButton *button, gpointer user_data)
{
  gtk_widget_destroy(saveDialog);
}


void on_scanResourcesButton_clicked(GtkButton *button, gpointer user_data)
{
  GtkWidget *file_selector;
  char *filename;
  char *directory;
  int res;
  GString *errorMessage;

#if GTK_MINOR_VERSION > 3 && USE_NEW_FILE_CHOOSER
  file_selector = gtk_file_chooser_dialog_new(_("Scan resources"),
					      GTK_WINDOW(mainWindow),
					      GTK_FILE_CHOOSER_ACTION_OPEN,
					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					      GTK_STOCK_OPEN, GTK_RESPONSE_OK,
					      NULL);
  directory = getLastOpenDirectory();
  if (directory)
    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(file_selector), directory);

  gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(file_selector),
				       FALSE);
#else
  /* Create the selector */
  file_selector = gtk_file_selection_new (_("Please select a file"));
  gtk_window_set_title (GTK_WINDOW (file_selector), _("Scan resources"));
   
  /* made all the options */
  directory = getLastOpenDirectory();
  if (directory)
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(file_selector), directory);
  gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(file_selector));
  gtk_file_selection_set_select_multiple(GTK_FILE_SELECTION(file_selector), FALSE);
#endif

  gtk_widget_set_name(file_selector, "filesel");
  gtk_window_set_position(GTK_WINDOW(file_selector), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (file_selector), TRUE);

  if (gtk_dialog_run (GTK_DIALOG (file_selector)) == GTK_RESPONSE_OK)
    {
#if GTK_MINOR_VERSION > 3 && USE_NEW_FILE_CHOOSER
      filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER (file_selector));
      directory = (char*)gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER (file_selector));
#else
      filename = (char*)gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_selector));
      directory = (char*)g_path_get_dirname(filename);
#endif
      setLastOpenDirectory(directory);
      g_free(directory);
      gtk_widget_destroy (file_selector);
      /* Flush the gtk queue before loading the signal. */
      visuWaitFunction();

      /* Load the resources. */
      errorMessage = (GString*)0;
      res = visuConfigFileLoad(VISU_CONFIGFILE_RESOURCE, filename, &errorMessage);
      if (!res && errorMessage)
	raiseAlertDialogWithScrollView(errorMessage->str);
      if (errorMessage)
	g_string_free(errorMessage, TRUE);

      if (res)
	g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
		       0 , NULL);
#if GTK_MINOR_VERSION > 3 && USE_NEW_FILE_CHOOSER
      g_free (filename);
#endif
    }
  else
    {
#if GTK_MINOR_VERSION > 3 && USE_NEW_FILE_CHOOSER
      directory = (char*)gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER (file_selector));
      setLastOpenDirectory(directory);
      g_free(directory);
#else
      setLastOpenDirectory((char*)gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_selector)));
#endif
      gtk_widget_destroy (file_selector);
    }
}


void on_redrawButton_clicked(GtkButton *button, gpointer user_data)
{
  unsigned int w, h;

  checkWindowSize(&w, &h);
  setNewWindowSize(w, h);
  raiseRenderWindow();
  g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLForceReDraw_signal_id,
		 0 , NULL);
}


void on_observeButton_clicked(GtkButton *button, gpointer user_data)
{
  GtkWidget* radio;
  gint posx, posy;

  DBG_fprintf(stderr, "Gtk Main : get panel position (%d,%d).\n", posx, posy);
  gtk_window_get_position(GTK_WINDOW(mainWindow), &posx, &posy);
  visuHideWindow(GTK_WINDOW(mainWindow));
  /* At the first call, if the window does not exist, we create it. */
  if (!observeWindow)
    buildObserveAndPickWindow();
  DBG_fprintf(stderr, "Gtk Main : set observe position (%d,%d).\n", posx, posy);
  gtk_window_move(GTK_WINDOW(observeWindow), posx, posy);
  visuShowWindow(GTK_WINDOW(observeWindow));

  initPickObserve(marksAndMesures, stopPickObserveGtk);
  initPickMesure();

  radio = lookup_widget(observeWindow, "radioObserve");
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio)))
    {
      setSensitivePreciseObserve(TRUE);
      beginObserve();
    }
  else
    {
      setSensitivePreciseObserve(FALSE);
      beginPick();
    }
  flagDisableChangePickObserve = 0;
}
void stopPickObserveGtk()
{
  GtkWidget *wd;

  if (flagDisableChangePickObserve)
    return;

  wd = lookup_widget(observeWindow, "radioObserve");
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wd)))
    {
      wd = lookup_widget(observeWindow, "radioPick");
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wd), TRUE);
    }
  else
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wd), TRUE );
}


void on_closebutton1_clicked(GtkButton *button, gpointer user_data)
{
  GtkWidget *radio;

  radio = lookup_widget(observeWindow, "radioObserve");
  if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio)))
    endPickMesure();

  flagDisableChangePickObserve = 1;
  endPickObserve();

  visuHideWindow(GTK_WINDOW(observeWindow));
  visuShowWindow(GTK_WINDOW(mainWindow));
  cleanPickObserve();
}


void on_radioObserve_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
  GtkWidget *labelHelp;
  char *helpObserve = _("left-button : rotations\nControl-left-button : rotation of the camera\nShift-left-button : translations\nmiddle-button (along Y) or wheel : size\nShift-middle-button (along Y) or shift-wheel : perspective\nkey 'r' : reset\nright-button : switch to pick");
  char *helpPick = _("left-button  : standard pick\nmiddle-button: pick (1st) reference\n   Shift-middle-button: pick 2nd reference\nright-button : switch to observe");

  DBG_fprintf(stderr, "Gtk Main : Toggle of the radio button in observe window.\n");
  flagDisableChangePickObserve = 1;
  labelHelp = lookup_widget(observeWindow, "labelInfoObservePick");
  if (gtk_toggle_button_get_active(togglebutton))
    {
      unSetReferencePickMesure();
      cleanPickObserve();
      setSensitivePreciseObserve(TRUE);
      gtk_label_set_text(GTK_LABEL(labelHelp), helpObserve);
      beginObserve();
    }
  else
    {
      setSensitivePreciseObserve(FALSE);
      gtk_label_set_text(GTK_LABEL(labelHelp), helpPick);
      beginPick();
    }
  flagDisableChangePickObserve = 0;
}

void on_radioPick_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{

}


void on_spinTheta_value_changed(GtkSpinButton *spinbutton, gpointer user_data)
{
  int reDrawNeeded;

  if (disableCallbacks)
    return;

  g_signal_handler_block(G_OBJECT(visu),
			 signalCameraChangedId);
  reDrawNeeded = setCameraThetaPhiOmega((float)gtk_spin_button_get_value(spinbutton),
					0., 0., MASK_THETA);
  g_signal_handler_unblock(G_OBJECT(visu),
			   signalCameraChangedId);
  if (reDrawNeeded)
    g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
		   0 , NULL);
}

void on_spinOmega_value_changed(GtkSpinButton *spinbutton, gpointer user_data)
{
  int reDrawNeeded;

  if (disableCallbacks)
    return;

  g_signal_handler_block(G_OBJECT(visu),
			 signalCameraChangedId);
  reDrawNeeded = setCameraThetaPhiOmega(0., 0., (float)gtk_spin_button_get_value(spinbutton),
					MASK_OMEGA);
  g_signal_handler_unblock(G_OBJECT(visu),
			   signalCameraChangedId);
  if (reDrawNeeded)
    g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
		   0 , NULL);
}


void on_spinDx_value_changed(GtkSpinButton *spinbutton, gpointer user_data)
{
  int reDrawNeeded;

  if (disableCallbacks)
    return;

  g_signal_handler_block(G_OBJECT(visu),
			 signalCameraChangedId);
  reDrawNeeded =
    setCameraXsYs((float)gtk_spin_button_get_value(spinbutton), 0., MASK_XS);
  g_signal_handler_unblock(G_OBJECT(visu),
			   signalCameraChangedId);
  if (reDrawNeeded)
    g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
		   0 , NULL);
}


void on_spinGross_value_changed(GtkSpinButton *spinbutton, gpointer user_data)
{
  int reDrawNeeded;

  if (disableCallbacks)
    return;

  g_signal_handler_block(G_OBJECT(visu),
			 signalCameraChangedId);
  reDrawNeeded =
    setCameraGross((float)gtk_spin_button_get_value(spinbutton));
  g_signal_handler_unblock(G_OBJECT(visu),
			   signalCameraChangedId);
  if (reDrawNeeded)
    g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
		   0 , NULL);
}


void on_spinPhi_value_changed(GtkSpinButton *spinbutton, gpointer user_data)
{
  int reDrawNeeded;

  if (disableCallbacks)
    return;

  g_signal_handler_block(G_OBJECT(visu),
			 signalCameraChangedId);
  reDrawNeeded = setCameraThetaPhiOmega(0.,
					(float)gtk_spin_button_get_value(spinbutton),
					0., MASK_PHI);
  g_signal_handler_unblock(G_OBJECT(visu),
			   signalCameraChangedId);
  if (reDrawNeeded)
    g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
		   0 , NULL);
}


void on_spinDy_value_changed(GtkSpinButton *spinbutton, gpointer user_data)
{
  int reDrawNeeded;

  if (disableCallbacks)
    return;

  g_signal_handler_block(G_OBJECT(visu),
			 signalCameraChangedId);
  reDrawNeeded =
    setCameraXsYs(0., (float)gtk_spin_button_get_value(spinbutton), MASK_YS);
  g_signal_handler_unblock(G_OBJECT(visu),
			   signalCameraChangedId);
  if (reDrawNeeded)
    g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
		   0 , NULL);
}


void on_spinPersp_value_changed(GtkSpinButton *spinbutton, gpointer user_data)
{
  int reDrawNeeded;

  if (disableCallbacks)
    return;

  g_signal_handler_block(G_OBJECT(visu),
			 signalCameraChangedId);
  reDrawNeeded =
    setCameraPersp((float)gtk_spin_button_get_value(spinbutton));
  g_signal_handler_unblock(G_OBJECT(visu),
			   signalCameraChangedId);
  if (reDrawNeeded)
    g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
		   0 , NULL);
}


void on_visuAbout_clicked(GtkButton *button, gpointer user_data)
{
  if (!aboutWindow)
    buildAboutWindow();
  gtk_widget_show(aboutWindow);
}

void updateDumpProgressBar()
{
  gdouble val;

  gtk_progress_bar_set_text(dumpBar, _("Saving image..."));
  val = gtk_progress_bar_get_fraction(dumpBar);
  if (val + 0.01 <= 1.0 && val >= 0.)
    gtk_progress_bar_set_fraction(dumpBar, val + 0.01);
  visuWaitFunction();
}

void on_dumpButton_clicked(GtkButton *button, gpointer user_data)
{
  DumpFileDialog *dialogDump;
  char *filename;
  DumpType *format;
  int res;
  GString *buffer;
  GdkCursor *cursorWatch;
  GtkDialog *dialog;

  dialogDump = gtkMainCreate_dumpDialog();
  if (!gtkMainRun_dumpDialog(dialogDump))
    {
      gtkMainFree_dumpDialog(dialogDump);
      return;
    }

  filename = gtkMainGet_dumpFileName(dialogDump);
  format = gtkMainGet_dumpFileFormat(dialogDump);

  if (!format || !filename)
    {
      fprintf(stderr, "INTERNAL ERROR! The dump dialog doesn't"
	      " return the desired format nor the chosen name.\n");
      exit(1);
    }
  DBG_fprintf(stderr, "Gtk Callbacks : dump image to the file '%s' (format : %s)\n",
	      filename, format->fileType->description);
  if (format->writeFunc)
    {
      cursorWatch = gdk_cursor_new(GDK_WATCH);
      dumpBar = gtkMainGet_dumpProgressBar(dialogDump);
      dialog = gtkMainGet_dumpDialog(dialogDump);
      gtk_progress_bar_set_fraction(dumpBar, 0.);
      buffer = g_string_new(_("Dumping the view to image file"));
      g_string_append_printf(buffer, _(" (format '%s').\n"), format->fileType->description);
      g_string_append_printf(buffer, _("Write to '%s'.\n\n"), filename);
      gtk_progress_bar_set_text(dumpBar,
				_("Waiting for generating image in memory..."));
      visuWaitFunction();
      gdk_window_set_cursor(GDK_WINDOW(((GtkWidget*)dialog)->window), cursorWatch);
      res = format->writeFunc(buffer, filename, updateDumpProgressBar);
      gdk_window_set_cursor(GDK_WINDOW(((GtkWidget*)dialog)->window), NULL);
      if (res)
	raiseAlertDialog(buffer->str);
      g_string_free(buffer, TRUE);
    }
  gtkMainFree_dumpDialog(dialogDump);
}


void on_pairsButton_clicked(GtkButton *button, gpointer user_data)
{
  if (!pairsDialog)
    buildPairsDialog();

  pairsIsHide = 0;
  if (pairsIsOutOfDate)
    rebuildPairsViewportChild((GObject*)0, (gpointer)0);

  visuShowWindow(GTK_WINDOW(pairsDialog));
}


void on_closebutton3_clicked(GtkButton *button, gpointer user_data)
{
  visuHideWindow(GTK_WINDOW(pairsDialog));
  pairsIsHide = 1;
}

gboolean on_pairsDialog_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  pairsDialog = (GtkWidget*)0;
  return FALSE;
}


void
on_buttonOpenParameters_clicked        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_buttonOpenResources_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{

}
