/*   EXTRAITS DE LA LICENCE
	Copyright CEA, contributeurs : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)
  
	Adresse ml :
	BILLARD, non joignable par ml ;
	CALISTE, damien P caliste AT cea P fr.

	Ce logiciel est un programme informatique servant  visualiser des
	structures atomiques dans un rendu pseudo-3D. 

	Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
	respectant les principes de diffusion des logiciels libres. Vous pouvez
	utiliser, modifier et/ou redistribuer ce programme sous les conditions
	de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
	sur le site "http://www.cecill.info".

	Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
	pris connaissance de la licence CeCILL, et que vous en avez accept les
	termes (cf. le fichier Documentation/licence.fr.txt fourni avec ce logiciel).
*/

/*   LICENCE SUM UP
	Copyright CEA, contributors : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)

	E-mail address:
	BILLARD, not reachable any more ;
	CALISTE, damien P caliste AT cea P fr.

	This software is a computer program whose purpose is to visualize atomic
	configurations in 3D.

	This software is governed by the CeCILL  license under French law and
	abiding by the rules of distribution of free software.  You can  use, 
	modify and/ or redistribute the software under the terms of the CeCILL
	license as circulated by CEA, CNRS and INRIA at the following URL
	"http://www.cecill.info". 

	The fact that you are presently reading this means that you have had
	knowledge of the CeCILL license and that you accept its terms. You can
	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
*/
#include "dataFile.h"

#include <visu_object.h>
#include <visu_tools.h>
#include <visu_data.h>
#include <opengl.h>

int dataFile_used;
char* dataFile_file;
int dataFile_nbColumns;
float dataFile_vectB[3], dataFile_vectA[3];
int dataFile_scaleType;
float dataFile_minMaxValues[2];
int dataFile_colUsed[3];
int dataFile_colorType;

void dataFileReset(GObject* obj, gpointer data);
void colorFromUserData(float rgba[4], VisuElement *ele, VisuNode* node);
float dataFileGet_valuesFromData(int column, float fromVal);

int initDataFileModule()
{
  int i;

  dataFile_used = 0;
  dataFile_file = (char*)0;
  dataFile_nbColumns = 0;
  for (i = 0 ; i < 3 ; i++)
    {
      dataFile_vectA[i] = 1.;
      dataFile_vectB[i] = 0.;
      dataFile_colUsed[i] = -1;
    }
  dataFile_minMaxValues[0] = 0.;
  dataFile_minMaxValues[1] = 1.;
  dataFile_scaleType = dataFile_normalize;
  dataFile_colorType = dataFile_hsv;
  
  g_signal_connect(G_OBJECT(visu), "dataReadyForRendering",
		   G_CALLBACK(dataFileReset), (gpointer)1);
  g_signal_connect(G_OBJECT(visu), "dataNotReadyForRendering",
		   G_CALLBACK(dataFileReset), (gpointer)0);

  return 1;
}

void dataFileReDraw()
{
  visuData_createAllNodes();
  if (setObjectList(1))
    g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
		   0 , NULL);
}


void dataFileReset(GObject* obj, gpointer data)
{
  DBG_fprintf(stderr, "Visu dataFile : set data informations to NULL (%d).\n",
	      GPOINTER_TO_INT(data));
  dataFileSet_file((char*)0, (GString*)0, (int*)0);
  dataFileSet_used(0);
}

int dataFileSet_file(char* filename, GString *message, int *errors)
{
  int i, j, res, err, nb;
  FILE *file;
  char line[MAX_LINE_LENGTH];
  char *msg;
  int nAtome;
  float *data, *minMaxVal;
  gchar **dataRead;
  gchar *fileUTF8;

  DBG_fprintf(stderr, "Visu dataFile : set data file to '%s'.\n", filename);
  if (!errors)
    errors = &err;
  *errors = 0;

  /* erase previously stored dataFile informations. */
  if (dataFile_file)
    free(dataFile_file);
  dataFile_file = (char*)0;
  dataFile_nbColumns = 0;
  for (i = 0; i < 3; i++)
    dataFile_colUsed[i] = -1;

  if (currentVisuData)
    {
      /* We clear previously stored user data from the current
	 visuData structure. */
      if (currentVisuData->nodes)
	visuDataRemove_nodeProperty(currentVisuData, "externalData");
    }

  if (!filename)
    return 1;

  file = fopen(filename, "r");
  if (!file)
    {
      fileUTF8 = getStringInUTF8(filename);
      g_string_append_printf(message, _("WARNING! Can't find the '%s' data file.\n"),
			     fileUTF8);
      g_free(fileUTF8);
      return 0;
    }
  nAtome = 0;
  minMaxVal = (float*)0;
  visuDataAdd_nodeProperty(currentVisuData, "externalData", freeFloat);
  while (!feof(file))
    {
      msg = fgets(line, MAX_LINE_LENGTH, file);
      if (!msg)
	continue;
      if (line[0] == '#' || line[0] == '!' || line[0] == '\n')
	continue;
      /* Test the number of columns. */
      if (dataFile_nbColumns == 0)
	{
	  dataRead = g_strsplit_set(msg, " \t;:\n", MAX_LINE_LENGTH);
	  DBG_fprintf(stderr, "Visu dataFile : looking for number of columns.\n");
	  for (i = 0; dataRead[i]; i++)
	    if (dataRead[i][0])
	      {
		DBG_fprintf(stderr, "                 | one token : '%s'\n", dataRead[i]);
		dataFile_nbColumns++;
	      }
	  g_strfreev(dataRead);
	  DBG_fprintf(stderr, "Visu dataFile : detected number of columns : %d\n",
		      dataFile_nbColumns);
	  minMaxVal = malloc(sizeof(float) * 2 * dataFile_nbColumns);
	  if (!minMaxVal)
	    {
	      allocationProblems();
	      exit(1);
	    }
	  visuDataSet_property(currentVisuData, "externalData-MinMax", (gpointer)minMaxVal);
	}
      data = malloc(sizeof(float) * dataFile_nbColumns);
      if (!data)
	{
	  allocationProblems();
	  exit(1);
	}
      if (nAtome >= currentVisuData->nbOfAllStoredNodes)
	{
	  g_string_append_printf(message, _("WARNING! There are more data than nodes.\n"));
	  *errors = 1;
	  break;
	}
      dataRead = g_strsplit_set(msg, " \t;:\n", MAX_LINE_LENGTH);
      res = 1;
      nb = 0;
      for (i = 0; dataRead[i]; i++)
	{
	  if (dataRead[i][0])
	    {
	      res = res && (1 == sscanf(dataRead[i], "%f", data + nb));
	      DBG_fprintf(stderr, "                 | one value : '%s' -> '%f'\n",
			  dataRead[i], data[nb]);
	      nb += 1;
	    }
	}
      res = res && (nb == dataFile_nbColumns);
      g_strfreev(dataRead);
      if (res)
	{
	  /* Associates the values to the node. */
	  visuDataSet_nodeProperty(currentVisuData, currentVisuData->fromNumberToVisuNode[nAtome],
				   "externalData", (gpointer)data);
	  /* Check for minMax values for each column. */
	  for (i = 0; i < dataFile_nbColumns; i++)
	    {
	      if (nAtome == 0)
		{
		  minMaxVal[i * 2 + 0] = data[i];
		  minMaxVal[i * 2 + 1] = data[i];
		}
	      else
		{
		  if (data[i] < minMaxVal[i * 2 + 0])
		    minMaxVal[i * 2 + 0] = data[i];
		  if (data[i] > minMaxVal[i * 2 + 1])
		    minMaxVal[i * 2 + 1] = data[i];
		}
	    }
	  nAtome++;
	}
      else
	free(data);
    }
  fclose(file);
  if (nAtome < currentVisuData->nbOfAllStoredNodes)
    {
      g_string_append_printf(message, _("WARNING! There are less data than nodes.\n"));
      *errors = 1;
    }
  dataFile_file = malloc(sizeof(char) * (strlen(filename) + 1));
  if (!dataFile_file)
    {
      allocationProblems();
      exit(1);
    }
  strcpy(dataFile_file, filename);

  if (!dataFile_nbColumns)
    {
      g_string_append_printf(message, _("WARNING! Can't find any columns with numbers.\n"
					"Valid format are as much numbers as desired, separated"
					" by any of the following characters : [ ;:\\t].\n"));
      return 0;
    }

  if (DEBUG)
    {
      DBG_fprintf(stderr, "Visu dataFile : min/max values :\n");
      for (i = 0; i < dataFile_nbColumns; i++)
      DBG_fprintf(stderr, "Visu dataFile :  col %d -> %f (min) and %f (max).\n",
		  i, minMaxVal[i * 2 + 0], minMaxVal[i * 2 + 1]);
    }

  return dataFile_used;
}
char* dataFileGet_file()
{
  return dataFile_file;
}
gboolean dataFileGet_fileMinMaxFromColumn(float minMax[2], int column)
{
  float *minMaxTable;
  if (!dataFile_file || column < 0 || column >= dataFile_nbColumns || !minMax)
    {
      fprintf(stderr, "WARNING! 'dataFileGet_fileMinMaxFromColumn' has been called"
	      " with a wrong 'column' argument or no file has been set.\n");
      return FALSE;
    }

  minMaxTable = (float*)visuDataGet_property(currentVisuData, "externalData-MinMax");
  if (!minMaxTable)
    return FALSE;

  minMax[0] = minMaxTable[column * 2 + 0];
  minMax[1] = minMaxTable[column * 2 + 1];
  return TRUE;
}


int dataFileSet_used(int val)
{
  DBG_fprintf(stderr, "Visu dataFile : set using flag for dataFile module to %d .\n", val);

  if (val == dataFile_used)
    return 0;

  dataFile_used = val;

  if (dataFile_used)
    visuElementSet_updateNodesOnMaterialChange();
  else
    visuElementUnset_updateNodesOnMaterialChange();

  if (currentVisuData)
    {
      if (val)
	visuDataSet_ColorFunc(currentVisuData, colorFromUserData);
      else
	visuDataSet_ColorFunc(currentVisuData, (setColorFunc)0);
    }
  
  if (dataFile_file)
    return 1;
  else
    return 0;
}
int dataFileGet_used()
{
  return dataFile_used;
}

int dataFileSet_scaleType(int scale)
{
  if (scale < dataFile_normalize || scale > dataFile_minMax)
    {
      fprintf(stderr, "WARNING! Wrong parameter scale in call of 'dataFileSet_scaleType'.\n");
      return 0;
    }
  DBG_fprintf(stderr, "Visu dataFile : set the scale type to %d (previuosly %d).\n",
	      scale, dataFile_scaleType);
  if (scale == dataFile_scaleType)
    return 0;
  dataFile_scaleType = scale;

  return (dataFile_used && dataFile_file);
}
int dataFileGet_scaleType()
{
  return dataFile_scaleType;
}
int dataFileSet_min(float min)
{
  if (min >= dataFile_minMaxValues[1])
    {
      fprintf(stderr, "WARNING! Can't put a minimum value equal or greater than"
			" the maximum value.\n");
      return 0;
    }
  DBG_fprintf(stderr, "Visu dataFile : set the min value to %f (previuosly %f).\n",
	      min, dataFile_minMaxValues[0]);
  if (dataFile_minMaxValues[0] == min)
    return 0;
  dataFile_minMaxValues[0] = min;

  return (dataFile_used && dataFile_file);
}
int dataFileSet_max(float max)
{
  if (max <= dataFile_minMaxValues[0])
    {
      fprintf(stderr, "WARNING! Can't put a maximum value equal or smaller than"
	      " the minimum value.\n");
      return 0;
    }
  DBG_fprintf(stderr, "Visu dataFile : set the max value to %f (previuosly %f).\n",
	      max, dataFile_minMaxValues[1]);
  if (dataFile_minMaxValues[1] == max)
    return 0;
  dataFile_minMaxValues[1] = max;

  return (dataFile_used && dataFile_file);
}
float dataFileGet_min()
{
  return dataFile_minMaxValues[0];
}
float dataFileGet_max()
{
  return dataFile_minMaxValues[1];
}

float dataFileGet_valuesFromData(int column, float fromVal)
{
  float res;
  float* minmax;

  switch (dataFile_scaleType)
    {
    case dataFile_normalize:
      minmax = (float*)visuDataGet_property(currentVisuData, "externalData-MinMax");
      if (!minmax)
	{
	  fprintf(stderr, _("WARNING! the data file extra function is"
			    " not correctly initialized since the currentVisuData"
			    " has no 'externalData-MinMax' property.\n"));
	  return 0.;
	}
      if (column < 0 || column >= dataFile_nbColumns)
	{
	  fprintf(stderr, _("WARNING! 'dataFileGet_valuesFromData' has been"
			    " call with a wrong 'column' argument.\n"));
	  return 0.;
	}
      res = ( (fromVal - minmax[column * 2 + 0]) /
	      (minmax[column * 2 + 1] - minmax[column * 2 + 0]) );
      return res;
    case dataFile_minMax:
      res = ( (fromVal - dataFile_minMaxValues[0]) /
	      (dataFile_minMaxValues[1] - dataFile_minMaxValues[0]) );
      if (res > 1.)
	res = 1.;
      else if (res < 0.)
	res = 0.;
      return res;
    }
  return 0.;
}

int dataFileGet_nbColumns()
{
  return dataFile_nbColumns;
}

int dataFileSet_vectA(float val, int pos)
{
  if (pos < 0 || pos > 2)
    {
      fprintf(stderr, "WARNING! Wrong parameter pos in call of 'dataFileSet_vectA'.\n");
      return 0;
    }
  DBG_fprintf(stderr, "Visu dataFile : set the %d value of vectA to %f (previuosly %f).\n",
	      pos, val, dataFile_vectA[pos]);
  if (dataFile_vectA[pos] == val)
    return 0;
  dataFile_vectA[pos] = val;

  return (dataFile_used && dataFile_file);
}
int dataFileSet_vectB(float val, int pos)
{
  if (pos < 0 || pos > 2)
    {
      fprintf(stderr, "WARNING! Wrong parameter pos in call of 'dataFileSet_vectB'.\n");
      return 0;
    }
  DBG_fprintf(stderr, "Visu dataFile : set the %d value of vectB to %f (previuosly %f).\n",
	      pos, val, dataFile_vectB[pos]);
  if (dataFile_vectB[pos] == val)
    return 0;
  dataFile_vectB[pos] = val;

  return (dataFile_used && dataFile_file);
}
float* dataFileGet_vectA()
{
  float *val;
  int i;

  val = malloc(sizeof(float) * 3);
  if (!val)
    {
      allocationProblems();
      exit(1);
    }
  for (i = 0; i < 3; i++)
    val[i] = dataFile_vectA[i];

  return val;
}
float* dataFileGet_vectB()
{
  float *val;
  int i;

  val = malloc(sizeof(float) * 3);
  if (!val)
    {
      allocationProblems();
      exit(1);
    }
  for (i = 0; i < 3; i++)
    val[i] = dataFile_vectB[i];

  return val;
}
int dataFileSet_colUsed(int val, int pos)
{
  if (pos < 0 || pos > 2)
    {
      fprintf(stderr, "WARNING! Wrong parameter pos (%d) in call of 'dataFileSet_colUsed'.\n",
	      pos);
      return 0;
    }
  if (val >= dataFile_nbColumns && val != -1)
    {
      fprintf(stderr, "WARNING! Wrong parameter val (%d) in call of 'dataFileSet_colUsed'.\n",
	      val);
      return 0;
    }
  DBG_fprintf(stderr, "Visu dataFile : channel %d uses column %d (previuosly %d).\n",
	      pos, val, dataFile_colUsed[pos]);
  if (dataFile_colUsed[pos] == val)
    return 0;
  dataFile_colUsed[pos] = val;

  return (dataFile_used && dataFile_file);
}
int* dataFileGet_colUsed()
{
  int *val;
  int i;

  val = malloc(sizeof(int) * 3);
  if (!val)
    {
      allocationProblems();
      exit(1);
    }
  for (i = 0; i < 3; i++)
    val[i] = dataFile_colUsed[i];

  return val;
}
int dataFileSet_colorType(int color)
{
  if (color < dataFile_rgb || color > dataFile_hsv)
    {
      fprintf(stderr, "WARNING! Wrong parameter scale in call of 'dataFileSet_colorType'.\n");
      return 0;
    }
  DBG_fprintf(stderr, "Visu dataFile : set the color type to %d (previuosly %d).\n",
	      color, dataFile_colorType);
  if (color == dataFile_colorType)
    return 0;
  dataFile_colorType = color;

  return (dataFile_used && dataFile_file);
}
int dataFileGet_colorType()
{
  return dataFile_colorType;
}
void dataFileGet_valuesTransformedInRGB(float* valuesTransformed, float* valuesFrom)
{
  int i;

  for (i = 0; i < 3; i++)
    {
      valuesTransformed[i] = dataFile_vectA[i] * valuesFrom[i] + dataFile_vectB[i];
      valuesTransformed[i] = (valuesTransformed[i] < 0.)?0.:valuesTransformed[i];
      valuesTransformed[i] = (valuesTransformed[i] > 1.)?1.:valuesTransformed[i];
    }
  switch(dataFile_colorType)
    {
    case dataFile_rgb:
      return;
    case dataFile_hsv:
      HSVtoRGB(valuesTransformed, valuesTransformed);
      break;
    }
}

/*******************/
/* Color functions */
/*******************/

void colorFromUserData(float rgba[4], VisuElement *ele, VisuNode* node)
{
  float val[3];
  int i;
  float *storedValues;

  if (!node || !ele)
    {
      fprintf(stderr, "WARNING! 'hsvColorFromUserData' has been called with"
	      " a null node or ele parameter.\n");
      return;
    }

  storedValues = (float*)visuDataGet_nodeProperty(currentVisuData, node, "externalData");
  if (!storedValues)
    return;

  for (i = 0; i < 3; i++)
    if (dataFile_colUsed[i] == -1)
      val[i] = 1.;
    else
      {
	val[i] = dataFileGet_valuesFromData(dataFile_colUsed[i],
					    storedValues[dataFile_colUsed[i]]);
/* 	val[i] = dataFileGet_valuesFromData(((float*)node->userData)[dataFile_colUsed[i]]); */
      }

  dataFileGet_valuesTransformedInRGB(rgba, val);
  rgba[3] = ele->rgb[3];
}
