/*   EXTRAITS DE LA LICENCE
	Copyright CEA, contributeurs : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)
  
	Adresse ml :
	BILLARD, non joignable par ml ;
	CALISTE, damien P caliste AT cea P fr.

	Ce logiciel est un programme informatique servant  visualiser des
	structures atomiques dans un rendu pseudo-3D. 

	Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
	respectant les principes de diffusion des logiciels libres. Vous pouvez
	utiliser, modifier et/ou redistribuer ce programme sous les conditions
	de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
	sur le site "http://www.cecill.info".

	Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
	pris connaissance de la licence CeCILL, et que vous en avez accept les
	termes (cf. le fichier Documentation/licence.fr.txt fourni avec ce logiciel).
*/

/*   LICENCE SUM UP
	Copyright CEA, contributors : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)

	E-mail address:
	BILLARD, not reachable any more ;
	CALISTE, damien P caliste AT cea P fr.

	This software is a computer program whose purpose is to visualize atomic
	configurations in 3D.

	This software is governed by the CeCILL  license under French law and
	abiding by the rules of distribution of free software.  You can  use, 
	modify and/ or redistribute the software under the terms of the CeCILL
	license as circulated by CEA, CNRS and INRIA at the following URL
	"http://www.cecill.info". 

	The fact that you are presently reading this means that you have had
	knowledge of the CeCILL license and that you accept its terms. You can
	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
*/
#include "plane.h"

#include <stdlib.h>
#include <math.h>

#include <opengl.h>
#include <visu_object.h>

int comparePolygonPoint(gconstpointer pointA, gconstpointer pointB, gpointer data);


struct Plane_struct
{
  /* Normal vector, unitary. */
  float nVect[3];
  /* Normal vector, user given. */
  float nVectUser[3];

  /* Distance between origin and intersection of the plane
     and the line made by origin and normal vector. */
  float dist;

  /* Color of that plane. */
  Color *color;

  /* Internal variables */
  /* Intersections with the bounding box.
     Consist of a GList of float[3], can be NULL if
     there is no interstection. */
  GList *inter;
  /* Isobarycenter G of all intersection points, required to order
     these points to form a convex polygon. */
  float pointG[3];
  /* The plane can hide the nodes on one of its side.
     This variable can be PLANE_SIDE_PLUS or PLANE_SIDE_MINUS or
     PLANE_SIDE_NONE. It codes the side of the plane which hides the nodes.
     If PLANE_SIDE_NONE is selected all nodes are rendered. */
  int hiddenSide;
};

Plane* planeNew(float vect[3], float dist, Color *color)
{
  Plane *plane;
  int res;

  if (!color)
    {
      fprintf(stderr, "WARNING! Can not create a new plane without any color.\n");
      return (Plane*)0;
    }

  plane = malloc(sizeof(Plane));
  if (!plane)
    {
      allocationProblems();
      return (Plane*)0;
    }
  DBG_fprintf(stderr, "Visu Planes : creating a new plane (%p).\n", (gpointer) plane);
  plane->inter = (GList*)0;
  plane->hiddenSide = PLANE_SIDE_NONE;
  plane->nVectUser[0] = 0.;
  plane->nVectUser[1] = 0.;
  plane->nVectUser[2] = 0.;
  plane->dist = 0.;
  res = (planeSet_normalVector(plane, vect) < 0);
  res = (planeSet_distanceFromOrigin(plane, dist) < 0) || res;
  res = (planeSet_color(plane, color) < 0) || res;
  res = planeComputeInter(plane) || res;
  
  if (res)
    {
      planeFree(plane);
      return (Plane*)0;
    }

  return plane;
}

void planeFree(Plane* plane)
{
  GList *tmpLst;

  if (!plane)
    {
      fprintf(stderr, "WARNING! '%s' has been called with a null value for parameter '%s'.\n",
	      "planeFree", "plane");
      return;
    }
  tmpLst = plane->inter;
  while (tmpLst)
    {
      free(tmpLst->data);
      tmpLst = g_list_next(tmpLst);
    }
  free(plane);
}

int planeSet_normalVector(Plane *plane, float vect[3])
{
  int i;
  float norm;

  if (!plane)
    {
      fprintf(stderr, "WARNING! '%s' has been called with a null value for parameter '%s'.\n",
	      "planeSet_normalVector", "plane");
      return -1;
    }
  if (vect[0] == plane->nVectUser[0] &&
      vect[1] == plane->nVectUser[1] &&
      vect[2] == plane->nVectUser[2])
    return 0;
  if (vect[0] * vect[0] + vect[1] * vect[1] + vect[2] * vect[2] == 0.)
    {
      fprintf(stderr, "WARNING! '%s' has been called with a null value for parameter '%s'.\n",
	      "planeSet_normalVector", "vect");
      return -1;
    }

  norm = 0.;
  for (i = 0; i < 3; i++)
    {
      norm += vect[i] * vect[i];
      plane->nVect[i] = vect[i];
      plane->nVectUser[i] = vect[i];
    }
  norm = sqrt(norm);
  for (i = 0; i < 3; i++)
    plane->nVect[i] /= norm;

  DBG_fprintf(stderr, "Visu Plane : set normal vector (%f,%f,%f) for plane %p.\n",
	      plane->nVect[0], plane->nVect[1], plane->nVect[2], (gpointer)plane);
  return 1;
}

int planeSet_distanceFromOrigin(Plane *plane, float dist)
{
  if (!plane)
    {
      fprintf(stderr, "WARNING! '%s' has been called with a null value for parameter '%s'.\n",
	      "planeSet_distanceFromOrigin", "plane");
      return -1;
    }
  if (plane->dist == dist)
    return 0;

  plane->dist = dist;
  DBG_fprintf(stderr, "Visu Plane : set distance from origin %f for plane %p.\n",
	      dist, (gpointer)plane);
  return 1;
}

int planeSet_color(Plane *plane, Color *color)
{
  if (!plane)
    {
      fprintf(stderr, "WARNING! '%s' has been called with a null value for parameter '%s'.\n",
	      "planeSet_color", "plane");
      return -1;
    }
  if (color == plane->color)
    return 0;

  plane->color = color;
  return 1;
}

int planeComputeInter(Plane* plane)
{
  float lambda, denom;
  float l[12][3], a[12][3];
  int i, j, n;
  float *inter;
  GList *tmpLst;

  if (!plane)
    {
      fprintf(stderr, "WARNING! '%s' has been called with a null value for parameter '%s'.\n",
	      "planeComputeInter", "plane");
      return 1;
    }
  /* RAZ old intersection list. */
  if (plane->inter)
    {
      tmpLst = plane->inter;
      while(tmpLst)
	{
	  free((float*)tmpLst->data);
	  tmpLst = g_list_next(tmpLst);
	}
      g_list_free(plane->inter);
      plane->inter = (GList*)0;
    }
  /* Compute, vector and position for the box. */
  for (i = 0; i < 3; i++)
    {
      l[0][i] = visuBox->p2[i] - visuBox->p1[i];
      l[1][i] = visuBox->p3[i] - visuBox->p2[i];
      l[2][i] = visuBox->p4[i] - visuBox->p3[i];
      l[3][i] = visuBox->p1[i] - visuBox->p4[i];
      l[4][i] = visuBox->p5[i] - visuBox->p1[i];
      l[5][i] = visuBox->p6[i] - visuBox->p2[i];
      l[6][i] = visuBox->p7[i] - visuBox->p3[i];
      l[7][i] = visuBox->p8[i] - visuBox->p4[i];
      l[8][i] = visuBox->p6[i] - visuBox->p5[i];
      l[9][i] = visuBox->p7[i] - visuBox->p6[i];
      l[10][i] = visuBox->p8[i] - visuBox->p7[i];
      l[11][i] = visuBox->p5[i] - visuBox->p8[i];
      a[0][i] = visuBox->p1[i];
      a[1][i] = visuBox->p2[i];
      a[2][i] = visuBox->p3[i];
      a[3][i] = visuBox->p4[i];
      a[4][i] = visuBox->p1[i];
      a[5][i] = visuBox->p2[i];
      a[6][i] = visuBox->p3[i];
      a[7][i] = visuBox->p4[i];
      a[8][i] = visuBox->p5[i];
      a[9][i] = visuBox->p6[i];
      a[10][i] = visuBox->p7[i];
      a[11][i] = visuBox->p8[i];
    }
  n = 0;
  for (i = 0; i < 3; i++)
    plane->pointG[i] = 0.;
  /*
    The plane is defined by n1x+n2y+n3z-d=0 with (n1,n2,n3) the normal vector (unitary)
    and d the algebric distance from the origin.
    A segment {P} of the box is defined by P=A+lambda.l with A a vertex, l a vector
    in the direction of the segment and lambda a real.
    If it exists a lambda that can solve P(lambda) in the plane equation, and this lambda
    is in [0;1] then this is the intersection.
   */
  for (i = 0; i < 12; i++)
    {
      denom = 0.;
      for (j = 0; j < 3; j++)
	denom += plane->nVect[j] * l[i][j];
      if (denom != 0.)
	{
	  lambda = plane->dist;
	  for (j = 0; j < 3; j++)
	    lambda -= plane->nVect[j] * a[i][j];
	  lambda /= denom;
	  if (lambda >= 0. && lambda <= 1.)
	    {
	      inter = malloc(sizeof(float) * 3);
	      if (!inter)
		{
		  allocationProblems();
		  exit(1);
		}
	      for (j = 0; j < 3; j++)
		{
		  inter[j] = a[i][j] + lambda * l[i][j];
		  plane->pointG[j] += inter[j];
		}
	      n += 1;
	      plane->inter = g_list_append(plane->inter, (gpointer)inter);
	      DBG_fprintf(stderr, "Visu Plane : a new intersection (%f,%f,%f) for plane %p.\n",
			  inter[0], inter[1], inter[2], (gpointer)plane);
	    }
	}
    }
  if (n > 0)
    {
      for (i = 0; i < 3; i++)
	plane->pointG[i] /= (float)n;
      plane->inter = g_list_sort_with_data(plane->inter, comparePolygonPoint, (gpointer)plane);
    }

  return 0;
}

void planeDraw(Plane* plane)
{
  GList *tmpLst;
  float mat[5] = {1., 0.0, 1.0, 0.0, 0.0};

  if (!plane)
    {
      fprintf(stderr, "WARNING! '%s' has been called with a null value for parameter '%s'.\n",
	      "planeDraw", "plane");
      return;
    }

  tmpLst = plane->inter;
  if (tmpLst)
    {
      glDisable(GL_CULL_FACE);
      setOpenGlMaterial(mat, plane->color->rgba);
      glPushMatrix();
      glTranslated(-visuBox->dxxs2, -visuBox->dyys2, -visuBox->dzzs2);
      glBegin(GL_POLYGON);
      while (tmpLst)
	{
	  glVertex3f(((float*)tmpLst->data)[0],
		     ((float*)tmpLst->data)[1],
		     ((float*)tmpLst->data)[2]);
	  tmpLst = g_list_next(tmpLst);
	}
      glEnd();
      glPopMatrix();  
      glEnable(GL_CULL_FACE);
      glCullFace(GL_BACK);
    }
}



int planeShowHide(Plane *plane, VisuData *visuData, gboolean hide, float side)
{
  int i, j, k;
  int reDraw;
  float point[3];
  float pScal;

  if (!plane || !visuData)
    {
      fprintf(stderr, "WARNING! '%s' has been called with a null"
	      " value for parameter '%s' or '%s'.\n",
	      "planeShowHide", "plane", "visuData");
      return 0;
    }
  
  reDraw = 0.;
  for (i = 0; i < visuData->ntype; i++)
    for(j = 0; j < visuData->numberOfStoredNodes[i]; j++)
    {
      if (hide)
	{
	  point[0] = (float)visuData->nodes[i][j].x;
	  point[1] = (float)visuData->nodes[i][j].y;
	  point[2] = (float)visuData->nodes[i][j].z;
	  pScal = 0.;
	  for (k = 0; k < 3; k++)
	    pScal += plane->nVect[k] * (point[k] - plane->pointG[k]);
	  if (pScal * side <= 0.)
	    reDraw = visuNodeSet_visibility(&visuData->nodes[i][j], FALSE) || reDraw;
	  else
	    reDraw = visuNodeSet_visibility(&visuData->nodes[i][j], TRUE) || reDraw;
	}
      else
	reDraw = visuNodeSet_visibility(&visuData->nodes[i][j], TRUE) || reDraw;
    }
  if (reDraw)
    {
      g_signal_emit (visu, VISU_GET_CLASS (visu)->NodeRenderedChanged_signal_id,
		     0 , (gpointer)0, NULL);
      visuData_createAllNodes();
      return 1;
    }
  else
    return 0;
}

int comparePolygonPoint(gconstpointer pointA, gconstpointer pointB, gpointer data)
{
  Plane *plane;
  float vectGA[3], vectGB[3];
  int i;
  float det;

  plane = (Plane*)data;
  for (i = 0; i < 3; i++)
    {
      vectGA[i] = ((float*)pointA)[i] - plane->pointG[i];
      vectGB[i] = ((float*)pointB)[i] - plane->pointG[i];
    }
  det = vectGA[0] * vectGB[1] * plane->nVect[2] + vectGB[0] * plane->nVect[1] * vectGA[2] +
    plane->nVect[0] * vectGA[1] * vectGB[2] - vectGA[2] * vectGB[1] * plane->nVect[0] -
    vectGA[1] * vectGB[0] * plane->nVect[2] - vectGA[0] * vectGB[2] * plane->nVect[1];
  if (det < 0.)
    return -1;
  else if (det > 0.)
    return 1;
  else
    return 0;
}

void planeGet_nVectUser(Plane *plane, float *vect)
{
  int i;

  if (!plane)
    {
      fprintf(stderr, "WARNING! '%s' has been called with a null value for parameter '%s'.\n",
	      "planeComputeInter", "plane");
      return;
    }
  for (i = 0; i < 3; i++)
    vect[i] = plane->nVectUser[i];
}

void planeGet_color(Plane *plane, Color **color)
{
  if (!plane)
    {
      fprintf(stderr, "WARNING! '%s' has been called with a null value for parameter '%s'.\n",
	      "planeGet_color", "plane");
      return;
    }
  *color = plane->color;
}

void planeGet_distanceFromOrigin(Plane *plane, float *dist)
{
  if (!plane)
    {
      fprintf(stderr, "WARNING! '%s' has been called with a null value for parameter '%s'.\n",
	      "planeComputeInter", "plane");
      return;
    }
  *dist = plane->dist;
}

int planeSet_hiddenState(Plane *plane, int side)
{
  if (!plane ||
      (side != PLANE_SIDE_NONE &&
       side != PLANE_SIDE_PLUS &&
       side != PLANE_SIDE_MINUS))
    {
      fprintf(stderr, "WARNING! 'planeSet_hiddenState' has been called with a null"
	      " value for argument 'plane' or has a wrong 'side' argument'.\n");
      return 0;
    }
  DBG_fprintf(stderr, "Visu Plane : hide state (%d, old %d) for plane %p.\n",
	      side, plane->hiddenSide, (gpointer)plane);
  if (plane->hiddenSide == side)
    return 0;
  plane->hiddenSide = side;
  return 1;
}
int planeGet_hiddenState(Plane *plane)
{
  if (!plane)
    {
      fprintf(stderr, "WARNING! 'planeGet_hiddenState' has been called with a null"
	      " value for argument 'plane'.\n");
      return 0;
    }
  return plane->hiddenSide;
}
int planeShowHide_all(VisuData *visuData, GList *listOfPlanes)
{
  int i, j, k;
  int reDraw;
  float point[3];
  float pScal;
  Plane *plane;

  reDraw = 0.;
  /* Set all nodes visibility to true. */
  for (i = 0; i < visuData->ntype; i++)
    for(j = 0; j < visuData->numberOfStoredNodes[i]; j++)
      reDraw = visuNodeSet_visibility(&visuData->nodes[i][j], TRUE ) || reDraw;
  /* Hide node depending on plane status. */
  while(listOfPlanes)
    {
      plane = ((Plane*)listOfPlanes->data);
      if (plane->hiddenSide != PLANE_SIDE_NONE)
	{
	  for (i = 0; i < visuData->ntype; i++)
	    for(j = 0; j < visuData->numberOfStoredNodes[i]; j++)
	      {
		visuDataGet_nodePosition(visuData, &visuData->nodes[i][j], point);
/* 		point[0] = (float)visuData->nodes[i][j].x; */
/* 		point[1] = (float)visuData->nodes[i][j].y; */
/* 		point[2] = (float)visuData->nodes[i][j].z; */
		pScal = 0.;
		for (k = 0; k < 3; k++)
		  pScal += plane->nVect[k] * (point[k] - plane->pointG[k]);
		if (pScal * plane->hiddenSide > 0.)
		  reDraw = visuNodeSet_visibility(&visuData->nodes[i][j], FALSE) || reDraw;
	      }
	}
      listOfPlanes = g_list_next(listOfPlanes);
    }
  if (reDraw)
    {
      g_signal_emit (visu, VISU_GET_CLASS (visu)->NodeRenderedChanged_signal_id,
		     0 , (gpointer)0, NULL);
      return 1;
    }
  else
    return 0;
}
