/*   EXTRAITS DE LA LICENCE
	Copyright CEA, contributeurs : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)
  
	Adresse ml :
	BILLARD, non joignable par ml ;
	CALISTE, damien P caliste AT cea P fr.

	Ce logiciel est un programme informatique servant  visualiser des
	structures atomiques dans un rendu pseudo-3D. 

	Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
	respectant les principes de diffusion des logiciels libres. Vous pouvez
	utiliser, modifier et/ou redistribuer ce programme sous les conditions
	de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
	sur le site "http://www.cecill.info".

	Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
	pris connaissance de la licence CeCILL, et que vous en avez accept les
	termes (cf. le fichier Documentation/licence.fr.txt fourni avec ce logiciel).
*/

/*   LICENCE SUM UP
	Copyright CEA, contributors : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)

	E-mail address:
	BILLARD, not reachable any more ;
	CALISTE, damien P caliste AT cea P fr.

	This software is a computer program whose purpose is to visualize atomic
	configurations in 3D.

	This software is governed by the CeCILL  license under French law and
	abiding by the rules of distribution of free software.  You can  use, 
	modify and/ or redistribute the software under the terms of the CeCILL
	license as circulated by CEA, CNRS and INRIA at the following URL
	"http://www.cecill.info". 

	The fact that you are presently reading this means that you have had
	knowledge of the CeCILL license and that you accept its terms. You can
	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
*/
/*
 * Initial main.c file generated by Glade. Edit as required.
 * Glade will not overwrite this file.
 */

#include <gtk/gtk.h>
#include <glib.h>
#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h> /* For the access markers R_OK, W_OK ... */

#include "interface.h"
#include "support.h"
#include "callbacks.h"
#include "gtk_main.h"
#include "gtk_observePick.h"
#include "gtk_pairs.h"
#include "gtk_about.h"
#include "extraGtkFunctions/gtk_colorComboBoxWidget.h"

#include "panelModules/externalModules.h"
#include "panelModules/externalRenderingMethods_gtk.h"
#include "renderingMethods/renderingAtomic.h"
#include "renderingMethods/renderingSpin.h"

#include "visu_object.h"
#include "visu_path.h"
#include "visu_rendering.h"
#include "visu_data.h"
#include "visu_configFile.h"
#include "visu_basic.h"            /* To have loadDataFromFile */
#include "visu_pairs.h"            /* To have setPairsOnOff */
#include "visu_commandLine.h"      /* To have getArgFilename and getXWindowGeometry*/
#include "OSOpenGL/visu_OSInterface.h"
#include "renderingBackend/visu_windowInterface.h"


/****************/
/* Private part */
/****************/

/* Initialisation */
int initGtkMain();
void createCallBacksMain();
gboolean initVisuPostGTK(gpointer data);

/* stocke le nombre d'lments de la liste, ce qui vite de
   reparcourir la liste pour le connatre. */
int nbElementsListeSubPanel;

/* Fuctions to rule the combobox. */
void initSubPanel();
void initListeSubPanel();

/* Fuctions to rule the notebook. */
void freeSubPanel(subPanel* panel);
void makeSubPanel(gpointer data, gpointer user_data);

/* pointer on different buttons. */
GtkWidget *loadButton;
GtkWidget *redrawButton;
GtkWidget *pairsButton;
GtkWidget *dumpButton;
GtkWidget *pickObserveButton;

/* Local callbacks */
void checkPairsChanged(GtkToggleButton *togglebutton, gpointer user_data);
void setRenderingButtonSensitive(GObject *obj, gpointer bool);
void setFileButtonSensitive(GObject *obj, gpointer bool);
void onResourcesLoaded(GObject *visu, gpointer data);

/* Parameter to store the position of windows. */
int rememberWindowPosition;
GHashTable *windowPosition;
#define PARAMETER_GTKMAIN_REMEMBER_DEFAULT 1

/* Store the last open directory. It is initialised
   to current working directory. */
gchar *lastOpenDirectory;


/* Store some informations on gtk method associated
   with different rendering methods. */
struct renderingMethod_gtkMain_struct
{
  createGtkWidgetFunc createOpenAddOn;
  createGtkLoadWidgetFunc createLoadAction;
};
GHashTable *renderingMethod_gtkMain;

/* Dumping structure */
struct DumpFileDialog_struct
{
  GtkWidget *progressBar;

  GtkWidget *comboType;
  DumpType *selectedFileFormat;

  GtkWidget *fileChooser;
  GtkWidget *hBoxFileFormat;

  GtkWidget *dialog;
  GtkWidget *cancelButton;

  gchar *dumpFileName;
};



int mainWithGtk(int argc, char *argv[])
{
  int res;
  int width, height;

  gtk_init (&argc, &argv);

  /* Rgle l'accs aux images */
  add_pixmap_directory(v_sim_pixmaps_dir);

  DBG_fprintf(stderr,"--- Initialising the rendering window ---\n");
  commandLineGet_XWindowGeometry(&width, &height);
  res = initRenderingWindow("V_Sim", "V_Sim", "v_sim_render", "V_Sim",
			    width, height);
  if (!res)
    exit(1);

  initVisuMain();

  DBG_fprintf(stderr,"--- Initialising the new gtk classes ---\n");
  /* Force the creation of the ColorComboBoxClass. */
  g_type_class_ref(colorComboBox_get_type());

  initGtkMain();

  /* Others elements linked to the main window. */
  DBG_fprintf(stderr, " | Gtk Pick and Observe");
  initObserveAndPickWindow();
  DBG_fprintf(stderr, " ... OK.\n");
  DBG_fprintf(stderr, " | Gtk Pairs");
  initPairsWindow();
  DBG_fprintf(stderr, " ... OK.\n");
  DBG_fprintf(stderr, " | Gtk About");
  initAboutWindow();
  DBG_fprintf(stderr, " ... OK.\n");

  DBG_fprintf(stderr, "GTK Main : adding loading parameters and resources"
	      " callbacks in the queue.\n");
  g_idle_add(initVisuPostGTK, (gpointer)0);

  createCallBacksMain();
  DBG_fprintf(stderr, "GTK Main : Creating callbacks.\n");

  DBG_fprintf(stderr, "Gtk Main : starting main GTK loop.\n");
  gtk_main ();

  return 0;
}

int initGtkMain()
{
  int i;

  /* Init the signal object for gtk stuffs. */
  visuGtkObject = g_object_new(VISU_GTK_TYPE, NULL);

  /* Set local variables */
  /* Retrieve the current working directory. */
  setLastOpenDirectory((char*)g_get_current_dir());
  /* Create the hashtable to store windows and their position. */
  windowPosition = g_hash_table_new_full(g_direct_hash,
					 g_direct_equal,
					 NULL, freeInt);
  setRememberPosition(PARAMETER_GTKMAIN_REMEMBER_DEFAULT);

  /* Create the hashtable of rendering method gtk functions. */
  renderingMethod_gtkMain = g_hash_table_new_full(g_direct_hash,
						  g_direct_equal,
						  NULL, freeGPointer);
  for (i = 0; listInitRendenringGtkFunc[i]; i++)
    listInitRendenringGtkFunc[i]();

  /* Initialise dependant gtk elements. */
  DBG_fprintf(stderr, "--- Initialising the GTK interface ---\n");
  DBG_fprintf(stderr, " | Gtk Main");
  mainWindow = create_main_window ();
  gtk_window_set_wmclass(GTK_WINDOW(mainWindow), "v_sim_commandPanel", "V_Sim");
  DBG_fprintf(stderr, " ... OK.\n");
  /* Signals and naming of widgets specific to the main window. */
  comboListeSubPanel = lookup_widget(mainWindow, "comboListeSubPanel");
  tabSubPanel = lookup_widget(mainWindow, "subPanel");
  loadButton = lookup_widget(mainWindow, "loadButton");
  redrawButton = lookup_widget(mainWindow, "redrawButton");
  pairsButton = lookup_widget(mainWindow, "pairsButton");
  pickObserveButton = lookup_widget(mainWindow, "pickObserveButton");
  dumpButton = lookup_widget(mainWindow, "dumpButton");
  /* init the sub panel contains. */
  initSubPanel();

  return 1;
}

void createCallBacksMain()
{
  GtkWidget *wd;

  /* Initialise the interface. */
  if (currentSubPanel->onEnter)
    currentSubPanel->onEnter((gpointer)0);
  gtk_widget_set_name(comboListeSubPanel, "combo");
  gtk_widget_set_name(GTK_COMBO(comboListeSubPanel)->entry, "comboEntry");
  wd = lookup_widget(mainWindow, "labelShowSub");
  gtk_widget_set_name(wd, "subpanel_title");
  wd = lookup_widget(mainWindow, "checkPairs");
  g_signal_connect(G_OBJECT(wd), "toggled",
		   G_CALLBACK(checkPairsChanged), (gpointer)0);

  g_signal_connect(G_OBJECT(visu), "dataReadyForRendering",
		   G_CALLBACK(setRenderingButtonSensitive),
		   (gpointer)1);
  g_signal_connect(G_OBJECT(visu), "dataNotReadyForRendering",
		   G_CALLBACK(setRenderingButtonSensitive),
		   (gpointer)0);
  g_signal_connect(G_OBJECT(visu), "renderingChanged",
		   G_CALLBACK(setFileButtonSensitive),
		   (gpointer)0);
  g_signal_connect(G_OBJECT(visu), "resourcesLoaded",
		   G_CALLBACK(onResourcesLoaded), (gpointer)0);
}

gboolean initVisuPostGTK(gpointer data)
{
  char* path;
  int res;
  char *filename, *spin_filename;
  GString *message;
  gchar *fileName, *dirname, *normDir;
  GtkWidget *wd;
  VisuData *newData;
  GString *errorMessage;

  DBG_fprintf(stderr,"Gtk Main : attach the OpenGL context to the rendering window.\n");
  res = setOpenGLContext(FALSE, (DumpImage**)0);
  if (res)
    {
      fprintf(stderr, "INTERNAL ERROR! Cannot set the OpenGL context"
	      " to the default rendering window.\n");
      exit(1);
    }

  DBG_fprintf(stderr,"--- Initialising the parameters ---\n");
  /* Save the gtk tag as a known tag to allow to read such parameters. */
  visuConfigFileAdd_knownTag("gtk");
  /* Look for parameters file */
  path = visuConfigFileGet_validPath(VISU_CONFIGFILE_PARAMETER, R_OK, 0);
  if (path)
    {
      errorMessage = (GString*)0;
      res = visuConfigFileLoad(VISU_CONFIGFILE_PARAMETER, path, &errorMessage);
      if (!res && errorMessage)
	raiseAlertDialogWithScrollView(errorMessage->str);
      if (errorMessage)
	g_string_free(errorMessage, TRUE);
      free(path);
    }
  else
    raiseAlertDialog("WARNING! Unable to find a valid parameters file. Starting with defaults.");

  DBG_fprintf(stderr,"--- Initialising resources ---\n");
  /* Look for resources file */
  path = visuConfigFileGet_validPath(VISU_CONFIGFILE_RESOURCE, R_OK, 0);
  if (path)
    {
      errorMessage = (GString*)0;
      res = visuConfigFileLoad(VISU_CONFIGFILE_RESOURCE, path, &errorMessage);
      if (!res && errorMessage)
	raiseAlertDialogWithScrollView(errorMessage->str);
      if (errorMessage)
	g_string_free(errorMessage, TRUE);
      free(path);
    }

  newData = (VisuData*)0;
  filename = commandLineGet_ArgFilename();
  spin_filename = commandLineGet_ArgSpinFileName();
  if (spin_filename && filename)
    {
      setRenderingMethodInUse(pointerOnRenderingSpinMethod);
      newData = visuDataNew();
      if (!newData)
	exit(1);

      dirname = g_path_get_dirname(filename);
      normDir = normalize_path(dirname);
      setLastOpenDirectory((char*)normDir);
      g_free(dirname);
      g_free(normDir);

      visuDataAdd_file(newData, filename, FILE_KIND_POSITION);
      visuDataAdd_file(newData, spin_filename, FILE_KIND_SPIN);
      free(spin_filename);
      free(filename);
      spin_filename = (char*)0;
      filename = (char*)0;

      DBG_fprintf(stderr, "GTK Main : adding the load call back in the queue.\n");
      g_idle_add(loadFileWithGtk, (gpointer)newData);
    }
  else if (filename)
    {
      setRenderingMethodInUse(pointerOnRenderingAtomicMethod);
      newData = visuDataNew();
      if (!newData)
	exit(1);

      dirname = g_path_get_dirname(filename);
      normDir = normalize_path(dirname);
      setLastOpenDirectory((char*)normDir);
      g_free(dirname);
      g_free(normDir);

      visuDataAdd_file(newData, filename, FILE_KIND_POSITION);
      free(filename);
      filename = (char*)0;

      DBG_fprintf(stderr, "GTK Main : adding the load call back in the queue.\n");
      g_idle_add(loadFileWithGtk, (gpointer)newData);
    }
  else
    setRenderingButtonSensitive((GObject*)0, (gpointer)0);
  if (spin_filename)
    free(spin_filename);
  if (filename)
    free(filename);

  /* Initialise the first subPanel now that resources have been loaded. */
  if (currentSubPanel->onEnter)
    currentSubPanel->onEnter((gpointer)0);
  
  /* Initialise the buttons of the main interface. */
  wd = lookup_widget(mainWindow, "checkPairs");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wd),
			       (gboolean)getPairsOnOff());


  gtk_widget_show (mainWindow);

  return FALSE;
}

/**********************************/
/* Functions to rule the combobox */
/**********************************/
gint lookForNameInComboList(gconstpointer a, gconstpointer b)
{
  return strcmp((char*)a, (char*)b);
}



/**********************************/
/* Fuctions to rule the notebook. */
/**********************************/
void freeSubPanel(subPanel* panel)
{
  if (panel)
    {
      if (panel->comboLabel)
	free(panel->comboLabel);
      if (panel->tabLabel)
	free(panel->tabLabel);
      free(panel);
    }
}
subPanel* createSubPanel(char* name, char *tabName, GtkWidget *child,
			 onChangePanelFunc enter, onChangePanelFunc leave)
{
  subPanel* panel;

  /* allocate the structure */
  panel = malloc(sizeof(subPanel));
  if (!panel)
    {
      allocationProblems();
      return (subPanel*)0;
    }
    

  /* initialise the values. */
  if (!name || !name[0] || !tabName || !tabName[0] || !child)
    return (subPanel*)0;
  /* allocate the variables */
  panel->comboLabel = malloc(sizeof(char) * (strlen(name) + 1));
  if (!panel->comboLabel)
    {
      allocationProblems();
      freeSubPanel(panel);
      return (subPanel*)0;
    }
  panel->tabLabel = malloc(sizeof(char) * (strlen(tabName) + 1));
  if (!panel->tabLabel)
    {
      allocationProblems();
      freeSubPanel(panel);
      return (subPanel*)0;
    }
  /* initialise the values. */
  strcpy(panel->comboLabel, name);
  strcpy(panel->tabLabel, tabName);
  panel->icon = (GtkWidget*)0;
  panel->child = child;
  panel->onEnter = enter;
  panel->onLeave = leave;
  
  return panel;
}
subPanel* createSubPanelWithIconFromPath(char* name, char *tabName,
					 char* iconPath, GtkWidget *child,
					 onChangePanelFunc enter, onChangePanelFunc leave)
{
  subPanel *panel;

  panel = createSubPanel(name, tabName, child, enter, leave);
  if (!panel)
    return (subPanel*)0;

  /* TODO : mettre un truc qui limite la taille des images
     effiches  20x20 par exemple en chargeant de GdkImage et en
     regardant sa taille. */
  panel->icon = create_pixmap((GtkWidget*)0,
			      (const gchar *)iconPath);
  return panel;
}
subPanel* createSubPanelWithIconFromStock(char* name, char *tabName,
					  gchar *id, GtkWidget *child,
					  onChangePanelFunc enter, onChangePanelFunc leave)
{
  subPanel *panel;

  panel = createSubPanel(name, tabName, child, enter, leave);
  if (!panel)
    return (subPanel*)0;

  panel->icon = gtk_image_new_from_stock(id, GTK_ICON_SIZE_MENU);
  return panel;
}

void makeSubPanel(gpointer data, gpointer user_data)
{
  GtkWidget *label;
  GtkWidget *hbox;

  subPanel *sb;
  char *lb;
  char *defaultLabel = "Void tab";

  sb = (subPanel*)data;
  if (sb->tabLabel)
    lb = sb->tabLabel;
  else
    lb = defaultLabel;

  label = gtk_label_new(lb);
  gtk_widget_show(label);
  hbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(hbox);
  if (sb->icon)
    {
      gtk_box_pack_start(GTK_BOX(hbox), sb->icon, FALSE, FALSE, 0);
      gtk_widget_show(sb->icon);
    }
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
  gtk_notebook_append_page(GTK_NOTEBOOK(tabSubPanel), sb->child, hbox);
  gtk_widget_show(sb->child);
  DBG_fprintf(stderr, "Gtk Main : created '%s' subpanel OK.\n", sb->comboLabel);
}

void addSubPanel(subPanel *sp)
{
  /* checked the structure has been allocated and
     rightly initialised. */
  if (!sp || !sp->comboLabel)
    {
      fprintf(stderr,_("WARNING! You tried to add a void subpanel. This is of course illegal.\n"));
      return;
    }

  /* add the comboLabel to the listeSubPanel */
  listeNomsSubPanel = g_list_append(listeNomsSubPanel,
				    sp->comboLabel);
  /* add the subpanel itself to the list of subpanels. */
  listeSubPanel = g_list_append(listeSubPanel,
				sp);
  nbElementsListeSubPanel++;
}

void initSubPanel()
{
  int i;
  subPanel *panel;

  for (i = 0; listSubPanelInitFunc[i]; i++)
    {
      panel = listSubPanelInitFunc[i]();
      if (!panel)
	{
	  fprintf(stderr, _("INTERNAL ERROR! Can't initialise subpanel number %d.\n"), i);
	  exit(1);
	}
      else
	DBG_fprintf(stderr, "Gtk Main : initialise '%s' subpanel OK.\n", panel->comboLabel);
    }

  /* create the elements of the combobox and
     of the subpanel. */
  gtk_combo_set_value_in_list(GTK_COMBO(comboListeSubPanel),
			      TRUE, FALSE);
  gtk_entry_set_editable(GTK_ENTRY
			 (GTK_COMBO(comboListeSubPanel)
			  ->entry),
			 FALSE);
  gtk_combo_set_popdown_strings (GTK_COMBO(comboListeSubPanel), listeNomsSubPanel);
  g_list_foreach(listeSubPanel, makeSubPanel, (gpointer)0);
  gtk_notebook_remove_page(GTK_NOTEBOOK(tabSubPanel), 0);
  /* Add the signals. */
  comboListeSubPanelSignalChanged =
    g_signal_connect((gpointer)
		     GTK_COMBO(comboListeSubPanel)->entry,
		     "changed",
		     GTK_SIGNAL_FUNC(comboChanged),
		     (gpointer)0);
  tabSubPanelSignalChanged =
    g_signal_connect((gpointer)tabSubPanel,
		     "switch-page",
		     GTK_SIGNAL_FUNC(subPanelChanged),
		     (gpointer)0);
  gtk_notebook_set_current_page(GTK_NOTEBOOK(tabSubPanel), 0);
  /* Call the refresh function for the viewed subPanel. */
  currentSubPanel = (subPanel*)listeSubPanel->data;
}



/*******************/
/* Local callbacks */
/*******************/

void setRenderingButtonSensitive(GObject *obj, gpointer bool)
{
  gtk_widget_set_sensitive(redrawButton, (gboolean)GPOINTER_TO_INT(bool));
  gtk_widget_set_sensitive(pairsButton, (gboolean)GPOINTER_TO_INT(bool));
  gtk_widget_set_sensitive(pickObserveButton, (gboolean)GPOINTER_TO_INT(bool));
  gtk_widget_set_sensitive(dumpButton, (gboolean)GPOINTER_TO_INT(bool));
}
void setFileButtonSensitive(GObject *obj, gpointer bool)
{
  if (getRenderingMethodInUse())
    gtk_widget_set_sensitive(loadButton, TRUE);
  else
    gtk_widget_set_sensitive(loadButton, FALSE);
  setRenderingButtonSensitive((GObject*)0, (gpointer)0);
}
void onResourcesLoaded(GObject *visu, gpointer data)
{
  GtkWidget *wd;

  wd = lookup_widget(mainWindow, "checkPairs");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wd), (gboolean)getPairsOnOff());
}
void checkPairsChanged(GtkToggleButton *togglebutton, gpointer user_data)
{
  int res;

  res = setPairsOnOff(gtk_toggle_button_get_active(togglebutton));
  if (res)
    g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
		   0 , NULL);
}



/*****************/
/* Miscellaneous */
/*****************/

char* getLastOpenDirectory()
{
  DBG_fprintf(stderr, "Gtk Main : get the last open directory : '%s'.\n", lastOpenDirectory);
  return lastOpenDirectory;
}
void setLastOpenDirectory(char* directory)
{
  if (lastOpenDirectory)
    g_free(lastOpenDirectory);

  if (!g_path_is_absolute(directory))
    lastOpenDirectory = g_build_filename(g_get_current_dir(), directory, NULL);
  else
    lastOpenDirectory = g_build_filename(directory, NULL);
  DBG_fprintf(stderr, "Gtk Main : set the last open directory to '%s', OK.\n", lastOpenDirectory);
}

/* This method loads the general function to load data from file
   and deals with errors with gtkDialogs. */
gboolean loadFileWithGtk(gpointer userData)
{
  GString *buffer;
  VisuData *data, *prevData;
  int res;
  gboolean changeElement;

  data = (VisuData*)userData;
  if (!data)
    return FALSE;

  prevData = currentVisuData;

  res = loadDataFromFile(data, &buffer);

  if (!res)
    {
      currentVisuData = (VisuData*)0;
      visuDataFree(data);
      raiseAlertDialog(buffer->str);
    }
  else
    currentVisuData = data;
  DBG_fprintf(stderr, "GTK Main : currentVisuData point now to %p.\n",
	      (gpointer)currentVisuData);

  if (prevData)
    {
      changeElement = visuData_compareElements(prevData, currentVisuData);
      visuDataSet_changeElementFlag(currentVisuData, changeElement);
      visuDataFree(prevData);
    }
  renderStoredData(currentVisuData);

  if (buffer)
    g_string_free(buffer, TRUE);

  return FALSE;
}

void setRememberPosition(int val)
{
  DBG_fprintf(stderr, "GTK Main : set the remember parameter to %d.\n", val);
  rememberWindowPosition = val;
}
int getRememberPosition()
{
  return rememberWindowPosition;
}


DumpFileDialog* gtkMainCreate_dumpDialog()
{
  DumpFileDialog *dialog;

  GList *format;
  GtkWidget *hbox2;
  GtkWidget *label;
  GtkWidget *wd;

  gchar *directory, *labelType;

  dialog = malloc(sizeof(DumpFileDialog));
  if (!dialog)
    {
      allocationProblems();
      exit(1);
    }

  dialog->dialog = gtk_dialog_new();
  gtk_window_set_modal(GTK_WINDOW(dialog->dialog), TRUE);
  gtk_window_set_transient_for(GTK_WINDOW(dialog->dialog), GTK_WINDOW(mainWindow));
  gtk_window_set_position(GTK_WINDOW(dialog->dialog), GTK_WIN_POS_CENTER_ON_PARENT);
  dialog->cancelButton = gtk_dialog_add_button(GTK_DIALOG(dialog->dialog),
					       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
  gtk_dialog_add_button(GTK_DIALOG(dialog->dialog), GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT);
  gtk_dialog_set_default_response(GTK_DIALOG(dialog->dialog), GTK_RESPONSE_ACCEPT);
  gtk_widget_set_name(dialog->dialog, "filesel");
  gtk_dialog_set_has_separator(GTK_DIALOG(dialog->dialog), FALSE);

  dialog->fileChooser = gtk_file_chooser_widget_new(GTK_FILE_CHOOSER_ACTION_SAVE);
  gtk_widget_show(dialog->fileChooser);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog->dialog)->vbox), dialog->fileChooser, TRUE, TRUE, 2);
  directory = getLastOpenDirectory();
  if (directory)
    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog->fileChooser), directory);
  gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(dialog->fileChooser),
				       FALSE);

  dialog->hBoxFileFormat = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(dialog->hBoxFileFormat);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog->dialog)->vbox), dialog->hBoxFileFormat, FALSE, FALSE, 2);
  /* Label to introduce the combobox which allow to choose the format */
  label = gtk_label_new(_("Choose the file format : "));
  gtk_misc_set_alignment(GTK_MISC(label), 0., 0.5);
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(dialog->hBoxFileFormat), label, TRUE, TRUE, 0);
  dialog->comboType = gtk_combo_box_new_text();
  /* Se the combo entry that let V_Sim detect the desired format through the extension. */
  gtk_combo_box_append_text(GTK_COMBO_BOX(dialog->comboType), _("Autodetect format"));
  format = getAllDumpModules();
  while (format)
    {
      labelType = fileFormatGet_label(((DumpType*)format->data)->fileType);
      if (labelType)
	gtk_combo_box_append_text(GTK_COMBO_BOX(dialog->comboType), labelType);
      format = g_list_next(format);
    }

  gtk_combo_box_set_active(GTK_COMBO_BOX(dialog->comboType), 0);
  gtk_widget_show(dialog->comboType);
  gtk_box_pack_start(GTK_BOX(dialog->hBoxFileFormat), dialog->comboType, FALSE, FALSE, 0);

  hbox2 = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(hbox2);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog->dialog)->vbox), hbox2, FALSE, FALSE, 2);
  /* Label to introduce the rogress bar */
  label = gtk_label_new(_("Dump progress : "));
  gtk_misc_set_alignment(GTK_MISC(label), 0., 0.5);
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 0);
  dialog->progressBar = gtk_progress_bar_new();
  gtk_widget_show(dialog->progressBar);
  gtk_box_pack_start(GTK_BOX(hbox2), dialog->progressBar, TRUE, TRUE, 2);

  dialog->selectedFileFormat = (DumpType*)0;
  dialog->dumpFileName = (gchar*)0;

  return dialog;
}
gboolean gtkMainRun_dumpDialog(DumpFileDialog *dialog)
{
  gchar *filename;
  int formatInt, i, ok;
  char* directory, *labelType;
  GList *dumpTypes;

  if (!dialog)
    {
      fprintf(stderr, "WARNING! 'gtkMainRun_dumpDialog' has been called"
	      " with a null 'dialog' argument.\n");
      return FALSE;
    }

  ok = 0;

  if (gtk_dialog_run(GTK_DIALOG(dialog->dialog)) == GTK_RESPONSE_ACCEPT)
    {
      filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog->fileChooser));
      /* get the directory. */
      directory = (char*)gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(dialog->fileChooser));
      setLastOpenDirectory(directory);
      g_free(directory);
      formatInt = gtk_combo_box_get_active(GTK_COMBO_BOX(dialog->comboType));
      ok = 1;
    }
  else
    {
      /* get the directory. */
      directory = (char*)gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(dialog->fileChooser));
      setLastOpenDirectory(directory);
      g_free(directory);
      ok = 0;
    }

  if (ok)
    {
      dumpTypes = getAllDumpModules();
      if (formatInt == 0)
	/* The automatic format has been selected, we look in the pattern list to
	   find a matching pattern. */
	while (dumpTypes && !fileFormatUse_match( ((DumpType*)dumpTypes->data)->fileType,
						  filename))
	  dumpTypes = g_list_next(dumpTypes);
      else
	for(i = 0; i < formatInt - 1; i++) dumpTypes = g_list_next(dumpTypes);
      if (!dumpTypes)
	{
	  /* Autodetect failed, no format match the given filename */
	  raiseAlertDialog(_("The filename doesn't match any know format."));
	  g_free(filename);
	  return FALSE;
	}

      dialog->dumpFileName = filename;
      dialog->selectedFileFormat = (DumpType*)dumpTypes->data;
      /* Put everything except the progress bar insensitive */
      gtk_widget_set_sensitive(dialog->fileChooser, FALSE);
      gtk_widget_set_sensitive(dialog->hBoxFileFormat, FALSE);
      DBG_fprintf(stderr, "Gtk Main : '%s' file format chosen.\n",
		  dialog->selectedFileFormat->fileType->description);

      return TRUE;
    }
  else
    return FALSE;
}
void gtkMainFree_dumpDialog(DumpFileDialog *dialog)
{
  if (!dialog)
    return;

  if (dialog->dumpFileName)
    g_free(dialog->dumpFileName);
  gtk_widget_destroy(dialog->dialog);
}
gchar* gtkMainGet_dumpFileName(DumpFileDialog *dialog)
{
  if (!dialog)
    {
      fprintf(stderr, "WARNING! 'gtkMainGet_dumpFileName' has been called"
	      " with a null 'dialog' argument.\n");
      return (gchar*)0;
    }
  return dialog->dumpFileName;
}
DumpType* gtkMainGet_dumpFileFormat(DumpFileDialog *dialog)
{
  if (!dialog)
    {
      fprintf(stderr, "WARNING! 'gtkMainGet_dumpFileFormat' has been called"
	      " with a null 'dialog' argument.\n");
      return (DumpType*)0;
    }
  return dialog->selectedFileFormat;
}
GtkProgressBar* gtkMainGet_dumpProgressBar(DumpFileDialog *dialog)
{
  if (!dialog)
    {
      fprintf(stderr, "WARNING! 'gtkMainGet_dumpProgressBar' has been called"
	      " with a null 'dialog' argument.\n");
      return (GtkProgressBar*)0;
    }
  return GTK_PROGRESS_BAR(dialog->progressBar);
}
GtkButton* gtkMainGet_dumpCancelButton(DumpFileDialog *dialog)
{
  if (!dialog)
    {
      fprintf(stderr, "WARNING! 'gtkMainGet_dumpCancelButton' has been called"
	      " with a null 'dialog' argument.\n");
      return (GtkButton*)0;
    }
  return GTK_BUTTON(dialog->cancelButton);
}
GtkDialog* gtkMainGet_dumpDialog(DumpFileDialog *dialog)
{
  if (!dialog)
    {
      fprintf(stderr, "WARNING! 'gtkMainGet_dumpDialog' has been called"
	      " with a null 'dialog' argument.\n");
      return (GtkDialog*)0;
    }
  return GTK_DIALOG(dialog->dialog);
}


gchar* getSelectedDirectory()
{
  GtkWidget *file_selector;
  gchar *dirname;
  char *directory;

  /* Create the selector */
  file_selector = gtk_file_chooser_dialog_new(_("Choose a directory"),
					      GTK_WINDOW(mainWindow),
					      GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					      GTK_STOCK_OPEN, GTK_RESPONSE_OK,
					      NULL);
  directory = getLastOpenDirectory();
  if (directory)
    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(file_selector), directory);
  gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(file_selector),
				       FALSE);

  gtk_widget_set_name(file_selector, "filesel");
  gtk_window_set_position(GTK_WINDOW(file_selector), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal(GTK_WINDOW (file_selector), TRUE);

  if (gtk_dialog_run (GTK_DIALOG (file_selector)) == GTK_RESPONSE_OK)
    {
      dirname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER (file_selector));
      setLastOpenDirectory(dirname);
    }
  else
    dirname = (gchar*)0;

  gtk_widget_destroy (file_selector);

  return dirname;
}


/* This method hides the specified window and save its coordinates if
   it is specified by a parameter. */
void visuHideWindow(GtkWindow *win)
{
  int *val;

  if (!win)
    return;

  if (rememberWindowPosition)
    {
      val = (int*)g_hash_table_lookup(windowPosition, win);
      if (!val)
	{
	  val = malloc(sizeof(int) * 2);
	  if (!val)
	    {
	      allocationProblems();
	      exit(1);
	    }
	  g_hash_table_insert(windowPosition, (gpointer)win, (gpointer)val);
	}
      gtk_window_get_position(win, &val[0], &val[1]);
      DBG_fprintf(stderr, "Gtk Main : store position (%d,%d) for window %d.\n", 
		  val[0], val[1], GPOINTER_TO_INT(win));
    }
  gtk_widget_hide(GTK_WIDGET(win));
}

/* This method shows the specified window and try to put it at its
   former position if the good parameter is used. */
void visuShowWindow(GtkWindow *win)
{
  int *val;

  if (!win)
    return;

  if (rememberWindowPosition)
    {
      val = (int*)g_hash_table_lookup(windowPosition, win);
      if (val)
	{
	  gtk_window_move(win, val[0], val[1]);
	  DBG_fprintf(stderr, "Gtk Main : set position (%d,%d) for window %d.\n", 
		      val[0], val[1], GPOINTER_TO_INT(win));
	}
    }
  gtk_widget_show(GTK_WIDGET(win));
}






gint compareStringsInGList(gconstpointer a, gconstpointer b)
{
  return strcmp((char*)a, (char*)b);
}


/* Draw the alert window with the specified message. */
void raiseAlertDialog(char *message)
{
  GtkWidget *alert;
  GtkWidget *label;
  GtkWidget *wd;

  alert = create_alertDialog ();
  label = lookup_widget(alert, "alertLabel");
  gtk_label_set_text(GTK_LABEL(label), message);
  gtk_widget_set_name(alert, "error");
  wd = lookup_widget(alert, "buttonAlert");
  gtk_widget_set_name(wd, "error_button");
  gtk_widget_show (alert);
  /* block in a loop waiting for reply. */
  gtk_dialog_run (GTK_DIALOG (alert));
  gtk_widget_destroy(alert);
}
/* Draw the alert window with the specified message. */
void raiseAlertDialogWithScrollView(char *message)
{
  GtkWidget *alert;
  GtkWidget *text;
  GtkWidget *wd;
  GtkTextBuffer *buf;

  alert = create_alertDialogLong ();
  text = lookup_widget(alert, "alertText");
  buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW(text));
  if (message)
    gtk_text_buffer_set_text (GTK_TEXT_BUFFER(buf), message, -1);
  else
    gtk_text_buffer_set_text (GTK_TEXT_BUFFER(buf), _("INTERNAL ERROR! No message available!"), -1);
  gtk_widget_set_name(alert, "error");
  wd = lookup_widget(alert, "buttonAlert");
  gtk_widget_set_name(wd, "error_button");
  gtk_widget_show (alert);
  /* block in a loop waiting for reply. */
  gtk_dialog_run (GTK_DIALOG (alert));
  gtk_widget_destroy(alert);
}
/* This function must be called in a blocking loop to update
   different things like the gtk for example. */
void visuWaitFunction()
{
  while(gtk_events_pending())
    gtk_main_iteration();
}

GList* gtkMainCreate_fileChooserFilter(GList *list, GtkFileFilter **filterDefault)
{
  GtkFileFilter *filter, *filterAll;
  GList *tmpLst, *tmpLst2;
  char *name;
  GList *returnedFilters;

  returnedFilters = (GList*)0;
  filterAll = gtk_file_filter_new ();
  gtk_file_filter_set_name(filterAll, _("All supported formats"));
  tmpLst = list;
  while(tmpLst)
    {
      filter = gtk_file_filter_new ();
      name = fileFormatGet_label((FileFormat*)tmpLst->data);
      if (name)
	gtk_file_filter_set_name(filter, name);
      else
	gtk_file_filter_set_name(filter, _("No description"));
      tmpLst2 = ((FileFormat*)tmpLst->data)->fileType;
      while (tmpLst2)
	{
	  gtk_file_filter_add_pattern (filter, (char*)tmpLst2->data);
	  gtk_file_filter_add_pattern (filterAll, (char*)tmpLst2->data);
	  tmpLst2 = g_list_next(tmpLst2);
	}
      tmpLst = g_list_next(tmpLst);
      returnedFilters = g_list_append(returnedFilters, filter);
    }
  returnedFilters = g_list_append(returnedFilters, filterAll);
  filter = gtk_file_filter_new ();
  gtk_file_filter_set_name(filter, "All files");
  gtk_file_filter_add_pattern (filter, "*");
  returnedFilters = g_list_append(returnedFilters, filter);
  
  *filterDefault = filterAll;

  return returnedFilters;
}
int gtkMainCreate_defaultFileChooser(VisuData *data, GtkFileChooser **fileChooser,
				     GtkWidget **fileWidget)
{
  GtkWidget *fileSelection;
  GList *filters, *tmpLst;
  GtkWidget *dialog;
  RenderingMethod *method;
  gchar* directory, *filename;
  GtkFileFilter *filterDefault;

  if (!data)
    return 0;

  filterDefault = (GtkFileFilter*)0;
  method = getRenderingMethodInUse();
  if (!method)
    filters = (GList*)0;
  else
    filters = gtkMainCreate_fileChooserFilter(method->fileType[0], &filterDefault);
  
  fileSelection = gtk_file_chooser_dialog_new(_("Load session"),
					      GTK_WINDOW(mainWindow),
					      GTK_FILE_CHOOSER_ACTION_OPEN,
					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					      GTK_STOCK_OPEN, GTK_RESPONSE_OK,
					      NULL);
  *fileChooser = GTK_FILE_CHOOSER(fileSelection);
  *fileWidget = fileSelection;
  directory = getLastOpenDirectory();
  if (directory)
    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(fileSelection), directory);
  tmpLst = filters;
  while(tmpLst)
    {
      gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(fileSelection),
				  (GtkFileFilter*)(tmpLst->data));
      tmpLst = g_list_next(tmpLst);
    }
  if (filters && filterDefault)
    gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(fileSelection), filterDefault);
  g_list_free(filters);

  gtk_widget_set_name(fileSelection, "filesel");
  gtk_window_set_position(GTK_WINDOW(fileSelection), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal(GTK_WINDOW(fileSelection), TRUE);

  if (gtk_dialog_run(GTK_DIALOG(fileSelection)) == GTK_RESPONSE_OK)
    {
      filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(fileSelection));
      visuDataAdd_file(data, filename, 0);
      g_free(filename);
      return 1;
    }
  else
    return 0;
}

void gtkMainSet_renderingSpecificMethods(RenderingMethod *method,
					 createGtkWidgetFunc createLoadWidget,
					 createGtkLoadWidgetFunc methodLoad)
{
  struct renderingMethod_gtkMain_struct *container;

  if (!method)
    return;

  container = malloc(sizeof(struct renderingMethod_gtkMain_struct));
  if (!container)
    {
      allocationProblems();
      exit(1);
    }
  container->createOpenAddOn = createLoadWidget;
  container->createLoadAction = methodLoad;
  
  g_hash_table_insert(renderingMethod_gtkMain, (gpointer)method, (gpointer)container);
}
createGtkLoadWidgetFunc gtkMainGet_renderingSpecificOpen(RenderingMethod *method)
{
  struct renderingMethod_gtkMain_struct *container;

  if (!method)
    return (createGtkLoadWidgetFunc)0;

  DBG_fprintf(stderr, "Gtk Main : looking for a specific load interface for rendering"
	      " method '%s'...\n", method->name);
  
  container = (struct renderingMethod_gtkMain_struct *)
    g_hash_table_lookup(renderingMethod_gtkMain, (gpointer)method);

  if (!container)
    return (createGtkLoadWidgetFunc)0;
  
  if (container->createLoadAction)
    return container->createLoadAction;
  else
    return gtkMainCreate_defaultFileChooser;
}
