/*   EXTRAITS DE LA LICENCE
	Copyright CEA, contributeurs : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)
  
	Adresse ml :
	BILLARD, non joignable par ml ;
	CALISTE, damien P caliste AT cea P fr.

	Ce logiciel est un programme informatique servant  visualiser des
	structures atomiques dans un rendu pseudo-3D. 

	Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
	respectant les principes de diffusion des logiciels libres. Vous pouvez
	utiliser, modifier et/ou redistribuer ce programme sous les conditions
	de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
	sur le site "http://www.cecill.info".

	Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
	pris connaissance de la licence CeCILL, et que vous en avez accept les
	termes (cf. le fichier Documentation/licence.fr.txt fourni avec ce logiciel).
*/

/*   LICENCE SUM UP
	Copyright CEA, contributors : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)

	E-mail address:
	BILLARD, not reachable any more ;
	CALISTE, damien P caliste AT cea P fr.

	This software is a computer program whose purpose is to visualize atomic
	configurations in 3D.

	This software is governed by the CeCILL  license under French law and
	abiding by the rules of distribution of free software.  You can  use, 
	modify and/ or redistribute the software under the terms of the CeCILL
	license as circulated by CEA, CNRS and INRIA at the following URL
	"http://www.cecill.info". 

	The fact that you are presently reading this means that you have had
	knowledge of the CeCILL license and that you accept its terms. You can
	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
*/
#include "gtk_observePick.h"
#include "callbacks.h"

#include "visu_pickMesure.h"

/* Callbacks. */
void visuDataChanged(GObject *obj, gpointer bool);
void visuElementRenderChanged(GObject *obj, VisuElement *element, gpointer data);
void translationChanged(GtkSpinButton* button, gpointer data);
void translationChecked(GtkToggleButton* button, gpointer data);
void observeMethodChanged(GtkToggleButton* button, gpointer data);
void applyTranslationsOnVisuDataChanged(GObject *obj, gpointer visuData, gpointer data);
gboolean onKillObserveEvent(GtkWidget *widget, GdkEvent *event, gpointer user_data);

/* Values of the translation */
float initialTranslation[3];
float currentUserTranslation[3];


/* Initialise the observe/pick window, connect the
   signals, give names to widgets... */
void initObserveAndPickWindow()
{
  observeWindow = (GtkWidget*)0;
  disableCallbacks = 0;
}

void buildObserveAndPickWindow()
{
  int i;

  DBG_fprintf(stderr, "GTK Observe and Pick : creating the window.\n");
  observeWindow = create_observeDialog();
  gtk_window_set_type_hint(GTK_WINDOW(observeWindow), GDK_WINDOW_TYPE_HINT_NORMAL);
  setNamesGtkWidgetObservePick();
  /* set the initial values for the spins. */
  refreshCameraSpinValues((GObject*)0, (gpointer)0);
  /* connect the signals to the spins. */
  connectSignalsObservePick();
  /* set the initial periodic stuff */
  visuDataChanged((GObject*)0, (gpointer)0);
}

void cleanPickObserve()
{
  gtk_label_set_text(GTK_LABEL(labelPickOut), "");
  gtk_label_set_text(GTK_LABEL(labelPickError), "");
}

void setSensitivePreciseObserve(gboolean value)
{
  GtkWidget *wd;

  gtk_widget_set_sensitive(spinTheta, value);
  gtk_widget_set_sensitive(spinPhi, value);
  if (openGLGet_preferedObserveMethod() == OPENGL_OBSERVE_CONSTRAINED)
    gtk_widget_set_sensitive(spinOmega, FALSE);
  else
    gtk_widget_set_sensitive(spinOmega, value);
  gtk_widget_set_sensitive(spinXs, value);
  gtk_widget_set_sensitive(spinYs, value);
  gtk_widget_set_sensitive(spinGross, value);
  gtk_widget_set_sensitive(spinPersp, value);
  wd = lookup_widget(observeWindow, "radioObserveConstrained");
  gtk_widget_set_sensitive(wd, value);
  wd = lookup_widget(observeWindow, "radioObserveWalker");
  gtk_widget_set_sensitive(wd, value);
}


/****************/
/* Private part */
/****************/

/* Connect the listeners on the signal emitted by the OpenGL server. */
void connectSignalsObservePick()
{
  GtkWidget *wd;

  signalCameraChangedId =
    g_signal_connect(G_OBJECT(visu), "OpenGLThetaPhi",
		     G_CALLBACK(refreshCameraSpinValues), (gpointer)0);
  signalCameraChangedId =
    g_signal_connect(G_OBJECT(visu), "OpenGLXsYs",
		     G_CALLBACK(refreshCameraSpinValues), (gpointer)0);
  signalCameraChangedId =
    g_signal_connect(G_OBJECT(visu), "OpenGLGross",
		     G_CALLBACK(refreshCameraSpinValues), (gpointer)0);
  signalCameraChangedId =
    g_signal_connect(G_OBJECT(visu), "OpenGLDRed",
		     G_CALLBACK(refreshCameraSpinValues), (gpointer)0);
  g_signal_connect(G_OBJECT(visu), "pickMesureAvailable",
		   G_CALLBACK(refreshMesureInfos), (gpointer)0);
  g_signal_connect(G_OBJECT(visu), "dataReadyForRendering",
		   G_CALLBACK(visuDataChanged), (gpointer)0);
  g_signal_connect(G_OBJECT(visu), "dataNotReadyForRendering",
		   G_CALLBACK(visuDataChanged), (gpointer)0);
  g_signal_connect(G_OBJECT(visu), "NodeRenderedChanged",
		   G_CALLBACK(visuElementRenderChanged), (gpointer)0);

  wd = lookup_widget(observeWindow, "checkAllowTranslations");
  g_signal_connect(G_OBJECT(wd), "toggled",
		   G_CALLBACK(translationChecked), (gpointer)0);
  wd = lookup_widget(observeWindow, "spinTransX");
  g_signal_connect(G_OBJECT(wd), "value-changed",
		   G_CALLBACK(translationChanged), GINT_TO_POINTER(0));
  currentUserTranslation[0] = gtk_spin_button_get_value(GTK_SPIN_BUTTON(wd));
  wd = lookup_widget(observeWindow, "spinTransY");
  g_signal_connect(G_OBJECT(wd), "value-changed",
		   G_CALLBACK(translationChanged), GINT_TO_POINTER(1));
  currentUserTranslation[1] = gtk_spin_button_get_value(GTK_SPIN_BUTTON(wd));
  wd = lookup_widget(observeWindow, "spinTransZ");
  g_signal_connect(G_OBJECT(wd), "value-changed",
		   G_CALLBACK(translationChanged), GINT_TO_POINTER(2));
  currentUserTranslation[2] = gtk_spin_button_get_value(GTK_SPIN_BUTTON(wd));
  g_signal_connect(G_OBJECT(visu), "dataLoaded",
		   G_CALLBACK(applyTranslationsOnVisuDataChanged), (gpointer)0);

  wd = lookup_widget(observeWindow, "radioObserveConstrained");
  g_signal_connect(G_OBJECT(wd), "toggled",
		   G_CALLBACK(observeMethodChanged), GINT_TO_POINTER(OPENGL_OBSERVE_CONSTRAINED));
  wd = lookup_widget(observeWindow, "radioObserveWalker");
  g_signal_connect(G_OBJECT(wd), "toggled",
		   G_CALLBACK(observeMethodChanged), GINT_TO_POINTER(OPENGL_OBSERVE_WALKER));

  g_signal_connect(G_OBJECT(observeWindow), "delete-event",
		   G_CALLBACK(onKillObserveEvent), (gpointer)0);
  g_signal_connect(G_OBJECT(observeWindow), "destroy-event",
		   G_CALLBACK(onKillObserveEvent), (gpointer)0);
}

void visuDataChanged(GObject* obj, gpointer data)
{
  gboolean periodic;

  DBG_fprintf(stderr, "Gtk Observe/Pick : caught 'dataReadyForRendering' or"
	      " 'dataNotReadyForRendering' signal, setting sensitivity.\n");
  if (currentVisuData)
    {
      periodic = visuDataGet_periodic(currentVisuData);
      gtk_widget_set_sensitive(lookup_widget(observeWindow, "hboxTranslations"),
			       periodic);
      gtk_widget_set_sensitive(lookup_widget(observeWindow, "checkAllowTranslations"),
			       periodic);
    }
}

void applyTranslation()
{
  float cartCoord[3];

  visuDataConvert_boxCoordinatestoXYZ(currentVisuData, cartCoord, currentUserTranslation);
/*   fprintf(stderr, "%f %f %f\n", cartCoord[0], cartCoord[1], cartCoord[2]); */
  visuDataSet_XYZtranslation(currentVisuData, cartCoord);
  visuData_constrainedInTheBox(currentVisuData);
  g_signal_emit (visu, VISU_GET_CLASS (visu)->NodePositionChanged_signal_id,
		 0 , NULL);
  visuData_createAllNodes();
  g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
		 0 , NULL);
}
void translationChecked(GtkToggleButton* button, gpointer data)
{
  float cartCoord[3];

  if (gtk_toggle_button_get_active(button))
    {
      applyTranslation();
    }
  else
    {
      cartCoord[0] = 0.;
      cartCoord[1] = 0.;
      cartCoord[2] = 0.;
      visuDataSet_XYZtranslation(currentVisuData, cartCoord);
      visuData_constrainedFree(currentVisuData);
      g_signal_emit (visu, VISU_GET_CLASS (visu)->NodePositionChanged_signal_id,
		     0 , NULL);
      visuData_createAllNodes();
      g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
		     0 , NULL);
    }
}

void translationChanged(GtkSpinButton* button, gpointer data)
{
  float trans;
  GtkWidget *wd;
  int i;
  int spin;

  spin = GPOINTER_TO_INT(data);
  if (spin < 0 || spin > 2)
    {
      fprintf(stderr, "WARNING! 'translationChanged' has been called with a"
	      " wrong 'data' argument.\n");
      return;
    }

  trans = gtk_spin_button_get_value(button);
  if (trans == currentUserTranslation[spin])
    return;

  currentUserTranslation[spin] = trans;

  wd = lookup_widget(observeWindow, "checkAllowTranslations");
  if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wd)))
    return;

  applyTranslation();
}

void applyTranslationsOnVisuDataChanged(GObject *obj, gpointer visuData, gpointer data)
{
  float cartCoord[3];
  GtkWidget *wd;
  
  if (!visuData)
    return;

  wd = lookup_widget(observeWindow, "checkAllowTranslations");
  if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wd)))
    return;

  DBG_fprintf(stderr, "Gtk Pick/Observe : Catch 'dataLoaded' signal,"
	      " applying current translations.\n");

  visuDataConvert_boxCoordinatestoXYZ((VisuData*)visuData, cartCoord, currentUserTranslation);
  visuDataSet_XYZtranslation((VisuData*)visuData, cartCoord);
  visuData_constrainedInTheBox((VisuData*)visuData);
}


void observeMethodChanged(GtkToggleButton* button, gpointer data)
{
  int method;

  if (!gtk_toggle_button_get_active(button))
    return;

  method = GPOINTER_TO_INT(data);
  openGLSet_preferedObserveMethod(method);
  if (method == OPENGL_OBSERVE_CONSTRAINED)
    {
      gtk_widget_set_sensitive(spinOmega, FALSE);
      if (setCameraThetaPhiOmega(0., 0., 0., MASK_OMEGA))
	g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
		       0 , NULL);
    }
  else
    gtk_widget_set_sensitive(spinOmega, TRUE);
}


/* Ask the OpenGL server to refresh the values of the spins
   controlling the position of the camera. */
void refreshCameraSpinValues(GObject *obj, gpointer data)
{
  disableCallbacks = 1;
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinTheta), (gdouble)visuCamera->theta);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinPhi), (gdouble)visuCamera->phi);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinOmega), (gdouble)visuCamera->omega);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinXs), (gdouble)visuCamera->xs);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinYs), (gdouble)visuCamera->ys);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinGross), (gdouble)visuCamera->gross);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinPersp), (gdouble)visuCamera->d_red);
  disableCallbacks = 0;
}

void refreshMesureInfos(GObject *obj, gpointer data)
{
  gchar *infos;
  gchar* errors;

  infos = getPickMesureInfos();
  errors = getPickMesureErrors();

  if (infos)
    gtk_label_set_text(GTK_LABEL(labelPickOut), infos);
  if (errors)
    gtk_label_set_text(GTK_LABEL(labelPickError), errors);
}

/* Give a name to the widgets present in this window
   to be able to affect them with a theme. */
void setNamesGtkWidgetObservePick()
{
  GtkWidget *wd;
  int method;

  gtk_widget_set_name(observeWindow, "message");
/*   wd = lookup_widget(observeWindow, "titreObserve"); */
/*   gtk_widget_set_name(wd, "message_title"); */
  wd = lookup_widget(observeWindow, "labelInfoObservePick");
  gtk_widget_set_name(wd, "label_info");

/*   wd = lookup_widget(observeWindow, "labelObserve"); */
/*   gtk_widget_set_name(wd, "label_head"); */
/*   wd = lookup_widget(observeWindow, "labelPick"); */
/*   gtk_widget_set_name(wd, "label_head"); */

  wd = lookup_widget(observeWindow, "labelTranslation");
  gtk_widget_set_name(wd, "label_head_2");
/*   wd = lookup_widget(observeWindow, "labelRotation"); */
/*   gtk_widget_set_name(wd, "label_head_2"); */
  wd = lookup_widget(observeWindow, "labelZoom");
  gtk_widget_set_name(wd, "label_head_2");

  wd = lookup_widget(observeWindow, "radioObserve");
  gtk_widget_set_name(wd, "message_radio");
  wd = lookup_widget(observeWindow, "radioPick");
  gtk_widget_set_name(wd, "message_radio");

  spinTheta = lookup_widget(observeWindow, "spinTheta");
  spinPhi = lookup_widget(observeWindow, "spinPhi");
  spinOmega = lookup_widget(observeWindow, "spinOmega");
  spinXs = lookup_widget(observeWindow, "spinDx");
  spinYs = lookup_widget(observeWindow, "spinDy");
  spinGross = lookup_widget(observeWindow, "spinGross");
  spinPersp = lookup_widget(observeWindow, "spinPersp");

  if (openGLGet_preferedObserveMethod() == OPENGL_OBSERVE_CONSTRAINED)
    gtk_widget_set_sensitive(spinOmega, FALSE);

  labelPickOut = lookup_widget(observeWindow, "pickInfo");
  labelPickError = lookup_widget(observeWindow, "pickComment");
  gtk_widget_set_name(labelPickError, "label_error");

  wd = lookup_widget(observeWindow, "checkAllowTranslations");
  gtk_widget_set_name(wd, "message_radio");

  method = openGLGet_preferedObserveMethod();
  wd = lookup_widget(observeWindow, "radioObserveConstrained");
  gtk_widget_set_name(wd, "message_radio");
  if (method == OPENGL_OBSERVE_CONSTRAINED)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wd), TRUE);
  wd = lookup_widget(observeWindow, "radioObserveWalker");
  gtk_widget_set_name(wd, "message_radio");
  if (method == OPENGL_OBSERVE_WALKER)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wd), TRUE);
}

gboolean onKillObserveEvent(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  DBG_fprintf(stderr, "Gtk Pick/Observe : delete or destroy event cancelled.\n");
  on_closebutton1_clicked((GtkButton*)0, (gpointer)0);

  return TRUE;
}

void visuElementRenderChanged(GObject *obj, VisuElement *element, gpointer data)
{
  if (element)
    {
      if (element->rendered)
	{
	  DBG_fprintf(stderr, "Gtk Observe/Pick : caught 'NodeRenderedChanged' applying"
		      " translation for element '%s'.\n", element->name);
	  if (visuData_constrainedElementInTheBox(currentVisuData, element))
	    g_signal_emit(visu, VISU_GET_CLASS (visu)->NodePositionChanged_signal_id,
			  0 , NULL);

	}
      else
	DBG_fprintf(stderr, "Gtk Observe/Pick : caught 'NodeRenderedChanged' but"
		    " do not apply translation since element '%s' is masked.\n",
		    element->name);
    }
  else
    DBG_fprintf(stderr, "Gtk Observe/Pick : caught 'NodeRenderedChanged' but"
		" do not apply translation since no element is masked or showed.\n");
}
