/*   EXTRAITS DE LA LICENCE
	Copyright CEA, contributeurs : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)
  
	Adresse ml :
	BILLARD, non joignable par ml ;
	CALISTE, damien P caliste AT cea P fr.

	Ce logiciel est un programme informatique servant  visualiser des
	structures atomiques dans un rendu pseudo-3D. 

	Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
	respectant les principes de diffusion des logiciels libres. Vous pouvez
	utiliser, modifier et/ou redistribuer ce programme sous les conditions
	de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
	sur le site "http://www.cecill.info".

	Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
	pris connaissance de la licence CeCILL, et que vous en avez accept les
	termes (cf. le fichier Documentation/licence.fr.txt fourni avec ce logiciel).
*/

/*   LICENCE SUM UP
	Copyright CEA, contributors : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)

	E-mail address:
	BILLARD, not reachable any more ;
	CALISTE, damien P caliste AT cea P fr.

	This software is a computer program whose purpose is to visualize atomic
	configurations in 3D.

	This software is governed by the CeCILL  license under French law and
	abiding by the rules of distribution of free software.  You can  use, 
	modify and/ or redistribute the software under the terms of the CeCILL
	license as circulated by CEA, CNRS and INRIA at the following URL
	"http://www.cecill.info". 

	The fact that you are presently reading this means that you have had
	knowledge of the CeCILL license and that you accept its terms. You can
	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>

#include "renderingBackend/visu_windowInterface.h"

#include "opengl.h"
#include "visu_extension.h"
#include "visu_tools.h"
#include "visu_object.h"
#include "visu_configFile.h"

/******************************************************************************/
/******************************************************************************/

/* static int stereo; */

/* This is a positive float that enable to increase or
   decrease the rendering load by modifying the
   number of facettes. */
#define FLAG_PARAMETER_OPENGL_DETAILS   "opengl_details"
#define DESC_PARAMETER_OPENGL_DETAILS   "Give a value to the quality of rendering (100 is normal) ; positive integer"
#define PARAMETER_OPENGL_DETAILS_DEFAULT 100
float precisionOfRendering;
gboolean readOpenGLPrecision(gchar **lines, int nbLines,
			     int position, GString *errorMessage);

#define FLAG_PARAMETER_OPENGL_IMMEDIATE   "opengl_immediateDrawing"
#define DESC_PARAMETER_OPENGL_IMMEDIATE   "If true, changes of parameters means immediate redrawing ; boolean 0 or 1"
#define PARAMETER_OPENGL_IMMEDIATE_DEFAULT 1
int opengl_immediate;
gboolean readOpenGLImmediate(gchar **lines, int nbLines,
			     int position, GString *errorMessage);
#define FLAG_PARAMETER_OPENGL_RENDERING "opengl_render"
#define DESC_PARAMETER_OPENGL_RENDERING "Rules the way OpenGl draws objects ; 3 strings possible : Wireframe, Flat or Smooth"
#define PARAMETER_OPENGL_RENDERING_DEFAULT Smooth
int renderingOption;
gboolean readOpenGLRendering(gchar **lines, int nbLines,
			     int position, GString *errorMessage);

#define FLAG_PARAMETER_OPENGL_ANTIALIAS "opengl_antialias"
#define DESC_PARAMETER_OPENGL_ANTIALIAS "If true, lines are drawn smoother ; boolean 0 or 1"
#define PARAMETER_OPENGL_ANTIALIAS_DEFAULT 0
int antialias;
gboolean readOpenGLAntialias(gchar **lines, int nbLines,
			     int position, GString *errorMessage);

#define FLAG_PARAMETER_OPENGL_FAKEBS "opengl_fakeBackingStore"
#define DESC_PARAMETER_OPENGL_FAKEBS "If true, V_Sim catches the Expose event from the X server and calls a redraw ; boolean 0 or 1"
#define PARAMETER_OPENGL_FAKEBS_DEFAULT 0
int fakeBackingStore;

#define FLAG_PARAMETER_OBSERVE_METHOD "opengl_observe_method"
#define DESC_PARAMETER_OBSERVE_METHOD "Choose the observe method ; integer (0: constrained mode, 1: walker mode)"
int openGL_preferedObserveMethod;
gboolean readOpenGLObserveMethod(gchar **lines, int nbLines,
				 int position, GString *errorMessage);


static char *renderingStrings[3] = {"Wireframe", "Flat", "Smooth"};
static float anglesDefault[3] = {40., -50., 0.};
static float translatDefault[2] = {0.5, 0.5};

struct observeData
{
  float delta_phi;
  float delta_theta;
  float delta_omega;
  float delta_xs;
  float delta_ys;
  float delta_gross;
  float delta_d_red;
  float old_theta;
  float old_phi;
  float old_omega;
  float old_xs;
  float old_ys;
  float old_gross;
  float old_d_red;
  float xi, yi;
};

/* moved to .h */
/* typedef struct light_ */
/* {   */
/*   gboolean enabled; */
/*   float ambient[4]; */
/*   float diffuse[4]; */
/*   float specular[4]; */
/*   float position[4]; */
/* } light; */


struct observeData *observeValues;

callBackOnNodeSelectedFunc callbackOnNodeSelected;
GList* pickObserveCallbackId;
CallbackFunctions callbackPickObserve;

void redrawForBackingStore(SimplifiedEvents *ev);
void glSetRendering();
void glSetLighting();
void glSetAntiAlias();
/* Private - This initialise the opengl redering (light...) */
void gl_init(void);


GList** light_get_list_location()
{
  static GList* v_sim_lights = NULL;

  return &v_sim_lights;
}

GList* light_get_list()
{
  return *light_get_list_location();
}

light* light_new(gboolean light_enabled, 
		 float ambient[4], float diffuse[4], float specular[4], 
		 float position[4])
{
  light *light0 = g_malloc(sizeof(light));
  int i;

  light0->enabled = light_enabled;
  for(i=0; i<4; i++)
    {
      light0->ambient[i] = ambient[i];
      light0->diffuse[i] = diffuse[i];
      light0->specular[i] = specular[i];
      light0->position[i] = position[i];
    }

  return light0;
}  

light* light_new_(gboolean light_enabled, float params[16])
{
  return light_new(light_enabled,
		   &params[0], &params[4], &params[8], &params[12]);
}  

light* light_new_default()
{
  float params[16] =  {1.0f, 1.0f, 1.0f, 1.0f,
		      1.0f, 1.0f, 1.0f, 1.0f,
		      1.0f, 1.0f, 1.0f, 1.0f,
		      3.0f, 2.0f, 1.7f, 0.0f};

  return light_new(TRUE,
		   &params[0], &params[4], &params[8], &params[12]);
}

light* light_new_by_position(float position[4])
{
  light *light0 = light_new_default();
  int i;
  
  light0->enabled = TRUE;
  for(i=0; i<4; i++)
    light0->position[i]=position[i];

  return light0;
}

void lighting_set(gpointer data, gpointer useless)
{
  light *light0 = data;
  int n = g_list_index(light_get_list(), light0);

  if(n < 0 || n >= GL_MAX_LIGHTS)
    return;

  if(light0->enabled == FALSE)
    {
      /* From glLightfv man page : "It is always the case that GL_LIGHTi = GL_LIGHT0 + i." */
      glDisable(GL_LIGHT0 + n);
      return;
    }

  glEnable(GL_LIGHT0 + n);
  glLightfv(GL_LIGHT0 + n, GL_AMBIENT, light0->ambient);
  glLightfv(GL_LIGHT0 + n, GL_DIFFUSE, light0->diffuse);
  glLightfv(GL_LIGHT0 + n, GL_SPECULAR, light0->specular);
  glLightfv(GL_LIGHT0 + n, GL_POSITION, light0->position);
  DBG_fprintf(stderr, "[%f][%f][%f][%f][%d][%d]\n", light0->position[0], light0->position[1], light0->position[2], light0->position[3], GL_LIGHT0+n, light0->enabled);
  

  DBG_fprintf(stderr, "V_Sim lighting : set light number %d\n", n);
}

void light_add(light *light0)
{
  GList **light_list = light_get_list_location();

  *light_list = g_list_append(*light_list, light0);
}  

int lighting_init()
{
  light_add(light_new_default());
  return 0; 
}

/******************************************************************************/

void putMaxExtens(gpointer key, gpointer value, gpointer data)
{
/*   VisuElement *ele; */
/*   RenderingMethod *method; */
/*   float size; */
  
/*   method = getRenderingMethodInUse(); */
/*   if (!method) */
/*     { */
/*       *((float*)data) = 0.; */
/*       return; */
/*     } */

/*   ele = (VisuElements*)value; */
/*   size = method->getExtensForElement(ele); */
/*   *((float*)data) = (*((float*)data) > size)?*((float*)data):size; */
}

/******************************************************************************/

void update_extens_etc(float extens_bare) {
   float rmax;
 
   /* Get the maximum extens of all the VisuElements. */
   rmax = 0.;
/*    g_hash_table_foreach(differentVisuElements, (GHFunc)putMaxExtens, (gpointer)&rmax); */
   visuBox->extens = extens_bare + (double)rmax;
   
   /* add some guard */
   visuBox->extens *= 1.1;
   
   /* define radius_max and pair_max for menus */
/*    radius_max = pair_max = 2.0f * extens; */
}

/******************************************************************************/

static void modelize(void) {
   double theta_rad = visuCamera->theta * PI180;
   double phi_rad   = visuCamera->phi   * PI180; 
   double sth, cth, sph, cph, com, som;
   double eyex, eyey, eyez;
   double distance;   
   double upx, upy, upz;
 
   /*  remplacer quand visuBox sera inclu dans visuData */
   if (!currentVisuData)
     return;

   distance = visuCamera->d_red * visuBox->extens;

   sth = sin(theta_rad);
   cth = cos(theta_rad);      
   sph = sin(phi_rad);
   cph = cos(phi_rad);
   com = cos(visuCamera->omega * PI180);
   som = sin(visuCamera->omega * PI180);

   /* La matrice de rotation est la suivante pour passer
      des coordonnes transformes aux coordonnes de l'cran :
      /cph.cth -sph cph.sth\
      |sph.cth  cph sph.sth|
      \   -sth   0      cth/
      Ainsi la camra qui est situ en (0,0,Distance) dans le repre transform
      devient dans le repre de l'cran : */
   eyex = distance*sth*cph;
   eyey = distance*sth*sph;
   eyez = distance*cth;
   
   /* Vecteur donnant la direction verticale.
      Dans le repre transform il est (-1,0,0). */
   upx = -cth*cph*com + sph*som;
   upy = -cth*sph*com - cph*som;
   upz = sth*com;
   
   glMatrixMode(GL_MODELVIEW); 
   glLoadIdentity();
   gluLookAt(eyex, eyey, eyez, 0.0, 0.0, 0.0, upx, upy, upz);
}

int rotateBox(float dTheta, float dPhi)
{
  return setCameraThetaPhiOmega(visuCamera->theta + dTheta,
				visuCamera->phi + dPhi,
				0., MASK_THETA | MASK_PHI);
}

void openGLSet_preferedObserveMethod(int method)
{
  if (method != OPENGL_OBSERVE_CONSTRAINED && method != OPENGL_OBSERVE_WALKER)
    {
      fprintf(stderr, "WARNING! 'openGLSet_preferedObserveMethod' has been"
	      " called with a wrong method argument.\n");
      return;
    }
  openGL_preferedObserveMethod = method;
}
int openGLGet_preferedObserveMethod()
{
  return openGL_preferedObserveMethod;
}


void matrixProduct(float matRes[3][3], float matA[3][3], float matB[3][3])
{
  int i, j, k;

  for (i = 0; i < 3; i++)
    for (j = 0; j < 3; j++)
      {
	matRes[i][j] = 0.;
	for (k = 0; k < 3; k++)
	  matRes[i][j] += matA[i][k] * matB[k][j];
      }
}
void vectorProduct(float vectRes[3], float mat[3][3], float vect[3])
{
  int i, j;

  for (i = 0; i < 3; i++)
    {
      vectRes[i] = 0.;
      for (j = 0; j < 3; j++)
	vectRes[i] += mat[i][j] * vect[j];
    }
}

int rotateCamera(float dTheta, float dPhi)
{
  double cth, sth, cph, sph, com, som;
  double cdth, sdth, cdph, sdph;
  double Theta, Phi, Omega;
  #define RADTODEG 57.29577951
  float MinRprime[3], MinR[3];
  float Mspherical[3];
  float matPhi[3][3], matTheta[3][3], matOmega[3][3], matdPhi[3][3], matdTheta[3][3];
  float matPhiPrime[3][3], matThetaPrime[3][3];
  float matTmp[3][3];
  float matRprime2R[3][3];
  float matRes[3][3], matRes2[3][3];

  cth = cos(visuCamera->theta * PI180);
  sth = sin(visuCamera->theta * PI180);
  cph = cos(visuCamera->phi * PI180);
  sph = sin(visuCamera->phi * PI180);
  com = cos(visuCamera->omega * PI180);
  som = sin(visuCamera->omega * PI180);

  cdth = cos(dTheta * PI180);
  sdth = sin(dTheta * PI180);
  cdph = cos(dPhi * PI180);
  sdph = sin(dPhi * PI180);


  matPhi[0][0] = cph;
  matPhi[1][0] = sph;
  matPhi[2][0] = 0.;
  matPhi[0][1] = -sph;
  matPhi[1][1] = cph;
  matPhi[2][1] = 0.;
  matPhi[0][2] = 0.;
  matPhi[1][2] = 0.;
  matPhi[2][2] = 1.;

  matTheta[0][0] = cth;
  matTheta[1][0] = 0.;
  matTheta[2][0] = -sth;
  matTheta[0][1] = 0.;
  matTheta[1][1] = 1.;
  matTheta[2][1] = 0.;
  matTheta[0][2] = sth;
  matTheta[1][2] = 0.;
  matTheta[2][2] = cth;

  matOmega[0][0] = com;
  matOmega[1][0] = som;
  matOmega[2][0] = 0.;
  matOmega[0][1] = -som;
  matOmega[1][1] = com;
  matOmega[2][1] = 0.;
  matOmega[0][2] = 0.;
  matOmega[1][2] = 0.;
  matOmega[2][2] = 1.;

  matdPhi[0][0] = 1.;
  matdPhi[1][0] = 0.;
  matdPhi[2][0] = 0.;
  matdPhi[0][1] = 0.;
  matdPhi[1][1] = cdph;
  matdPhi[2][1] = -sdph;
  matdPhi[0][2] = 0.;
  matdPhi[1][2] = sdph;
  matdPhi[2][2] = cdph;

  matdTheta[0][0] = cdth;
  matdTheta[1][0] = 0.;
  matdTheta[2][0] = -sdth;
  matdTheta[0][1] = 0.;
  matdTheta[1][1] = 1.;
  matdTheta[2][1] = 0.;
  matdTheta[0][2] = sdth;
  matdTheta[1][2] = 0.;
  matdTheta[2][2] = cdth;

  matrixProduct(matRes, matdPhi, matdTheta);
  matrixProduct(matRes2, matOmega, matRes);
  matrixProduct(matRes, matTheta, matRes2);
  matrixProduct(matRprime2R, matPhi, matRes);

  MinRprime[0] = 0.;
  MinRprime[1] = 0.;
  MinRprime[2] = 1.;
  vectorProduct(MinR, matRprime2R, MinRprime);
/*   fprintf(stderr, "M : %f %f %f -> %f\n", MinR[0], MinR[1], MinR[2], */
/* 	  MinR[0]*MinR[0] + MinR[1]*MinR[1] + MinR[2]*MinR[2]); */

/*   cartesian_to_spherical(Mspherical, MinR); */
  Mspherical[0] = sqrt(MinR[0]*MinR[0] + MinR[1]*MinR[1] + MinR[2]*MinR[2]);
  if (MinR[1] == 0 && MinR[0] == 0)
    {
      Mspherical[1] = (MinR[2] > 0.)?0.:180.;
      Mspherical[2] = visuCamera->phi;
    }
  else
    {
      Mspherical[1] = acos(MinR[2] / Mspherical[0]) * RADTODEG;
      if (MinR[0] == 0.)
	Mspherical[2] = (MinR[1] > 0.)?90.:-90.;
      else
	{
	  Mspherical[2] = atan(MinR[1] / MinR[0]) * RADTODEG;
	  if (MinR[0] < 0.)
	    Mspherical[2] += 180.;
	}
    }
/*   fprintf(stderr, "avant %f %f\n", Mspherical[1], Mspherical[2]); */
  while (Mspherical[1] - visuCamera->theta < -90.)
    Mspherical[1] += 360.;
  while (Mspherical[1] - visuCamera->theta > 90.)
    Mspherical[1] -= 360.;
  while (Mspherical[2] - visuCamera->phi < -90.)
    Mspherical[2] += 360.;
  while (Mspherical[2] - visuCamera->phi > 90.)
    Mspherical[2] -= 360.;
/*   fprintf(stderr, "aprs %f %f\n", Mspherical[1], Mspherical[2]); */

  Theta = Mspherical[1];
  Phi = Mspherical[2];

/*   fprintf(stderr, "%f %f, %f %f\n", visuCamera->theta, visuCamera->phi, Theta, Phi); */
/*   fprintf(stderr, "%f %f, %f %f\n", dTheta, dPhi, Theta - visuCamera->theta, Phi -  visuCamera->phi); */

  cth = cos(Theta * PI180);
  sth = sin(Theta * PI180);
  cph = cos(Phi * PI180);
  sph = sin(Phi * PI180);

  matPhiPrime[0][0] = cph;
  matPhiPrime[1][0] = -sph;
  matPhiPrime[2][0] = 0.;
  matPhiPrime[0][1] = sph;
  matPhiPrime[1][1] = cph;
  matPhiPrime[2][1] = 0.;
  matPhiPrime[0][2] = 0.;
  matPhiPrime[1][2] = 0.;
  matPhiPrime[2][2] = 1.;

  matThetaPrime[0][0] = cth;
  matThetaPrime[1][0] = 0.;
  matThetaPrime[2][0] = sth;
  matThetaPrime[0][1] = 0.;
  matThetaPrime[1][1] = 1.;
  matThetaPrime[2][1] = 0.;
  matThetaPrime[0][2] = -sth;
  matThetaPrime[1][2] = 0.;
  matThetaPrime[2][2] = cth;

  matrixProduct(matRes2, matPhiPrime, matRprime2R);
  matrixProduct(matRes, matThetaPrime, matRes2);

  MinRprime[0] = 0.;
  MinRprime[1] = 1.;
  MinRprime[2] = 0.;
  vectorProduct(MinR, matRes, MinRprime);
/*   fprintf(stderr, "vect u : %f %f %f -> %f\n", MinR[0], MinR[1], MinR[2], */
/* 	  MinR[0]*MinR[0] + MinR[1]*MinR[1] + MinR[2]*MinR[2]); */
  Omega = acos(MinR[1]) * RADTODEG;
  if (MinR[0] > 0.)
    Omega = -Omega;
  while (Omega - visuCamera->omega < -90.)
    Omega += 360.;
  while (Omega - visuCamera->omega > 90.)
    Omega -= 360.;

/*   fprintf(stderr, "Theta phi omega : %f %f %f\n", Theta, Phi, Omega); */

  return setCameraThetaPhiOmega(Theta, Phi, Omega, MASK_THETA | MASK_PHI | MASK_OMEGA);
}

/******************************************************************************/

void gl_new_data(char *fn)
{
   double p1_x, p2_x, p3_x, p4_x, p5_x, p6_x, p7_x, p8_x;
   double p1_y, p2_y, p3_y, p4_y, p5_y, p6_y, p7_y, p8_y;
   double p1_z, p2_z, p3_z, p4_z, p5_z, p6_z, p7_z, p8_z;
   double sc, su;
/*    int i; */
 
   DBG_fprintf(stderr, "Visu OpenGL : a new file have been loaded, recreating the geometry.\n");

   visuBox->dxxs2 = 0.5 * (visuDataGet_boxGeometry(currentVisuData, VISU_DATA_BOX_DXX) +
			   visuDataGet_boxGeometry(currentVisuData, VISU_DATA_BOX_DYX) +
			   visuDataGet_boxGeometry(currentVisuData, VISU_DATA_BOX_DZX));
   visuBox->dyys2 = 0.5 * (visuDataGet_boxGeometry(currentVisuData, VISU_DATA_BOX_DYY) +
			   visuDataGet_boxGeometry(currentVisuData, VISU_DATA_BOX_DZY));
   visuBox->dzzs2 = 0.5 * (visuDataGet_boxGeometry(currentVisuData, VISU_DATA_BOX_DZZ));
   
   p1_x = p1_y = p1_z = 0.0;
   p2_x = visuDataGet_boxGeometry(currentVisuData, VISU_DATA_BOX_DXX);
   p2_y = p2_z = 0.0;
   p4_x = visuDataGet_boxGeometry(currentVisuData, VISU_DATA_BOX_DYX);
   p4_y = visuDataGet_boxGeometry(currentVisuData, VISU_DATA_BOX_DYY);
   p4_z = 0.0;
   p5_x = visuDataGet_boxGeometry(currentVisuData, VISU_DATA_BOX_DZX);
   p5_y = visuDataGet_boxGeometry(currentVisuData, VISU_DATA_BOX_DZY);
   p5_z = visuDataGet_boxGeometry(currentVisuData, VISU_DATA_BOX_DZZ);
      
   p3_x = p2_x + p4_x;
   p3_y = p2_y + p4_y;
   p3_z = p2_z + p4_z;
      
   p6_x = p5_x + p2_x;
   p6_y = p5_y + p2_y;
   p6_z = p5_z + p2_z;
      
   p7_x = p5_x + p3_x;
   p7_y = p5_y + p3_y;
   p7_z = p5_z + p3_z;
      
   p8_x = p5_x + p4_x;
   p8_y = p5_y + p4_y;
   p8_z = p5_z + p4_z;
   visuBox->p1[0] = p1_x;
   visuBox->p1[1] = p1_y;
   visuBox->p1[2] = p1_z;

   visuBox->p2[0] = p2_x;
   visuBox->p2[1] = p2_y;
   visuBox->p2[2] = p2_z;

   visuBox->p3[0] = p3_x;
   visuBox->p3[1] = p3_y;
   visuBox->p3[2] = p3_z;

   visuBox->p4[0] = p4_x;
   visuBox->p4[1] = p4_y;
   visuBox->p4[2] = p4_z;

   visuBox->p5[0] = p5_x;
   visuBox->p5[1] = p5_y;
   visuBox->p5[2] = p5_z;

   visuBox->p6[0] = p6_x;
   visuBox->p6[1] = p6_y;
   visuBox->p6[2] = p6_z;

   visuBox->p7[0] = p7_x;
   visuBox->p7[1] = p7_y;
   visuBox->p7[2] = p7_z;

   visuBox->p8[0] = p8_x;
   visuBox->p8[1] = p8_y;
   visuBox->p8[2] = p8_z;

   /* calculate bare = 1/2 radius of centered sample */
   su = (p7_x-p1_x)*(p7_x-p1_x)+
        (p7_y-p1_y)*(p7_y-p1_y)+
        (p7_z-p1_z)*(p7_z-p1_z);
   sc = (p8_x-p2_x)*(p8_x-p2_x)+
        (p8_y-p2_y)*(p8_y-p2_y)+
        (p8_z-p2_z)*(p8_z-p2_z);
   if (sc > su) su = sc;
   sc = (p5_x-p3_x)*(p5_x-p3_x)+
        (p5_y-p3_y)*(p5_y-p3_y)+
        (p5_z-p3_z)*(p5_z-p3_z);
   if (sc > su) su = sc;
   sc = (p6_x-p4_x)*(p6_x-p4_x)+
        (p6_y-p4_y)*(p6_y-p4_y)+
        (p6_z-p4_z)*(p6_z-p4_z);
   if (sc > su) su = sc;
   
   update_extens_etc(0.5f*(float)sqrt(su));

   if (!OpenGLIsPixmap())
     rename_window(fn);
   
   g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLExtens_signal_id,
		  0 , NULL);
}

/******************************************************************************/

void project(void) { 
   double xmin, xmax, ymin, ymax;
   double rap_usr, rap_win;
       
   /* OpenGL maps near plane with viewport
      But I prefer to map 0 (fixed) plane with viewport
   */

   /*  remplacer quand visuBox intgrera visuData */
   if (!currentVisuData)
     return;
    
   visuWindow->near = (visuCamera->d_red - 1.0) * visuBox->extens;
   visuWindow->far  = (visuCamera->d_red + 1.0) * visuBox->extens;

   g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLNearFar_signal_id,
		  0 , NULL);

   xmin = (    - visuCamera->xs) * 2.0 *
     visuBox->extens/visuCamera->gr/visuCamera->gross;
   xmax = (1.0 - visuCamera->xs) * 2.0 *
     visuBox->extens/visuCamera->gr/visuCamera->gross;
   ymin = (    - visuCamera->ys) * 2.0 *
     visuBox->extens/visuCamera->gr/visuCamera->gross;
   ymax = (1.0 - visuCamera->ys) * 2.0 *
     visuBox->extens/visuCamera->gr/visuCamera->gross;
   visuWindow->left   = xmin;
   visuWindow->bottom = ymin;
   rap_usr = (1.0*(ymax - ymin))/(xmax - xmin);
   rap_win = (1.0*visuWindow->height)/visuWindow->width;
    
   if ( rap_usr > rap_win ) {
      visuWindow->top   = ymax;
      visuWindow->right = xmin + (ymax - ymin) / rap_win;
   }
   else { 
      visuWindow->right = xmax;
      visuWindow->top   = ymin + (xmax - xmin) * rap_win;
   }

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glFrustum(visuWindow->left, visuWindow->right, visuWindow->bottom,
	     visuWindow->top, visuWindow->near, visuWindow->far);
   glMatrixMode(GL_MODELVIEW); 
        
}

/******************************************************************************/

int initOpengl(int width, int height)
{
  VisuConfigFileEntry *resourceEntry;

  DBG_fprintf(stderr, "OpenGl : initialization.\n");
  visuCamera = malloc(sizeof(OpenGLCamera));
  visuBox = malloc(sizeof(OpenGLBox));
  visuWindow = malloc(sizeof(OpenGLWindow));
  if (!visuCamera || !visuBox || !visuWindow)
    {
      allocationProblems();
      return 0;
    }

  visuWindow->width = width;
  visuWindow->height = height;
  visuWindow->near = 0.;
  visuWindow->far = 0.;

  visuCamera->theta = 40.;
  visuCamera->phi = -50.;
  visuCamera->omega = 0.;
  visuCamera->xs = 0.5;
  visuCamera->ys = 0.5;
  visuCamera->gross = 1.;
  visuCamera->d_red = 5.;
  visuCamera->gr = visuCamera->d_red / (visuCamera->d_red - 1.);

  observeValues = malloc(sizeof(struct observeData));
  if (!observeValues)
    {
      allocationProblems();
      return 0;
    }

  gl_init();

  /* Resources */
  resourceEntry = visuConfigFileAdd_entry(VISU_CONFIGFILE_RESOURCE,
					  FLAG_RESOURCE_OPENGL_ANGLES,
					  DESC_RESOURCE_OPENGL_ANGLES,
					  1, readOpenGLThetaPhi);
  resourceEntry = visuConfigFileAdd_entry(VISU_CONFIGFILE_RESOURCE,
					  FLAG_RESOURCE_OPENGL_TRANSLAT,
					  DESC_RESOURCE_OPENGL_TRANSLAT,
					  1, readOpenGLXsYs);
  resourceEntry = visuConfigFileAdd_entry(VISU_CONFIGFILE_RESOURCE,
					  FLAG_RESOURCE_OPENGL_GROSS,
					  DESC_RESOURCE_OPENGL_GROSS,
					  1, readOpenGLGross);
  resourceEntry = visuConfigFileAdd_entry(VISU_CONFIGFILE_RESOURCE,
					  FLAG_RESOURCE_OPENGL_PERSP,
					  DESC_RESOURCE_OPENGL_PERSP,
					  1, readOpenGLPersp);
  visuConfigFileAdd_exportFunction(VISU_CONFIGFILE_RESOURCE,
				   exportResourcesOpenGL);
  /* Parameters */
  resourceEntry = visuConfigFileAdd_entry(VISU_CONFIGFILE_PARAMETER,
					  FLAG_PARAMETER_OPENGL_DETAILS,
					  DESC_PARAMETER_OPENGL_DETAILS,
					  1, readOpenGLPrecision);
  resourceEntry = visuConfigFileAdd_entry(VISU_CONFIGFILE_PARAMETER,
					  FLAG_PARAMETER_OPENGL_RENDERING,
					  DESC_PARAMETER_OPENGL_RENDERING,
					  1, readOpenGLRendering);
  resourceEntry = visuConfigFileAdd_entry(VISU_CONFIGFILE_PARAMETER,
					  FLAG_PARAMETER_OPENGL_ANTIALIAS,
					  DESC_PARAMETER_OPENGL_ANTIALIAS,
					  1, readOpenGLAntialias);
  resourceEntry = visuConfigFileAdd_entry(VISU_CONFIGFILE_PARAMETER,
					  FLAG_PARAMETER_OPENGL_IMMEDIATE,
					  DESC_PARAMETER_OPENGL_IMMEDIATE,
					  1, readOpenGLImmediate);
  resourceEntry = visuConfigFileAdd_entry(VISU_CONFIGFILE_PARAMETER,
					  FLAG_PARAMETER_OBSERVE_METHOD,
					  DESC_PARAMETER_OBSERVE_METHOD,
					  1, readOpenGLObserveMethod);
  visuConfigFileAdd_exportFunction(VISU_CONFIGFILE_PARAMETER,
				   exportParametersOpenGL);


  setPrecision((float)PARAMETER_OPENGL_DETAILS_DEFAULT / 100.);

  isOBJECTReady = 0;
  renderingOption = PARAMETER_OPENGL_RENDERING_DEFAULT;
  antialias = PARAMETER_OPENGL_ANTIALIAS_DEFAULT;
  fakeBackingStore = PARAMETER_OPENGL_FAKEBS_DEFAULT;
  opengl_immediate = PARAMETER_OPENGL_IMMEDIATE_DEFAULT;
  openGL_preferedObserveMethod = OPENGL_OBSERVE_CONSTRAINED;


  /* Add a callback to reinitialisate the data when a new file is loaded. */
  g_signal_connect(G_OBJECT(visu), "dataReadyForRendering",
		   G_CALLBACK(newFileLoaded), (gpointer)0);
  g_signal_connect(G_OBJECT(visu), "OpenGLExtensionChanged",
		   G_CALLBACK(reDraw), (gpointer)0);
  g_signal_connect(G_OBJECT(visu), "OpenGLAskForReDraw",
		   G_CALLBACK(reDraw), (gpointer)0);
  g_signal_connect(G_OBJECT(visu), "OpenGLForceReDraw",
		   G_CALLBACK(reDraw), (gpointer)1);

  return 1;
}

/******************************************************************************/

/* Set the specified identifier as the OBJECT list that is drawn. If
   identifier is null then the OBJECT list is deleted. */
int setObjectList(int identifier)
{
  if (!identifier)
    {
      drawToEmpty();
      isOBJECTReady = 0;
      return 0;
    }
  else
    {
      isOBJECTReady = 1;
      return 1;
    }
}

/******************************************************************************/
void glSetLighting()
{
/*    float ambient[4] =  {1.0f, 1.0f, 1.0f, 1.0f}; */
/*    float diffuse[4] =  {1.0f, 1.0f, 1.0f, 1.0f}; */
/*    float specular[4] = {1.0f, 1.0f, 1.0f, 1.0f}; */
/*    float position[4] = {3.0f, 2.0f, 1.7f, 0.0f}; /\* w = 0 directional *\/ */
   float lmodel_ambient[4] = {0.2f, 0.2f, 0.2f, 1.0f};
      
/*    glLightfv(GL_LIGHT0, GL_AMBIENT, ambient); */
/*    glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse); */
/*    glLightfv(GL_LIGHT0, GL_SPECULAR, specular); */
/*    glLightfv(GL_LIGHT0, GL_POSITION, position); */

/*    glLightfv(GL_LIGHT1, GL_AMBIENT, ambient); */
/*    glLightfv(GL_LIGHT1, GL_DIFFUSE, diffuse); */
/*    glLightfv(GL_LIGHT1, GL_SPECULAR, specular); */
/*    glLightfv(GL_LIGHT1, GL_POSITION, position1); */
   glPushMatrix();
   glLoadIdentity();

   glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
   glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_FALSE);
   glEnable(GL_LIGHTING);
/*    glEnable(GL_LIGHT0); */

   g_list_foreach(light_get_list(), lighting_set, NULL);
   
   glPopMatrix();
}

void setOpenGlMaterial(float* material, float* rgba)
{
  float mm[4] = {0.0f, 0.0f, 0.0f, 0.0f};

  mm[3] = rgba[3];
  mm[0] = material[amb] * rgba[0];
  mm[1] = material[amb] * rgba[1];
  mm[2] = material[amb] * rgba[2];
  glMaterialfv(GL_FRONT, GL_AMBIENT, mm);
  mm[0] = material[dif] * rgba[0];
  mm[1] = material[dif] * rgba[1];
  mm[2] = material[dif] * rgba[2];
  glMaterialfv(GL_FRONT, GL_DIFFUSE, mm);
  glMaterialf(GL_FRONT, GL_SHININESS, material[shi] * 128.0f);
  mm[0] = material[spe];
  mm[1] = material[spe];
  mm[2] = material[spe];
  glMaterialfv(GL_FRONT, GL_SPECULAR, mm);
  mm[0] = material[emi] * rgba[0];
  mm[1] = material[emi] * rgba[1];
  mm[2] = material[emi] * rgba[2];
  glMaterialfv(GL_FRONT, GL_EMISSION, mm);
}


void gl_init(void)
{
/*   glEnable(GL_DEPTH_TEST); */
/*   glDepthFunc(GL_LEQUAL); */
/*   glDepthRange(0.0, 1.0); */
/*   glClearDepth(1.0); */

/*   glEnable(GL_CULL_FACE); */
/*   glCullFace(GL_BACK); */

  obj = gluNewQuadric();
   
  glViewport(0, 0, visuWindow->width, visuWindow->height);

  /* Initialisation of the table that stores the user
     glObjectList. */
  glObjectListRegistered = malloc(sizeof(int) * GLOBJECTLIST_INCREMENT);
  if (!glObjectListRegistered)
    {
      allocationProblems();
      exit(1);
    }
  glObjectListRegisteredSize = GLOBJECTLIST_INCREMENT;
  nbGlObjectListRegistered = 0;
  lastGlObjectListSize = 0;

  lighting_init();
}

int setOpenGLContext(gboolean contextIsPixmap, DumpImage **dumpData)
{
  int res;

  DBG_fprintf(stderr, "Opengl : changing Opengl context.\n");
  if (contextIsPixmap)
    {
      /* Change the context to have a indirect rendering to
	 be able to write to a glxpixmap. */
      *dumpData = dumpImageNew(visuWindow->width, visuWindow->height);
      if (!*dumpData)
	return 1;
  
      res = attachOpenGlContext(TRUE, dumpImageGet_buffer(*dumpData));
      if (!res)
	{
	  dumpImageFree(*dumpData);
	  return 1;
	}
    }
  else
    {
      res = attachOpenGlContext(FALSE, (gpointer)0);
      if (!res)
	return 1;
    }

  /* Set the openGL flags. */
  glEnable(GL_DEPTH_TEST);
  glDepthFunc(GL_LEQUAL);
  glDepthRange(0.0, 1.0);
  glClearDepth(1.0);

  glEnable(GL_CULL_FACE);
  glCullFace(GL_BACK);

  glEnable(GL_NORMALIZE);

  glSetLighting();
  glSetRendering();
  glSetAntiAlias();
  return 0;
}

/******************************************************************************/

void reDraw(GObject* obj, gpointer data)
{
  if (!isOBJECTReady)
    {
      DBG_fprintf(stderr, "Redraw rejected since we've got no valid OBJECT list.\n");
      return;
    }
  if (!opengl_immediate && !GPOINTER_TO_INT(data))
    {
      DBG_fprintf(stderr, "Redraw rejected since drawing is deferred and not forced.\n");
      return;
    }
    
  DBG_fprintf(stderr, "Redraw accepted let's go...\n");

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  callAllExtensionsLists();

  /* display (and prepare next buffer for write in) */
  callSwapGLBuffer();
}


void drawToEmpty()
{
  glClearColor(0., 0., 0., 0.0f);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  callSwapGLBuffer();
} 

void drawEraseMark(int which, int xx, int yy, int draw_it)
{
#define pick_mark_half_size 4
   static int pick_mark_x[2], pick_mark_y[2];
   static int mark_present[2] = {0, 0};
      
   if(draw_it) {
      pick_mark_x[which] = xx - pick_mark_half_size;
      pick_mark_y[which] = (visuWindow->height - yy) - pick_mark_half_size;
      mark_present[which] = 1;
   }
   else {
      if(!mark_present[which]) return;
      mark_present[which] = 0;
   }
   
   glDrawBuffer(GL_FRONT);
   
   glMatrixMode(GL_PROJECTION);
   glPushMatrix();
   glLoadIdentity();
   gluOrtho2D(0.0, (float)visuWindow->width, 0.0, (float)visuWindow->height);
   
   glMatrixMode(GL_MODELVIEW);
   glPushMatrix();
   glLoadIdentity();
   
   glDisable(GL_DEPTH_TEST);
   glDisable(GL_LIGHTING);
/*    glDisable(GL_FOG); */
   glDisable(GL_DITHER);
   glEnable(GL_BLEND);
   glBlendFunc(GL_ONE_MINUS_DST_COLOR, GL_ZERO);
   glColor3f(1.0f, 1.0f, 1.0f);
   if(which == 0) {
      glRecti(pick_mark_x[which], pick_mark_y[which],
              pick_mark_x[which]+2*pick_mark_half_size,
              pick_mark_y[which]+2*pick_mark_half_size);
   }
   else {
      glRecti(pick_mark_x[which], pick_mark_y[which],
              pick_mark_x[which]+pick_mark_half_size,
              pick_mark_y[which]+pick_mark_half_size);
   }
   glEnable(GL_DEPTH_TEST);  
   glEnable(GL_LIGHTING);
/*    glEnable(GL_FOG); */
   glEnable(GL_DITHER); /* WARNING: it is the default! */
   /* We need to recall the state of the blend for the
      antialiasing or not. */
  if (antialias)
    {
      /* Antialias */
      glEnable(GL_LINE_SMOOTH);
      glEnable(GL_BLEND);
      glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
      glHint(GL_LINE_SMOOTH_HINT, GL_DONT_CARE);
    }
  else
    {
      glDisable(GL_LINE_SMOOTH);
      glDisable(GL_BLEND);
    }
   
   glPopMatrix();
   
   glMatrixMode(GL_PROJECTION);
   glPopMatrix();
   glMatrixMode(GL_MODELVIEW);
   
   glDrawBuffer(GL_BACK);
   
   glFlush();
}

/******************************************************************************/

void newFileLoaded(GObject *obj, gpointer data)
{
  gchar *nom, *file;

  /* Entraine un changement d'extens. */
  nom = visuDataGet_file(currentVisuData, 0);
  if (nom)
    {
      file = g_path_get_basename(nom);
      gl_new_data(file);
      g_free(file);
    }
  else
    {
      fprintf(stderr, _("WARNING! Can't find the filename to label the rendering window.\n"));
      gl_new_data(_("No filename"));
    }

  /* effectue les projections. */
  project();
  modelize();
  g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLFacetteChanged_signal_id,
		 0 , NULL);
}

int setNewWindowSize(unsigned int width, unsigned int height)
{
  if (visuWindow->width != width || visuWindow->height != height)
    {
      visuWindow->width = width;
      visuWindow->height = height;
      glViewport(0, 0, visuWindow->width, visuWindow->height);
      project();
      g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLWidthHeight_signal_id,
		     0 , NULL);
      g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLFacetteChanged_signal_id,
		     0 , NULL);
      return 1;
    }
  return 0;
}


/******************************************************************************/

/* It returns the number that can be used to link glObjectList.
   This number is also added to the list of glObjectList to be displayed.
   The size parameter is the number of lists that could be used by
   the user for this glObjectList. Then the next call to this function
   will return the last value plus the size plus one. */
int createNewObjectList(int size)
{
  if (nbGlObjectListRegistered >= glObjectListRegisteredSize)
    {
      glObjectListRegistered = realloc(glObjectListRegistered,
				       sizeof(int) * (glObjectListRegisteredSize + 
						      GLOBJECTLIST_INCREMENT));
      if (!glObjectListRegistered)
	{
	  allocationProblems();
	  exit(1);
	}
      glObjectListRegisteredSize += GLOBJECTLIST_INCREMENT;
    }
  if (nbGlObjectListRegistered > 0)
    glObjectListRegistered[nbGlObjectListRegistered] = 
      glObjectListRegistered[nbGlObjectListRegistered - 1] + lastGlObjectListSize + 1;
  else
    glObjectListRegistered[0] = GLOBJECTLIST_FIRST;
  lastGlObjectListSize += size;
  nbGlObjectListRegistered++;
  return glObjectListRegistered[nbGlObjectListRegistered - 1];
}

/******************************************************************************/

void observe(SimplifiedEvents *ev)
{
  int reDrawNeeded;
  int sign_theta;
  int dx, dy;

  if (!ev)
    {
      fprintf(stderr, _("INTERNAL ERROR! 'observe' has been called"
			" with a null SimplifiedEvents parameter.\n"));
      return;
    }

/*   fprintf(stderr, "%d %d, %d\n", ev->x, ev->y, ev->button); */
/*   fprintf(stderr, "%d %d, %d\n", ev->shiftMod, ev->controlMod, ev->motion); */
  reDrawNeeded = 0;
  if (ev->button && !ev->motion)
    {
      observeValues->xi = ev->x;
      observeValues->yi = ev->y;
      /* Support de la roulette en zoom et perspective. */
      if (ev->button == 4 && !ev->shiftMod)
	reDrawNeeded = setCameraGross(visuCamera->gross * 1.1);
      else if (ev->button == 4 && ev->shiftMod)
	reDrawNeeded = setCameraPersp(visuCamera->d_red / 1.1);
      else if (ev->button == 5 && !ev->shiftMod)
	reDrawNeeded = setCameraGross(visuCamera->gross / 1.1);
      else if (ev->button == 5 && ev->shiftMod)
	reDrawNeeded = setCameraPersp(visuCamera->d_red * 1.1);
    }
  else if (ev->motion)
    {
      dx =   ev->x - observeValues->xi;
      dy = -(ev->y - observeValues->yi);
      if(ev->button == 1 && !ev->shiftMod && !ev->controlMod)
	{
	  if (openGL_preferedObserveMethod == OPENGL_OBSERVE_CONSTRAINED)
	    {
	      if(visuCamera->theta > 0.0)
		sign_theta = 1;
	      else
		sign_theta = -1;
	      reDrawNeeded = rotateBox(dy * observeValues->delta_theta,
				       -dx * observeValues->delta_phi * sign_theta);
	    }
	  else if (openGL_preferedObserveMethod == OPENGL_OBSERVE_WALKER)
	    reDrawNeeded = rotateCamera(dy * observeValues->delta_theta,
					-dx * observeValues->delta_phi);
	}
      else if(ev->button == 1 && ev->shiftMod && !ev->controlMod)
	reDrawNeeded =
	  setCameraXsYs(visuCamera->xs + dx * observeValues->delta_xs,
			visuCamera->ys + dy * observeValues->delta_ys,
			MASK_XS | MASK_YS);
      else if(ev->button == 1 && ev->controlMod && !ev->shiftMod)
	{
	  if (abs(dx) > abs(dy))
	    reDrawNeeded = setCameraThetaPhiOmega(0., 0.,
						  visuCamera->omega + dx *
						  observeValues->delta_omega, MASK_OMEGA);
	  else
	    reDrawNeeded = setCameraThetaPhiOmega(0., 0.,
						  visuCamera->omega + dy *
						  observeValues->delta_omega, MASK_OMEGA);
	}
      else if(ev->button == 2 && !ev->shiftMod)
	reDrawNeeded = setCameraGross(visuCamera->gross *
				      (1. + (float)dy *
				       observeValues->delta_gross));
      else if(ev->button == 2 && ev->shiftMod)
	reDrawNeeded = setCameraPersp(visuCamera->d_red *
				      (1. - (float)dy *
				       observeValues->delta_d_red));
      observeValues->xi = ev->x;
      observeValues->yi = ev->y;
    }
  else if (ev->letter == 'r')
    {
      reDrawNeeded = setCameraThetaPhiOmega(observeValues->old_theta,
					    observeValues->old_phi,
					    observeValues->old_omega,
					    MASK_THETA | MASK_PHI | MASK_OMEGA) ||
	reDrawNeeded;
      reDrawNeeded = setCameraXsYs(observeValues->old_xs,
				   observeValues->old_ys,
				   MASK_XS | MASK_YS) ||
	reDrawNeeded;
      reDrawNeeded = setCameraGross(observeValues->old_gross) ||
	reDrawNeeded;
      reDrawNeeded = setCameraPersp(observeValues->old_d_red) ||
	reDrawNeeded;
    }
  if (reDrawNeeded)
    reDraw((GObject*)0, (gpointer)1);
}

void initPickObserve(callBackOnNodeSelectedFunc handler,
		     GDestroyNotify stopPickObserve)
{
  observeValues->delta_phi = 180.0f / visuWindow->width;
  observeValues->delta_theta = 180.0f / visuWindow->height;
  observeValues->delta_omega = 180.0f / visuWindow->width;
  observeValues->delta_xs = 1.0f / visuWindow->width;
  observeValues->delta_ys = 1.0f / visuWindow->height;
  observeValues->delta_gross = 3.0f / visuWindow->height;
  observeValues->delta_d_red = 5.0f / visuWindow->height;
  observeValues->old_theta = visuCamera->theta;
  observeValues->old_phi = visuCamera->phi;
  observeValues->old_omega = visuCamera->omega;
  observeValues->old_xs = visuCamera->xs;
  observeValues->old_ys = visuCamera->ys;
  observeValues->old_gross = visuCamera->gross;
  observeValues->old_d_red = visuCamera->d_red;

  stopStandardEventListener();

  callbackOnNodeSelected = handler;

  pickObserveCallbackId = (GList*)0;

  callbackPickObserve.action = observe;
  callbackPickObserve.stop = stopPickObserve;
}

void beginObserve(void)
{
  removeEventListener(pickObserveCallbackId);
  pickObserveCallbackId = (GList*)0;

  callbackPickObserve.action = observe;
  pickObserveCallbackId = setObserveEventListener(&callbackPickObserve);
}

void endPickObserve(void)
{
  removeEventListener(pickObserveCallbackId);
  pickObserveCallbackId = (GList*)0;

  restoreStandardEventListener();
}

/******************************************************************************/

void pick(SimplifiedEvents *ev)
{
  VisuPick nodeInfo;
      
  nodeInfo.button = ev->button;
  nodeInfo.x = ev->x;
  nodeInfo.y = ev->y;
  nodeInfo.shiftMod = ev->shiftMod;
  nodeInfo.node = getSelectElement(ev->x, ev->y);
  if (DEBUG)
    {
      if (nodeInfo.node)
	{
	  DBG_fprintf(stderr, "Picked node %d, coord %f %f %f\n",
		      nodeInfo.node->number, nodeInfo.node->x,
		      nodeInfo.node->y, nodeInfo.node->z);
	}
      else
	{
	  DBG_fprintf(stderr, "You picked nothing.\n");
	}
    }
  if (callbackOnNodeSelected)
    callbackOnNodeSelected(&nodeInfo);
  else
    fprintf(stderr, _("WARNING! No method has been specified to detail what"
		      " to do when a node is selected.\n"));
}

void beginPick()
{
  removeEventListener(pickObserveCallbackId);
  pickObserveCallbackId = (GList*)0;

  callbackPickObserve.action = pick;
  pickObserveCallbackId = setPickEventListener(&callbackPickObserve);
}

VisuNode* getSelectElement(int x, int y)
{
#define bufsize 512
   GLuint select_buf[bufsize];
   GLint viewport[4] = {0, 0, 0, 0};
#define wpck 2.0
#define hpck 2.0
   int hits, names, ptr, i;
   unsigned int z1;
   unsigned int z1_sauve = UINT_MAX;
   int number;
   int found;
   VisuNode* res;

   res = (VisuNode*)0;

   glSelectBuffer(bufsize, select_buf);
   hits = glRenderMode(GL_SELECT);
   glInitNames();
   glPushName(-1);
      
   viewport[2] = visuWindow->width;
   viewport[3] = visuWindow->height;
   glNewList(10, GL_COMPILE);
      gluPickMatrix(1.0*x, 1.0*(visuWindow->height-y), wpck, hpck, viewport);
   glEndList();

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glCallList(10);
   glFrustum(visuWindow->left, visuWindow->right, visuWindow->bottom,
	     visuWindow->top, visuWindow->near, visuWindow->far);
   glMatrixMode(GL_MODELVIEW); 

   glCallList(visuDataGet_objectList());
   glFlush();

   hits = glRenderMode(GL_RENDER);
   DBG_fprintf(stderr, "%d elements are on the z buffer.\n", hits);
   ptr = 0;

   /* return the buffer to normal */
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glFrustum(visuWindow->left, visuWindow->right, visuWindow->bottom,
	     visuWindow->top, visuWindow->near, visuWindow->far);
   glMatrixMode(GL_MODELVIEW); 

   found = 0;
   number = -1;
   for(i=0; i<hits; i++) {
      names = select_buf[ptr];
      if (names != 1) {
	(void)fprintf(stderr, _("WARNING! OpenGL picking is not working???\n"));
	 return (VisuNode*)0;
      }
      ptr = ptr + 1;
      z1 = select_buf[ptr];
      ptr = ptr + 2;
      if (z1 < z1_sauve) {
         z1_sauve = z1;
	 number = (int)select_buf[ptr];
	 found = 1;
      }
      ptr = ptr + 1;
   }
   if (found && number >= 0)
     res = visuDataGet_fromNumber(currentVisuData, number);
   return res;
}

/******************************************************************************/

unsigned char * dumpViewToImage(GString *buffer)
{
  char *row_tab;
  int row_length, m, n1, n2, j;
  unsigned char *image;
  int res;
  DumpImage *dumpData;
  int resetOpenGl;

  resetOpenGl = !OpenGLIsPixmap();
  if (resetOpenGl)
    {
      DBG_fprintf(stderr, "Opengl : changing Opengl context to"
		  " indirect rendering for dump process.\n");
      storeOpenGLContext();
      res = setOpenGLContext(TRUE, &dumpData);
      if (res)
	return (unsigned char*)0;
    }

  /* Allocate our buffer for one row */
  row_length = 3 * visuWindow->width;
  row_tab = malloc(row_length*sizeof(char));
  if(!row_tab)
    {
      g_string_append(buffer, _("Cannot allocate memory for row_tab:\n"
				"Window dump cancelled"));
      dumpImageFree(dumpData);
      return (unsigned char*)0;
    }
       
  /* Allocate our buffer for the image */
  image = 0; /* because of gcc on portable PC !?? */
  image = malloc(row_length * visuWindow->height * sizeof(unsigned char));
  if(!image)
    {
      g_string_append(buffer, _("Cannot allocate memory for image:\n"
				"Window dump cancelled"));
      dumpImageFree(dumpData);
      return (unsigned char*)0;
    }
      
/*   image_width = width; */
/*   image_height = height; */

  DBG_fprintf(stderr, "Opengl : Beginning dump process.\n");

  /* Recreate the buffer... not necessary but...*/
  if (resetOpenGl)
    {
      project();
      modelize();
      rebuildAllExtensionsLists();
    }
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  callAllExtensionsLists();
/*   callSwapGLBuffer(); */

  glPixelStorei(GL_PACK_ALIGNMENT,1); /* just in case */

  /* Copy the image into our buffer */
  n2 = 0;
  for(m = visuWindow->height-1; m>= 0; m--)
    {
      glReadPixels(0,m,visuWindow->width,1,GL_RGB,GL_UNSIGNED_BYTE,row_tab);
      n1 = n2;
      n2 = n1 + row_length;
      for(j=n1; j<n2; j++)
	image[j] = row_tab[j-n1];
    }
  
  (void)free(row_tab);

  if (resetOpenGl)
    {
      DBG_fprintf(stderr, "Opengl : changing back Opengl context to direct rendering.\n");
      restoreOpenGLContext();
      dumpImageFree(dumpData);
/*       project(); */
/*       modelize(); */
/*       glSetLighting(); */
/*       glSetRendering(); */
/*       glSetAntiAlias(); */
/*       rebuildAllExtensionsLists(); */
    }
   
  return image;
}

/******************************************************************************/


/******************************************************************************/

/* This is a function to get the number of facettes advised
   by the server (according to its policy on rendering)
   to draw an object according to a given dimension. */
int getNumberOfFacettes(float dimension)
{
   int rsize;     
   int nlat;
#define NLAT_MIN 12
#define NLAT_MAX 50
#define RSIZE_MIN  10
#define RSIZE_MAX 250

#define NLAT_V_MIN 0
#define NLAT_V_MAX (NLAT_MIN)
#define RSIZE_V_MIN  0
#define RSIZE_V_MAX (RSIZE_MIN)

#define NLAT_MINI 3
#define NLAT_MAXI 100

   static float fac = -1.0f, fac_v = -1.0f;

   if (!visuCamera->gr || !visuCamera->gross)
     return -1;
   if (dimension <= 0.)
     return -1;

   /* calculate once fac and fac_v!... */
   if(fac < 0.0f) {
      fac = ((float)(NLAT_MAX - NLAT_MIN))/(RSIZE_MAX - RSIZE_MIN);
      fac_v = ((float)(NLAT_V_MAX - NLAT_V_MIN))/(RSIZE_V_MAX - RSIZE_V_MIN);
   }

   rsize = (int) (visuWindow->width * (dimension /
				    (2.0 * visuBox->extens / visuCamera->gr /
				     visuCamera->gross)));

   if(rsize < RSIZE_MIN) {
      nlat = (int)(NLAT_V_MIN + fac_v * (rsize - RSIZE_V_MIN));
      if(nlat < NLAT_MINI) nlat = NLAT_MINI;
   }
   else if(rsize > RSIZE_MAX) {
      nlat = NLAT_MAX;
   }
   else {
      nlat = (int)(NLAT_MIN + fac * (rsize - RSIZE_MIN));
   }

   nlat = (int)((float)nlat * precisionOfRendering);
   if (nlat < NLAT_MINI)
     nlat = NLAT_MINI;
   if (nlat > NLAT_MAXI)
     nlat = NLAT_MAXI;
   
   return nlat;
}

/* This function change the value of the parameter precisionOfRendering. */
int setPrecision(float value)
{
  if (value <= 0. || value == precisionOfRendering)
    return 0;
  
  precisionOfRendering = value;

  g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLFacetteChanged_signal_id,
		 0 , NULL);
  return 1;
}
/* This function retrieve the value of the parameter precisionOfRendering. */
float getPrecision()
{
  return precisionOfRendering;
}

/* To set the antialiasing on lines. */
void glSetAntiAlias()
{
  if (antialias)
    {
      /* Set blend if not present */
      glEnable(GL_BLEND);
      glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

      /* Antialias */
      glEnable(GL_LINE_SMOOTH);
      glHint(GL_LINE_SMOOTH_HINT, GL_DONT_CARE);
    }
  else
    {
      glDisable(GL_LINE_SMOOTH);
/*       glDisable(GL_BLEND); */
    }
}
int setAntialias(int value)
{
  if (value == antialias)
    return 0;

  antialias = value;
  glSetAntiAlias();
  
  return 1;
}
/* Get the value of the antialiasing parameter. */
int getAntialias()
{
  return antialias;
}

/* This function change the value of the parameter renderingOption. */
void glSetRendering()
{
  switch (renderingOption)
    {
    case Wireframe:
      glShadeModel(GL_FLAT);
      glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
      break;
    case Flat:
      glShadeModel(GL_FLAT);
      glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
      break;
    case Smooth:
      glShadeModel(GL_SMOOTH);
      glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
      break;
    }
}
int setRenderingOption(int value)
{
  if (value < 0 || value > 2)
    return 0;
  
  if (value == renderingOption)
    return 0;

  renderingOption = value;
  glSetRendering();

  return 1;
}
/* This function change the value of the parameter renderingOption. */
int setRenderingOptionFromString(char* value)
{
  int i;

  if (!value)
    return 0;

  i = 0;
  while (i < 3 && strcmp(value, renderingStrings[i])) i++;
  if (i == 3)
    return 0;
  return setRenderingOption(i);
}
/* This function retrieve the value of the parameter renderingOption. */
int getRenderingOption()
{
  return renderingOption;
}
/* This function retrieve the value of the parameter renderingOption. */
char* getRenderingOptionAsString()
{
  return renderingStrings[renderingOption];
}
/* Return a GList with all the name of the options available. */
GList* getRenderingOptionAsList()
{
  GList *resultat;
  int i;

  resultat = (GList*)0;
  for (i = 0; i < 3; i++)
      resultat = g_list_append(resultat, renderingStrings[i]);

  return resultat;
}

/* If on this create a function that call redraw if
   the expose event is raised. */
void setFakeBackingStore(int bool)
{
  if (bool == fakeBackingStore)
    return;

  fakeBackingStore = bool;

  if (bool)
    {
      stopBackingStore();
      /* Add the expose event to the existing mask. */
      addExposureEvent(redrawForBackingStore);
    }
  else
    {
      startBackingStore();
      /* Remove the expose event from the existing mask. */
      removeExposureEvent();
    }
}
int getFakeBackingStore()
{
  return fakeBackingStore;
}
void redrawForBackingStore(SimplifiedEvents *ev)
{
  reDraw((GObject*)0, (gpointer)1);
}

/* If true all changes are applied only when the refresh button
   is pushed. */
void setImmediateDrawing(int bool)
{
  if (bool == opengl_immediate)
    return;

  opengl_immediate = bool;
}
int getImmediateDrawing()
{
  return opengl_immediate;
}


/***************************/
/* Dealing with parameters */
/***************************/

gboolean readOpenGLPrecision(gchar **lines, int nbLines,
			     int position, GString *errorMessage)
{
  int val;
  int res;

  res = sscanf(lines[0],"%d", &val);
  if (res != 1)
    {
      if (errorMessage)
	g_string_append_printf(errorMessage, _("WARNING! Parse error at line %d,"
					       " 1 integer value (v > 0) must appear"
					       " after the %s markup.\n"),
			       position, FLAG_PARAMETER_OPENGL_DETAILS);
      setPrecision((float)PARAMETER_OPENGL_DETAILS_DEFAULT / 100.);
      return FALSE;
    }

  setPrecision((float)val / 100.);
  return TRUE;
}
gboolean readOpenGLRendering(gchar **lines, int nbLines,
			     int position, GString *errorMessage)
{
  int res;
  int i;

  res = 0;
  lines[0] = g_strstrip(lines[0]);
  if (!lines[0][0])
    {
      if (errorMessage)
	g_string_append_printf(errorMessage, _("WARNING! Parse error at line %d,"
					       " 1 string value must appear"
					       " after the %s markup.\n"),
			       position, FLAG_PARAMETER_OPENGL_RENDERING);
      setRenderingOption(PARAMETER_OPENGL_RENDERING_DEFAULT);
      res = 1;
    }
  else
    {
      i = 0;
      while (i < 3 && strcmp(lines[0], renderingStrings[i])) i++;
      if (i == 3)
	{
	  if (errorMessage)
	    g_string_append_printf(errorMessage, _("WARNING! Unknown value (%s) for the"
						   " '%s' parameter at line %d.\n"),
				   lines[0], FLAG_PARAMETER_OPENGL_RENDERING, position);
	  setRenderingOption(PARAMETER_OPENGL_RENDERING_DEFAULT);
	  res = 1;
	}
      else
	setRenderingOption(i);
    }
  
  if (res)
    return FALSE;
  else
    return TRUE;
}
gboolean readOpenGLAntialias(gchar **lines, int nbLines,
			     int position, GString *errorMessage)
{
  int res;
  int val;

  res = sscanf(lines[0],"%d", &val);
  if (res != 1)
    {
      if (errorMessage)
	g_string_append_printf(errorMessage, _("WARNING! Parse error at line %d,"
					       " 1 integer value (v > 0) must appear"
					       " after the %s markup.\n"),
			       position, FLAG_PARAMETER_OPENGL_ANTIALIAS);
      setAntialias(PARAMETER_OPENGL_ANTIALIAS_DEFAULT);
      return FALSE;
    }
  else
    setAntialias(val);

  return TRUE;
}
gboolean readOpenGLObserveMethod(gchar **lines, int nbLines,
				 int position, GString *errorMessage)
{
  int res;
  int val;

  res = sscanf(lines[0],"%d", &val);
  if (res != 1 || (val != OPENGL_OBSERVE_CONSTRAINED && val != OPENGL_OBSERVE_WALKER))
    {
      if (errorMessage)
	g_string_append_printf(errorMessage, _("WARNING! Parse error at line %d,"
					       " 1 integer value (v > 0) must appear"
					       " after the %s markup.\n"),
			       position, FLAG_PARAMETER_OBSERVE_METHOD);
      openGLSet_preferedObserveMethod(OPENGL_OBSERVE_CONSTRAINED);
      return FALSE;
    }
  else
    openGLSet_preferedObserveMethod(val);

  return TRUE;
}

gboolean readOpenGLImmediate(gchar **lines, int nbLines,
			     int position, GString *errorMessage)
{
  int res;
  int val;

  res = sscanf(lines[0],"%d", &val);
  if (res != 1)
    {
      if (errorMessage)
	g_string_append_printf(errorMessage, _("WARNING! Parse error at line %d, 1 integer"
					       " value (v > 0) must appear"
					       " after the %s markup.\n"),
			       position, FLAG_PARAMETER_OPENGL_IMMEDIATE);
      setAntialias(PARAMETER_OPENGL_IMMEDIATE_DEFAULT);
      return FALSE;
    }
  else
    setImmediateDrawing(val);

  return TRUE;
}
gboolean readOpenGLThetaPhi(gchar **lines, int nbLines,
			    int position, GString *errorMessage)
{
  int res;
  float val[3];
  
  res = sscanf(lines[0],"%f %f %f", &val[0], &val[1], &val[2]);
  if (res != 3)
    {
      if (errorMessage)
	g_string_append_printf(errorMessage, _("WARNING! Parse error at line %d, 2 floating "
					       "point values must appear"
					       " after the %s markup.\n"),
			       position, FLAG_RESOURCE_OPENGL_ANGLES);
      setCameraThetaPhiOmega(anglesDefault[0], anglesDefault[1], anglesDefault[2],
			     MASK_THETA | MASK_PHI | MASK_OMEGA);
      return FALSE;
    }
  setCameraThetaPhiOmega(val[0], val[1], val[2], MASK_THETA | MASK_PHI | MASK_OMEGA);

  return TRUE;
}
gboolean readOpenGLXsYs(gchar **lines, int nbLines,
			int position, GString *errorMessage)
{
  int res;
  float val[2];
  
  res = sscanf(lines[0],"%f %f", &val[0], &val[1]);
  if (res != 2)
    {
      if (errorMessage)
	g_string_append_printf(errorMessage, _("WARNING! Parse error at line %d, 2 floating "
					       "point values must appear"
					       " after the %s markup.\n"),
			       position, FLAG_RESOURCE_OPENGL_TRANSLAT);
      setCameraXsYs(translatDefault[0], translatDefault[1], MASK_XS | MASK_YS);
      return FALSE;
    }
    setCameraXsYs(val[0], val[1], MASK_XS | MASK_YS);

  return TRUE;
}
gboolean readOpenGLGross(gchar **lines, int nbLines,
			 int position, GString *errorMessage)
{
  int res;
  float val;
  
  res = sscanf(lines[0],"%f", &val);
  if (res != 1)
    {
      if (errorMessage)
	g_string_append_printf(errorMessage, _("WARNING! Parse error at line %d, 1 floating "
					       "point value (0. < v <= 10)"
					       " must appear after the %s"
					       " markup.\n"), position,
			       FLAG_RESOURCE_OPENGL_GROSS);
      setCameraGross(RESOURCE_OPENGL_GROSS_DEFAULT);
      return FALSE;
    }
    setCameraGross(val);

  return TRUE;
}
gboolean readOpenGLPersp(gchar **lines, int nbLines,
			 int position, GString *errorMessage)
{
  int res;
  float val;
  
  res = sscanf(lines[0],"%f", &val);
  if (res != 1)
    {
      if (errorMessage)
	g_string_append_printf(errorMessage, _("WARNING! Parse error at line %d, 1 floating "
					       "point value (0. < v <= 10)"
					       " must appear after the %s"
					       " markup.\n"), position,
			       FLAG_RESOURCE_OPENGL_PERSP);
      setCameraGross(RESOURCE_OPENGL_PERSP_DEFAULT);
      return FALSE;
    }
  setCameraPersp(val);

  return TRUE;
}
gboolean exportParametersOpenGL(GString *data, int *nbLinesWritten)
{
  g_string_append_printf(data, "# %s\n", DESC_PARAMETER_OPENGL_DETAILS);
  g_string_append_printf(data, "%s: %d\n\n", FLAG_PARAMETER_OPENGL_DETAILS,
	  (int)(precisionOfRendering * 100.));
  g_string_append_printf(data, "# %s\n", DESC_PARAMETER_OPENGL_RENDERING);
  g_string_append_printf(data, "%s: %s\n\n", FLAG_PARAMETER_OPENGL_RENDERING,
	  renderingStrings[renderingOption]);
  g_string_append_printf(data, "# %s\n", DESC_PARAMETER_OPENGL_ANTIALIAS);
  g_string_append_printf(data, "%s: %d\n\n", FLAG_PARAMETER_OPENGL_ANTIALIAS,
	  antialias);
  g_string_append_printf(data, "# %s\n", DESC_PARAMETER_OPENGL_IMMEDIATE);
  g_string_append_printf(data, "%s: %d\n\n", FLAG_PARAMETER_OPENGL_IMMEDIATE,
	  opengl_immediate);
  g_string_append_printf(data, "# %s\n", DESC_PARAMETER_OBSERVE_METHOD);
  g_string_append_printf(data, "%s: %d\n\n", FLAG_PARAMETER_OBSERVE_METHOD,
	  openGL_preferedObserveMethod);
  *nbLinesWritten = 15;
  return TRUE;
}
gboolean exportResourcesOpenGL(GString *data, int *nbLinesWritten)
{
  g_string_append_printf(data, "# %s\n", DESC_RESOURCE_OPENGL_ANGLES);
  g_string_append_printf(data, "%s:\n", FLAG_RESOURCE_OPENGL_ANGLES);
  g_string_append_printf(data, "    %9.3f %9.3f %9.3f\n",
	  visuCamera->theta, visuCamera->phi, visuCamera->omega);
  g_string_append_printf(data, "# %s\n", DESC_RESOURCE_OPENGL_TRANSLAT);
  g_string_append_printf(data, "%s:\n", FLAG_RESOURCE_OPENGL_TRANSLAT);
  g_string_append_printf(data, "    %9.3f %9.3f\n",
	  visuCamera->xs, visuCamera->ys);
  g_string_append_printf(data, "# %s\n", DESC_RESOURCE_OPENGL_GROSS);
  g_string_append_printf(data, "%s:\n", FLAG_RESOURCE_OPENGL_GROSS);
  g_string_append_printf(data, "    %9.3f\n", visuCamera->gross);
  g_string_append_printf(data, "# %s\n", DESC_RESOURCE_OPENGL_PERSP);
  g_string_append_printf(data, "%s:\n", FLAG_RESOURCE_OPENGL_PERSP);
  g_string_append_printf(data, "    %9.3f\n", visuCamera->d_red);
  g_string_append_printf(data, "\n");
  *nbLinesWritten = 10;

  return TRUE;
}





/********************/
/* In/out functions */
/********************/

/* Change the value of the theta and phi angles. */
int setCameraThetaPhiOmega(float valueTheta, float valuePhi, float valueOmega, int mask)
{
  float valT, valP, valO;
  int idem;
  
  idem = 1;
  if (mask & MASK_THETA)
    {
      valT = valueTheta;
      while (valT < -180.)
	valT += 360.;
      while (valT > 180.)
	valT -= 360.;

      if (visuCamera->theta != valT)
	{
	  idem = 0;
	  visuCamera->theta = valT;
	}
    }
  if (mask & MASK_PHI)
    {
      valP = valuePhi;
      while (valP < -180.)
	valP += 360.;
      while (valP > 180.)
	valP -= 360.;

      if (visuCamera->phi != valP)
	{
	  idem = 0;
	  visuCamera->phi = valP;
	}
    }
  if (mask & MASK_OMEGA)
    {
      valO = valueOmega;
      while (valO < -180.)
	valO += 360.;
      while (valO > 180.)
	valO -= 360.;

      if (visuCamera->omega != valO)
	{
	  idem = 0;
	  visuCamera->omega = valO;
	}
    }
  if (idem)
    return 0;

  g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLThetaPhi_signal_id,
		 0 , NULL);
  modelize();

  return 1;
}

/* Change the value of the camera position. */
int setCameraXsYs(float valueX, float valueY, int mask)
{
  float valX, valY;
  int idem;
  
  idem = 1;
  if (mask & MASK_XS)
    {
      valX = valueX;
      if (valX < -3.)
	valX = -3.;
      while (valX > 3.)
	valX = 3.;

      if (visuCamera->xs != valX)
	{
	  idem = 0;
	  visuCamera->xs = valX;
	}
    }
  if (mask & MASK_YS)
    {
      valY = valueY;
      while (valY < -3.)
	valY = -3.;
      while (valY > 3.)
	valY = 3.;

      if (visuCamera->ys != valY)
	{
	  idem = 0;
	  visuCamera->ys = valY;
	}
    }
  if (idem)
    return 0;
  g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLXsYs_signal_id,
		 0 , NULL);
  project();

  return 1;
}

/* Change the value of the camera zoom value. */
int setCameraGross(float value)
{
  float val;
  
  val = value;
  if (val < 0.02)
    val = 0.02;
  else if (val > 10)
    val = 10;

  if (visuCamera->gross == val)
    return 0;

  visuCamera->gross = val;
  g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLGross_signal_id,
		 0 , NULL);
  project();
  
  g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLFacetteChanged_signal_id,
		 0 , NULL);
  return 1;
}
/* Change the value of the camera perspective value. */
int setCameraPersp(float value)
{
  float val;
  
  val = value;
  if (val < 1.1)
    val = 1.1;
  else if (val > 1000.)
    val = 1000.;

  if (visuCamera->d_red == val)
    return 0;

  visuCamera->d_red = val;
  visuCamera->gr = val / (val - 1.0);
  g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLDRed_signal_id,
		 0 , NULL);
  g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLFacetteChanged_signal_id,
		 0 , NULL);
  project();
  modelize();
  return 1;
}
