/*   EXTRAITS DE LA LICENCE
	Copyright CEA, contributeurs : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)
  
	Adresse ml :
	BILLARD, non joignable par ml ;
	CALISTE, damien P caliste AT cea P fr.

	Ce logiciel est un programme informatique servant  visualiser des
	structures atomiques dans un rendu pseudo-3D. 

	Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
	respectant les principes de diffusion des logiciels libres. Vous pouvez
	utiliser, modifier et/ou redistribuer ce programme sous les conditions
	de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
	sur le site "http://www.cecill.info".

	Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
	pris connaissance de la licence CeCILL, et que vous en avez accept les
	termes (cf. le fichier Documentation/licence.fr.txt fourni avec ce logiciel).
*/

/*   LICENCE SUM UP
	Copyright CEA, contributors : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)

	E-mail address:
	BILLARD, not reachable any more ;
	CALISTE, damien P caliste AT cea P fr.

	This software is a computer program whose purpose is to visualize atomic
	configurations in 3D.

	This software is governed by the CeCILL  license under French law and
	abiding by the rules of distribution of free software.  You can  use, 
	modify and/ or redistribute the software under the terms of the CeCILL
	license as circulated by CEA, CNRS and INRIA at the following URL
	"http://www.cecill.info". 

	The fact that you are presently reading this means that you have had
	knowledge of the CeCILL license and that you accept its terms. You can
	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
*/
#include "panelBrowser.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <gtk/gtk.h>

#include <gtk_main.h>
#include <visu_object.h>
#include <visu_tools.h>
#include <extraFunctions/dataFile.h>
#include <support.h>

subPanel *panelDataFile;
int nbColumns;
int gcOk;

#define RED_DATA_LABEL   _("R")
#define GREEN_DATA_LABEL _("G")
#define BLUE_DATA_LABEL  _("B")
#define HUE_DATA_LABEL   _("H")
#define SAT_DATA_LABEL   _("S")
#define VAL_DATA_LABEL   _("V")

char* labelRGB[3];
char* labelHSV[3];

#define COLOR_PREVIEW_WIDTH  150
#define COLOR_PREVIEW_HEIGHT 15
#define COLOR_PREVIEW_BITS   8

#define COLOR_TRANSFORMATION_WIDTH  60
#define COLOR_TRANSFORMATION_HEIGHT 40

GtkWidget *createInteriorDataFile();
void createCallBacksDataFile();
void makeMinMaxLabels();
void makeColumnSelection();
void makeColorPreview();
void makeGC();

/* Linkable widgets */
GtkWidget *containerDataFilePanel;
GtkWidget *checkbuttonData;
GtkWidget *statusbarDataFile;
guint statusDataFileContextId;
GtkWidget *radioNormalized;
GtkWidget *radioMinMax;
GtkWidget *entryDataMax;
GtkWidget *entryDataMin;
GtkWidget *spinbuttonDataChA[3];
GtkWidget *spinbuttonDataChB[3];
GtkWidget *comboboxDataCh[3];
GtkWidget *radiobuttonDataRGB;
GtkWidget *radiobuttonDataHSV;
GtkWidget *labelChannel[3];
GdkPixbuf *pixbufColorPreview;
GtkWidget *colorPreview;
GdkGC *GCBack;
GdkGC *GCCurves[3];
GtkWidget *colorTransformation;
GtkWidget *expanderMinMax;

/* Callbacks */
void visuFileReset(GObject *obj, gpointer data);
void loadDataFile(GtkButton *button, gpointer data);
void useDataFileColor(GtkToggleButton *toggle, gpointer data);
gboolean checkDataFilesOnEnter(gpointer data);
void onScaleTypeChange(GtkToggleButton *toggle, gpointer data);
void onEntryMinMaxChangeValue(GtkEntry *entry, gpointer user_data);
void onSpinChAChangeValue(GtkSpinButton *spinbutton, gpointer user_data);
void onSpinChBChangeValue(GtkSpinButton *spinbutton, gpointer user_data);
void onComboColChange(GtkComboBox *combo, gpointer data);
void onColorTypeChange(GtkToggleButton *toggle, gpointer data);
gboolean exposeColorTransformation(GtkWidget *widget, GdkEventExpose *event, gpointer data);

void numericEntrySetValue(GtkEntry *entry, float val, gchar *format);



subPanel* initPanelDataFile()
{
  char *cl = _("Colorize with data");
  char *tl = _("Data color");
  GtkWidget *child;

  labelRGB[0] = RED_DATA_LABEL;
  labelRGB[1] = GREEN_DATA_LABEL;
  labelRGB[2] = BLUE_DATA_LABEL;
  labelHSV[0] = HUE_DATA_LABEL;
  labelHSV[1] = SAT_DATA_LABEL;
  labelHSV[2] = VAL_DATA_LABEL;

  child = createInteriorDataFile();
  panelDataFile = createSubPanelWithIconFromStock(cl, tl, GTK_STOCK_SELECT_COLOR,
						  child, checkDataFilesOnEnter ,
						  (onChangePanelFunc)0);
  if (!panelDataFile)
    return (subPanel*)0;
						 
  /* Initialise the values. */
  gtk_widget_set_sensitive(containerDataFilePanel, FALSE);
  gtk_widget_set_sensitive(entryDataMax, FALSE);
  gtk_widget_set_sensitive(entryDataMin, FALSE);
  gcOk = 0;

  /* Create the callbacks of all the sensitive widgets. */
  createCallBacksDataFile();

  /* call the visu add routine */
  addSubPanel(panelDataFile);

  return panelDataFile;
}

GtkWidget *createInteriorDataFile()
{
  GtkWidget *scrolledwindow1;
  GtkWidget *viewport1;
  GtkWidget *vbox;
  GtkWidget *vbox1;
  GtkWidget *hbox2;
  GtkWidget *alignment6;
  GtkWidget *hbox9;
  GtkWidget *image3;
  GtkWidget *label19;
  GtkWidget *alignment2;
  GtkWidget *buttonOpenDataFile;
  GtkWidget *hbox8;
  GtkWidget *frame1;
  GtkWidget *label9;
  GtkWidget *hbox7;
  GSList *radioNormalized_group = NULL;
  GtkWidget *alignment3;
  GtkWidget *table2;
  GtkObject *spinbuttonDataMax_adj;
  GtkObject *spinbuttonDataMin_adj;
  GtkWidget *label7;
  GtkWidget *label8;
  GtkWidget *hbox6;
  GtkWidget *label22;
  GSList *radiobuttonDataRGB_group = NULL;
  GtkWidget *table3;
  GtkWidget *label;
  GtkWidget *label14;
  GtkWidget *label15;
  GtkWidget *label16;
  GtkWidget *label20;
  GtkWidget *label21;
  GtkObject *spinbuttonDataChA1_adj;
  GtkObject *spinbuttonDataChA2_adj;
  GtkObject *spinbuttonDataChA3_adj;
  GtkObject *spinbuttonDataChB1_adj;
  GtkObject *spinbuttonDataChB2_adj;
  GtkObject *spinbuttonDataChB3_adj;
  GtkWidget *alignment5;

  /* include from glade */
  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  containerDataFilePanel = vbox1;

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox2, FALSE, FALSE, 2);

  checkbuttonData = gtk_check_button_new ();
  gtk_widget_show (checkbuttonData);
  gtk_box_pack_start (GTK_BOX (hbox2), checkbuttonData, FALSE, FALSE, 0);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbuttonData), (gboolean)dataFileGet_used());

  alignment6 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment6);
  gtk_container_add (GTK_CONTAINER (checkbuttonData), alignment6);

  hbox9 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox9);
  gtk_container_add (GTK_CONTAINER (alignment6), hbox9);

  image3 = gtk_image_new_from_stock ("gtk-select-color", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image3);
  gtk_box_pack_start (GTK_BOX (hbox9), image3, FALSE, FALSE, 0);

  label19 = gtk_label_new_with_mnemonic (_("Use color scheme"));
  gtk_widget_set_name(label19, "label_head");
  gtk_widget_show (label19);
  gtk_box_pack_start (GTK_BOX (hbox9), label19, FALSE, FALSE, 0);

  alignment2 = gtk_alignment_new (1, 0.5, 0, 1);
  gtk_widget_show (alignment2);
  gtk_box_pack_start (GTK_BOX (hbox2), alignment2, TRUE, TRUE, 5);

  buttonOpenDataFile = gtk_button_new_from_stock ("gtk-open");
  gtk_widget_show (buttonOpenDataFile);
  gtk_container_add (GTK_CONTAINER (alignment2), buttonOpenDataFile);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow1);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow1, TRUE, TRUE, 0);

  viewport1 = gtk_viewport_new (NULL, NULL);
  gtk_widget_show (viewport1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), viewport1);

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox);
  gtk_container_add (GTK_CONTAINER (viewport1), vbox);
/*   gtk_box_pack_start (GTK_BOX (vbox1), vbox, TRUE, TRUE, 0); */

  hbox7 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox7);
  gtk_box_pack_start (GTK_BOX (vbox), hbox7, FALSE, FALSE, 0);

  label = gtk_label_new_with_mnemonic (_("Normalize data"));
  gtk_widget_set_name(label, "label_head");
  gtk_misc_set_alignment (GTK_MISC (label), 0., 0.5);
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox7), label, TRUE, TRUE, 0);

  radioNormalized = gtk_radio_button_new_with_mnemonic (NULL, _("auto"));
  gtk_widget_show (radioNormalized);
  gtk_box_pack_start (GTK_BOX (hbox7), radioNormalized, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radioNormalized), radioNormalized_group);
  radioNormalized_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radioNormalized));

  radioMinMax = gtk_radio_button_new_with_mnemonic (NULL, _("manual"));
  gtk_widget_show (radioMinMax);
  gtk_box_pack_start (GTK_BOX (hbox7), radioMinMax, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radioMinMax), radioNormalized_group);
  radioNormalized_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radioMinMax));

  alignment3 = gtk_alignment_new (0.5, 0.5, 0.8, 1);
  gtk_widget_show (alignment3);
  gtk_box_pack_start (GTK_BOX (vbox), alignment3, TRUE, TRUE, 0);

  table2 = gtk_table_new (1, 4, FALSE);
  gtk_widget_show (table2);
  gtk_container_add (GTK_CONTAINER (alignment3), table2);
  gtk_table_set_col_spacings (GTK_TABLE (table2), 3);

  entryDataMax = gtk_entry_new();
  gtk_entry_set_text(GTK_ENTRY(entryDataMax), "1.");
  gtk_entry_set_width_chars(GTK_ENTRY(entryDataMax), 6);
  gtk_table_attach (GTK_TABLE (table2), entryDataMax, 3, 4, 0, 1,
                    GTK_FILL | GTK_EXPAND, (GtkAttachOptions)0, 0, 0);
  gtk_widget_show(entryDataMax);

  entryDataMin = gtk_entry_new();
  gtk_entry_set_text(GTK_ENTRY(entryDataMin), "0.");
  gtk_entry_set_width_chars(GTK_ENTRY(entryDataMin), 6);
  gtk_table_attach (GTK_TABLE (table2), entryDataMin, 1, 2, 0, 1,
                    GTK_FILL | GTK_EXPAND, (GtkAttachOptions)0, 0, 0);
  gtk_widget_show(entryDataMin);

  label7 = gtk_label_new (_("Min:"));
  gtk_widget_show (label7);
  gtk_table_attach (GTK_TABLE (table2), label7, 0, 1, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label7), 1, 0.5);

  label8 = gtk_label_new (_("Max:"));
  gtk_widget_show (label8);
  gtk_table_attach (GTK_TABLE (table2), label8, 2, 3, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label8), 1, 0.5);

  expanderMinMax = gtk_expander_new_with_mnemonic(_("_Show/hide read min/max values"));
  gtk_widget_show(expanderMinMax);
  gtk_box_pack_start(GTK_BOX (vbox), expanderMinMax, FALSE, FALSE, 0);

  hbox6 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox6);
  gtk_box_pack_start (GTK_BOX (vbox), hbox6, FALSE, FALSE, 0);

  label22 = gtk_label_new (_("Transformation"));
  gtk_widget_set_name(label22, "label_head");
  gtk_widget_show (label22);
  gtk_box_pack_start (GTK_BOX (hbox6), label22, TRUE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (label22), 0, 0.5);

  radiobuttonDataRGB = gtk_radio_button_new_with_mnemonic (NULL, _("RGB"));
  gtk_widget_show (radiobuttonDataRGB);
  gtk_box_pack_start (GTK_BOX (hbox6), radiobuttonDataRGB, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobuttonDataRGB), radiobuttonDataRGB_group);
  radiobuttonDataRGB_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobuttonDataRGB));

  radiobuttonDataHSV = gtk_radio_button_new_with_mnemonic (NULL, _("HSV"));
  gtk_widget_show (radiobuttonDataHSV);
  gtk_box_pack_start (GTK_BOX (hbox6), radiobuttonDataHSV, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobuttonDataHSV), radiobuttonDataRGB_group);
  radiobuttonDataRGB_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobuttonDataHSV));

  table3 = gtk_table_new (3, 7, FALSE);
  gtk_widget_show (table3);
  gtk_box_pack_start (GTK_BOX (vbox), table3, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (table3), 3);
  gtk_table_set_row_spacings (GTK_TABLE (table3), 3);

  labelChannel[0] = gtk_label_new (_("R"));
  gtk_widget_show (labelChannel[0]);
  gtk_table_attach (GTK_TABLE (table3), labelChannel[0], 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (labelChannel[0]), 0, 0.5);

  labelChannel[1] = gtk_label_new (_("G"));
  gtk_widget_show (labelChannel[1]);
  gtk_table_attach (GTK_TABLE (table3), labelChannel[1], 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (labelChannel[1]), 0, 0.5);

  labelChannel[2] = gtk_label_new (_("B"));
  gtk_widget_show (labelChannel[2]);
  gtk_table_attach (GTK_TABLE (table3), labelChannel[2], 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (labelChannel[2]), 0, 0.5);

  label14 = gtk_label_new (_("="));
  gtk_widget_show (label14);
  gtk_table_attach (GTK_TABLE (table3), label14, 1, 2, 0, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 2, 0);

  label15 = gtk_label_new (_("+"));
  gtk_widget_show (label15);
  gtk_table_attach (GTK_TABLE (table3), label15, 3, 4, 0, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_padding (GTK_MISC (label15), 2, 0);

  label16 = gtk_label_new (_("\303\227"));
  gtk_widget_show (label16);
  gtk_table_attach (GTK_TABLE (table3), label16, 5, 6, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label16), 0, 0.5);

  label20 = gtk_label_new (_("\303\227"));
  gtk_widget_show (label20);
  gtk_table_attach (GTK_TABLE (table3), label20, 5, 6, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label20), 0, 0.5);

  label21 = gtk_label_new (_("\303\227"));
  gtk_widget_show (label21);
  gtk_table_attach (GTK_TABLE (table3), label21, 5, 6, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label21), 0, 0.5);

  spinbuttonDataChA1_adj = gtk_adjustment_new (1, -1, 1, 0.01, 0.1, 0.1);
  spinbuttonDataChA[0] = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonDataChA1_adj), 0.001, 2);
  gtk_widget_show (spinbuttonDataChA[0]);
  gtk_table_attach (GTK_TABLE (table3), spinbuttonDataChA[0], 4, 5, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbuttonDataChA[0]), TRUE);

  spinbuttonDataChA2_adj = gtk_adjustment_new (1, -1, 1, 0.01, 0.1, 0.1);
  spinbuttonDataChA[1] = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonDataChA2_adj), 0.001, 2);
  gtk_widget_show (spinbuttonDataChA[1]);
  gtk_table_attach (GTK_TABLE (table3), spinbuttonDataChA[1], 4, 5, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbuttonDataChA[1]), TRUE);

  spinbuttonDataChA3_adj = gtk_adjustment_new (1, -1, 1, 0.01, 0.1, 0.1);
  spinbuttonDataChA[2] = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonDataChA3_adj), 0.001, 2);
  gtk_widget_show (spinbuttonDataChA[2]);
  gtk_table_attach (GTK_TABLE (table3), spinbuttonDataChA[2], 4, 5, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbuttonDataChA[2]), TRUE);

  spinbuttonDataChB1_adj = gtk_adjustment_new (0, -1, 1, 0.01, 0.1, 0.1);
  spinbuttonDataChB[0] = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonDataChB1_adj), 0.001, 2);
  gtk_widget_show (spinbuttonDataChB[0]);
  gtk_table_attach (GTK_TABLE (table3), spinbuttonDataChB[0], 2, 3, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbuttonDataChB[0]), TRUE);

  spinbuttonDataChB2_adj = gtk_adjustment_new (0, -1, 1, 0.01, 0.1, 0.1);
  spinbuttonDataChB[1] = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonDataChB2_adj), 0.001, 2);
  gtk_widget_show (spinbuttonDataChB[1]);
  gtk_table_attach (GTK_TABLE (table3), spinbuttonDataChB[1], 2, 3, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbuttonDataChB[1]), TRUE);

  spinbuttonDataChB3_adj = gtk_adjustment_new (0, -1, 1, 0.01, 0.1, 0.1);
  spinbuttonDataChB[2] = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonDataChB3_adj), 0.001, 2);
  gtk_widget_show (spinbuttonDataChB[2]);
  gtk_table_attach (GTK_TABLE (table3), spinbuttonDataChB[2], 2, 3, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbuttonDataChB[2]), TRUE);

  comboboxDataCh[0] = gtk_combo_box_new_text ();
  gtk_combo_box_append_text(GTK_COMBO_BOX(comboboxDataCh[0]), "1.");

  comboboxDataCh[1] = gtk_combo_box_new_text ();
  gtk_combo_box_append_text(GTK_COMBO_BOX(comboboxDataCh[1]), "1.");

  comboboxDataCh[2] = gtk_combo_box_new_text ();
  gtk_combo_box_append_text(GTK_COMBO_BOX(comboboxDataCh[2]), "1.");
  gtk_widget_show (comboboxDataCh[0]);
  gtk_table_attach (GTK_TABLE (table3), comboboxDataCh[0], 6, 7, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_show (comboboxDataCh[1]);
  gtk_table_attach (GTK_TABLE (table3), comboboxDataCh[1], 6, 7, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_show (comboboxDataCh[2]);
  gtk_table_attach (GTK_TABLE (table3), comboboxDataCh[2], 6, 7, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  hbox8 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox8);
  gtk_box_pack_start (GTK_BOX (vbox), hbox8, FALSE, FALSE, 0);

  alignment5 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment5);
  gtk_box_pack_start (GTK_BOX (hbox8), alignment5, TRUE, FALSE, 0);

  frame1 = gtk_frame_new (NULL);
  gtk_widget_show (frame1);
  gtk_container_add (GTK_CONTAINER (alignment5), frame1);

  pixbufColorPreview = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE,
				      COLOR_PREVIEW_BITS,
				      COLOR_PREVIEW_WIDTH,
				      COLOR_PREVIEW_HEIGHT);

  colorPreview = create_pixmap (mainWindow, NULL);
  gtk_widget_show (colorPreview);
  gtk_container_add (GTK_CONTAINER (frame1), colorPreview);
  gtk_widget_set_size_request (colorPreview, COLOR_PREVIEW_WIDTH,
			       COLOR_PREVIEW_HEIGHT);
  gtk_misc_set_padding (GTK_MISC (colorPreview), 0, 3);

  label9 = gtk_label_new (_("Color range preview"));
  gtk_widget_show (label9);
  gtk_frame_set_label_widget (GTK_FRAME (frame1), label9);

  alignment5 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment5);
  gtk_box_pack_start (GTK_BOX (hbox8), alignment5, TRUE, FALSE, 0);

  frame1 = gtk_frame_new (NULL);
  gtk_widget_show (frame1);
  gtk_container_add (GTK_CONTAINER (alignment5), frame1);

  colorTransformation = gtk_drawing_area_new();
  gtk_widget_show (colorTransformation);
  gtk_container_add (GTK_CONTAINER (frame1), colorTransformation);
  gtk_widget_set_size_request (colorTransformation, COLOR_TRANSFORMATION_WIDTH,
			       COLOR_TRANSFORMATION_HEIGHT);

  label = gtk_label_new (_("Trans."));
  gtk_widget_show (label);
  gtk_frame_set_label_widget (GTK_FRAME (frame1), label);

  statusbarDataFile = gtk_statusbar_new ();
  gtk_widget_show (statusbarDataFile);
  gtk_box_pack_end (GTK_BOX (vbox1), statusbarDataFile, FALSE, FALSE, 0);
  gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (statusbarDataFile), FALSE);
  statusDataFileContextId = gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbarDataFile),
							 _("Description of loaded data file."));

  g_signal_connect(G_OBJECT(buttonOpenDataFile), "clicked",
		   G_CALLBACK(loadDataFile), (gpointer)0);
  g_signal_connect(G_OBJECT(checkbuttonData), "toggled",
		   G_CALLBACK(useDataFileColor), (gpointer)0);

  return containerDataFilePanel;
}

void createCallBacksDataFile()
{
  int i;

  g_signal_connect(G_OBJECT(visu), "dataReadyForRendering",
		   G_CALLBACK(visuFileReset), (gpointer)1);
  g_signal_connect(G_OBJECT(visu), "dataNotReadyForRendering",
		   G_CALLBACK(visuFileReset), (gpointer)0);
  g_signal_connect(G_OBJECT(radioNormalized), "toggled",
		   G_CALLBACK(onScaleTypeChange), (gpointer)dataFile_normalize);
  g_signal_connect(G_OBJECT(radioMinMax), "toggled",
		   G_CALLBACK(onScaleTypeChange), (gpointer)dataFile_minMax);
  g_signal_connect(G_OBJECT(entryDataMin), "activate",
		   G_CALLBACK(onEntryMinMaxChangeValue), (gpointer)0);
  g_signal_connect(G_OBJECT(entryDataMax), "activate",
		   G_CALLBACK(onEntryMinMaxChangeValue), (gpointer)1);
  for (i = 0; i < 3; i++)
    {
      g_signal_connect(G_OBJECT(spinbuttonDataChA[i]), "value-changed",
		       G_CALLBACK(onSpinChAChangeValue), GINT_TO_POINTER(i));
      g_signal_connect(G_OBJECT(spinbuttonDataChB[i]), "value-changed",
		       G_CALLBACK(onSpinChBChangeValue), GINT_TO_POINTER(i));
      g_signal_connect(G_OBJECT(comboboxDataCh[i]), "changed",
		       G_CALLBACK(onComboColChange), GINT_TO_POINTER(i));

    }
  g_signal_connect(G_OBJECT(radiobuttonDataRGB), "toggled",
		   G_CALLBACK(onColorTypeChange), (gpointer)dataFile_rgb);
  g_signal_connect(G_OBJECT(radiobuttonDataHSV), "toggled",
		   G_CALLBACK(onColorTypeChange), (gpointer)dataFile_hsv);
  g_signal_connect(G_OBJECT(colorTransformation), "expose_event",  
		   G_CALLBACK(exposeColorTransformation), NULL);
}

void makeMinMaxLabels()
{
  char *file;
  GtkWidget *childMinMax;
  int nbColumns;
  int i;
  GtkWidget *table, *label;
  GString *labelStr;
  float minMax[2];

  DBG_fprintf(stderr, "Panel dataFile : rebuilding min/max labels.\n");
  /* empty the child of the expander */
  childMinMax = gtk_bin_get_child(GTK_BIN(expanderMinMax));
  if (childMinMax)
    gtk_widget_destroy(childMinMax);

  file = dataFileGet_file();
  nbColumns = dataFileGet_nbColumns();
  if (file && nbColumns > 0)
    {
      table = gtk_table_new(nbColumns + 1, 3, FALSE);
      gtk_widget_show(table);
      label = gtk_label_new(_("Column number"));
      gtk_widget_set_name(label, "label_head");
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, GTK_SHRINK, GTK_SHRINK, 2, 0);
      label = gtk_label_new(_("Min value"));
      gtk_widget_set_name(label, "label_head");
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table), label, 1, 2, 0, 1, GTK_EXPAND, GTK_SHRINK, 2, 0);
      label = gtk_label_new(_("Max value"));
      gtk_widget_set_name(label, "label_head");
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table), label, 2, 3, 0, 1, GTK_EXPAND, GTK_SHRINK, 2, 0);

      labelStr = g_string_new("");
      for (i = 0; i < nbColumns; i++)
	{
	  g_string_printf(labelStr, _("Column %d"), i + 1);
	  label = gtk_label_new(labelStr->str);
	  gtk_widget_show(label);
	  gtk_table_attach(GTK_TABLE(table), label, 0, 1, i + 1, i + 2,
			   GTK_SHRINK, GTK_SHRINK, 2, 0);

	  if (dataFileGet_fileMinMaxFromColumn(minMax, i))
	    {
	      g_string_printf(labelStr, _("%g"), minMax[0]);
	      label = gtk_label_new(labelStr->str);
	      gtk_widget_show(label);
	      gtk_table_attach(GTK_TABLE(table), label, 1, 2, i + 1, i + 2,
			       GTK_SHRINK, GTK_SHRINK, 2, 0);

	      g_string_printf(labelStr, _("%g"), minMax[1]);
	      label = gtk_label_new(labelStr->str);
	      gtk_widget_show(label);
	      gtk_table_attach(GTK_TABLE(table), label, 2, 3, i + 1, i + 2,
			       GTK_SHRINK, GTK_SHRINK, 2, 0);
	    }
	  else
	    fprintf(stderr, "WARNING! Can't retrieve min/max values for column %d.\n", i);
	}
      g_string_free(labelStr, TRUE);

      gtk_container_add(GTK_CONTAINER(expanderMinMax), table);
    }
}
void makeColumnSelection()
{
  char *file;
  int i, j;
  GString *label;
  int *selected;

  DBG_fprintf(stderr, "Panel dataFile : rebuilding comboboxes.\n");
  file = dataFileGet_file();
  /* empty combobox */
  for (j = nbColumns; j > 0; j--)
    {
      for (i = 0; i < 3; i++)
	gtk_combo_box_remove_text(GTK_COMBO_BOX(comboboxDataCh[i]), j);
    }
  if (file)
    {
      label = g_string_new("");
      nbColumns = dataFileGet_nbColumns();
      for (j = 0; j < nbColumns; j++)
	{
	  g_string_printf(label, _("Col. %d"), j + 1);
	  for (i = 0; i < 3; i++)
	    gtk_combo_box_append_text(GTK_COMBO_BOX(comboboxDataCh[i]), label->str);
	}
      g_string_free(label, TRUE);
      selected = dataFileGet_colUsed();
      if (selected)
	{
	  for (i = 0; i < 3; i++)
	    {
	      DBG_fprintf(stderr, "Panel dataFile : Channel %d has selection %d.\n",
			  i, selected[i]);
	      gtk_combo_box_set_active(GTK_COMBO_BOX(comboboxDataCh[i]), selected[i] + 1);
	    }
	  free(selected);
	}
      else
	for (i = 0; i < 3; i++)
	  {
	    DBG_fprintf(stderr, "Panel dataFile : Channel %d has selection %d.\n",
			i, -1);
	    gtk_combo_box_set_active(GTK_COMBO_BOX(comboboxDataCh[i]), 0);
	  }
    }
  else
    for (i = 0; i < 3; i++)
      {
	DBG_fprintf(stderr, "Panel dataFile : Channel %d has selection %d.\n",
		    i, -1);
	gtk_combo_box_set_active(GTK_COMBO_BOX(comboboxDataCh[i]), 0);
      }
}

void makeColorPreview()
{
  int *valCols;
  int nb, i;
  int rowstride, x, y;
  guchar *pixels, *p;
  float rgbValues[COLOR_PREVIEW_WIDTH][3];
  float fromVal[3];
  int ind;

  valCols = dataFileGet_colUsed();
  if (!valCols)
    return;

  nb = 0;
  ind = -1;
  for (i = 0; i < 3; i++)
    if (valCols[i] != -1)
      {
	nb += 1;
	ind = i;
      }
  if (nb != 1 || ind < 0)
    {
      gtk_image_set_from_pixbuf(GTK_IMAGE(colorPreview), NULL);
      exposeColorTransformation(colorTransformation, NULL, (gpointer)0);
    }
  else
    {
      DBG_fprintf(stderr, "Panel dataFile : making color preview bitmap.\n");
      rowstride = gdk_pixbuf_get_rowstride(pixbufColorPreview);
      pixels = gdk_pixbuf_get_pixels(pixbufColorPreview);

      for (i = 0; i < 3; i++)
	fromVal[i] = 1.;
      for (i = 0; i < COLOR_PREVIEW_WIDTH; i++)
	{
	  fromVal[ind] = (float)i / (float)(COLOR_PREVIEW_WIDTH - 1);
	  dataFileGet_valuesTransformedInRGB(rgbValues[i], fromVal);
/* 	  fprintf(stderr, "% d ----> %f %f %f -> %f %f %f\n",i, fromVal[0], fromVal[1], fromVal [2], rgbValues[i][0], rgbValues[i][1], rgbValues[i][2]); */
	}

      for (y = 0; y < COLOR_PREVIEW_HEIGHT; y++)
	for (x = 0; x < COLOR_PREVIEW_WIDTH; x++)
	  {
	    p = pixels + y * rowstride + x * 3;
	    p[0] = (guchar)(rgbValues[x][0] * 255);
	    p[1] = (guchar)(rgbValues[x][1] * 255);
	    p[2] = (guchar)(rgbValues[x][2] * 255);
	  }
      gtk_image_set_from_pixbuf(GTK_IMAGE(colorPreview), pixbufColorPreview);

      /* Creating the curves. */
      exposeColorTransformation(colorTransformation, NULL, (gpointer)0);
    }
}

void makeGC()
{
  GdkColor colorR, colorG, colorB, colorGrey;

  if (!colorTransformation->window)
    return;

  colorGrey.red = 49151;
  colorGrey.green = 49151;
  colorGrey.blue = 49151;
  GCBack = gdk_gc_new(colorTransformation->window);
  gdk_gc_set_rgb_fg_color(GCBack, &colorGrey);
  gdk_gc_set_rgb_bg_color(GCBack, &colorGrey);
  g_object_ref(GCBack);

  colorR.red = 65535;
  colorR.green = 0;
  colorR.blue = 0;
  GCCurves[0] = gdk_gc_new(colorTransformation->window);
  g_object_ref(GCCurves[0]);
  gdk_gc_set_rgb_fg_color(GCCurves[0], &colorR);
  gdk_gc_set_rgb_bg_color(GCCurves[0], &colorGrey);
  colorG.red = 0;
  colorG.green = 65535;
  colorG.blue = 0;
  GCCurves[1] = gdk_gc_new(colorTransformation->window);
  g_object_ref(GCCurves[1]);
  gdk_gc_set_rgb_fg_color(GCCurves[1], &colorG);
  gdk_gc_set_rgb_bg_color(GCCurves[1], &colorGrey);
  colorB.red = 0;
  colorB.green = 0;
  colorB.blue = 65535;
  GCCurves[2] = gdk_gc_new(colorTransformation->window);
  gdk_gc_set_rgb_fg_color(GCCurves[2], &colorB);
  gdk_gc_set_rgb_bg_color(GCCurves[2], &colorGrey);
  g_object_ref(GCCurves[2]);

  gcOk = 1;
}


/*************/
/* Callbacks */
/*************/
void visuFileReset(GObject *obj, gpointer data)
{
  DBG_fprintf(stderr, "Panel dataFile : catch dataReadyForRendering or"
	      " dataNotReadyForRendering.\n");
  if (GPOINTER_TO_INT(data))
    gtk_widget_set_sensitive(containerDataFilePanel, TRUE);
  else
    gtk_widget_set_sensitive(containerDataFilePanel, FALSE);

  gtk_statusbar_pop(GTK_STATUSBAR(statusbarDataFile),
		    statusDataFileContextId);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbuttonData),
			       (gboolean)dataFileGet_used());
  makeMinMaxLabels();
  makeColumnSelection();
  makeColorPreview();
}
void loadDataFile(GtkButton *button, gpointer data)
{
  GtkWidget *file_selector;
  char *filename;
  gchar *fileUTF8;
  char *directory;
  int res, errors;
  GString *message;

  file_selector = gtk_file_chooser_dialog_new(_("Load data file"),
					      GTK_WINDOW(mainWindow),
					      GTK_FILE_CHOOSER_ACTION_OPEN,
					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					      GTK_STOCK_OPEN, GTK_RESPONSE_OK,
					      NULL);
  directory = getLastOpenDirectory();
  if (directory)
    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(file_selector), directory);
  gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(file_selector),
				       FALSE);


  gtk_widget_set_name(file_selector, "filesel");
  gtk_window_set_position(GTK_WINDOW(file_selector), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (file_selector), TRUE);

  if (gtk_dialog_run (GTK_DIALOG (file_selector)) == GTK_RESPONSE_OK)
    {
      filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER (file_selector));
      directory = (char*)gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER (file_selector));
      setLastOpenDirectory(directory);
      g_free(directory);
      gtk_widget_destroy (file_selector);

      message = g_string_new(_("Loading data file...\n"));
      res = dataFileSet_file(filename, message, &errors);

      if (res)
	dataFileReDraw();
      
      if (errors)
	raiseAlertDialogWithScrollView(message->str);
      g_string_free(message, TRUE);

      res = dataFileGet_nbColumns();
      if (res)
	{
	  fileUTF8 = getStringInUTF8(filename);
	  message = g_string_new(_("Data file: "));
	  if (g_utf8_strlen(fileUTF8, 16) > 15)
	    g_string_append_printf(message, _(" (...)%s :: %d column(s)"),
				   g_utf8_offset_to_pointer(fileUTF8,
							    g_utf8_strlen(fileUTF8, -1) - 15),
				   res);
	  else
	    g_string_append_printf(message, _(" %s (%d col)"),
				   fileUTF8, res);
	  gtk_statusbar_pop(GTK_STATUSBAR(statusbarDataFile),
			    statusDataFileContextId);
	  gtk_statusbar_push(GTK_STATUSBAR(statusbarDataFile),
			     statusDataFileContextId,
			     message->str);
	  g_string_free(message, TRUE);
	  g_free(fileUTF8);
	}

      g_free (filename);
    }
  else
    {
      directory = (char*)gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER (file_selector));
      setLastOpenDirectory(directory);
      g_free(directory);
      gtk_widget_destroy (file_selector);
    }
  makeMinMaxLabels();
  makeColumnSelection();
  makeColorPreview();
}
void useDataFileColor(GtkToggleButton *toggle, gpointer data)
{
  gboolean isOn;
  int res;

  isOn = gtk_toggle_button_get_active(toggle);
  res = dataFileSet_used((gboolean)isOn);
  if (res)
    dataFileReDraw();
}
gboolean checkDataFilesOnEnter(gpointer data)
{
  float *val;
  int i;

  if (dataFileGet_used())
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbuttonData), TRUE);
  else
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbuttonData), FALSE);

  switch (dataFileGet_scaleType())
    {
    case dataFile_normalize:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radioNormalized), TRUE);
      break;
    case dataFile_minMax:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radioMinMax), TRUE);
      break;
    default:
      break;
    }

  numericEntrySetValue(GTK_ENTRY(entryDataMin), dataFileGet_min(), (gchar*)0);
  numericEntrySetValue(GTK_ENTRY(entryDataMax), dataFileGet_max(), (gchar*)0);
  val = dataFileGet_vectA();
  if (val)
    {
      for (i = 0; i < 3; i++)
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbuttonDataChA[i]), val[i]);
      free(val);
    }
  val = dataFileGet_vectB();
  if (val)
    {
      for (i = 0; i < 3; i++)
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbuttonDataChB[i]), val[i]);
      free(val);
    }

  switch (dataFileGet_colorType())
    {
    case dataFile_rgb:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radiobuttonDataRGB), TRUE);
      for (i = 0; i < 3; i++)
	gtk_label_set_text(GTK_LABEL(labelChannel[i]), labelRGB[i]);
      break;
    case dataFile_hsv:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radiobuttonDataHSV), TRUE);
      for (i = 0; i < 3; i++)
	gtk_label_set_text(GTK_LABEL(labelChannel[i]), labelHSV[i]);
      break;
    default:
      break;
    }

  return FALSE;
}
void onScaleTypeChange(GtkToggleButton *toggle, gpointer data)
{
  int scale, res;

  if (!gtk_toggle_button_get_active(toggle))
    return;

  scale = GPOINTER_TO_INT(data);
  if (scale < dataFile_normalize || scale > dataFile_minMax)
    {
      fprintf(stderr, "WARNING! 'onScaleTypeChange' has been called with"
	      " a wrong data parameter (%d).\n", scale);
      return;
    }
  res = dataFileSet_scaleType(scale);
  if (scale != dataFile_minMax)
    {
      gtk_widget_set_sensitive(entryDataMax, FALSE);
      gtk_widget_set_sensitive(entryDataMin, FALSE);
    }
  else
    {
      gtk_widget_set_sensitive(entryDataMax, TRUE);
      gtk_widget_set_sensitive(entryDataMin, TRUE);
    }
  if (res)
    dataFileReDraw();
}
void numericEntrySetValue(GtkEntry *entry, float val, gchar *format)
{
  GString *str;
  
  g_return_if_fail(entry);
  str = g_string_new("");
  if (format)
    g_string_printf(str, format, val);
  else
    g_string_printf(str, "%f", val);
  gtk_entry_set_text(GTK_ENTRY(entry), str->str);
  g_string_free(str, TRUE);
}
void onEntryMinMaxChangeValue(GtkEntry *entry, gpointer user_data)
{
  int minMax, res;
  gchar *last;
  gdouble valueFloat;

  minMax = GPOINTER_TO_INT(user_data);
  g_return_if_fail(minMax == 0 || minMax == 1);
  DBG_fprintf(stderr, "Panel Data File : grep value from GtkEntry for"
	      " the minMax value (%d).\n", minMax);

  valueFloat = g_ascii_strtod(gtk_entry_get_text(GTK_ENTRY(entry)),
			      &last);
  if (*last != '\0' || valueFloat <= 0.)
    {
      fprintf(stderr, "WARNING! Only strictly positive floating point values are possible.\n");
      if (minMax == 0)
	numericEntrySetValue(entry, dataFileGet_min(), (gchar*)0);
      else
	numericEntrySetValue(entry, dataFileGet_max(), (gchar*)0);
      return;
    }
  switch (minMax)
    {
    case 0:
      res = dataFileSet_min(valueFloat);
      break;
    case 1:
      res = dataFileSet_max(valueFloat);
      break;
    default:
      res = 0;
      break;
    }
  if (res)
    dataFileReDraw();
}
void onSpinChAChangeValue(GtkSpinButton *spinbutton, gpointer user_data)
{
  int pos;
  int res;

  pos = GPOINTER_TO_INT(user_data);
  if (pos < 0 || pos > 2)
    {
      fprintf(stderr, "WARNING! 'onSpinChAChangeValue' has been called with"
	      " a wrong data parameter (%d).\n", pos);
      return;
    }
  res = dataFileSet_vectA(gtk_spin_button_get_value(spinbutton), pos);
  makeColorPreview();
  if (res)
    dataFileReDraw();
}
void onSpinChBChangeValue(GtkSpinButton *spinbutton, gpointer user_data)
{
  int pos;
  int res;

  pos = GPOINTER_TO_INT(user_data);
  if (pos < 0 || pos > 2)
    {
      fprintf(stderr, "WARNING! 'onSpinChBChangeValue' has been called with"
	      " a wrong data parameter (%d).\n", pos);
      return;
    }
  res = dataFileSet_vectB(gtk_spin_button_get_value(spinbutton), pos);
  makeColorPreview();
  if (res)
    dataFileReDraw();
}

void onComboColChange(GtkComboBox *combo, gpointer data)
{
  int pos;
  int res;

  pos = GPOINTER_TO_INT(data);
  if (pos < 0 || pos > 2)
    {
      fprintf(stderr, "WARNING! 'onComboColChange' has been called with"
	      " a wrong data parameter (%d).\n", pos);
      return;
    }
  res = dataFileSet_colUsed(gtk_combo_box_get_active(combo) - 1, pos);
  makeColorPreview();
  if (res)
    dataFileReDraw();
}
void onColorTypeChange(GtkToggleButton *toggle, gpointer data)
{
  int color, res, i;

  if (!gtk_toggle_button_get_active(toggle))
    return;

  color = GPOINTER_TO_INT(data);
  if (color < dataFile_rgb || color > dataFile_hsv)
    {
      fprintf(stderr, "WARNING! 'onColorTypeChange' has been called with"
	      " a wrong data parameter (%d).\n", color);
      return;
    }
  res = dataFileSet_colorType(color);
  if (color == dataFile_rgb)
    for (i = 0; i < 3; i++)
      gtk_label_set_text(GTK_LABEL(labelChannel[i]), labelRGB[i]);
  else
    for (i = 0; i < 3; i++)
      gtk_label_set_text(GTK_LABEL(labelChannel[i]), labelHSV[i]);
  makeColorPreview();
  if (res)
    dataFileReDraw();
}
gboolean exposeColorTransformation(GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
  int *valCols;
  int i;
  float x0, x1;
  int xl0, yl0, xl1, yl1;
  float *vectA, *vectB;

  if (!colorTransformation->window)
    return TRUE;

  if (!gcOk)
    makeGC();
 
  valCols = dataFileGet_colUsed();
  if (!valCols)
    {
      if (gcOk)
	gdk_draw_rectangle(colorTransformation->window, GCBack,
			   TRUE, 2, 2, COLOR_TRANSFORMATION_WIDTH - 4,
			   COLOR_TRANSFORMATION_HEIGHT - 4);
      return TRUE;
    }

  gdk_draw_rectangle(colorTransformation->window, GCBack,
		     TRUE, 2, 2, COLOR_TRANSFORMATION_WIDTH - 4,
		     COLOR_TRANSFORMATION_HEIGHT - 4);

  vectA = dataFileGet_vectA();
  vectB = dataFileGet_vectB();
  for (i = 0; i < 3; i++)
    {
      if (valCols[i] != -1)
	{
	  if (vectA[i] == 0)
	    {
	      yl1 = COLOR_TRANSFORMATION_HEIGHT - 3 -
		(int)(vectB[i] * (float)(COLOR_TRANSFORMATION_HEIGHT - 5));
	      gdk_draw_line(colorTransformation->window, GCBack,
			    2, yl1, COLOR_TRANSFORMATION_WIDTH - 3, yl1);
	    }
	  else
	    {
	      x0 = - vectB[i] / vectA[i];
	      x1 = (1. - vectB[i]) / vectA[i];
	      x0 = (x0 < 0.)?0.:x0;
	      x0 = (x0 > 1.)?1.:x0;
	      x1 = (x1 < 0.)?0.:x1;
	      x1 = (x1 > 1.)?1.:x1;
	      if (x0 != x1)
		{
		  xl0 = 2 + (int)(x0 * (float)(COLOR_TRANSFORMATION_WIDTH - 5));
		  xl1 = 2 + (int)(x1 * (float)(COLOR_TRANSFORMATION_WIDTH - 5));
		  yl0 = COLOR_TRANSFORMATION_HEIGHT - 3 -
		    (int)((vectA[i] * x0 + vectB[i]) * (float)(COLOR_TRANSFORMATION_HEIGHT - 5));
		  yl1 = COLOR_TRANSFORMATION_HEIGHT - 3 -
		    (int)((vectA[i] * x1 + vectB[i]) * (float)(COLOR_TRANSFORMATION_HEIGHT - 5));
		  gdk_draw_line(colorTransformation->window, GCCurves[i],
				xl0, yl0, xl1, yl1);
		}
	    }
	}
      else
	{
	  x0 = vectA[i] + vectB[i];
	  x0 = (x0 < 0.)?0.:x0;
	  x0 = (x0 > 1.)?1.:x0;
	  yl1 = COLOR_TRANSFORMATION_HEIGHT - 3 -
	    (int)(x0 * (float)(COLOR_TRANSFORMATION_HEIGHT - 5));
	  gdk_draw_line(colorTransformation->window, GCCurves[i],
			2, yl1, COLOR_TRANSFORMATION_WIDTH - 3, yl1);
	}
    }
  free(vectA);
  free(vectB);

  return TRUE;
}
