/*   EXTRAITS DE LA LICENCE
	Copyright CEA, contributeurs : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)
  
	Adresse ml :
	BILLARD, non joignable par ml ;
	CALISTE, damien P caliste AT cea P fr.

	Ce logiciel est un programme informatique servant  visualiser des
	structures atomiques dans un rendu pseudo-3D. 

	Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
	respectant les principes de diffusion des logiciels libres. Vous pouvez
	utiliser, modifier et/ou redistribuer ce programme sous les conditions
	de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
	sur le site "http://www.cecill.info".

	Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
	pris connaissance de la licence CeCILL, et que vous en avez accept les
	termes (cf. le fichier Documentation/licence.fr.txt fourni avec ce logiciel).
*/

/*   LICENCE SUM UP
	Copyright CEA, contributors : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)

	E-mail address:
	BILLARD, not reachable any more ;
	CALISTE, damien P caliste AT cea P fr.

	This software is a computer program whose purpose is to visualize atomic
	configurations in 3D.

	This software is governed by the CeCILL  license under French law and
	abiding by the rules of distribution of free software.  You can  use, 
	modify and/ or redistribute the software under the terms of the CeCILL
	license as circulated by CEA, CNRS and INRIA at the following URL
	"http://www.cecill.info". 

	The fact that you are presently reading this means that you have had
	knowledge of the CeCILL license and that you accept its terms. You can
	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
*/
#include "panelOpenGL.h"

#include <visu_object.h>

int disableCallbacksOpenGL;

static GtkWidget *light_tree_view;
static GtkListStore *light_list_store;

subPanel* initPanelOpenGL()
{
  char *cl = _("Set OpenGL parameters");
  char *tl = _("OpenGL");
  GtkWidget *child;
/*   char *nR[] = {(char*)0}; */
/*   char *dR[] = {(char*)0}; */
/*   readValueOfResourceFunc rR[] = {(readValueOfResourceFunc)0}; */
/*   char *nP[] = {(char*)0}; */
/*   char *dP[] = {(char*)0}; */
/*   readValueOfParameterFunc rP[] = {(readValueOfParameterFunc)0}; */

  child = createInteriorOpenGL();
  panelOpenGL = createSubPanelWithIconFromStock(cl, tl, GTK_STOCK_EXECUTE,
						child, refreshPanelOpenGL,
						(onChangePanelFunc)0);
  if (!panelOpenGL)
    return (subPanel*)0;

  /* Create the callbacks of all the sensitive widgets. */
  createCallBacksOpenGL();

  /* call the visu add routine */
  addSubPanel(panelOpenGL);

  /* Private parameters. */
  disableCallbacksOpenGL = 0;

  return panelOpenGL;
}

enum {
  LIGHT_NUMBER=0,
  LIGHT_ENABLED,
  LIGHT_POSITION0,
  LIGHT_POSITION1,
  LIGHT_POSITION2,
  LIGHT_POSITION3,
  LIGHT_N_PARAMETERS
};

void light_store_in_list_store(gpointer data, gpointer useless)
{
  light *light0 = data;
  GtkTreeIter iter;

  gtk_list_store_insert(light_list_store, &iter, g_list_index(light_get_list(), light0));
  gtk_list_store_set(light_list_store, &iter,
		     LIGHT_NUMBER, g_list_index(light_get_list(), light0),
		     LIGHT_ENABLED, light0->enabled,
		     LIGHT_POSITION0, light0->position[0],
		     LIGHT_POSITION1, light0->position[1],
		     LIGHT_POSITION2, light0->position[2], 
		     LIGHT_POSITION3, light0->position[3], -1);
}

void glSetLighting();
void light_tree_show_hide(GtkCellRendererToggle *cell_renderer, 
			  gchar *string_path, gpointer user_data)
{
  GtkTreePath* path = gtk_tree_path_new_from_string(string_path);
  GtkTreeIter iter;
  gboolean status;
  int light_number;
  light *light0;

  if(gtk_tree_model_get_iter(GTK_TREE_MODEL(light_list_store), &iter, path) == FALSE)
    return;

  status = gtk_cell_renderer_toggle_get_active(cell_renderer) ? FALSE : TRUE;
  gtk_list_store_set(light_list_store, &iter,
		     LIGHT_ENABLED, status, -1);
  gtk_tree_model_get(GTK_TREE_MODEL(light_list_store), &iter,
		     LIGHT_NUMBER, &light_number, -1);

  light0 = g_list_nth_data(light_get_list(), light_number);
  light0->enabled = status;

  glSetLighting();
  g_signal_emit(visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id, 0, NULL);
}

void light_update(GtkCellRendererText *cellrenderertext,
		  gchar *path, gchar *text, gpointer user_data)
{
  GtkTreeIter iter;
  int light_number;
  float new_value = atof(text);
  light *light0;

  if(gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(light_list_store),
					 &iter, path))
    {      
      gtk_list_store_set(light_list_store, &iter,
			 GPOINTER_TO_INT(user_data), new_value, -1);
      gtk_tree_model_get(GTK_TREE_MODEL(light_list_store), &iter, 
			 LIGHT_NUMBER, &light_number, -1);
    }

  light0 = g_list_nth_data(light_get_list(), light_number);
  light0->position[GPOINTER_TO_INT(user_data)-LIGHT_POSITION0] = new_value;
			 
  glSetLighting();
  g_signal_emit(visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id, 0, NULL);
}

void light_sync_lists()
{
  gtk_list_store_clear(light_list_store);
  g_list_foreach(light_get_list(), light_store_in_list_store, NULL);  
}

void lights_make_tree_view()
{
  GtkTreeViewColumn* column; 
  GtkCellRenderer *cell_show = gtk_cell_renderer_toggle_new();
  GtkCellRenderer *cell_float;
  light_list_store =
    gtk_list_store_new (LIGHT_N_PARAMETERS, G_TYPE_INT, G_TYPE_BOOLEAN, G_TYPE_FLOAT, G_TYPE_FLOAT, G_TYPE_FLOAT, G_TYPE_FLOAT);
  light_tree_view =
    gtk_tree_view_new_with_model (GTK_TREE_MODEL (light_list_store));

  light_sync_lists();

  g_signal_connect(cell_show, "toggled", G_CALLBACK(light_tree_show_hide), NULL);

  column = gtk_tree_view_column_new_with_attributes(_("Use"), cell_show,
						    "active", LIGHT_ENABLED, NULL);
  gtk_tree_view_column_set_expand(column, FALSE);
  gtk_tree_view_column_set_alignment(column, 0.5);
  gtk_tree_view_append_column (GTK_TREE_VIEW (light_tree_view), column);

/*   column = gtk_tree_view_column_new_with_attributes(_("n"), gtk_cell_renderer_text_new(), */
/* 						    "text", LIGHT_NUMBER, NULL); */
/*   gtk_tree_view_append_column (GTK_TREE_VIEW (light_tree_view), column); */

  cell_float = gtk_cell_renderer_text_new();
  g_object_set(G_OBJECT(cell_float), "editable", TRUE, NULL);
  g_signal_connect(G_OBJECT(cell_float), "edited",
		   G_CALLBACK(light_update), GINT_TO_POINTER(LIGHT_POSITION0));
  column = gtk_tree_view_column_new_with_attributes(_("x"), cell_float,
						    "text", LIGHT_POSITION0, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (light_tree_view), column);
  
  cell_float = gtk_cell_renderer_text_new();
  g_object_set(G_OBJECT(cell_float), "editable", TRUE, NULL);
  g_signal_connect(G_OBJECT(cell_float), "edited",
		   G_CALLBACK(light_update), GINT_TO_POINTER(LIGHT_POSITION1));
  column = gtk_tree_view_column_new_with_attributes(_("y"), cell_float,
						    "text", LIGHT_POSITION1, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (light_tree_view), column);

  cell_float = gtk_cell_renderer_text_new();
  g_object_set(G_OBJECT(cell_float), "editable", TRUE, NULL);
  g_signal_connect(G_OBJECT(cell_float), "edited",
		   G_CALLBACK(light_update), GINT_TO_POINTER(LIGHT_POSITION2));
  column = gtk_tree_view_column_new_with_attributes(_("z"), cell_float,
						    "text", LIGHT_POSITION2, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (light_tree_view), column);

  cell_float = gtk_cell_renderer_text_new();
  g_object_set(G_OBJECT(cell_float), "editable", TRUE, NULL);
  g_signal_connect(G_OBJECT(cell_float), "edited",
		   G_CALLBACK(light_update), GINT_TO_POINTER(LIGHT_POSITION3));
  column = gtk_tree_view_column_new_with_attributes(_("?"), cell_float,
						    "text", LIGHT_POSITION3, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (light_tree_view), column);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (light_tree_view), TRUE);
}
  
void light_add_new()
{
  light *light0;

  if(g_list_position(light_get_list(), g_list_last(light_get_list())) >= GL_MAX_LIGHTS)
    return;

  light0 = light_new_default();
  light_add(light0);

  light_sync_lists(); 

  glSetLighting();
  g_signal_emit(visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id, 0, NULL);
}

GtkWidget *createInteriorOpenGL()
{
  GtkWidget *vbox, *hbox;
  GtkWidget *label;
  GtkWidget *table;
  GtkWidget *align;
  GtkObject *adj;
  GtkWidget *scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  GtkWidget *add_button = gtk_button_new();
  GtkWidget *image_add = gtk_image_new_from_stock ("gtk-add", GTK_ICON_SIZE_BUTTON);
  GtkWidget *add_vbox = gtk_vbox_new(FALSE, 0);

  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), 
				 GTK_POLICY_AUTOMATIC, 
				 GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window), GTK_SHADOW_IN);

  vbox = gtk_vbox_new(FALSE, 0);

  hbox = gtk_hbox_new(FALSE, 0);
  label = gtk_label_new(_("Redraw immediately after changes :"));
  gtk_widget_set_name(label, "label_head");
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
  gtk_widget_show(label);
  checkImmediateDrawing = gtk_check_button_new();
/*   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkImmediateDrawing), */
/* 			       (gboolean)get); */
  gtk_box_pack_start(GTK_BOX(hbox), checkImmediateDrawing, FALSE, FALSE, 2);
  gtk_widget_show(checkImmediateDrawing);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
  gtk_widget_show(hbox);

  table = gtk_table_new(2, 1, FALSE);
  gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 5);
  gtk_widget_show(table);
  align = gtk_alignment_new(0., 0.5, 0., 0.);
  gtk_table_attach(GTK_TABLE(table), align, 0, 1, 0, 1, GTK_FILL | GTK_EXPAND, GTK_SHRINK, 2, 0);
  gtk_widget_show(align);
  label = gtk_label_new(_("Choose a rendering option :"));
  gtk_widget_set_name(label, "label_head");
  gtk_container_add(GTK_CONTAINER(align), label);
  gtk_widget_show(label);
  align = gtk_alignment_new(1.0, 0.5, 0., 0.);
  gtk_table_attach(GTK_TABLE(table), align, 0, 1, 1, 2, GTK_FILL | GTK_EXPAND, GTK_SHRINK, 2, 0);
  gtk_widget_show(align);
  openGLRendering = getRenderingOptionAsList();
  comboOpenGLRendering = gtk_combo_new();
  gtk_combo_set_value_in_list(GTK_COMBO(comboOpenGLRendering),
			      TRUE, FALSE);
  gtk_entry_set_editable(GTK_ENTRY
			 (GTK_COMBO(comboOpenGLRendering)->entry),
			 FALSE);
  gtk_combo_set_popdown_strings (GTK_COMBO(comboOpenGLRendering), openGLRendering);
  gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(comboOpenGLRendering)->entry),
		     getRenderingOptionAsString());
  gtk_container_add(GTK_CONTAINER(align), comboOpenGLRendering);
  gtk_widget_show(comboOpenGLRendering);


  hbox = gtk_hbox_new(FALSE, 0);
  label = gtk_label_new(_("Precision (100 is normal) : "));
  gtk_widget_set_name(label, "label_head");
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
  gtk_widget_show(label);
  adj = gtk_adjustment_new(5., 10., 500., 5., 1.0, 1.0);
  spinPrecision = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.01, 0);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinPrecision),
			    getPrecision() * 100.);
  gtk_box_pack_start(GTK_BOX(hbox), spinPrecision, FALSE, FALSE, 2);
  gtk_widget_show(spinPrecision);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
  gtk_widget_show(hbox);

  hbox = gtk_hbox_new(FALSE, 0);
  label = gtk_label_new(_("Antialiase lines :"));
  gtk_widget_set_name(label, "label_head");
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
  gtk_widget_show(label);
  checkAntialiasing = gtk_check_button_new();
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkAntialiasing),
			       (gboolean)getAntialias());
  gtk_box_pack_start(GTK_BOX(hbox), checkAntialiasing, FALSE, FALSE, 2);
  gtk_widget_show(checkAntialiasing);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
  gtk_widget_show(hbox);

  hbox = gtk_hbox_new(FALSE, 0);
  label = gtk_label_new(_("Dirty hack to fake the backingstore :"));
  gtk_widget_set_name(label, "label_head");
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
  gtk_widget_show(label);
  checkFakeBackingStore = gtk_check_button_new();
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkFakeBackingStore),
			       (gboolean)getFakeBackingStore());
  gtk_box_pack_start(GTK_BOX(hbox), checkFakeBackingStore, FALSE, FALSE, 2);
  gtk_widget_show(checkFakeBackingStore);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
  gtk_widget_show(hbox);

  gtk_widget_show(vbox);

/*   hbox = gtk_hbox_new(FALSE, 0); */
/*   label = gtk_label_new(_("Light sources :")); */
/*   gtk_widget_set_name(label, "label_head"); */
/*   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0); */
/*   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0); */
/*   gtk_widget_show_all(hbox); */
  hbox = gtk_hbox_new(FALSE, 0);
  lights_make_tree_view();
  gtk_container_add (GTK_CONTAINER (scrolled_window), light_tree_view);
  gtk_box_pack_start(GTK_BOX(hbox), scrolled_window, TRUE, TRUE, 0);
  gtk_container_add (GTK_CONTAINER (add_button), image_add);
  gtk_box_pack_start(GTK_BOX(hbox), add_vbox, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(add_vbox), add_button, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 5);
  g_signal_connect(add_button, "clicked",
		   G_CALLBACK(light_add_new), NULL);
  gtk_widget_show_all(hbox);
  



  return vbox;
}

void createCallBacksOpenGL()
{
  g_signal_connect((gpointer)spinPrecision, "value-changed",
		   G_CALLBACK(precisionChanged), (gpointer)0);
  g_signal_connect((gpointer)GTK_COMBO(comboOpenGLRendering)->entry,
		   "changed",
		   GTK_SIGNAL_FUNC(comboOpenGLRenderingChanged),
		   (gpointer)0);
  g_signal_connect((gpointer)checkAntialiasing, "toggled",
		   G_CALLBACK(antialisingCheckChanged), (gpointer)0);
  g_signal_connect((gpointer)checkFakeBackingStore, "toggled",
		   G_CALLBACK(fakeBakingSoreCheckChanged), (gpointer)0);
  g_signal_connect((gpointer)checkImmediateDrawing, "toggled",
		   G_CALLBACK(immediateDrawingCheckChanged), (gpointer)0);
}

/*************/
/* Callbacks */
/*************/
void immediateDrawingCheckChanged(GtkToggleButton *button, gpointer data)
{
  if (disableCallbacksOpenGL)
    return;

  setImmediateDrawing(gtk_toggle_button_get_active(button));
}
void comboOpenGLRenderingChanged(GtkEditable *entry, gpointer data)
{
  char *label;
  
  if (disableCallbacksOpenGL)
    return;

  label = (char*)gtk_entry_get_text(GTK_ENTRY(entry));
  if (!label || label[0] == '\0')
    return;

  if (setRenderingOptionFromString(label))
    g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
		   0 , NULL);
}
void antialisingCheckChanged(GtkToggleButton *button, gpointer data)
{
  if (disableCallbacksOpenGL)
    return;

  if (setAntialias(gtk_toggle_button_get_active(button)))
    g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
		   0 , NULL);
}
void precisionChanged(GtkButton *button, gpointer user_data)
{
  float prec;

  if (disableCallbacksOpenGL)
    return;

  prec = (float)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinPrecision));
  if (setPrecision(prec / 100.))
    g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
		   0 , NULL);
}
void fakeBakingSoreCheckChanged(GtkToggleButton *button, gpointer data)
{
  if (disableCallbacksOpenGL)
    return;

  setFakeBackingStore(gtk_toggle_button_get_active(button));
}
gboolean refreshPanelOpenGL(gpointer data)
{
  disableCallbacksOpenGL = 1;
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkImmediateDrawing),
			       (gboolean)getImmediateDrawing());
  gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(comboOpenGLRendering)->entry),
		     getRenderingOptionAsString());
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinPrecision),
			    getPrecision() * 100.);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkAntialiasing),
			       (gboolean)getAntialias());
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkFakeBackingStore),
			       (gboolean)getFakeBackingStore());
  disableCallbacksOpenGL = 0;

  return FALSE;
}
