/*   EXTRAITS DE LA LICENCE
	Copyright CEA, contributeurs : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)
  
	Adresse ml :
	BILLARD, non joignable par ml ;
	CALISTE, damien P caliste AT cea P fr.

	Ce logiciel est un programme informatique servant  visualiser des
	structures atomiques dans un rendu pseudo-3D. 

	Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
	respectant les principes de diffusion des logiciels libres. Vous pouvez
	utiliser, modifier et/ou redistribuer ce programme sous les conditions
	de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
	sur le site "http://www.cecill.info".

	Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
	pris connaissance de la licence CeCILL, et que vous en avez accept les
	termes (cf. le fichier Documentation/licence.fr.txt fourni avec ce logiciel).
*/

/*   LICENCE SUM UP
	Copyright CEA, contributors : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)

	E-mail address:
	BILLARD, not reachable any more ;
	CALISTE, damien P caliste AT cea P fr.

	This software is a computer program whose purpose is to visualize atomic
	configurations in 3D.

	This software is governed by the CeCILL  license under French law and
	abiding by the rules of distribution of free software.  You can  use, 
	modify and/ or redistribute the software under the terms of the CeCILL
	license as circulated by CEA, CNRS and INRIA at the following URL
	"http://www.cecill.info". 

	The fact that you are presently reading this means that you have had
	knowledge of the CeCILL license and that you accept its terms. You can
	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
*/
#include "panelPlanes.h"

#include <GL/gl.h>
#include <GL/glu.h> 

#include <visu_object.h>
#include <support.h>
#include <gtk_main.h>
#include <opengl.h>
#include <visu_extension.h>

#include <extraFunctions/plane.h>
#include <extraGtkFunctions/gtk_colorComboBoxWidget.h>

#include "visuConfig.h"


GtkWidget *vBoxPlanes;
GtkWidget *checkUsePlanes;
GtkWidget *spinbuttonNVect[3];
GtkWidget *spinbuttonDistance;
GtkWidget *widgetColorPlane;
GtkWidget *treeviewPlanes;
GtkListStore *listStoredPlanes;
GtkWidget *labelDistance;
GtkWidget *buttonRotate;

gboolean checkPlanesOnEnter(gpointer data);
void onPlaneUsed(GtkToggleButton *button, gpointer data);
void onPlaneAdd(GtkButton *button, gpointer data);
void onPlaneRemove(GtkButton *button, gpointer data);
void onSpinNVectChange(GtkSpinButton *spin, gpointer data);
void onSpinDistanceChange(GtkSpinButton *spin, gpointer data);
void onPlaneColorChange(ColorComboBox *combo, Color *selectedColor, gpointer data);
gboolean onTreePlaneClick(GtkWidget *widget, GdkEventButton *event, gpointer user_data);
void rebuildListPlanes(GObject *obj, gpointer data);
void setPlaneSensitive(GObject *obj, gpointer data);
void onNewFileLoaded(GObject *obj, gpointer data);
void onTreeSelectionChanged(GtkTreeSelection *tree, gpointer data);
void onNodePositionChanged(GObject *obj, gpointer data);
void onSetCameraPosition(GtkButton *button, gpointer data);
void onSpinBoundsChangeDistance(GObject *obj, gfloat val, gpointer data);

gboolean isPlanesInitialised;

OpenGLExtension *extensionPlanes;
int identifierAllPlanes;

enum
  {
    COLUMN_PLANE_DRAWN,
    COLUMN_PLANE_LABEL,
    COLUMN_PLANE_HIDE_IS_ON,
    COLUMN_PLANE_HIDDEN_SIDE,
    COLUMN_PLANE_COLOR_PIXBUF,
    COLUMN_PLANE_POINTER,
    N_COLUMN_PLANE
  };

/* String used to labelled planes, dist. means 'distance' and
   norm. means 'normal' (50 chars max). */
#define LABEL_PLANE _("norm.:(%3d;%3d;%3d)\ndistance: %6.2f")

subPanel* initPanelPlanes()
{
  /* Long description */
  char *cl = _("Drawing planes");
  /* Short description */
  char *tl = _("Planes");
  subPanel *panelPlanes;

  vBoxPlanes = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_sensitive(vBoxPlanes, FALSE);
  panelPlanes = createSubPanelWithIconFromPath(cl, tl, "stock-channels-16.png",
					       vBoxPlanes, checkPlanesOnEnter,
					       (onChangePanelFunc)0);
  if (!panelPlanes)
    return (subPanel*)0;

  /* call the visu add routine */
  addSubPanel(panelPlanes);

  isPlanesInitialised = FALSE;

  /* Book some OpenGL list to draw planes. */
  identifierAllPlanes = createNewObjectList(1);
  extensionPlanes = OpenGLExtension_new(_("Planes"), _("Draw some planes."),
					identifierAllPlanes, rebuildListPlanes);
  extensionPlanes->used = 0;
  OpenGLExtensionSet_priority(extensionPlanes, OPENGL_EXTENSION_PRIORITY_NORMAL);
  registerOpenGLExtension(extensionPlanes);

  /* Add the signal for the vBoxPlanes. */
  g_signal_connect(G_OBJECT(visu), "dataReadyForRendering",
		   G_CALLBACK(setPlaneSensitive), GINT_TO_POINTER(1));
  g_signal_connect(G_OBJECT(visu), "dataNotReadyForRendering",
		   G_CALLBACK(setPlaneSensitive), GINT_TO_POINTER(0));

  return panelPlanes;
}

gboolean drawPlanes(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
{
  Plane *plane;
  gboolean used;

  used = FALSE;
  gtk_tree_model_get(model, iter, COLUMN_PLANE_DRAWN, &used, -1);
  if (used)
    {
      plane = (Plane*)0;
      gtk_tree_model_get(model, iter, COLUMN_PLANE_POINTER, &plane, -1);
      if (plane)
	planeDraw(plane);
    }  
  return FALSE;
}

void rebuildListPlanes(GObject *obj, gpointer data)
{
  DBG_fprintf(stderr, "Panel planes : rebuilding object list for planes.\n");
  glNewList(identifierAllPlanes, GL_COMPILE);

  /* Set blend if not present. */
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

  gtk_tree_model_foreach(GTK_TREE_MODEL(listStoredPlanes), drawPlanes, (gpointer)0);
  glEndList();
}

gboolean checkPlanesOnEnter(gpointer data)
{
  if (!isPlanesInitialised)
    {
      createInteriorPlanes();
      isPlanesInitialised = TRUE;
    }

  return FALSE;
}

void createInteriorPlanes()
{
  GtkWidget *alignment1;
  GtkWidget *hbox1;
  GtkWidget *image1;
  GtkWidget *label1;
  GtkWidget *scrolledPlanes;
  GtkWidget *hbox2;
  GtkWidget *alignment3;
  GtkWidget *vbox3;
  GtkWidget *hbox4;
  GtkWidget *label2;
  GtkObject *spinbuttonNVect_adj[3];
  GtkWidget *hbox5;
  GtkObject *spinbuttonDistance_adj;
  GtkWidget *hbox3;
  GtkWidget *label4;
  GtkWidget *alignment2;
  GtkWidget *vbox2;
  GtkWidget *buttonPlaneAdd;
  GtkWidget *image2;
  GtkWidget *buttonPlaneRemove;
  GtkWidget *image4;
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *column;
  GtkWidget *image;
  GtkTooltips *tooltips;

  int i;

  tooltips = gtk_tooltips_new ();

  checkUsePlanes = gtk_check_button_new ();
  gtk_widget_show (checkUsePlanes);
  gtk_box_pack_start (GTK_BOX (vBoxPlanes), checkUsePlanes, FALSE, FALSE, 0);

  alignment1 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment1);
  gtk_container_add (GTK_CONTAINER (checkUsePlanes), alignment1);

  hbox1 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox1);
  gtk_container_add (GTK_CONTAINER (alignment1), hbox1);

  image1 = create_pixmap (mainWindow, "stock-channels-16.png");
  gtk_widget_show (image1);
  gtk_box_pack_start (GTK_BOX (hbox1), image1, FALSE, FALSE, 0);

  label1 = gtk_label_new_with_mnemonic (_("Use planes"));
  gtk_widget_set_name(label1, "label_head");
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (hbox1), label1, FALSE, FALSE, 0);

  scrolledPlanes = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledPlanes);
  gtk_box_pack_start (GTK_BOX (vBoxPlanes), scrolledPlanes, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledPlanes), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledPlanes), GTK_SHADOW_IN);

  treeviewPlanes = gtk_tree_view_new ();
  gtk_widget_show (treeviewPlanes);
  gtk_container_add (GTK_CONTAINER (scrolledPlanes), treeviewPlanes);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeviewPlanes), TRUE);

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (vBoxPlanes), hbox2, FALSE, FALSE, 0);

  
  vbox3 = gtk_vbox_new (FALSE, 3);
  gtk_widget_show (vbox3);
  gtk_box_pack_start(GTK_BOX(hbox2), vbox3, FALSE, FALSE, 2);
  buttonRotate = gtk_button_new();
  gtk_tooltips_set_tip(tooltips, buttonRotate, _("Set the camera to look in the direction"
						 " of the normal of the selected plane."), NULL);
  gtk_box_pack_start(GTK_BOX(vbox3), buttonRotate, TRUE, FALSE, 0);
  gtk_widget_show(buttonRotate);
  image = create_pixmap(mainWindow, "stock_rotate_20.png");
  gtk_widget_show(image);
  gtk_container_add(GTK_CONTAINER(buttonRotate), image);
  gtk_widget_set_sensitive(buttonRotate, FALSE);
  

  alignment3 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment3);
  gtk_box_pack_start (GTK_BOX (hbox2), alignment3, TRUE, TRUE, 0);

  vbox3 = gtk_vbox_new (FALSE, 3);
  gtk_widget_show (vbox3);
  gtk_container_add (GTK_CONTAINER (alignment3), vbox3);

  hbox4 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox4);
  gtk_box_pack_start (GTK_BOX (vbox3), hbox4, TRUE, TRUE, 0);

  label2 = gtk_label_new (_("Normal: "));
  gtk_widget_show (label2);
  gtk_box_pack_start (GTK_BOX (hbox4), label2, TRUE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (label2), 1, 0.5);

  for (i = 0; i < 3; i++)
    {
      spinbuttonNVect_adj[i] = gtk_adjustment_new (1, -20, 20, 1, 3, 3);
      spinbuttonNVect[i] = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonNVect_adj[i]), 1, 0);
      gtk_widget_show (spinbuttonNVect[i]);
      gtk_box_pack_start (GTK_BOX (hbox4), spinbuttonNVect[i], FALSE, FALSE, 0);
      gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbuttonNVect[i]), TRUE);
    }

  hbox5 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox5);
  gtk_box_pack_start (GTK_BOX (vbox3), hbox5, TRUE, TRUE, 0);

  labelDistance = gtk_label_new ("");
  if (panelConfigGet_spinBoundsValue() == 1.)
    gtk_label_set_text(GTK_LABEL(labelDistance),
		       _("Distance from origin: "));
  else
    {
      gtk_label_set_text(GTK_LABEL(labelDistance),
			 _("Dist. from origin <span style=\"italic\" size=\"smaller\""
			   " foreground=\"red\">\303\227 mult.</span>: "));
      gtk_label_set_use_markup(GTK_LABEL(labelDistance), TRUE);
    }
  gtk_widget_show (labelDistance);
  gtk_box_pack_start (GTK_BOX (hbox5), labelDistance, TRUE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (labelDistance), 1, 0.5);

  spinbuttonDistance_adj = gtk_adjustment_new (1, -1000, 1000, 1, 10, 10);
  spinbuttonDistance = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonDistance_adj), 1, 2);
  gtk_widget_show (spinbuttonDistance);
  gtk_box_pack_start (GTK_BOX (hbox5), spinbuttonDistance, FALSE, FALSE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbuttonDistance), TRUE);

  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (vbox3), hbox3, TRUE, TRUE, 0);

  label4 = gtk_label_new (_("Color: "));
  gtk_widget_show (label4);
  gtk_box_pack_start (GTK_BOX (hbox3), label4, TRUE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (label4), 1, 0.5);

  widgetColorPlane = colorComboBox_new(TRUE);
  gtk_widget_show(widgetColorPlane);
  gtk_box_pack_start(GTK_BOX(hbox3), widgetColorPlane, FALSE, FALSE, 0);


  alignment2 = gtk_alignment_new (0.5, 0.5, 1, 0);
  gtk_widget_show (alignment2);
  gtk_box_pack_start (GTK_BOX (hbox2), alignment2, FALSE, FALSE, 0);

  vbox2 = gtk_vbox_new (FALSE, 3);
  gtk_widget_show (vbox2);
  gtk_container_add (GTK_CONTAINER (alignment2), vbox2);
  gtk_container_set_border_width (GTK_CONTAINER (vbox2), 3);

  buttonPlaneAdd = gtk_button_new ();
  gtk_widget_show (buttonPlaneAdd);
  gtk_box_pack_start (GTK_BOX (vbox2), buttonPlaneAdd, FALSE, FALSE, 0);

  image2 = gtk_image_new_from_stock ("gtk-add", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image2);
  gtk_container_add (GTK_CONTAINER (buttonPlaneAdd), image2);

  buttonPlaneRemove = gtk_button_new ();
  gtk_widget_show (buttonPlaneRemove);
  gtk_box_pack_start (GTK_BOX (vbox2), buttonPlaneRemove, FALSE, FALSE, 0);

  image4 = gtk_image_new_from_stock ("gtk-remove", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image4);
  gtk_container_add (GTK_CONTAINER (buttonPlaneRemove), image4);

  /* Add the callback methods. */
  g_signal_connect(G_OBJECT(checkUsePlanes), "toggled",
		   G_CALLBACK(onPlaneUsed), (gpointer)0);

  g_signal_connect(G_OBJECT(buttonPlaneAdd), "clicked",
		   G_CALLBACK(onPlaneAdd), (gpointer)0);
  g_signal_connect(G_OBJECT(buttonPlaneRemove), "clicked",
		   G_CALLBACK(onPlaneRemove), (gpointer)0);

  for (i = 0; i < 3; i++)
    g_signal_connect(G_OBJECT(spinbuttonNVect[i]), "value-changed",
		     G_CALLBACK(onSpinNVectChange), GINT_TO_POINTER(i));

  g_signal_connect(G_OBJECT(spinbuttonDistance), "value-changed",
		   G_CALLBACK(onSpinDistanceChange), (gpointer)0);

  g_signal_connect(G_OBJECT(treeviewPlanes), "button-release-event",
		   G_CALLBACK(onTreePlaneClick), (gpointer)0);

  g_signal_connect(G_OBJECT(widgetColorPlane), "color-selected",
		   G_CALLBACK(onPlaneColorChange), (gpointer)0);

  g_signal_connect(G_OBJECT(visu), "dataReadyForRendering",
		   G_CALLBACK(onNewFileLoaded), (gpointer)0);

  g_signal_connect(G_OBJECT(visu), "NodePositionChanged",
		   G_CALLBACK(onNodePositionChanged), (gpointer)0);

  g_signal_connect(gtk_tree_view_get_selection(GTK_TREE_VIEW(treeviewPlanes)),
		   "changed", G_CALLBACK(onTreeSelectionChanged), (gpointer)0);

  g_signal_connect(G_OBJECT(buttonRotate), "clicked",
		   G_CALLBACK(onSetCameraPosition), (gpointer)0);

  g_signal_connect(G_OBJECT(visuGtkObject), "spinBoundsChanged",
		   G_CALLBACK(onSpinBoundsChangeDistance), (gpointer)0);


  /* Create the listModel */
  listStoredPlanes = gtk_list_store_new (N_COLUMN_PLANE,
					 G_TYPE_BOOLEAN,
					 G_TYPE_STRING,
					 G_TYPE_BOOLEAN,
					 G_TYPE_BOOLEAN,
					 GDK_TYPE_PIXBUF,
					 G_TYPE_POINTER);

  /* Render the associated tree */
  renderer = gtk_cell_renderer_toggle_new ();
  column = gtk_tree_view_column_new_with_attributes (_("Drawn"),
						     renderer,
						     "active", COLUMN_PLANE_DRAWN,
						     NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW(treeviewPlanes), column);
  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes (_("Parameters"),
						     renderer,
						     "text", COLUMN_PLANE_LABEL,
						     NULL);
  gtk_tree_view_column_set_expand(column, TRUE);
  gtk_tree_view_column_set_alignment(column, 0.5);
  gtk_tree_view_append_column (GTK_TREE_VIEW (treeviewPlanes), column);
  renderer = gtk_cell_renderer_toggle_new ();
  column = gtk_tree_view_column_new_with_attributes (_("Mask"),
						     renderer,
						     "active", COLUMN_PLANE_HIDE_IS_ON,
						     NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW(treeviewPlanes), column);
  renderer = gtk_cell_renderer_toggle_new ();
  column = gtk_tree_view_column_new_with_attributes (_("Invert"),
						     renderer,
						     "active", COLUMN_PLANE_HIDDEN_SIDE,
						     NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW(treeviewPlanes), column);
  renderer = gtk_cell_renderer_pixbuf_new ();
  column = gtk_tree_view_column_new_with_attributes (_("Color"),
						     renderer,
						     "pixbuf", COLUMN_PLANE_COLOR_PIXBUF,
						     NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW(treeviewPlanes), column);

  gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(treeviewPlanes)),
			      GTK_SELECTION_SINGLE);
  gtk_tree_view_set_model(GTK_TREE_VIEW(treeviewPlanes), GTK_TREE_MODEL(listStoredPlanes));
}

GList* getListOfAllPlanes()
{
  GList *planeList;
  GtkTreeIter iter;
  Plane *plane;
  
  DBG_fprintf(stderr, "Panel Plane : building plane list...\n");
  /* Build the list of all planes. */
  planeList = (GList*)0;
  if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(listStoredPlanes), &iter))
    do
      {
	gtk_tree_model_get(GTK_TREE_MODEL(listStoredPlanes),
			   &iter, COLUMN_PLANE_POINTER, &plane, -1);
	planeList = g_list_append(planeList, plane);
	DBG_fprintf(stderr, " | found one plane %p.\n", (gpointer)plane);
      }
    while (gtk_tree_model_iter_next(GTK_TREE_MODEL(listStoredPlanes), &iter));

  return planeList;
}

void onPlaneUsed(GtkToggleButton *button, gpointer data)
{
  gboolean used;
  int i, j, val;
  GList *planeList;
  GtkTreeIter iter;

  used = gtk_toggle_button_get_active(button);
  if (extensionPlanes->used == used)
    return;

  extensionPlanes->used = used;
  
  /* Test if any planes are stored. */
  if (!gtk_tree_model_get_iter_first(GTK_TREE_MODEL(listStoredPlanes), &iter))
    return;

  /* Show/hide nodes accordingly. */
  planeList = (GList*)0;
  if (used)
    planeList = getListOfAllPlanes();

  val = planeShowHide_all(currentVisuData, planeList);
  if (val)
    visuData_createAllNodes();

  if (used)
    g_list_free(planeList);
  
  g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
		 0 /* details */,
		 NULL);
}
void onPlaneAdd(GtkButton *button, gpointer userData)
{
  Plane* plane;
  float vect[3];
  int i;
  GtkTreeIter iter;
  Color *color;
  gchar str[50];
  float dist;
  GdkPixbuf *pixbufColorAlphaBox;

  pixbufColorAlphaBox = colorComboBoxGet_selectedPixbuf(COLOR_COMBOX(widgetColorPlane));
  color = colorComboBoxGet_selectedColor(COLOR_COMBOX(widgetColorPlane));
  for (i = 0; i < 3; i++)
    vect[i] = (float)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbuttonNVect[i]));
  dist = (float)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbuttonDistance));
  plane = planeNew(vect, dist, color);
  
  /* String used to labelled planes, dist. means 'distance' and
     norm. means 'normal' (50 chars max). */
  sprintf(str, LABEL_PLANE, (int)vect[0], (int)vect[1], (int)vect[2], dist);
  gtk_list_store_append(listStoredPlanes, &iter);
  gtk_list_store_set(listStoredPlanes, &iter,
		     COLUMN_PLANE_DRAWN, TRUE,
		     COLUMN_PLANE_LABEL, str,
		     COLUMN_PLANE_HIDE_IS_ON, FALSE,
		     COLUMN_PLANE_HIDDEN_SIDE, FALSE,
		     COLUMN_PLANE_COLOR_PIXBUF, (gpointer)pixbufColorAlphaBox,
		     COLUMN_PLANE_POINTER, (gpointer)plane,
		     -1);

  rebuildListPlanes((GObject*)0, (gpointer)0);
  gtk_tree_selection_select_iter(gtk_tree_view_get_selection(GTK_TREE_VIEW(treeviewPlanes)), &iter);
  if (extensionPlanes->used)
    g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
		   0 /* details */,
		   NULL);
}
void onPlaneRemove(GtkButton *button, gpointer data)
{
  gboolean res;
  GtkTreeModel *tmpList;
  GtkTreeIter iter;
  int val;
  GList *planeList;

  DBG_fprintf(stderr, "Panel Plane : removing plane...\n");
  res = gtk_tree_selection_get_selected(gtk_tree_view_get_selection(GTK_TREE_VIEW(treeviewPlanes)),
					&tmpList, &iter);
  if (res)
    {
      res = TRUE;
      if (!gtk_list_store_remove(GTK_LIST_STORE(listStoredPlanes), &iter))
	res = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(listStoredPlanes), &iter);

      DBG_fprintf(stderr, "Panel Plane : OK plane found and removed.\n");

      rebuildListPlanes((GObject*)0, (gpointer)0);
      /* Build the list of all planes. */
      planeList = getListOfAllPlanes();
      /* Modify nodes visibility accordingly. */
      val = planeShowHide_all(currentVisuData, planeList);
      g_list_free(planeList);
      if (val)
	visuData_createAllNodes();
      if (extensionPlanes->used)
	g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
		       0 /* details */,
		       NULL);

      if (res)
	gtk_tree_selection_select_iter(gtk_tree_view_get_selection(GTK_TREE_VIEW(treeviewPlanes)),
				       &iter);
    }
}
void onSpinNVectChange(GtkSpinButton *spin, gpointer data)
{
  gboolean res;
  GtkTreeModel *tmpList;
  GtkTreeIter iter;
  Plane *plane;
  char str[50];
  float dist;
  float vect[3];
  gboolean hide, side;
  int i;
  GList *planeList;
  int val;

  i = GPOINTER_TO_INT(data);
  if (i < 0 || i > 2)
    return;

  res = gtk_tree_selection_get_selected(gtk_tree_view_get_selection(GTK_TREE_VIEW(treeviewPlanes)),
					&tmpList, &iter);
  if (res)
    {
      plane = (Plane*)0;
      hide = FALSE;
      side = FALSE;
      gtk_tree_model_get(GTK_TREE_MODEL(listStoredPlanes), &iter,
			 COLUMN_PLANE_POINTER, &plane,
			 COLUMN_PLANE_HIDE_IS_ON, &hide,
			 COLUMN_PLANE_HIDDEN_SIDE, &side,
			 -1);
      if (plane)
	{
	  planeGet_nVectUser(plane, vect);
	  vect[i] = (float)gtk_spin_button_get_value(spin);
	  val = planeSet_normalVector(plane, vect);
	  if (val < 0)
	    {
	      fprintf(stderr, "WARNING! Something goes wrong when assigning"
		      " the new normal vector : (%f; %f; %f).\n",
		      vect[0], vect[1], vect[2]);
	      planeGet_nVectUser(plane, vect);
	      for (i = 0; i < 3; i++)
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbuttonNVect[i]), vect[i]);
	      return;
	    }
	  else if (val == 0)
	    return;
	  planeComputeInter(plane);
	  planeGet_distanceFromOrigin(plane, &dist);
	  sprintf(str, LABEL_PLANE,
		  (int)vect[0], (int)vect[1], (int)vect[2], dist);
	  gtk_list_store_set(listStoredPlanes, &iter, COLUMN_PLANE_LABEL, str, -1);
	  rebuildListPlanes((GObject*)0, (gpointer)0);
	  /* Build the list of all planes. */
	  planeList = getListOfAllPlanes();
	  /* Modify nodes visibility accordingly. */
	  val = planeShowHide_all(currentVisuData, planeList);
	  g_list_free(planeList);
	  if (val)
	    visuData_createAllNodes();
	  if (extensionPlanes->used)
	    g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
			   0 /* details */,
			   NULL);
	}
    }
}
void onSpinDistanceChange(GtkSpinButton *spin, gpointer data)
{
  gboolean res;
  GtkTreeModel *tmpList;
  GtkTreeIter iter;
  Plane *plane;
  char str[50];
  float dist, mult;
  float vect[3];
  gboolean hide, side;
  GList *planeList;
  int val;

  res = gtk_tree_selection_get_selected(gtk_tree_view_get_selection(GTK_TREE_VIEW(treeviewPlanes)),
					&tmpList, &iter);
  if (res)
    {
      plane = (Plane*)0;
      hide = FALSE;
      side = FALSE;
      gtk_tree_model_get(GTK_TREE_MODEL(listStoredPlanes), &iter,
			 COLUMN_PLANE_POINTER, &plane,
			 COLUMN_PLANE_HIDE_IS_ON, &hide,
			 COLUMN_PLANE_HIDDEN_SIDE, &side,
			 -1);
      if (plane)
	{
	  mult = panelConfigGet_spinBoundsValue();
	  dist = (float)gtk_spin_button_get_value(spin);
	  val = planeSet_distanceFromOrigin(plane, dist * mult);
	  if (val < 0)
	    {
	      fprintf(stderr, "WARNING! Something goes wrong when assigning"
		      " the new distance from origin : %f.\n",
		      dist);
	      gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbuttonDistance), 0.);
	      return;
	    }
	  else if (val == 0)
	    return;
	  planeComputeInter(plane);
	  planeGet_nVectUser(plane, vect);
	  sprintf(str, LABEL_PLANE,
		  (int)vect[0], (int)vect[1], (int)vect[2], dist);
	  gtk_list_store_set(listStoredPlanes, &iter, COLUMN_PLANE_LABEL, str, -1);
	  rebuildListPlanes((GObject*)0, (gpointer)0);
	  /* Build the list of all planes. */
	  planeList = getListOfAllPlanes();
	  /* Modify nodes visibility accordingly. */
	  val = planeShowHide_all(currentVisuData, planeList);
	  g_list_free(planeList);
	  if (val)
	    visuData_createAllNodes();
	  if (extensionPlanes->used)
	    g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
			   0 /* details */,
			   NULL);
	}
    }
}
void onSpinBoundsChangeDistance(GObject *obj, gfloat mult, gpointer data)
{
  DBG_fprintf(stderr, "Panel Plane : catch the 'spinBoundsChanged' signal,"
	      " should adjust the distance from origin (multiplier : %f).\n", mult);

  if (panelConfigGet_spinBoundsValue() == 1.)
    gtk_label_set_text(GTK_LABEL(labelDistance),
		       _("Distance from origin: "));
  else
    {
      gtk_label_set_text(GTK_LABEL(labelDistance),
			 _("Dist. from origin <span style=\"italic\" size=\"smaller\""
			   " foreground=\"red\">\303\227 mult.</span>: "));
      gtk_label_set_use_markup(GTK_LABEL(labelDistance), TRUE);
    }
}
gboolean onTreePlaneClick(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
  GtkTreePath *currentPath;
  GtkTreeIter iter;
  GtkTreeViewColumn *col;
  gboolean res;
  Plane *plane;
  gboolean checked, side;
  gboolean reDraw;
  int val;
  GList *planeList;

  DBG_fprintf(stderr, "Panel Plane : Tree clicked (button %d).\n", event->button);
  if (event->button == 1)
    {
      res = gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW (treeviewPlanes),
					  event->x, event->y, &currentPath,
					  &col, NULL, NULL);
      if (!res)
	return FALSE;

      /* One row is selected, print values on spins... */
      gtk_tree_model_get_iter(GTK_TREE_MODEL(listStoredPlanes), &iter, currentPath);
      plane = (Plane*)0;
      gtk_tree_model_get(GTK_TREE_MODEL(listStoredPlanes),
			 &iter, COLUMN_PLANE_POINTER, &plane, -1);

      reDraw = FALSE;
      if (col == gtk_tree_view_get_column(GTK_TREE_VIEW(treeviewPlanes), 0))
	{
	  gtk_tree_model_get(GTK_TREE_MODEL(listStoredPlanes), &iter,
			     COLUMN_PLANE_DRAWN, &checked, -1);
	  gtk_list_store_set(listStoredPlanes, &iter,
			     COLUMN_PLANE_DRAWN, !checked, -1);
	  rebuildListPlanes((GObject*)0, (gpointer)0);
	  reDraw = TRUE;
	  DBG_fprintf(stderr, "Panel planes : use state (%d) for plane %d.\n",
		      !checked, GPOINTER_TO_INT(plane));
	}
      else if (col == gtk_tree_view_get_column(GTK_TREE_VIEW(treeviewPlanes), 2) ||
	       col == gtk_tree_view_get_column(GTK_TREE_VIEW(treeviewPlanes), 3))
	{
	  gtk_tree_model_get(GTK_TREE_MODEL(listStoredPlanes), &iter,
			     COLUMN_PLANE_HIDE_IS_ON, &checked,
			     COLUMN_PLANE_HIDDEN_SIDE, &side, -1);
	  if (col == gtk_tree_view_get_column(GTK_TREE_VIEW(treeviewPlanes), 2))
	    {
	      checked = !checked;
	      gtk_list_store_set(listStoredPlanes, &iter,
				 COLUMN_PLANE_HIDE_IS_ON, checked, -1);
	    }
	  else
	    {
	      side = !side;
	      gtk_list_store_set(listStoredPlanes, &iter,
				 COLUMN_PLANE_HIDDEN_SIDE, side, -1);
	    }
	  reDraw = TRUE;
	  /* Set the hidden status */
	  if (!checked)
	    val = planeSet_hiddenState(plane, PLANE_SIDE_NONE);
	  else
	    {
	      if (side)
		val = planeSet_hiddenState(plane, PLANE_SIDE_MINUS);
	      else
		val = planeSet_hiddenState(plane, PLANE_SIDE_PLUS);
	    }
	  if (val)
	    {
	      /* Build the list of all planes. */
	      planeList = getListOfAllPlanes();
	      /* Modify nodes visibility accordingly. */
	      val = planeShowHide_all(currentVisuData, planeList);
	      g_list_free(planeList);
	      if (val)
		visuData_createAllNodes();
	    }
	}
      if (reDraw)
	{
	  if (extensionPlanes->used)
	    g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
			   0 /* details */,
			   NULL);
	}
      gtk_tree_path_free(currentPath);
    }

  return FALSE;
}
void onPlaneColorChange(ColorComboBox *combo, Color *color, gpointer data)
{
  gboolean res;
  GtkTreeModel *tmpList;
  GtkTreeIter iter;
  Plane *plane;
  GtkTreeModel *tree;
  int selected, val;
  GdkPixbuf *pixbufColorAlphaBox;

  DBG_fprintf(stderr, "Panel Plane : Catch the 'color-selected' signal.\n");

  pixbufColorAlphaBox = colorComboBoxGet_selectedPixbuf(combo);

  /* Get the plane */
  res = gtk_tree_selection_get_selected(gtk_tree_view_get_selection(GTK_TREE_VIEW(treeviewPlanes)),
					&tmpList, &iter);
  if (!res)
    return;
  plane = (Plane*)0;
  gtk_tree_model_get(GTK_TREE_MODEL(listStoredPlanes), &iter, COLUMN_PLANE_POINTER, &plane, -1);
  if (!plane)
    return;

  /* Change the color value. */
  val = planeSet_color(plane, color);
  if (val < 0)
    {
      fprintf(stderr, "WARNING! Something goes wrong when assigning"
	      " the new color : %p.\n", (gpointer)color);
      return;
    }
  else if (val == 0)
    return;

  DBG_fprintf(stderr, "Panel Plane : color combobox changed to %p (%f,%f,%f,%f).\n",
	      (gpointer)color, color->rgba[0], color->rgba[1], color->rgba[2], color->rgba[3]);
  DBG_fprintf(stderr, "Panel Plane : Changed color (%p) for plane : %p.\n",
	      (gpointer)color, (gpointer)plane);

  gtk_list_store_set(listStoredPlanes, &iter, COLUMN_PLANE_COLOR_PIXBUF, pixbufColorAlphaBox, -1);
  rebuildListPlanes((GObject*)0, (gpointer)0);
  if (extensionPlanes->used)
    g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
		   0 /* details */,
		   NULL);
}

void setPlaneSensitive(GObject *obj, gpointer data)
{
  DBG_fprintf(stderr, "Panel Plane : catch dataReadyForRendering or"
	      " dataNotReadyForRendering signal (%d).\n", GPOINTER_TO_INT(data));
  if (GPOINTER_TO_INT(data))
    gtk_widget_set_sensitive(vBoxPlanes, TRUE);
  else
    gtk_widget_set_sensitive(vBoxPlanes, FALSE);
}

void onNewFileLoaded(GObject *obj, gpointer data)
{
  GList *list, *tmplst;
  int val;

  if (!extensionPlanes->used)
    return;

  DBG_fprintf(stderr, "Panel Plane : catch 'dataReadyForRendering' signal,"
	      " recalculating intersections and masking set.\n");
  
  list = getListOfAllPlanes();
  tmplst = list;
  while (tmplst)
    {
      /* Compute the intersection with the new box */
      planeComputeInter((Plane*)tmplst->data);
      tmplst = g_list_next(tmplst);
    }
  /* Compute the masking state */
  val = planeShowHide_all(currentVisuData, list);
  g_list_free(list);
  rebuildListPlanes((GObject*)0, (gpointer)0);
}

void onNodePositionChanged(GObject *obj, gpointer data)
{
  GList *list;

  DBG_fprintf(stderr, "Panel Plane : catch 'NodePositionChanged' signal,"
	      " recalculating intersections.\n");
  
  list = getListOfAllPlanes();
  planeShowHide_all(currentVisuData, list);
  g_list_free(list);
  rebuildListPlanes((GObject*)0, (gpointer)0);
}


void onTreeSelectionChanged(GtkTreeSelection *tree, gpointer data)
{
  gboolean res;
  GtkTreeModel *tmpList;
  GtkTreeIter iter;
  Plane *plane;
  float val, nPlane[3], dist, mult;
  int i;
  Color *colorPlane, *colorCombo;

  DBG_fprintf(stderr, "Panel Plane : catch 'changed' signal from "
	      " the gtkTreeSelection.\n");

  res = gtk_tree_selection_get_selected(tree, &tmpList, &iter);
  if (res)
    {
      plane = (Plane*)0;
      gtk_tree_model_get(GTK_TREE_MODEL(listStoredPlanes), &iter,
			 COLUMN_PLANE_POINTER, &plane,
			 -1);
      planeGet_nVectUser(plane, nPlane);
      for (i = 0; i < 3; i++)
	{
	  val = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbuttonNVect[i]));
	  if (val != nPlane[i])
	    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbuttonNVect[i]), nPlane[i]);
	}
      planeGet_distanceFromOrigin(plane, &dist);
      mult = panelConfigGet_spinBoundsValue();
      val = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbuttonDistance));
      if (val != dist / mult)
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbuttonDistance), dist / mult);
      planeGet_color(plane, &colorPlane);
      colorCombo = colorComboBoxGet_selectedColor(COLOR_COMBOX(widgetColorPlane));
      if (colorPlane != colorCombo)
	colorComboBoxSet_selectionByColor(COLOR_COMBOX(widgetColorPlane), colorPlane);

      gtk_widget_set_sensitive(buttonRotate, TRUE);
    }
  else
    gtk_widget_set_sensitive(buttonRotate, FALSE);
}

void onSetCameraPosition(GtkButton *button, gpointer data)
{
  gboolean res;
  GtkTreeModel *tmpList;
  GtkTreeIter iter;
  float spherical[3];
  Plane *plane;
  float nPlane[3];

  res = gtk_tree_selection_get_selected(gtk_tree_view_get_selection(GTK_TREE_VIEW(treeviewPlanes)),
					&tmpList, &iter);
  if (res)
    {
      plane = (Plane*)0;
      gtk_tree_model_get(GTK_TREE_MODEL(listStoredPlanes), &iter,
			 COLUMN_PLANE_POINTER, &plane,
			 -1);
      DBG_fprintf(stderr, "Panel Planes : Set the camera position to be"
		  " normal to the selected plane (%p).\n", (gpointer)plane);
      planeGet_nVectUser(plane, nPlane);
      cartesian_to_spherical(spherical, nPlane);
      setCameraThetaPhiOmega(spherical[1], spherical[2], 0.,
			     MASK_THETA | MASK_PHI);
      g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
		     0 , NULL);
    }
}
