/*   EXTRAITS DE LA LICENCE
	Copyright CEA, contributeurs : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)
  
	Adresse ml :
	BILLARD, non joignable par ml ;
	CALISTE, damien P caliste AT cea P fr.

	Ce logiciel est un programme informatique servant  visualiser des
	structures atomiques dans un rendu pseudo-3D. 

	Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
	respectant les principes de diffusion des logiciels libres. Vous pouvez
	utiliser, modifier et/ou redistribuer ce programme sous les conditions
	de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
	sur le site "http://www.cecill.info".

	Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
	pris connaissance de la licence CeCILL, et que vous en avez accept les
	termes (cf. le fichier Documentation/licence.fr.txt fourni avec ce logiciel).
*/

/*   LICENCE SUM UP
	Copyright CEA, contributors : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)

	E-mail address:
	BILLARD, not reachable any more ;
	CALISTE, damien P caliste AT cea P fr.

	This software is a computer program whose purpose is to visualize atomic
	configurations in 3D.

	This software is governed by the CeCILL  license under French law and
	abiding by the rules of distribution of free software.  You can  use, 
	modify and/ or redistribute the software under the terms of the CeCILL
	license as circulated by CEA, CNRS and INRIA at the following URL
	"http://www.cecill.info". 

	The fact that you are presently reading this means that you have had
	knowledge of the CeCILL license and that you accept its terms. You can
	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "visu_windowInterface.h"

#if X11 == 1
#include <gdk/gdkx.h>
#endif

#if WIN32 == 1
#include <windows.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include <stdio.h>
#include <stdlib.h>

#include <pixmaps/icone-observe.xpm>
#include <visu_tools.h>
#include <OSOpenGL/visu_OSInterface.h>
#include <support.h>


GtkWidget *gtkWindow;

GdkCursor *cursorRotate;
GdkCursor *cursorWatch;
GdkCursor *cursorPointer;
GdkCursor *cursorPirate;
GdkCursor *currentCursor;

static gulong exposureEventId;
int exposureIsBlocked;

CallbackFunctions standardCallback;

#if X11 == 1
XID XWindowId;
#endif

#if WIN32 == 1
struct _GdkDrawableImplWin32
{
  GdkDrawable parent_instance;

  GdkDrawable *wrapper;

  GdkColormap *colormap;

  HANDLE handle;
};
typedef struct _GdkDrawableImplWin32 GdkDrawableImplWin32;
#endif


gboolean onButtonPressed(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
  CallbackFunctions *callbacks;
  SimplifiedEvents ev;

  callbacks = (CallbackFunctions*)user_data;
  if (!callbacks)
    {
      fprintf(stderr, "INTERNAL ERROR! 'onButtonPressed' has been called"
	      " with a null data parameter.\n");
      return FALSE;
    }
  ev.button = 0;
  ev.motion = 0;
  ev.letter = '\0';

  gdk_window_set_cursor(GDK_WINDOW(gtkWindow->window), cursorWatch);
  ev.x = event->x;
  ev.y = event->y;
  ev.button = event->button;
  ev.shiftMod = event->state & GDK_SHIFT_MASK;
  ev.controlMod = event->state & GDK_CONTROL_MASK;

  if (ev.button == 3)
    callbacks->stop((gpointer)0);
  else
    callbacks->action(&ev);
  gdk_window_set_cursor(GDK_WINDOW(gtkWindow->window), currentCursor);

  return TRUE;
}
gboolean onScrollEvent(GtkWidget *widget, GdkEventScroll *event, gpointer user_data)
{
  CallbackFunctions *callbacks;
  SimplifiedEvents ev;

  callbacks = (CallbackFunctions*)user_data;
  if (!callbacks)
    {
      fprintf(stderr, "INTERNAL ERROR! 'onScrollEvent' has been called"
	      " with a null data parameter.\n");
      return FALSE;
    }
  ev.button = 0;
  ev.motion = 0;
  ev.letter = '\0';

  ev.x = event->x;
  ev.y = event->y;
  if (event->direction == GDK_SCROLL_UP)
    ev.button = 4;
  else if (event->direction == GDK_SCROLL_DOWN)
    ev.button = 5;
  ev.shiftMod = event->state & GDK_SHIFT_MASK;
  ev.controlMod = event->state & GDK_CONTROL_MASK;

  if (ev.button)
    {
      gdk_window_set_cursor(GDK_WINDOW(gtkWindow->window), cursorWatch);
      callbacks->action(&ev);
      gdk_window_set_cursor(GDK_WINDOW(gtkWindow->window), currentCursor);
    }

  return TRUE;
}
gboolean onMouseMotion(GtkWidget *widget, GdkEventMotion *event, gpointer user_data)
{
  GdkWindow *windowMove;
  gint x, y;
  GdkModifierType mask;
  CallbackFunctions *callbacks;
  SimplifiedEvents ev;

  callbacks = (CallbackFunctions*)user_data;
  if (!callbacks)
    {
      fprintf(stderr, "INTERNAL ERROR! 'onMouseMotion' has been called"
	      " with a null data parameter.\n");
      return FALSE;
    }
  ev.button = 0;
  ev.motion = 1;
  ev.letter = '\0';

  windowMove = gdk_window_get_pointer(GDK_WINDOW(gtkWindow->window),
				      &x, &y, &mask);

  ev.x = x;
  ev.y = y;
  if (event->state & GDK_BUTTON1_MASK)
    ev.button = 1;
  else if (event->state & GDK_BUTTON2_MASK)
    ev.button = 2;
  else if (event->state & GDK_BUTTON3_MASK)
    ev.button = 3;
  ev.shiftMod = event->state & GDK_SHIFT_MASK;
  ev.controlMod = event->state & GDK_CONTROL_MASK;

  if (ev.button == 3)
    callbacks->stop((gpointer)0);
  else if (ev.button != 0)
    {
      gdk_window_set_cursor(GDK_WINDOW(gtkWindow->window), cursorWatch);
      callbacks->action(&ev);
      gdk_window_set_cursor(GDK_WINDOW(gtkWindow->window), currentCursor);
    }

  return TRUE;
}
gboolean onKeyPressed(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
  CallbackFunctions *callbacks;
  SimplifiedEvents ev;

  callbacks = (CallbackFunctions*)user_data;
  if (!callbacks)
    {
      fprintf(stderr, "INTERNAL ERROR! 'onKeyPressed' has been called"
	      " with a null data parameter.\n");
      return FALSE;
    }
  ev.button = 0;
  ev.motion = 0;
  ev.letter = '\0';

  gdk_window_set_cursor(GDK_WINDOW(gtkWindow->window), cursorWatch);
  if(event->keyval == GDK_r || event->keyval == GDK_R)
    ev.letter = 'r';

  callbacks->action(&ev);
  gdk_window_set_cursor(GDK_WINDOW(gtkWindow->window), currentCursor);

  return TRUE;
}

gboolean onExposure(GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
{
  DBG_fprintf(stderr, "Visu window GTK : expose event catched.\n");
  standardCallback.action((SimplifiedEvents*)0);
  return TRUE;
}

GList* setObserveEventListener(CallbackFunctions *userCallbacks)
{
  GList *listId;
  guint *idClic, *idMove, *idKey, *idScroll;

  gdk_window_set_cursor(GDK_WINDOW(gtkWindow->window), cursorRotate);
  currentCursor = cursorRotate;

  listId = (GList*)0;
  idClic = malloc(sizeof(guint));
  idMove = malloc(sizeof(guint));
  idKey = malloc(sizeof(guint));
  idScroll = malloc(sizeof(guint));
  if (!idClic || !idMove || !idKey || !idScroll)
    {
      allocationProblems();
      exit(1);
    }
  *idClic = g_signal_connect(G_OBJECT(gtkWindow), "button-press-event",
			     G_CALLBACK(onButtonPressed), (gpointer)userCallbacks);
  *idMove = g_signal_connect(G_OBJECT(gtkWindow), "motion-notify-event",
			     G_CALLBACK(onMouseMotion), (gpointer)userCallbacks);
  *idKey = g_signal_connect(G_OBJECT(gtkWindow), "key-press-event",
			    G_CALLBACK(onKeyPressed), (gpointer)userCallbacks);
  *idScroll = g_signal_connect(G_OBJECT(gtkWindow), "scroll-event",
			       G_CALLBACK(onScrollEvent), (gpointer)userCallbacks);
  listId = g_list_append(listId, (gpointer)idClic);
  listId = g_list_append(listId, (gpointer)idMove);
  listId = g_list_append(listId, (gpointer)idKey);
  listId = g_list_append(listId, (gpointer)idScroll);
    
  return listId;
}

GList* setPickEventListener(CallbackFunctions *userCallbacks)
{
  GList *listId;
  guint *idClic;

  gdk_window_set_cursor(GDK_WINDOW(gtkWindow->window), cursorPointer);
  currentCursor = cursorPointer;

  listId = (GList*)0;
  idClic = malloc(sizeof(guint));
  if (!idClic)
    {
      allocationProblems();
      exit(1);
    }
  *idClic = g_signal_connect(G_OBJECT(gtkWindow), "button-press-event",
			     G_CALLBACK(onButtonPressed), (gpointer)userCallbacks);
  listId = g_list_append(listId, (gpointer)idClic);

  return listId;
}

void removeEventListener(GList *listId)
{
  GList* ptList;

  ptList = listId;
  while (ptList)
    {
      g_signal_handler_disconnect(G_OBJECT(gtkWindow), *(guint*)ptList->data);
      free(ptList->data);
      ptList = g_list_next(ptList);
    }
  if (listId)
    g_list_free(listId);
  listId = (GList*)0;
}

void restoreStandardEventListener()
{
  gdk_window_set_cursor(GDK_WINDOW(gtkWindow->window),
			cursorPirate);
  if (!exposureIsBlocked)
    {
      DBG_fprintf(stderr, "Visu window GTK : connecting exposure callback.\n");
      g_signal_handler_unblock(G_OBJECT(gtkWindow),
			       exposureEventId);
    }
}
void stopStandardEventListener()
{
  if (!exposureIsBlocked)
    {
      DBG_fprintf(stderr, "Visu window GTK : disconnecting exposure callback.\n");
      g_signal_handler_block(G_OBJECT(gtkWindow),
			     exposureEventId);
    }
}

void addExposureEvent(ActionFunc action)
{
  standardCallback.action = action;
  DBG_fprintf(stderr, "Visu window GTK : connecting exposure callback.\n");
  if (exposureIsBlocked)
    g_signal_handler_unblock(G_OBJECT(gtkWindow),
			     exposureEventId);
  exposureIsBlocked = 0;
}
void removeExposureEvent()
{
  standardCallback.action = (ActionFunc)0;
  DBG_fprintf(stderr, "Visu window GTK : disconnecting exposure callback.\n");
  if (!exposureIsBlocked)
    g_signal_handler_block(G_OBJECT(gtkWindow),
			   exposureEventId);
  exposureIsBlocked = 1;
}

gboolean onKillEvent(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  DBG_fprintf(stderr, "Visu window GTK : delete or destroy event cancelled.\n");
  return TRUE;
}

int initRenderingWindow(char* windowRef, char* iconRef, char* window_name,
			char* class_name, int width, int height)
{
  GdkPixbuf *iconPixBuf;
  GtkWidget *backLogo;
  int res;
#if WIN32 == 1
  gpointer windowHandler;
  GdkDrawable *impl;
#endif

  DBG_fprintf(stderr, "Visu window GTK : Building rendering window.\n");
  gtkWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(gtkWindow), windowRef);
  gtk_window_set_wmclass(GTK_WINDOW(gtkWindow), window_name, class_name);
  gtk_window_set_default_size(GTK_WINDOW(gtkWindow), width, height);
  
  iconPixBuf = gdk_pixbuf_new_from_xpm_data((const char**)icone_observe_xpm);
  gtk_window_set_icon(GTK_WINDOW(gtkWindow), iconPixBuf);

  /* the GDK_POINTER_MOTION_HINT_MASK is necessary to let the
     program redraw the window before listening to motion-event again. */
  gtk_widget_set_events(gtkWindow, GDK_BUTTON_PRESS_MASK |
			GDK_SCROLL_MASK |
			GDK_KEY_PRESS_MASK |
			GDK_POINTER_MOTION_MASK |
			GDK_POINTER_MOTION_HINT_MASK |
			GDK_EXPOSURE_MASK);

  g_signal_connect(G_OBJECT(gtkWindow), "delete-event",
		   G_CALLBACK(onKillEvent), (gpointer)0);
  g_signal_connect(G_OBJECT(gtkWindow), "destroy-event",
		   G_CALLBACK(onKillEvent), (gpointer)0);
  exposureEventId = g_signal_connect(G_OBJECT(gtkWindow), "expose-event",
				     G_CALLBACK(onExposure),
				     (gpointer)&standardCallback);

  gtk_widget_show(gtkWindow);

  backLogo = create_pixmap(gtkWindow, "logo_grey.png");
  gtk_container_add(GTK_CONTAINER(gtkWindow), backLogo);
  gtk_widget_show(backLogo);

  cursorPirate = gdk_cursor_new(GDK_PIRATE);
  cursorRotate = gdk_cursor_new(GDK_BOX_SPIRAL);
  cursorWatch = gdk_cursor_new(GDK_WATCH);
  cursorPointer = gdk_cursor_new(GDK_DOTBOX);

  gdk_window_set_cursor(GDK_WINDOW(gtkWindow->window), cursorPirate);

  res = 0;
#if X11 == 1
  res = setDisplay((gpointer)gdk_x11_drawable_get_xdisplay(GDK_DRAWABLE(gtkWindow->window)));
  res = setScreen(gdk_x11_screen_get_screen_number(gdk_drawable_get_screen(GDK_DRAWABLE(gtkWindow->window)))) || res;
  XWindowId = gdk_x11_drawable_get_xid(GDK_DRAWABLE(gtkWindow->window));
  res = setWindowHandler((gpointer)&XWindowId) || res;
#endif
#if WIN32 == 1
  impl = ((GdkWindowObject*)gtkWindow->window)->impl;
  windowHandler = &((GdkDrawableImplWin32*)impl)->handle;
  res = setWindowHandler(windowHandler);
  res = initDisplay() || res;
#endif

  res = initOpenGLContext() || res;

  /* Now that the glx tunnel has been added, we need
     to specify again that we want a backing store because until now
     the backing store is only for the X window (and thus is black)
     but not for the glx screen. */
  startBackingStore();

  g_signal_handler_block(G_OBJECT(gtkWindow),
			 exposureEventId);
  exposureIsBlocked = 1;

  DBG_fprintf(stderr, "Visu window GTK : Building OK.\n");
  return 1;
}

/* gpointer getWindowHandler(gpointer window) */
/* { */
/*   gpointer windowHandler; */
/*   GdkDrawable *impl; */

/*   if (!window) */
/*     window = gtkWindow; */
/*   impl = ((GdkWindowObject*)((GtkWidget*)window)->window)->impl; */
/* #if X11 == 1 */
/*   windowHandler = &((GdkDrawableImplX11*)impl)->xid; */
/* #endif */
/* #if WIN32 == 1 */
/*   windowHandler = &(HWND)((GdkDrawableImplWin32*)impl)->handle; */
/* #endif */

/*   return windowHandler; */
/* } */


void checkWindowSize(unsigned int *width, unsigned int *height)
{
  gtk_window_get_size(GTK_WINDOW(gtkWindow), (gint*)width, (gint*)height);
}

void raiseRenderWindow()
{
  gdk_window_raise(GDK_WINDOW(gtkWindow->window));
}

void rename_window(char *nom)
{
  gtk_window_set_title(GTK_WINDOW(gtkWindow), nom);
}
