/*   EXTRAITS DE LA LICENCE
	Copyright CEA, contributeurs : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)
  
	Adresse ml :
	BILLARD, non joignable par ml ;
	CALISTE, damien P caliste AT cea P fr.

	Ce logiciel est un programme informatique servant  visualiser des
	structures atomiques dans un rendu pseudo-3D. 

	Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
	respectant les principes de diffusion des logiciels libres. Vous pouvez
	utiliser, modifier et/ou redistribuer ce programme sous les conditions
	de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
	sur le site "http://www.cecill.info".

	Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
	pris connaissance de la licence CeCILL, et que vous en avez accept les
	termes (cf. le fichier Documentation/licence.fr.txt fourni avec ce logiciel).
*/

/*   LICENCE SUM UP
	Copyright CEA, contributors : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)

	E-mail address:
	BILLARD, not reachable any more ;
	CALISTE, damien P caliste AT cea P fr.

	This software is a computer program whose purpose is to visualize atomic
	configurations in 3D.

	This software is governed by the CeCILL  license under French law and
	abiding by the rules of distribution of free software.  You can  use, 
	modify and/ or redistribute the software under the terms of the CeCILL
	license as circulated by CEA, CNRS and INRIA at the following URL
	"http://www.cecill.info". 

	The fact that you are presently reading this means that you have had
	knowledge of the CeCILL license and that you accept its terms. You can
	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
*/
#include "visu_commandLine.h"

#include <stdlib.h>
#include <getopt.h>
#include <stdio.h>

#include <renderingMethods/renderingSpin.h>

char *argFileName;

char *argSpinFileName;

char *argExportFileName;

int withGtk;

int xWindowWidth, xWindowHeight;

void printInfoMessage()
{
  fprintf(stdout, _("V_Sim is a software to visualize atomic structures with"
		    " OpenGl rendering.\n\n"));
  fprintf(stdout, _("usage: v_sim [-h|-e file|-g size|-s file|-m] [fileToRender]\n"));
  fprintf(stdout, _("-h, --help             show this little help.\n"));
  fprintf(stdout, _("-e, --export file      made an image from the fileToRender\n"));
  fprintf(stdout, _("                       argument. The format is specified through\n"));
  fprintf(stdout, _("                       the extension of the file parameter.\n"));
  fprintf(stdout, _("-g, --geometry size    specify the size of the rendering window,\n"));
  fprintf(stdout, _("                       the size parameter must have the following\n"));
  fprintf(stdout, _("                       format : <width>x<height> with positive non\n"));
  fprintf(stdout, _("                       null values.\n"));
  fprintf(stdout, _("-s, --spin-file file   use the given file as a spin indicator. If this\n"));
  fprintf(stdout, _("                       option is used, V_Sim switches automatically to\n"));
  fprintf(stdout, _("                       spin rendering whatever method is specified in\n"));
  fprintf(stdout, _("                       the parameter file.\n"));
  fprintf(stdout, _("-m, --hiding-mode      when loading a spin file, hides element containing\n"));
  fprintf(stdout, _("                       null module values.\n"));
}

int parseCommandLine(int argc, char **argv)
{
  struct option long_options[] = {
    {"export", 1, 0, 'e'}, /* export : name, 1 : arg required */
    {"help", 0, 0, 'h'}, /* help : name, 0 : arg not required */
    {"geometry", 1, 0, 'g'}, /* help : name, 0 : arg not required */
    {"spin-file", 1, 0, 's'},
    {"hiding-mode", 0, 0, 'm'},
    {0, 0, 0, 0}
  };
  int res;
  int option_index;


  argFileName = (char*)0;
  argSpinFileName = (char *)0;
  argExportFileName = (char*)0;
  xWindowWidth = 600;
  xWindowHeight = 600;

  withGtk = 1;

  option_index = 0;
  while (1)
    {
      res = getopt_long (argc, argv, "e:hg:s:m",
			 long_options, &option_index);

      /* test if there's no more option available. */
      if (res == -1)
	break;

      switch (res)
	{
	case 'e':
	  DBG_fprintf(stderr, "Visu Command : option '%s' found with arg '%s'.\n",
		      long_options[option_index].name, optarg);
	  if (!optarg)
	    {
	      fprintf(stderr, _("OPTION ERROR! The option 'export' needs a parameter.\n"));
	      exit(1);
	    }
	  else
	    {
	      argExportFileName = malloc(sizeof(char) * (strlen(optarg) + 1));
	      if (!argExportFileName)
		{
		  allocationProblems();
		  exit(1);
		}
	      strcpy(argExportFileName, optarg);
	    }
	  /* This option does not need gtk. */
	  withGtk = 0;
	  break;
	case 'h':
	  DBG_fprintf(stderr, "Visu Command : option '%s' found.\n",
		      long_options[option_index].name);
	  printInfoMessage();
	  exit(0);
	  break;
	case 'g':
	  DBG_fprintf(stderr, "Visu Command : set the geometry of the X window (%s).\n", optarg);
	  res = sscanf(optarg, "%dx%d", &xWindowWidth, &xWindowHeight);
	  if (res != 2 || xWindowWidth <= 0 || xWindowHeight <=0)
	    {
	      fprintf(stderr, _("WARNING! Wrong format for geometry"
				" option (<width>x<height> awaited).\n"));
	      xWindowWidth = 600;
	      xWindowHeight = 600;
	      break;
	    }
	  break;
	case 's':
	  DBG_fprintf(stderr, "Visu Command : set the filenane for"
		      " spin rendering to '%s'.\n", optarg);
	  if (!optarg)
	    {
	      fprintf(stderr, _("OPTION ERROR! The option 'spin-file' needs a parameter.\n"));
	      exit(1);
	    }
	  argSpinFileName = g_strdup(optarg);
	  break;
	case 'm':
	  DBG_fprintf(stderr, "Visu Command : set hiding-mode to TRUE in spin rendering.\n");
	  *(int *)rspin_global_resource_get("hiding_mode") = 1;
	  break;
	default:
	  fprintf (stderr, _("OPTION ERROR! unknown option.\n"));
	  printInfoMessage();
	  exit(0);
	}
    }

  if (argc - optind == 1)
    {
      DBG_fprintf(stderr, "Visu Command : there is one argument '%s'.\n", argv[optind]);
      argFileName = g_strdup(argv[optind]);
    }
  else if (argc - optind == 0)
    return 0;
  else
    fprintf (stderr, _("OPTION ERROR! This program allows only up to one argument.\n"));

  return 0;
}

char* commandLineGet_ArgFilename()
{
  return argFileName;
}

char* commandLineGet_ArgSpinFileName()
{
  return argSpinFileName;
}

int commandLineGet_WithGtk()
{
  return withGtk;
}

char* commandLineGet_ExportFileName()
{
  return argExportFileName;
}

void commandLineGet_XWindowGeometry(int *width, int *height)
{
  *width = xWindowWidth;
  *height = xWindowHeight;
}
