/*   EXTRAITS DE LA LICENCE
	Copyright CEA, contributeurs : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)
  
	Adresse ml :
	BILLARD, non joignable par ml ;
	CALISTE, damien P caliste AT cea P fr.

	Ce logiciel est un programme informatique servant  visualiser des
	structures atomiques dans un rendu pseudo-3D. 

	Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
	respectant les principes de diffusion des logiciels libres. Vous pouvez
	utiliser, modifier et/ou redistribuer ce programme sous les conditions
	de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
	sur le site "http://www.cecill.info".

	Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
	pris connaissance de la licence CeCILL, et que vous en avez accept les
	termes (cf. le fichier Documentation/licence.fr.txt fourni avec ce logiciel).
*/

/*   LICENCE SUM UP
	Copyright CEA, contributors : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)

	E-mail address:
	BILLARD, not reachable any more ;
	CALISTE, damien P caliste AT cea P fr.

	This software is a computer program whose purpose is to visualize atomic
	configurations in 3D.

	This software is governed by the CeCILL  license under French law and
	abiding by the rules of distribution of free software.  You can  use, 
	modify and/ or redistribute the software under the terms of the CeCILL
	license as circulated by CEA, CNRS and INRIA at the following URL
	"http://www.cecill.info". 

	The fact that you are presently reading this means that you have had
	knowledge of the CeCILL license and that you accept its terms. You can
	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
*/
#include "visu_data.h"


#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include <GL/gl.h>
#include <GL/glu.h> 

#include "visu_object.h"
#include "visu_tools.h"
#include "visu_extension.h"
#include "visu_rendering.h"
#include "opengl.h"


struct nodesProperties_struct
{
  /* This table has the same size and structure
     than the nodes table of a #VisuData object. */
  gpointer **data;

  /* These store the allocated size of the data table. */
  int sizeElement;
  int *sizeNode;

  /* This method is called for each stored token if not NULL when the table is freed. */
  GDestroyNotify freeTokenFunc;
};
void freeNodesPropertiesStruct(gpointer data);

void visuData_rebuildList(GObject* obj, gpointer data);

/* This is the maximum size (in drawing mode) of all the VisuElements
   that have at least one VisuNode registered. */
/* float maxDrawingSizeOfVisuElements; */

/* This function is called when a renderingChanged signal is
   emitted. It frees all the VisuNode. */
void freeAllNodesOnRenderingChanged(GObject *obj, gpointer data);



int initVisuData()
{
  char *name = "Visu Data";
  /* Description of the visuModule visu_data. */
  char *description = _("Draw all the nodes.");
  OpenGLExtension *extensionData;

  /* when a new rendering method is loaded, all
     VisuNode are freed. */
  /* Connect the signal emitted by visu when
     a new rendering method is selected. */
  g_signal_connect(G_OBJECT(visu), "renderingChanged",
		   G_CALLBACK(freeAllNodesOnRenderingChanged), (gpointer)0);

  /* Get an OpenGL identifier to store all the materials. */
  identifierAllNodes = createNewObjectList(NMAX_TP + 1);
  extensionData = OpenGLExtension_new(name, description,
				      identifierAllNodes, visuData_rebuildList);
  OpenGLExtensionSet_priority(extensionData, OPENGL_EXTENSION_PRIORITY_FIRST);
  extensionData->used = 1;
  registerOpenGLExtension(extensionData);
  /* Initialse the maximum size occupied by a VisuElement. */
/*   setMaxExtens(0.); */

  return 1;
}


/* This function is called when a renderingChanged signal is
   emitted. It frees all the VisuNode. */
void freeAllNodesOnRenderingChanged(GObject *obj, gpointer data)
{
  if (!currentVisuData)
    return;

  visuDataFree(currentVisuData);
  currentVisuData = (VisuData*)0;
  /* Since currentData is set to NULL, we emit this signal. */
  g_signal_emit (visu, VISU_GET_CLASS (visu)->dataNotReadyForRendering_signal_id,
		 0 /* details */,
		 NULL);

  /* The list of VisuElement to draw is reduced to zero. */
  setObjectList(0);
  DBG_fprintf(stderr, "Visu Data : freeing all data on rendering change, OK.\n");

  /* Since the rendering method is changed and all the
     VisuNodes are freed, maxDrawingSizeOfVisuElements must
     be lowered to zero. */
/*   setMaxExtens(0.); */
}






/***************/
/* OpenGL Part */
/***************/

void visuData_createNodes(VisuData *data, VisuElement *ele)
{
  RenderingMethod *currentRenderingMethod;
  int *nEle;
  int j;
  int mat;

  if (!data || !ele)
    {
      fprintf(stderr, "WARNING! 'visuData_createNodes' has been"
	      " called with a null 'data' or a null 'ele' argument.\n");
      return;
    }
  nEle = (int*)g_hash_table_lookup(data->fromVisuElementToInt, (gpointer)ele);
  if (!nEle)
    {
      fprintf(stderr, "WARNING! 'visuData_createNodes' has been"
	      " called but the given VisuElement %p-'%s' is not used.\n",
	      (gpointer)ele, ele->name);
      return;
    }
  currentRenderingMethod = getRenderingMethodInUse();
  if (!currentRenderingMethod)
    {
      fprintf(stderr, "WARNING! 'visuData_createNodes' has been called but no"
	      " rendering method has been selected.\n");
      return;
    }
  if (!currentRenderingMethod->createNode)
    {
      fprintf(stderr, "WARNING! 'visuData_createNodes' has been called but"
	      " the current rendering method has no createNode method.\n");
      return;
    }

  if (!ele->materialIsUpToDate)
    visuElement_createMaterial((gpointer)0, (gpointer)ele, (gpointer)0);
  
  glNewList(identifierAllNodes + *nEle + 1, GL_COMPILE);
  if (ele->rendered)
    {
      DBG_fprintf(stderr, "Visu Data : creating glObjectList of nodes for '%s' - %d.\n",
		  ele->name, ele->openGLIdentifier);
      /* Set the material by default if available. */
      mat = visuElementGet_identifierMaterial(ele);
      if (!mat)
	fprintf(stderr, _("WARNING! Can't find the material identifier for the"
			  " element '%s'.\n"), ele->name);
      else
	glCallList(mat);
      for(j = 0; j < currentVisuData->numberOfStoredNodes[*nEle]; j++)
	{
	  if (currentVisuData->nodes[*nEle][j].rendered)
	    {
	      glLoadName((GLuint)currentVisuData->nodes[*nEle][j].number);
	      currentRenderingMethod->createNode(&currentVisuData->nodes[*nEle][j], ele);
	    }
	}
    }
  else
    DBG_fprintf(stderr, "Visu Data : skipping glObjectList of nodes for '%s' - %d.\n",
		ele->name, ele->openGLIdentifier);
  glEndList();
}

void visuData_createAllNodes()
{
  int i, j;
  int mat, obj;
#if DEBUG == 1
  GTimer *timer;
  gulong fractionTimer;
#endif

  if (!currentVisuData)
    {
      DBG_fprintf(stderr, "Visu Data : A file has just been loaded, but the"
		  " currentVisuData variable is empty!\n");
      return;
    }

  DBG_fprintf(stderr, "Visu Data : 'visuData_createAllNodes' called.\n");

#if DEBUG == 1
  timer = g_timer_new();
  g_timer_start(timer);
#endif

  for (i = 0; i < currentVisuData->ntype; i++)
    visuData_createNodes(currentVisuData, currentVisuData->fromIntToVisuElement[i]);

  glNewList(identifierAllNodes, GL_COMPILE);
  glLineWidth(1);
  for (i = 0; i < currentVisuData->ntype; i++)
    {
      DBG_fprintf(stderr, "Visu Data : add list %d.\n", identifierAllNodes + i + 1);
/*       if (currentVisuData->fromIntToVisuElement[i]->rendered) */
      glCallList(identifierAllNodes + i + 1);
    }
  glEndList();
#if DEBUG == 1
  g_timer_stop(timer);
  fprintf(stderr, "Visu Data : lists built in %gs.\n", g_timer_elapsed(timer, &fractionTimer)/1e-6);
  g_timer_destroy(timer);
#endif
}
void visuData_createAllElements(VisuData *data)
{
  int i;
  RenderingMethod *method;

  if (!data)
    {
      fprintf(stderr, "WARNING! 'visuData_createAllElements' has"
	      " been called with a null 'data' argument.\n");
      return;
    }
  DBG_fprintf(stderr, "Visu Data : creating OpenGl elements for"
	      " all VisuElement used in given VisuData %p.\n", (gpointer)data);
  method = getRenderingMethodInUse();
  if (!method)
    {
      fprintf(stderr, "WARNING! 'visuData_createAllElements' has"
	      " been called but there is no selected rendering method.\n");
      return;
    }
  for (i = 0; i < data->ntype; i++)
    method->createElement((VisuElement*)data->fromIntToVisuElement[i]);
}

void deleteAllNodes(GObject *obj, gpointer data)
{
  setObjectList(0);
}

int visuDataGet_objectList()
{
  return identifierAllNodes;
}

void visuData_rebuildList(GObject *obj, gpointer data)
{
  int i;

  DBG_fprintf(stderr, "Visu Data : rebuilding object list from visuData.\n");
  for (i = 0; i < currentVisuData->ntype; i++)
    visuElement_createMaterial((gpointer)currentVisuData->fromIntToVisuElement[i]->name,
			       (gpointer)currentVisuData->fromIntToVisuElement[i],
			       data);
  visuData_createAllElements(currentVisuData);
  visuData_createAllNodes();
}




/************************************************/
/* Methods to deal with the VisuData structure. */
/************************************************/

/**
 * visuDataNew:
 *
 * This creates an empty #VisuData object.
 *
 * Returns: a newly created #VisuData object.
 */
VisuData* visuDataNew()
{
  VisuData *data;
  int i;
  
  data = malloc(sizeof(VisuData));
  if (!data)
    return (VisuData*)0;

  DBG_fprintf(stderr, "Visu Data : creating new VisuData %p", (gpointer)data);

  data->ntype= 0;
  data->files = (GHashTable*)0;
  data->fromVisuElementToInt = (GHashTable*)0;
  data->fromNumberToVisuNode = (VisuNode**)0;
  data->fromIntToVisuElement = (VisuElement**)0;
  data->numberOfStoredNodes = (int*)0;
  data->numberOfNodes = (int*)0;
  data->nbOfAllStoredNodes = 0;
  data->nodes = (VisuNode**)0;

  for (i = 0; i < 3; i++)
    data->translation[i] = 0.;
  for (i = 0; i < 6; i++)
    data->boxGeometry[i] = 0.;
  data->isBoxPeriodic = FALSE;

  data->files = g_hash_table_new_full(g_int_hash, g_int_equal, freeInt, freeString);
  if (!data->files)
    {
      DBG_fprintf(stderr, " ... Error!\nVisu Data : allocate files failed.\n");
      visuDataFree(data);
      return (VisuData*)0;
    }
  data->nodesProperties = g_hash_table_new_full(g_str_hash, g_str_equal,
						NULL, freeNodesPropertiesStruct);
  if (!data->nodesProperties)
    {
      DBG_fprintf(stderr, " ... Error!\nVisu Data : allocate nodesProperties failed.\n");
      visuDataFree(data);
      return (VisuData*)0;
    }
  data->setColor = (setColorFunc)0;
  data->properties = g_hash_table_new_full(g_str_hash, g_str_equal,
					   NULL, freeGPointer);
  if (!data->properties)
    {
      DBG_fprintf(stderr, " ... Error!\nVisu Data : allocate properties failed.\n");
      visuDataFree(data);
      return (VisuData*)0;
    }
  data->fromVisuElementToInt = g_hash_table_new_full(g_direct_hash, g_direct_equal,
						     NULL, freeInt);
  if (!data->fromVisuElementToInt)
    {
      DBG_fprintf(stderr, " ... Error!\nVisu Data : allocate fromVisuElementToInt failed.\n");
      visuDataFree(data);
      return (VisuData*)0;
    }
  DBG_fprintf(stderr, " ... OK\n");

  return data;
}

int visuDataSet_population(VisuData *data, int nbOfTypes, int* nbOfNodesPerVisuElement,
			   VisuElement **visuElementUsed)
{
  int i, j;
  int *pos;
  int nb;

  DBG_fprintf(stderr, "Visu Data : populating VisuData %p", (gpointer)data);

  data->fromIntToVisuElement = malloc(sizeof(VisuElement*) * nbOfTypes);
  if (!data->fromIntToVisuElement)
    {
      DBG_fprintf(stderr, " ... Error!\nVisu Data : allocate fromIntToVisuElement failed.\n");
      return 0;
    }

  data->ntype = nbOfTypes;
  if (data->ntype <= 0)
    {
      DBG_fprintf(stderr, " ... Error!\nVisu Data : ntype <= 0 failed.\n");
      return 0;
    }
  data->nodes = malloc(sizeof(VisuNode*) * data->ntype);
  if (!data->nodes)
    {
      DBG_fprintf(stderr, " ... Error!\nVisu Data : allocate nodes failed.\n");
      return 0;
    }
  data->numberOfNodes = malloc(sizeof(int) * data->ntype);
  if (!data->numberOfNodes)
    {
      DBG_fprintf(stderr, " ... Error!\nVisu Data : allocate numberOfNodes failed.\n");
      return 0;
    }
  data->numberOfStoredNodes = malloc(sizeof(int) * data->ntype);
  if (!data->numberOfStoredNodes)
    {
      DBG_fprintf(stderr, " ... Error!\nVisu Data : allocate numberOfStoredNodes failed.\n");
      return 0;
    }

  nb = 0;
  for(i = 0; i < data->ntype; i++)
    {
      if (nbOfNodesPerVisuElement[i] <= 0)
	{
	  DBG_fprintf(stderr, " ... Error!\nVisu Data : nb nodes for '%s' <= 0.\n",
		      data->fromIntToVisuElement[i]->name);
	  return 0;
	}
      data->nodes[i] = malloc(sizeof(VisuNode) * nbOfNodesPerVisuElement[i]);
      if (!data->nodes[i])
	{
	  DBG_fprintf(stderr, " ... Error!\nVisu Data : allocate nodes[%d] failed.\n", i);
	  return 0;
	}
      nb += nbOfNodesPerVisuElement[i];
      data->numberOfNodes[i] = nbOfNodesPerVisuElement[i];
      data->numberOfStoredNodes[i] = 0;
      pos = malloc(sizeof(int));
      if (!pos)
	{
	  DBG_fprintf(stderr, " ... Error!\nVisu Data : allocate pos (%d) failed.\n", i);
	  return 0;
	}
      *pos = i;
      g_hash_table_insert(data->fromVisuElementToInt, (gpointer)visuElementUsed[i], (gpointer)pos);
      data->fromIntToVisuElement[i] = visuElementUsed[i];
    }
  data->nbOfAllStoredNodes = 0;
  data->fromNumberToVisuNode = malloc(sizeof(VisuNode*) * nb);
  if (!data->fromNumberToVisuNode)
    {
      DBG_fprintf(stderr, " ... Error!\nVisu Data : allocate fromNumberToVisuNode failed.\n");
      return 0;
    }

  DBG_fprintf(stderr, " ... OK\n");
  return 1;
}
/**
 * visuDataFree:
 * @data: a VisuData to be freed.
 *
 * This method frees all the allocated memory of the VisuData
 * specified as parameter.
 */
void visuDataFree(VisuData *data)
{
  int i;

  if (!data)
    return;

  DBG_fprintf(stderr, "Visu Data : freeing the VisuData %p ...\n", (gpointer)data);

  if (data->files)
    g_hash_table_destroy(data->files);

  if (data->fromVisuElementToInt)
    g_hash_table_destroy(data->fromVisuElementToInt);
  if (data->fromIntToVisuElement)
    free(data->fromIntToVisuElement);

  if (data->fromNumberToVisuNode)
    free(data->fromNumberToVisuNode);
  
  if (data->nodes)
    {
      for(i = 0; i < data->ntype; i++)
	free(data->nodes[i]);
      free(data->nodes);
    }
  if (data->numberOfNodes)
    free(data->numberOfNodes);
  if (data->numberOfStoredNodes)
    free(data->numberOfStoredNodes);

  if (data->nodesProperties)
    g_hash_table_destroy(data->nodesProperties);
  if (data->properties)
    g_hash_table_destroy(data->properties);

  free(data);
  DBG_fprintf(stderr, "Visu Data : freeing ... OK.\n");
}
void visuDataFree_population(VisuData *data)
{
  int i;

  if (!data)
    return;

  DBG_fprintf(stderr, "Visu Data : freeing the population of VisuData %p ...\n", (gpointer)data);

  if (data->fromVisuElementToInt)
    g_hash_table_destroy(data->fromVisuElementToInt);
  data->fromVisuElementToInt = g_hash_table_new_full(g_direct_hash, g_direct_equal,
						     NULL, freeInt);
  if (data->fromIntToVisuElement)
    free(data->fromIntToVisuElement);
  data->fromIntToVisuElement = (VisuElement**)0;

  if (data->fromNumberToVisuNode)
    free(data->fromNumberToVisuNode);
  data->fromNumberToVisuNode = (VisuNode**)0;
  
  if (data->nodes)
    {
      for(i = 0; i < data->ntype; i++)
	free(data->nodes[i]);
      free(data->nodes);
    }
  data->nodes = (VisuNode**)0;

  if (data->numberOfNodes)
    free(data->numberOfNodes);
  data->numberOfNodes = (int*)0;
  if (data->numberOfStoredNodes)
    free(data->numberOfStoredNodes);
  data->numberOfStoredNodes = (int*)0;

  if (data->nodesProperties)
    g_hash_table_destroy(data->nodesProperties);
  data->nodesProperties = g_hash_table_new_full(g_str_hash, g_str_equal,
						NULL, freeNodesPropertiesStruct);

  DBG_fprintf(stderr, "Visu Data : freeing ... OK.\n");
}

void visuDataAdd_file(VisuData *data, gchar* file, int kind)
{
  gchar *cpFile;
  int *key;

  if (!data)
    {
      fprintf(stderr, "WARNING! No data parameter specified in"
	      " the call of 'visuDataAddFileFunc'.\n");
      return;
    }
  cpFile = g_strdup(file);
  key = malloc(sizeof(int));
  *key = kind;
  g_hash_table_insert(data->files, key, (gpointer)cpFile);
  DBG_fprintf(stderr, "Visu Data : adding '%s' filename (key: %d) to the VisuData %p.\n",
	      cpFile, kind, (gpointer)data);
}
void visuDataRemove_allFiles(VisuData *data)
{
  if (!data)
    {
      fprintf(stderr, "WARNING! No data parameter specified in"
	      " the call of 'visuDataAddFileFunc'.\n");
      return;
    }
  g_hash_table_destroy(data->files);
  data->files = g_hash_table_new_full(g_int_hash, g_int_equal, freeInt, freeString);
}
gchar* visuDataGet_file(VisuData *data, int kind)
{
  if (!data)
    {
      fprintf(stderr, "WARNING! No data parameter specified in"
	      " the call of 'visuDataAddFileFunc'.\n");
      return (gchar*)0;
    }
  DBG_fprintf(stderr, "Visu Data : retrieve filename with key %d from the VisuData %p.\n",
	      kind, (gpointer)data);
  return g_hash_table_lookup(data->files, (gpointer)&kind);
}


int visuNodeSet_visibility(VisuNode* node, gboolean visibility)
{
  if (!node)
    {
      fprintf(stderr, "WARNING! '%s' has been called with a null value for parameter '%s'.\n",
	      "visuNodeSet_visibility", "node");
      return 0;
    }
  if (node->rendered == (int)visibility)
    return 0;
  node->rendered = (int)visibility;
  return 1;
}
gboolean visuNodeGet_visibility(VisuNode* node)
{
  if (!node)
    {
      fprintf(stderr, "WARNING! '%s' has been called with a null value for parameter '%s'.\n",
	      "visuNodeGet_visibility", "node");
      return FALSE;
    }
  if (node->rendered)
    return TRUE;
  else
    return FALSE;
}

void visuNodeSetThetaPhi(VisuNode *node, double theta, double phi)
{
  if(!node)
    {
      fprintf(stderr, "WARNING! '%s' has been called with a null value for parameter '%s'.\n",
	      "visuNodeSet_newValues", "node");
      return;
    }
  node->theta = theta;
  node->phi = phi;
}

void visuNodeSet_newValues(VisuNode *node, double x, double y, double z, 
			   double theta, double phi, int number,
			   int rendered, int posEle, int posNod)
{
  int k;

  if (!node)
    {
      fprintf(stderr, "WARNING! '%s' has been called with a null value for parameter '%s'.\n",
	      "visuNodeSet_newValues", "node");
      return;
    }
  node->x = x;
  node->y = y;
  node->z = z;
  node->phi = phi;
  node->theta = theta;
  for (k = 0; k < 3; k++)
    node->translation[k] = 0.;
  node->number = number;
  node->rendered = rendered;
  node->posElement = posEle;
  node->posNode = posNod;
}

gboolean visuData_compareElements(VisuData *data1, VisuData *data2)
{
  int i;

  g_return_val_if_fail(data1 && data2, FALSE);

  DBG_fprintf(stderr, "Visu Data : comparing composition of '%p' and '%p'.\n",
	      (gpointer)data1, (gpointer)data2);
  if (data1->ntype != data2->ntype)
    return FALSE;

  for (i = 0; i< data1->ntype; i++)
    if (!g_hash_table_lookup(data2->fromVisuElementToInt,
			     (gpointer)data1->fromIntToVisuElement[i]))
      return FALSE;

  for (i = 0; i< data2->ntype; i++)
    if (!g_hash_table_lookup(data1->fromVisuElementToInt,
			     (gpointer)data2->fromIntToVisuElement[i]))
      return FALSE;

  return TRUE;
}
void visuDataSet_changeElementFlag(VisuData *data, gboolean changeElement)
{
  gboolean *val;

  g_return_if_fail(data);

  /* Test if already exists */
  val = (gboolean*)visuDataGet_property(data, "changeElementListFlag");
  if (!val)
    {
      val = g_malloc(sizeof(gboolean));
      visuDataSet_property(data, "changeElementListFlag", val);
    }
  *val = changeElement;
}
gboolean visuDataGet_changeElementFlag(VisuData *data)
{
  gboolean *val;

  g_return_val_if_fail(data, FALSE);

  val = (gboolean*)visuDataGet_property(data, "changeElementListFlag");
  if (val)
    return *val;
  else
    return FALSE;
}


void visuDataSet_ColorFunc(VisuData *data, setColorFunc func)
{
  if (!data)
    {
      fprintf(stderr, _("WARNING! 'visuDataSet_ColorFunc' has been called with"
			" a null data parameter.\n"));
      return;
    }
  DBG_fprintf(stderr, "Visu Data : set the color method to %d.\n",
	      GPOINTER_TO_INT(func));
  data->setColor = func;
}

/**
 * visuDataAdd_VisuElement:
 * @data: the #VisuData where to add the new #VisuNode ;
 * @ele: the #VisuElement kind of the new #VisuNode ;
 * @x: its x coordinate ;
 * @y: its y coordinate ;
 * @z: its z coordinate ;
 * @number: its position in the input file (or anything else that can uniqualy
 *          identify this new #VisuNode).
 *
 * This method adds a new #VisuNode to the specified #VisuData. Rotation is set to 0.
 *
 * Returns: 0 if everything is allright.
 */
int visuDataAdd_VisuElement(VisuData *data, VisuElement *ele,
			    double x, double y, double z, int number)
{
  int *pos;

  if (!data)
    return 1;

  pos = (int*)g_hash_table_lookup(data->fromVisuElementToInt, (gpointer)ele);
  if (!pos)
    return 1;

  if (*pos > data->ntype || *pos < 0)
    {
      fprintf(stderr, _("INTERNAL ERROR! The hashtable from VisuElement to position"
			" of the VisuData %d is malformed since the VisuElement '%s' has a"
			" position %d.\n"), GPOINTER_TO_INT(data), ele->name, *pos);
      return 1;
    }
  data->nbOfAllStoredNodes += 1;
  visuNodeSet_newValues(data->nodes[*pos] + data->numberOfStoredNodes[*pos],
			x, y, z, 0., 0., data->nbOfAllStoredNodes, 1, *pos,
			data->numberOfStoredNodes[*pos]);
  data->fromNumberToVisuNode[data->nbOfAllStoredNodes - 1] =
    data->nodes[*pos] + data->numberOfStoredNodes[*pos];
  data->numberOfStoredNodes[*pos] += 1;
  return 0;
}
/**
 * visuDataAdd_VisuElementWithRotation:
 * @data: the #VisuData where to add the new #VisuNode ;
 * @ele: the #VisuElement kind of the new #VisuNode ;
 * @x: its x coordinate ;
 * @y: its y coordinate ;
 * @z: its z coordinate ;
 * @theta: rotation around the z axis;
 * @phi: rotation around the y axis;
 * @number: its position in the input file (or anything else that can uniqualy
 *          identify this new #VisuNode).
 *
 * This method adds a new #VisuNode to the specified #VisuData.
 *
 * Returns: 0 if everything is allright.
 */
int visuDataAdd_VisuElementWithRotation(VisuData *data, VisuElement *ele,
					double x, double y, double z, 
					double theta, double phi, int number)
{
  int *pos;

  if (!data)
    return 1;

  pos = (int*)g_hash_table_lookup(data->fromVisuElementToInt, (gpointer)ele);
  if (!pos)
    return 1;

  if (*pos > data->ntype || *pos < 0)
    {
      fprintf(stderr, _("INTERNAL ERROR! The hashtable from VisuElement to position"
			" of the VisuData %d is malformed since the VisuElement '%s' has a"
			" position %d.\n"), GPOINTER_TO_INT(data), ele->name, *pos);
      return 1;
    }
  data->nbOfAllStoredNodes += 1;
  visuNodeSet_newValues(data->nodes[*pos] + data->numberOfStoredNodes[*pos],
			x, y, z, theta, phi, data->nbOfAllStoredNodes, 1,
			*pos, data->numberOfStoredNodes[*pos]);
  data->fromNumberToVisuNode[data->nbOfAllStoredNodes - 1] =
    data->nodes[*pos] + data->numberOfStoredNodes[*pos];
  data->numberOfStoredNodes[*pos] += 1;
  return 0;
}
/**
 * visuDataAdd_int:
 * @data: the #VisuData where to add the new #VisuNode ;
 * @position: a integer corresponding to the position of
 *            a #VisuElement in the array **nodes in the structure;
 * @x: its x coordinate ;
 * @y: its y coordinate ;
 * @z: its z coordinate ;
 * @number: its position in the input file (or anything else that can uniqualy
 *          identify this new #VisuNode).
 *
 * This method adds a new #VisuNode to the specified #VisuData. Position must be
 * chosen between 0 and (ntype - 1) and corresponds to the position of the array
 * of #VisuNodes of a #VisuElement.
 *
 * Returns: 0 if everything is allright.
 */
int visuDataAdd_int(VisuData *data, int position,
		    double x, double y, double z, int number)
{
  if (!data)
    return 1;

  if (position > data->ntype || position < 0)
    return 1;

  data->nbOfAllStoredNodes += 1;
  visuNodeSet_newValues(data->nodes[position] + data->numberOfStoredNodes[position],
			x, y, z, 0., 0., data->nbOfAllStoredNodes, 1,
			position, data->numberOfStoredNodes[position]);
  data->fromNumberToVisuNode[data->nbOfAllStoredNodes - 1] =
    data->nodes[position] + data->numberOfStoredNodes[position];
  data->numberOfStoredNodes[position] += 1;
  return 0;
}
/**
 * visuDataGet_fromNumber:
 * @number: an integer.
 *
 * This methods retrieves the #VisuNode identified by the integer @number.
 *
 * Returns: the found #VisuNode or NULL if none corresponds to number.
 */
VisuNode* visuDataGet_fromNumber(VisuData *data, int number)
{
  VisuNode *node;

  if (!data)
    return (VisuNode*)0;

  if (number <= 0 || number > data->nbOfAllStoredNodes)
    node = (VisuNode*)0;
  else
    node = data->fromNumberToVisuNode[number - 1];
  
  return node;
}

void visuDataSet_nodeProperty(VisuData* data, VisuNode* node, char* key, gpointer value)
{
  struct nodesProperties_struct *storedData;

  if (!data || !node || !key)
    {
      fprintf(stderr, _("WARNING! 'visuDataSet_nodeProperty' has been called with"
			" a null data/node parameter or a null key parameter.\n"));
      return;
    }
  storedData = (struct nodesProperties_struct*)g_hash_table_lookup(data->nodesProperties,
								   (gpointer)key);
  if (!storedData)
    {
      fprintf(stderr, _("WARNING! the data %p has no nodesProperties"
			" identified by the key '%s'\n"), (gpointer)data, key);
      return;
    }
/*   DBG_fprintf(stderr, "Visu Data : set a new value to the node" */
/* 	      " property called '%s' : %p.\n", key, value); */
  if (storedData->freeTokenFunc)
    storedData->freeTokenFunc(storedData->data[node->posElement][node->posNode]);
  storedData->data[node->posElement][node->posNode] = value;
}
gpointer visuDataGet_nodeProperty(VisuData* data, VisuNode* node, char* key)
{
  struct nodesProperties_struct *storedData;
  
  if (!data || !node || !key)
    {
      fprintf(stderr, _("WARNING! 'visuDataGet_nodeProperty' has been called with"
			" a null data/node parameter or a null key parameter.\n"));
      return (gpointer)0;
    }

  storedData = (struct nodesProperties_struct*)g_hash_table_lookup(data->nodesProperties,
								   (gpointer)key);
  if (!storedData)
    return (gpointer*)0;
  else
    return storedData->data[node->posElement][node->posNode];
}
void freeNodesPropertiesStruct(gpointer data)
{
  struct nodesProperties_struct *storedData;
  int i, j;

  DBG_fprintf(stderr, "Visu Data : freeing the allocated area of"
	      " a nodes property.\n");
  storedData = (struct nodesProperties_struct *)data;
  if (storedData->freeTokenFunc)
    for (i = 0; i < storedData->sizeElement; i++)
      for (j = 0; j < storedData->sizeNode[i]; j++)
	if (storedData->data[i][j])
	  storedData->freeTokenFunc(storedData->data[i][j]);
  for (i = 0; i < storedData->sizeElement; i++)
    free(storedData->data[i]);
  free(storedData->data);
  free(storedData->sizeNode);
}
void visuDataRemove_nodeProperty(VisuData* data, char* key)
{
  if (!data || !key)
    {
      fprintf(stderr, _("WARNING! 'visuDataRemove_nodePropertry' has been called with"
			" a null data parameter or a null key parameter.\n"));
      return;
    }
  g_hash_table_remove(data->nodesProperties, key);
  DBG_fprintf(stderr, "Visu Data : removing the nodes property called '%s'.\n", key);
}
void visuDataAdd_nodeProperty(VisuData* data, char* key, GDestroyNotify freeFunc)
{
  struct nodesProperties_struct *storedData;
  int i, j;
  
  if (!data || !key)
    {
      fprintf(stderr, _("WARNING! 'visuDataAdd_nodePropertry' has been called with"
			" a null data parameter or a null key parameter.\n"));
      return;
    }
  storedData = g_hash_table_lookup(data->nodesProperties, key);
  if (storedData)
    {
      fprintf(stderr, _("WARNING! The key '%s' is already associated.\n"), key);
      return;
    }
  DBG_fprintf(stderr, "Visu Data : adding a new nodes property, called '%s'.\n", key);
  storedData = malloc(sizeof(struct nodesProperties_struct));
  if (!storedData)
    {
      allocationProblems();
      exit(1);
    }
  storedData->freeTokenFunc = freeFunc;
  storedData->data = malloc(sizeof(gpointer*) * data->ntype);
  if (!storedData->data)
    {
      allocationProblems();
      exit(1);
    }
  storedData->sizeElement = data->ntype;
  storedData->sizeNode = malloc(sizeof(int) * data->ntype);
  if (!storedData->sizeNode)
    {
      allocationProblems();
      exit(1);
    }
  for (i = 0; i < data->ntype; i++)
    {
      storedData->data[i] = malloc(sizeof(gpointer) * data->numberOfNodes[i]);
      if (!storedData->data[i])
	{
	  allocationProblems();
	  exit(1);
	}
      storedData->sizeNode[i] = data->numberOfNodes[i];
      for (j = 0; j < data->numberOfNodes[i]; j++)
	storedData->data[i][j] = (gpointer)0;
    }
  g_hash_table_insert(data->nodesProperties, (gpointer)key, (gpointer)storedData);
}

void visuDataSet_property(VisuData* data, char* key, gpointer value)
{
  if (!data || !key)
    {
      fprintf(stderr, "WARNING! 'visuDataSet_property' has been called with"
	      " a null data parameter or a null key parameter.\n");
      return;
    }
  DBG_fprintf(stderr, "Visu Data : set a new value to the "
	      " property called '%s' : %p.\n", key, value);
  g_hash_table_insert(data->properties, (gpointer)key, (gpointer)value);
}
gpointer visuDataGet_property(VisuData* data, char* key)
{
  if (!data || !key)
    {
      fprintf(stderr, "WARNING! 'visuDataGet_property' has been called with"
	      " a null data parameter or a null key parameter.\n");
      return (gpointer)0;
    }
  DBG_fprintf(stderr, "Visu Data : get the value associated to the "
	      " property called '%s'.\n", key);
  return g_hash_table_lookup(data->properties, (gpointer)key);
}
void visuDataRemove_property(VisuData* data, char* key)
{
  if (!data || !key)
    {
      fprintf(stderr, "WARNING! 'visuDataRemove_property' has been called with"
	      " a null data parameter or a null key parameter.\n");
      return;
    }
  g_hash_table_remove(data->properties, (gpointer)key);
}


float* visuDataGet_XYZtranslation(VisuData* data)
{
  float *trans;
  int i;

  if (!data)
    {
      fprintf(stderr, "WARNING! 'visuDataGet_translation' has been called with no 'data'"
	      " attribute.\n");
      return (float*)0;
    }
  trans = malloc(sizeof(float) * 3);
  if (!trans)
    {
      allocationProblems();
      exit(1);
    }
  for (i = 0; i < 3; i++)
    trans[i] = data->translation[i];
  return trans;
}
int visuDataSet_XYZtranslation(VisuData* data, float xyz[3])
{
  int i, res;

  if (!data || !xyz)
    {
      fprintf(stderr, "WARNING! 'visuDataAdd_translation' has been called with no 'data'"
	      " attribute.\n");
      return 0;
    }

  res = 0;
  for (i = 0; i < 3; i++)
    {
      if (data->translation[i] != xyz[i])
	{
	  data->translation[i] = xyz[i];
	  res = 1;
	}
    }
  DBG_fprintf(stderr, "Visu Data : set translation to : %f %f %f\n",
	      data->translation[0], data->translation[1], data->translation[2]);

  return res;
}

void visuDataConvert_XYZtoBoxCoordinates(VisuData *data, float boxCoord[3], float xyz[3])
{
  int i, j;

  if (!data || !boxCoord || !xyz)
    {
      fprintf(stderr, "WARNING! 'visuDataConvert_XYZtoBoxCoordinates' has been called"
	      " with no 'data' or no 'boxCoord' or no 'xyz' argument.\n");
      return;
    }
  for (i = 0; i < 3; i++)
    {
      boxCoord[i] = 0.;
      for (j = 0; j < 3; j++)
	boxCoord[i] += data->transformXYZtoBoxCoordinates[i][j] * xyz[j];
    }
}

void visuDataConvert_boxCoordinatestoXYZ(VisuData *data, float xyz[3], float boxCoord[3])
{
  float transformMatrix[3][3];
  int i, j;

  if (!data || !boxCoord || !xyz)
    {
      fprintf(stderr, "WARNING! 'visuDataConvert_boxCoordinatestoXYZ' has been called"
	      " with no 'data' or no 'boxCoord' or no 'xyz' argument.\n");
      return;
    }
  for (i = 0; i < 3; i++)
    {
      xyz[i] = 0.;
      for (j = 0; j < 3; j++)
	xyz[i] += data->transformBoxCoordinatestoXYZ[i][j] * boxCoord[j];
    }
}

void visuDataSet_boxGeometry(VisuData *data, float geometry[6], gboolean periodic)
{
  int i;
  float normY, normZ;

  if (!data || !geometry)
    {
      fprintf(stderr, "WARNING! 'visuDataSet_boxGeometry' has been called with no 'data'"
	      " or no 'geometry' attribute.\n");
      return;
    }
  data->isBoxPeriodic = periodic;
  for (i = 0; i < 6; i++)
    data->boxGeometry[i] = geometry[i];
  normY = sqrt(geometry[VISU_DATA_BOX_DYX] * geometry[VISU_DATA_BOX_DYX] +
	       geometry[VISU_DATA_BOX_DYY] * geometry[VISU_DATA_BOX_DYY]);
  normZ = sqrt(geometry[VISU_DATA_BOX_DZX] * geometry[VISU_DATA_BOX_DZX] +
	       geometry[VISU_DATA_BOX_DZY] * geometry[VISU_DATA_BOX_DZY] +
	       geometry[VISU_DATA_BOX_DZZ] * geometry[VISU_DATA_BOX_DZZ]);
  data->boxNorm[0] = 1.;
  for (i = 1; i < 3; i++)
    data->boxNorm[i] = geometry[i] / normY;
  for (i = 3; i < 6; i++)
    data->boxNorm[i] = geometry[i] / normZ;
  DBG_fprintf(stderr, "Visu Data : the bounding box is set to :\n %f %f %f\n %f %f %f\n",
	      geometry[0], geometry[1], geometry[2], geometry[3],
	      geometry[4], geometry[5]);

  /* Create the transformation matrix. */
  data->transformXYZtoBoxCoordinates[0][0] = 1 / data->boxGeometry[VISU_DATA_BOX_DXX];
  data->transformXYZtoBoxCoordinates[0][1] = - data->boxGeometry[VISU_DATA_BOX_DYX] /
    data->boxGeometry[VISU_DATA_BOX_DXX] / data->boxGeometry[VISU_DATA_BOX_DYY];
  data->transformXYZtoBoxCoordinates[0][2] = - (data->boxGeometry[VISU_DATA_BOX_DZX] /
						data->boxGeometry[VISU_DATA_BOX_DXX] +
						data->boxGeometry[VISU_DATA_BOX_DYX] *
						data->boxGeometry[VISU_DATA_BOX_DZY] / 
						data->boxGeometry[VISU_DATA_BOX_DXX] / 
						data->boxGeometry[VISU_DATA_BOX_DYY] ) /
    data->boxGeometry[VISU_DATA_BOX_DZZ];
  data->transformXYZtoBoxCoordinates[1][0] = 0.;
  data->transformXYZtoBoxCoordinates[1][1] = 1 / data->boxGeometry[VISU_DATA_BOX_DYY];
  data->transformXYZtoBoxCoordinates[1][2] = - data->boxGeometry[VISU_DATA_BOX_DZY] /
    data->boxGeometry[VISU_DATA_BOX_DYY] / data->boxGeometry[VISU_DATA_BOX_DZZ];
  data->transformXYZtoBoxCoordinates[2][0] = 0.;
  data->transformXYZtoBoxCoordinates[2][1] = 0.;
  data->transformXYZtoBoxCoordinates[2][2] = 1 / data->boxGeometry[VISU_DATA_BOX_DZZ];

  data->transformBoxCoordinatestoXYZ[0][0] = data->boxGeometry[VISU_DATA_BOX_DXX];
  data->transformBoxCoordinatestoXYZ[0][1] = data->boxGeometry[VISU_DATA_BOX_DYX];
  data->transformBoxCoordinatestoXYZ[0][2] = data->boxGeometry[VISU_DATA_BOX_DZX];
  data->transformBoxCoordinatestoXYZ[1][0] = 0.;
  data->transformBoxCoordinatestoXYZ[1][1] = data->boxGeometry[VISU_DATA_BOX_DYY];
  data->transformBoxCoordinatestoXYZ[1][2] = data->boxGeometry[VISU_DATA_BOX_DZY];
  data->transformBoxCoordinatestoXYZ[2][0] = 0.;
  data->transformBoxCoordinatestoXYZ[2][1] = 0.;
  data->transformBoxCoordinatestoXYZ[2][2] = data->boxGeometry[VISU_DATA_BOX_DZZ];
}
float visuDataGet_boxGeometry(VisuData *data, int vector)
{
  if (!data)
    {
      fprintf(stderr, "WARNING! 'visuDataGet_boxGeometry' has been called with no 'data'"
	      " attribute.\n");
      return -1.;
    }
  if (vector < 0 || vector >= 6)
    {
      fprintf(stderr, "WARNING! 'visuDataGet_boxGeometry' has been called"
	      " with a wrong 'vector' attribute (%d).\n", vector);
      return -1.;
    }
  return data->boxGeometry[vector];
}
gboolean visuDataGet_periodic(VisuData *data)
{
  if (!data)
    {
      fprintf(stderr, "WARNING! 'visuDataGet_boxGeometry' has been called with no 'data'"
	      " attribute.\n");
      return FALSE;
    }
  return data->isBoxPeriodic;
}
int visuData_constrainedElementInTheBox(VisuData *data, VisuElement *element)
{
  int *indexEle, i, j, k;
  gboolean moved, changed;
  float cartCoord[3], boxCoord[3];

  if (!data || !element)
    {
      fprintf(stderr, "WARNING! 'visuData_constrainedElementInTheBox' has been"
	      " called with no 'data' or no 'element' attribute.\n");
      return 0;
    }
  indexEle = (int*)g_hash_table_lookup(data->fromVisuElementToInt, (gpointer)element);
  if (!indexEle)
    {
      fprintf(stderr, "WARNING! 'visuData_constrainedElementInTheBox' has been"
	      " called with an 'element' attribute that is not in 'data' attribute.\n");
      return 0;
    }
  i = *indexEle;

  if (!data->fromIntToVisuElement[i]->rendered)
    return 0;

  DBG_fprintf(stderr, "Visu Data : Checking for nodes of element '%s' to be in the box.\n",
	      element->name);
  changed = FALSE;
  for(j = 0; j < data->numberOfStoredNodes[i]; j++)
    {
      cartCoord[0] = data->nodes[i][j].x + data->translation[0] +
	data->nodes[i][j].translation[0];
      cartCoord[1] = data->nodes[i][j].y + data->translation[1] +
	data->nodes[i][j].translation[1];
      cartCoord[2] = data->nodes[i][j].z + data->translation[2] +
	data->nodes[i][j].translation[2];
      visuDataConvert_XYZtoBoxCoordinates(data, boxCoord, cartCoord);
      moved = FALSE;
      for (k = 0; k < 3; k++)
	{
	  while (boxCoord[k] < 0.)
	    {
	      moved = TRUE;
	      boxCoord[k] += 1.;
	    }
	  while (boxCoord[k] > 1.)
	    {
	      moved = TRUE;
	      boxCoord[k] -= 1.;
	    }
	}
      if (moved)
	{
	  changed = TRUE;
	  visuDataConvert_boxCoordinatestoXYZ(data, cartCoord, boxCoord);
	  data->nodes[i][j].translation[0] = cartCoord[0] - data->nodes[i][j].x -
	    data->translation[0];
	  data->nodes[i][j].translation[1] = cartCoord[1] - data->nodes[i][j].y -
	    data->translation[1];
	  data->nodes[i][j].translation[2] = cartCoord[2] - data->nodes[i][j].z -
	    data->translation[2];
	}
    }
  return changed;
}
int visuData_constrainedInTheBox(VisuData *data)
{
  int i;
  gboolean changed;

  if (!data)
    {
      fprintf(stderr, "WARNING! 'visuData_constrainedInTheBox' has been called with no 'data'"
	      " attribute.\n");
      return 0;
    }
  changed = FALSE;
  for (i = 0; i < data->ntype; i++)
    changed = visuData_constrainedElementInTheBox(data, data->fromIntToVisuElement[i]) || changed;
  if (changed)
    return 1;
  else
    return 0;
}
int visuData_constrainedFree(VisuData *data)
{
  int i, j, k;

  if (!data)
    {
      fprintf(stderr, "WARNING! 'visuData_constrainedInTheBox' has been called with no 'data'"
	      " attribute.\n");
      return 0;
    }
  for (i = 0; i < data->ntype; i++)
    for(j = 0; j < data->numberOfStoredNodes[i]; j++)
      for (k = 0; k < 3; k++)
	data->nodes[i][j].translation[k] = 0.;
  return 1;
}

void visuDataGet_nodePosition(VisuData *data, VisuNode *node, float coord[3])
{
  if (!data || !node || !coord)
    {
      fprintf(stderr, "WARNING! 'visuDataGet_nodePosition' has been called"
	      " with no 'data' or no 'node' or no 'coord' argument.\n");
      return;
    }
  coord[0] = node->x + node->translation[0] + data->translation[0];
  coord[1] = node->y + node->translation[1] + data->translation[1];
  coord[2] = node->z + node->translation[2] + data->translation[2];
}

/**
 * visuDataTrace:
 * @data: the #VisuData to trace.
 *
 * This method gives informations about the argument.
 */
void visuDataTrace(VisuData *data)
{
  VisuElement *ele;
  int i;

  if (!data)
    return;

  fprintf(stderr, "Visu Data : data %d.\n", GPOINTER_TO_INT(data));
  fprintf(stderr, " | nb VisuElements : %d\n", data->ntype);
  for (i = 0; i < data->ntype; i++)
    {
      ele = data->fromIntToVisuElement[i];
      fprintf(stderr, " | VisuElements '%s' (%d), %d allocated VisuNodes and %d stored.\n",
	      data->fromIntToVisuElement[i]->name, GPOINTER_TO_INT(data->fromIntToVisuElement[i]),
	      data->numberOfNodes[i], data->numberOfStoredNodes[i]);
    }
}







/**********************/
/* Obsolete funstions */
/**********************/

/* Return the maxium size occupied by a VisuElement that has
   at least one VisuNode. */
/* float getMaxExtens() */
/* { */
/*   return maxDrawingSizeOfVisuElements; */
/* } */
/* Store a new value in maxDrawingSizeOfVisuElements that controls
   the maximum size occupied by a VisuElement. This function emits
   a signal () that is tarpped by opengl.c. Value must be positive or
   nul. */
/* void setMaxExtends(float value) */
/* { */
/*   if (value < 0.) */
/*     return; */

/*   maxDrawingSizeOfVisuElements = value; */

/*   g_signal_emit (visu, VISU_GET_CLASS (visu)->VisuElementSizeChanged, */
/* 		 0 , NULL); */

/* } */
