/*   EXTRAITS DE LA LICENCE
	Copyright CEA, contributeurs : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)
  
	Adresse ml :
	BILLARD, non joignable par ml ;
	CALISTE, damien P caliste AT cea P fr.

	Ce logiciel est un programme informatique servant  visualiser des
	structures atomiques dans un rendu pseudo-3D. 

	Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
	respectant les principes de diffusion des logiciels libres. Vous pouvez
	utiliser, modifier et/ou redistribuer ce programme sous les conditions
	de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
	sur le site "http://www.cecill.info".

	Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
	pris connaissance de la licence CeCILL, et que vous en avez accept les
	termes (cf. le fichier Documentation/licence.fr.txt fourni avec ce logiciel).
*/

/*   LICENCE SUM UP
	Copyright CEA, contributors : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)

	E-mail address:
	BILLARD, not reachable any more ;
	CALISTE, damien P caliste AT cea P fr.

	This software is a computer program whose purpose is to visualize atomic
	configurations in 3D.

	This software is governed by the CeCILL  license under French law and
	abiding by the rules of distribution of free software.  You can  use, 
	modify and/ or redistribute the software under the terms of the CeCILL
	license as circulated by CEA, CNRS and INRIA at the following URL
	"http://www.cecill.info". 

	The fact that you are presently reading this means that you have had
	knowledge of the CeCILL license and that you accept its terms. You can
	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
*/
#ifndef VISU_DATA_H
#define VISU_DATA_H

#include <glib.h>

#include "visu_elements.h"

/* This header file contains the globale variables
   that describes what visu basicaly controls :
    - a list of elements (which can be atoms or spin
      or anything else) ;
    - a list of coordinates to position the element ;
    - the size of the box which contains all the elements ;
*/

/***************/
/* Public part */
/***************/

/* Structure to store the coordinates and the element of
   all stuffs rendered by visu. */
struct node_struct;
/**
 * VisuNode:
 *
 * Short way to address #node_struct objects.
 */
typedef struct node_struct VisuNode;

/**
 * visuDataGet_objectList:
 *
 * All the drawned nodes are stored in an OpenGL list. The identifier of
 * this list can be access with the present method.
 *
 * Returns: an integer that identifies the OpenGL list of all drawn nodes.
 */
int visuDataGet_objectList();





/**
 * node_struct:
 * @x: a floating point value that positions the node on x axis ;
 * @y: a floating point value that positions the node on y axis ;
 * @z: a floating point value that positions the node on z axis ;
 * @theta: a floating point value that rotates the node in spherical coordinates ;
 * @phi: a floating point value that rotates the node in spherical coordinates ;
 * @translation: an array of three floating point values that translates the
 *               node to its drawn position from (x, y, z) ;
 * @number: an integer that corresponds to its position in the entry file, it references
 *          also the node itself in the array 'fromNumberToVisuNode' of the #VisuData
 *          that contains the node ;
 * @posElement: an integer that is the position of the #VisuElement of the node
 *              in the array 'fromIntToVisuElement' of the #VisuData object that
 *              contains the node ;
 * @posNode: an integer that is the position of the node itself in the array
 *           'nodes' of the #VisuData object that contains the node ;
 * @rendered: a boolean to store if the node is drwn or not.
 *
 * Structure to store primary data of a node.
 */
struct node_struct
{
  /* coordinates of the node. */
  double x, y, z;

  /* rotation around the x and z axis */
  double theta, phi;

  /* translation */
  float translation[3];

  /* Number of this element in the input file. */
  int number;
  /* Position in the #VisuData structure. */
  int posElement, posNode;

  /* A boolean to specify if if this node is rendered or not. */
  int rendered;
};

/**
 * visuNodeSet_newValues:
 * @node: an allocated #VisuNode object ;
 * @x: its x coordinate ;
 * @y: its y coordinate ;
 * @z: its z coordinate ;
 * @theta: its theta rotation (around z axis) ;
 * @phi: its phi rotation (around transformed y axis) ;
 * @number: its position in the input file (or anything else that can uniqualy
 *          identify this new #VisuNode) ;
 * @rendered: 1 if draw, 0 otherwise ;
 * @posEle: an int corresponding the number of the #VisuElement
 *          it is associated with ;
 * @posNod: an int corresponding to the position of the node in the
 *          node table of the #VisuData in which it is allocated ;
 * 
 * This method doesn't allocate the node it works on. It should be already
 * allocated. Moreover it should not have been already initialised
 * since nothing from the node is freed before giving the values.
 */
void visuNodeSet_newValues(VisuNode *node, double x, double y, double z, 
			   double theta, double phi, int number,
			   int rendered, int posEle, int posNod);
/**
 * visuNodeSet_visibility:
 * @node: a #VisuNode object ;
 * @visibility: a boolean.
 *
 * This method is used to turn on or off the drawing of the specified node.
 *
 * Returns: true if the calling method should recreate the node list with
 * a call to the createAllNodes() method.
 */
int visuNodeSet_visibility(VisuNode* node, gboolean visibility);
/**
 * visuNodeGet_visibility:
 * @node: a #VisuNode object.
 *
 * This method is used get the status of the drawing state of a node.
 *
 * Returns: true if the node is rendered, false otherwise.
 */
gboolean visuNodeGet_visibility(VisuNode* node);
/**
 * visuNodeSetThetaPhi:
 * @node: a #VisuNode object ;
 * @theta: an angle in spherical coordinates ;
 * @phi: an angle in spherical coordinates.
 *
 * Set the rotation of a #VisuNode in spherical coordinates.
 */
void visuNodeSetThetaPhi(VisuNode *node, double theta, double phi);



/************************************************/
/* Methods to deal with the VisuData structure. */
/************************************************/

enum visuData_boxVector
  {
    VISU_DATA_BOX_DXX,
    VISU_DATA_BOX_DYX,
    VISU_DATA_BOX_DYY,
    VISU_DATA_BOX_DZX,
    VISU_DATA_BOX_DZY,
    VISU_DATA_BOX_DZZ
  };

/**
 * setColorFunc:
 * @ele: a #VisuElement ;
 * @node: a #VisuNode ;
 * @rgba: an 4 allocated float area to store the return values.
 *
 * This prototype is used to specify an optional method
 * to associate a color with external values to each node.
 */
typedef void (*setColorFunc)(float rgba[4], VisuElement *ele, VisuNode* node);

/**
 * VisuData_struct:
 * @files: a #GHashTable to store the files corresponding to the #VisuData,
 *         the keys are integers and correspond to the kind of file ;
 * @ntype: the number of #VisuElement object used by this #VisuData ;
 * @fromVisuElementToInt: a table to correspond #VisuElement to the position
 *                        in the array 'nodes' ;
 * @fromIntToVisuElement: a table to correspond the position in the array 'nodes'
 *                        to a pointer to the corresponding #VisuElement ;
 * @fromNumberToVisuNode: a table to correspond the 'number' entry in the #VisuNode
 *                        structure to the right #VisuNode ;
 * @nbOfAllStoredNodes: the number of #VisuNode object stored in this #VisuData ;
 * @numberOfNodes: the number of #VisuNode object allocated for each #VisuElement
 *                 used by the #VisuData ;
 * @numberOfStoredNodes: the same but the number of stored nodes instead of allocated nodes ;
 * @nodes: the array containing all #VisuNodes ;
 * @boxGeometry: the size of the bounding box ;
 * @boxNorm: the same vectors than in boxGeometry but leading to unitary vectors ;
 * @transformXYZtoBoxCoordinates: a matrix that transforms cartesian coordinates to
 *        coordinates in the box geometry. Use visuDataConvert_XYZtoBoxCoordinates()
 *        to access this matrix ;
 * @transformBoxCoordinatestoXYZ: a matrix that transform box coordinates to
 *        cartesian coordinates. Use visuDataConvert_boxCoordinatestoXYZ()
 *        to access this matrix.
 * @isBoxPeriodic: a boolean that states if the bounding box gives periodic conditions
 *                 to the nodes ;
 * @translation: a translation apply to all the nodes when rendered ;
 * @nodesProperties: a #GHashTable to store some properties with the nodes, the keys
 *                   are strings ;
 * @properties: a #GHashTable to store some properties with the #VisuData, the keys
 *              are strings ;
 * @setColor: a method to change the color of the #VisuNode (a bit tricky, should change
 *            in future release).
 *
 * All data from a file are stored in a structure, containing
 * the number of VisuElements used by this file and the arrays
 * of nodes present is the file and many other things related to
 * the data read from input file(s).
 */
struct VisuData_struct
{
  /********************/
  /* Files attributes */
  /********************/
  /* Files that correspond to that VisuData.
     They are identified by a kind which is an integer. */
  GHashTable *files;


  /********************/
  /* Nodes attributes */
  /********************/
  /* Number of VisuElements used by these data. */
  int ntype;
  /* This hashtable gives access to the good array  of VisuNodes
     when one has the VisuElement. */
  GHashTable *fromVisuElementToInt;
  /* This is a corresponding table from the nodes array indice
     to the associated VisuElement. */
  VisuElement **fromIntToVisuElement;

  /* This array gives access to the good VisuNode
     when one has its number. */
  VisuNode **fromNumberToVisuNode;
  /* The total of stored VisuNodes. */
  int nbOfAllStoredNodes;

  /* Number of nodes allocated (size of the nodes array) per VisuElement. */
  int* numberOfNodes;
  /* Number of nodes physically present in the array per VisuElement. */
  int* numberOfStoredNodes;
  /* Coordinates of all these nodes. */
  VisuNode **nodes;


  /********************/
  /* Box   attributes */
  /********************/
  /* This is the geometry of the box. Array position 1 to 6
     correspond to xx, xy, yy, zx, zy and zz. */
  float boxGeometry[6];
  /* This is the unitary vector corresponding to boxGeometry. */
  float boxNorm[6];
  /* This is the matrix that transform cartesian coordinates to
     coordinates in the box geometry. Use visuDataConvert_XYZtoBoxCoordinates()
     to access this matrix. */
  float transformXYZtoBoxCoordinates[3][3];
  /* This is the matrix that transform box coordinates to
     cartesian coordinates. Use visuDataConvert_boxCoordinatestoXYZ()
     to access this matrix. */
  float transformBoxCoordinatestoXYZ[3][3];
  /* TRUE if the box is periodic. It is FALSE by default. */
  gboolean isBoxPeriodic;
  /* Translation applied to all nodes when rendered. */
  float translation[3];


  /********************/
  /* Misc. attributes */
  /********************/
  /* This is a table to store data, reachable with string keys.
     It should be accessed via visuDataSet_nodeProperty
     and visuDataGet_nodeProperty. */
  GHashTable *nodesProperties;

  /* Define a method to set the color of each node.
     If this method is NULL, the color of the element is used. */
  setColorFunc setColor;

  /* This hashtable is usefull to store some data associated with the VisuData.
     It should be accessed with the visuDataSet_property()
     and the visuDataGet_property methods. */
  GHashTable *properties;
};
/**
 * VisuData:
 *
 * A short way to name #VisuData_struct objects.
 */
typedef struct VisuData_struct VisuData;

/**
 * visuDataNew:
 *
 * This creates an empty #VisuData object.
 *
 * Returns: a newly created #VisuData object.
 */
VisuData* visuDataNew();
/**
 * visuDataSet_population:
 * @data: a #VisuData object ;
 * @nbOfTypes: number of #VisuElement ;
 * @nbOfNodesPerVisuElement: number of #VisuNode per VisuElement ;
 * @visuElementUsed: pointers to #VisuElement in the same order that
 *                   @nbOfNodesPerVisuElement.
 *
 * This method allocates the storing part of the given #VisuData structure and
 * store all the #VisuNodes.
 *
 * Returns: 1 if everything goes right.
 */
int visuDataSet_population(VisuData *data, int nbOfTypes, int* nbOfNodesPerVisuElement,
			   VisuElement **visuElementUsed);
/**
 * visuDataFree:
 * @data: a VisuData to be freed.
 *
 * This method frees all the allocated memory of the VisuData
 * specified as parameter.
 */
void visuDataFree(VisuData *data);
/**
 * visuDataFree_population:
 * @data: a VisuData to be freed.
 *
 * This method frees only the allocated memory that deals with
 * the nodes (i.e. everything except the data of the files,
 * the properties and the setColor method.
 */
void visuDataFree_population(VisuData *data);
/**
 * visuDataAdd_file:
 * @data: a #VisuData object ;
 * @file: a string that points to a file ;
 * @kind: an integer to qualify the file to add.
 *
 * This method is used to add files
 * of type @kind to the @data. The @file
 * attribute is copied.
 */
void visuDataAdd_file(VisuData *data, gchar* file, int kind);
/**
 * visuDataRemove_allFiles:
 * @data: a #VisuData object.
 *
 * This method is used to empty the list of
 * known file from the given @data.
 */
void visuDataRemove_allFiles(VisuData *data);
/**
 * visuDataGet_file:
 * @data: a #VisuData object.
 * @kind: an integer to qualify the required file.
 *
 * This prototype is used to retrieve stored
 * files identify by their @kind.
 *
 * Returns: the name of a file (it should not be deleted).
 */
gchar* visuDataGet_file(VisuData *data, int kind);

/**
 * visuData_compareElements:
 * @data1: a #VisuData object ;
 * @data2: an other #VisuData object.
 *
 * This method is used to compare the composition of the given two #VisuData objects.
 * The test is only done on #VisuElement lists.
 *
 * Returns: TRUE if the two objects contains exactly the same #VisuElemet objects (not
 *          one more or one less or one different), FALSE otherwise.
 */
gboolean visuData_compareElements(VisuData *data1, VisuData *data2);
/**
 * visuDataSet_changeElementFlag:
 * @data: a #VisuData object ;
 * @changeElement: a boolean.
 *
 * This method is mainly used by internal gears to set a flag. This flag control
 * if the @data object has the same #VisuElement objects than the previously rendered one.
 */
void visuDataSet_changeElementFlag(VisuData *data, gboolean changeElement);
/**
 * visuDataGet_changeElementFlag:
 * @data: a #VisuData object.
 *
 * V_Sim can use a flag set on @data object to know if @data has exactly the same
 * #VisuElement list than the previously rendered one.
 *
 * Returns: TRUE if the previously rendered #VisuData object has had the same
 *          #VisuElement list than the given one, FALSE otherwise.
 */
gboolean visuDataGet_changeElementFlag(VisuData *data);

/**
 * visuDataAdd_VisuElement:
 * @data: the #VisuData where to add the new #VisuNode ;
 * @ele: the #VisuElement kind of the new #VisuNode ;
 * @x: its x coordinate ;
 * @y: its y coordinate ;
 * @z: its z coordinate ;
 * @number: its position in the input file (or anything else that can uniqualy
 *          identify this new #VisuNode).
 *
 * This method adds a new #VisuNode to the specified #VisuData.
 *
 * Returns: 0 if everything is allright.
 */
int visuDataAdd_VisuElement(VisuData *data, VisuElement *ele,
			    double x, double y, double z, int number);
/**
 * visuDataAdd_VisuElementWithRotation:
 * @data: the #VisuData where to add the new #VisuNode ;
 * @ele: the #VisuElement kind of the new #VisuNode ;
 * @x: its x coordinate ;
 * @y: its y coordinate ;
 * @z: its z coordinate ;
 * @theta: its theta rotation (around z axis) ;
 * @phi: its phi rotation (around transformed y axis) ;
 * @number: its position in the input file (or anything else that can uniqualy
 *          identify this new #VisuNode).
 *
 * This method adds a new #VisuNode to the specified #VisuData.
 *
 * Returns: 0 if everything is allright.
 */
int visuDataAdd_VisuElementWithRotation(VisuData *data, VisuElement *ele,
					double x, double y, double z, 
					double theta, double phi, int number);
/**
 * visuDataAdd_int:
 * @data: the #VisuData where to add the new #VisuNode ;
 * @position: a integer corresponding to the position of
 *            a #VisuElement in the array **nodes in the structure;
 * @x: its x coordinate ;
 * @y: its y coordinate ;
 * @z: its z coordinate ;
 * @number: its position in the input file (or anything else that can uniqualy
 *          identify this new #VisuNode).
 *
 * This method adds a new #VisuNode to the specified #VisuData. Position must be
 * chosen between 0 and (ntype - 1) and corresponds to the position of the array
 * of #VisuNodes of a #VisuElement.
 *
 * Returns: 0 if everything is alright.
 */
int visuDataAdd_int(VisuData *data, int position,
		    double x, double y, double z, int number);
/**
 * visuDataGet_fromNumber:
 * @data: a %VisuData structure which stores the nodes.
 * @number: an integer.
 *
 * This methods retrieves the #VisuNode identified by the integer @number.
 *
 * Returns: the found #VisuNode or NULL if none corresponds to number.
 */
VisuNode* visuDataGet_fromNumber(VisuData *data, int number);

/**
 * visuDataSet_nodeProperty:
 * @data: a #VisuData object ;
 * @node: a #VisuNode object ;
 * @key: a string ;
 * @value: an allocated area to store data.
 *
 * This method is used to store some values associated with
 * the given @node of the given @data. These values are anything allocated and will be
 * freed (with free()) when the node is deleted. These values
 * are described by the @key, and can be retrieved with the
 * #visuDataGet_nodeProperty method. The @key is used by the #VisuData
 * and should not be freed. Moreover it is not freed when the node
 * is destroyed.
 */
void visuDataSet_nodeProperty(VisuData* data, VisuNode* node, char* key, gpointer value);
/**
 * visuDataGet_nodeProperty:
 * @data: a #VisuData object ;
 * @node: a #VisuNode object ;
 * @key: a string.
 *
 * This method is used to retrieve some data associated to
 * the specified @node, stored in the given @data. These return data
 * should not be freed after used.
 *
 * Returns: some data associated to the key.
 */
gpointer visuDataGet_nodeProperty(VisuData* data, VisuNode* node, char* key);
/**
 * visuDataAdd_nodeProperty:
 * @data: a #VisuData object ;
 * @key: a string ;
 * @freeFunc: a method to free each token.
 * 
 * This method adds and allocates a new area to store nodes associated data that
 * could be retrieve with the @key. When the property is removed with the
 * #visuDataRemove_nodePropertry the area is freed and @freeFunc is called for
 * each token.
 */
void visuDataAdd_nodeProperty(VisuData* data, char* key, GDestroyNotify freeFunc);
/**
 * visuDataRemove_nodeProperty:
 * @data: a #VisuData object ;
 * @key: a string.
 * 
 * This method remove all the data associated with the string @key, of all nodes.
 * They are freed.
 */
void visuDataRemove_nodeProperty(VisuData* data, char* key);

/**
 * visuDataSet_property:
 * @data: a #VisuData object ;
 * @key: a string ;
 * @value: an allocated area to store data.
 *
 * This method is used to store some values associated with
 * the given @data. These values are anything allocated and will be
 * freed (with free()) when the @data is deleted. These values
 * are described by the @key, and can be retrieved with the
 * #visuDataGet_property() method. The @key is used by the #VisuData
 * and should not be freed. Moreover it is not freed when the node
 * is destroyed.
 */
void visuDataSet_property(VisuData* data, char* key, gpointer value);
/**
 * visuDataGet_property:
 * @data: a #VisuData object ;
 * @key: a string.
 *
 * This method is used to retrieve some data associated to
 * the specified @data. These return data should not be freed after used.
 *
 * Returns: some data associated to the key.
 */
gpointer visuDataGet_property(VisuData* data, char* key);
/**
 * visuDataRemove_property:
 * @data: a #VisuData object ;
 * @key: a string.
 * 
 * This method remove the data associated with the string @key.
 * They are freed.
 */
void visuDataRemove_property(VisuData* data, char* key);

/**
 * visuDataSet_ColorFunc:
 * @data: a #VisuData object ;
 * @func: a method that colorize the nodes.
 *
 * This is a little trick to colorized the nodes. It should not be used since it
 * will probably be different in future release.
 */
void visuDataSet_ColorFunc(VisuData *data, setColorFunc func);

/**
 * visuDataGet_XYZtranslation:
 * @data: a #VisuData object.
 *
 * The nodes are rendered at thier coordinates plus a translation. This method
 * allows to retrieve that translation.
 *
 * Returns: a newly allocated array of 3 floats. It should be freed with a call
 *          to free() after use.
 */
float* visuDataGet_XYZtranslation(VisuData* data);
/**
 * visuDataSet_XYZtranslation:
 * @data: a #VisuData object ;
 * @xyz: an array of floating point values.
 *
 * This set the translations of the specified #VisuData whatever previous values.
 * The translation is done in the orthonormal referential, not the referential of
 * the box.
 *
 * Returns: if returns 1, visuData_createAllNodes() should be called and
 *          the 'NodePositionChanged' should be emitted.
 */
int visuDataSet_XYZtranslation(VisuData* data, float xyz[3]);
/**
 * visuDataConvert_XYZtoBoxCoordinates:
 * @data: a #VisuData object ;
 * @boxCoord: an array of floating point values to store the result ;
 * @xyz: an array of floating point values describing coordinates in cartesian.
 *
 * Use this method to transform cartesian coordinates to the box coordinates.
 */
void visuDataConvert_XYZtoBoxCoordinates(VisuData *data, float boxCoord[3], float xyz[3]);
/**
 * visuDataConvert_boxCoordinatestoXYZ:
 * @data: a #VisuData object ;
 * @xyz: an array of floating point values to store the result ;
 * @boxCoord: an array of floating point values that describes the box coordinates.
 *
 * Use this method to transform box coordinates into cartesian.
 */
void visuDataConvert_boxCoordinatestoXYZ(VisuData *data, float xyz[3], float boxCoord[3]);
/**
 * visuDataSet_boxGeometry:
 * @data: a #VisuData object ;
 * @geometry: a 6 floating point array ;
 * @periodic: a boolean.
 *
 * This methods set the size of the box that defines the viewport. If the @periodic
 * argument is TRUE, the box is also the limit of a periodic system.
 */
void visuDataSet_boxGeometry(VisuData *data, float geometry[6], gboolean periodic);
/**
 * visuDataGet_boxGeometry:
 * @data: a #VisuData object ;
 * @vector: an int corresponding to a vector of the box.
 *
 * Retrieve the value of a vector defining the bounding box. The vector
 * is chosen with an int, see the #visuData_boxVector enum for more
 * details.
 *
 * Returns: the value of the required vector (always a positive value),
 *          a negative value if something goes wrong.
 */
float visuDataGet_boxGeometry(VisuData *data, int vector);
/**
 * visuDataGet_periodic:
 * @data: a #VisuData object.
 * 
 * The bounding box can be just useful to set the OpenGl viewport,
 * or it can define a limit for periodic conditions.
 *
 * Returns: if the box is a periodic limit or not.
 */
gboolean visuDataGet_periodic(VisuData *data);
/**
 * visuData_constrainedElementInTheBox:
 * @data: a #VisuData object ;
 * @element: a #VisuElement object.
 *
 * Check all the nodes of the specified @element and change their coordinates if they are out
 * of the bounding box. The position of each node is the result of the
 * sum of their own position and of the box translation.
 *
 * Returns: 1 if visuData_createAllNodes() should be called and
 *          the 'NodePositionChanged' should be emitted.
 */
int visuData_constrainedElementInTheBox(VisuData *data, VisuElement *element);
/**
 * visuData_constrainedInTheBox:
 * @data: a #VisuData object.
 *
 * It does the same things that visuData_constrainedElementInTheBox() but for all
 * the #VisuElement of the given @data. I.e. it checks all the nodes and changes
 * their coordinates if they are out of the bounding box.
 * The position of each node is the result of the
 * sum of their own position and of the box translation.
 *
 * Returns: 1 if visuData_createAllNodes() should be called and
 *          the 'NodePositionChanged' should be emitted.
 */
int visuData_constrainedInTheBox(VisuData *data);
/**
 * visuData_constrainedFree:
 * @data: a #VisuData object.
 *
 * Return all the nodes to their original position, except for the
 * global translation.
 *
 * Returns: 1 if visuData_createAllNodes() should be called and
 *          the 'NodePositionChanged' should be emitted.
 */
int visuData_constrainedFree(VisuData *data);
/**
 * visuDataGet_nodePosition:
 * @data: a #VisuData object ;
 * @node: a #VisuNode object ;
 * @coord: an array of 3 floating point values to store the position.
 *
 * Position of nodes are subject to various translations and different transformations.
 * Their coordinates should not be access directly through node.[xyz]. This method
 * is used to retrieve the given node position.
 */
void visuDataGet_nodePosition(VisuData *data, VisuNode *node, float coord[3]);

/**
 * visuDataTrace:
 * @data: the #VisuData to trace.
 *
 * This method gives informations about the argument.
 */
void visuDataTrace(VisuData *data);


/**
 * initVisuData:
 *
 * Initialise all the variables of this module. Should not be used
 * except by the main program.
 *
 * Returns: 1 if everything goes well, 0 otherwise.
 */
int initVisuData();

/****************/
/* Private part */
/****************/

/* This is a pointer to the current loaded data. */
VisuData *currentVisuData;


/***************/
/* OpenGL Part */
/***************/

int identifierAllNodes;

/**
 * visuData_createNodes:
 * @data: a #VisuData object ;
 * @ele: a #VisuElement object.
 *
 * This create the glObjectList registered at (%identifierAllNodes + the position
 * of the @ele in the @data object + 1) that contains all the nodes of the given #VisuElement.
 */
void visuData_createNodes(VisuData *data, VisuElement *ele);
/**
 * visuData_createAllNodes:
 *
 * This create the glObjectList registered at %identifierAllNodes
 * that contains all the nodes. This glObjectList is made of all
 * nodes of all element that has a flag %rendered at true translated
 * to their own positions.
 *
 * This method asks for redraw by emitting the %OpenGLAskForReDraw.
 */
void visuData_createAllNodes();
/**
 * visuData_createAllElements:
 * @data: a #VisuData object.
 *
 * This method will call the createOpenGLElementFunc() method of the current #RenderingMethod
 * on all the nodes of the given #VisuData.
 */
void visuData_createAllElements(VisuData *data);
/**
 * deleteAllNodes:
 *
 * This delete the glObjectList registered at %identifierAllNodes
 * that contains all the nodes.
 */
void deleteAllNodes();

/* Return the maxium size occupied by a VisuElement that has
   at least one VisuNode. */
/* float getMaxExtens(); */
/* Store a new value in maxDrawingSizeOfVisuElements that controls
   the maximum size occupied by a VisuElement. This function emits
   a signal () that is tarpped by opengl.c. Value must be positive or
   nul. */
/* void setMaxExtends(float value); */

#endif
