/*   EXTRAITS DE LA LICENCE
	Copyright CEA, contributeurs : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)
  
	Adresse ml :
	BILLARD, non joignable par ml ;
	CALISTE, damien P caliste AT cea P fr.

	Ce logiciel est un programme informatique servant  visualiser des
	structures atomiques dans un rendu pseudo-3D. 

	Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
	respectant les principes de diffusion des logiciels libres. Vous pouvez
	utiliser, modifier et/ou redistribuer ce programme sous les conditions
	de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
	sur le site "http://www.cecill.info".

	Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
	pris connaissance de la licence CeCILL, et que vous en avez accept les
	termes (cf. le fichier Documentation/licence.fr.txt fourni avec ce logiciel).
*/

/*   LICENCE SUM UP
	Copyright CEA, contributors : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)

	E-mail address:
	BILLARD, not reachable any more ;
	CALISTE, damien P caliste AT cea P fr.

	This software is a computer program whose purpose is to visualize atomic
	configurations in 3D.

	This software is governed by the CeCILL  license under French law and
	abiding by the rules of distribution of free software.  You can  use, 
	modify and/ or redistribute the software under the terms of the CeCILL
	license as circulated by CEA, CNRS and INRIA at the following URL
	"http://www.cecill.info". 

	The fact that you are presently reading this means that you have had
	knowledge of the CeCILL license and that you accept its terms. You can
	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
*/
#ifndef VISU_EXTENSION_H
#define VISU_EXTENSION_H

#include <glib.h>
#include <glib-object.h>

/***************/
/* Public part */
/***************/

/**
 * OPENGL_EXTENSION_PRIORITY_FIRST
 *
 * An extension with this priority is drawn first.
 */
#define OPENGL_EXTENSION_PRIORITY_FIRST 0
/**
 * OPENGL_EXTENSION_PRIORITY_HIGH
 *
 * An extension with this priority is quickly drawn.
 */
#define OPENGL_EXTENSION_PRIORITY_HIGH 20
/**
 * OPENGL_EXTENSION_PRIORITY_NORMAL
 *
 * An extension with this priority is drawn after the higher priorities.
 */
#define OPENGL_EXTENSION_PRIORITY_NORMAL 50
/**
 * OPENGL_EXTENSION_PRIORITY_LOW
 *
 * An extension with this priority is drawn among last extensions.
 */
#define OPENGL_EXTENSION_PRIORITY_LOW 80
/**
 * OPENGL_EXTENSION_PRIORITY_LAST
 *
 * An extension with this priority is drawn last.
 */
#define OPENGL_EXTENSION_PRIORITY_LAST 100

/**
 * OpenGLExtension_struct:
 * 
 * This structure allows the user to store data
 * about an OpenGL extension.
 */
struct OpenGLExtension_struct;
/**
 * OpenGLExtension:
 * 
 * Common name to refer to a #OpenGLExtension_struct.
 */
typedef struct OpenGLExtension_struct OpenGLExtension;

/**
 * rebuildObjectListFunc:
 * @obj: the object than triggered the signal ;
 * @data: some user informations.
 *
 * Prototypes used to recreate OpenGL list of objects for each extension.
 */
typedef void (*rebuildObjectListFunc)(GObject* obj, gpointer data);

/**
 * OpenGLExtension_new:
 * @name: name of the extension (must be non null),
 * @description: a brief description of the extension (can be null),
 * @objectListId: an int to identify an list of OpenGL objects (null
 * if this extension as no OpenGL object,
 * @rebuild: handler to a method that is called every time V_Sim needs
 * to create again the OpenGL object list. If NULL, nothing is called.
 *
 * Create a new OpenGLExtension with the specified name, description
 * and OpenGL object list. The name must be unique since it's used by
 * v_sim to store this extension. The priority is set to
 * #OPENGL_EXTENSION_PRIORITY_NORMAL.
 *
 * Returns: the new OpenGLExtension or null if something wrong happens.
 */
OpenGLExtension* OpenGLExtension_new(char* name, char* description,
				     int objectListId, rebuildObjectListFunc rebuild);
/**
 * OpenGLExtension_free:
 * @extension: the extension to delete.
 *
 * Free all the allocated attributes of the specified method.
 */
void OpenGLExtension_free(OpenGLExtension* extension);

/**
 * OpenGLExtensionSet_priority:
 * @extension: a #OpenGLExtension object ;
 * @priority: an integer value.
 *
 * Extentions are drawn in an order that depends on their priority.
 * The lower is the number, the sooner the extension is drawn. Flags,
 * such as #OPENGL_EXTENSION_PRIORITY_NORMAL or #OPENGL_EXTENSION_PRIORITY_LOW,
 * can be used or user defined values are also possible.
 */
void OpenGLExtensionSet_priority(OpenGLExtension* extension, int priority);

/**
 * OpenGLExtensionSet_saveOpenGLState:
 * @extension: a #OpenGLExtension object ;
 * @saveState: an boolean value.
 *
 * If the extension needs to change some OpenGL state (to disable the fog for example,
 * or the cullface), a flag should be set to enable V_Sim to restore the right values after
 * the extensio have been called. Because the OpenGL list of an extension is just GL_COMPILE
 * the extension can't just save and restore state itself because when the list is called,
 * the state can have been changed.
 */
void OpenGLExtensionSet_saveOpenGLState(OpenGLExtension *extension, gboolean saveState);

/**
 * OpenGLExtension_get_active:
 * @extension: the extension.
 *
 * Get if the extension is used or not. If not its ObjectList
 * is not rendered.
 *
 * Returns: 1 if used, 0 otherwise.
 */
int OpenGLExtension_get_active(OpenGLExtension* extension);
/**
 * OpenGLExtension_set_active:
 * @extension: the extension,
 * @value: the new value.
 *
 * Set if an extension is actually used or not.
 */
void OpenGLExtension_set_active(OpenGLExtension* extension, int value);

/**
 * registerOpenGLExtension:
 * @extension: an extension.
 *
 * A method used by user to registered a new extension.
 */
void registerOpenGLExtension(OpenGLExtension *extension);

/**
 * callAllExtensionsLists:
 *
 * For each registered extension that has a valid list (id > 1000),
 * it calls it with glCallList.
 */
void callAllExtensionsLists();
/**
 * rebuildAllExtensionsLists:
 *
 * For each registered extension that has a valid rebuild method,
 * it calls it.
 */
void rebuildAllExtensionsLists();




/****************/
/* Private area */
/****************/

/* Explicit definition of the structure. */
struct OpenGLExtension_struct
{
  /* Some variable to describe this OpenGL extension.
     The attribute name is mandatory since it is
     used to identify the method. */
  char* name;
  char* description;

  /* The id of the possible objects list brings by
     the extension is refered by this int. */
  int objectListId;

  /* Function called to rebuild the object list of the extension. */
  rebuildObjectListFunc rebuild;

  /* A priority for the extension. */
  int priority;

  /* If set, V_Sim save the OpenGL state before the list
     id is called and restore all states after. */
  gboolean saveState;

  /* A boolean to know if this extension is actually used
     or not. */
  int used;
};

/**
 * initOpenGLExtensions:
 *
 * Initialise all the variable of this part. It calls all
 * the elements in %listInitExtensionFunc (that stores the
 * init function of the extensions). If these elements return
 * valid OpenGLExtension, they are registered through a call
 * to registerOpenGLExtension().
 *
 * Returns: 1 if everything goes allright during the initialisation.
 */
int initOpenGLExtensions();
/**
 * loadExtensions:
 *
 * This method is used when V_Sim is initialized and should
 * not be called elsewhere. For each valid init method found
 * in #listInitExtensionFunc, it is called.
 */
void loadExtensions();


#endif
