/*   EXTRAITS DE LA LICENCE
	Copyright CEA, contributeurs : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)
  
	Adresse ml :
	BILLARD, non joignable par ml ;
	CALISTE, damien P caliste AT cea P fr.

	Ce logiciel est un programme informatique servant  visualiser des
	structures atomiques dans un rendu pseudo-3D. 

	Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
	respectant les principes de diffusion des logiciels libres. Vous pouvez
	utiliser, modifier et/ou redistribuer ce programme sous les conditions
	de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
	sur le site "http://www.cecill.info".

	Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
	pris connaissance de la licence CeCILL, et que vous en avez accept les
	termes (cf. le fichier Documentation/licence.fr.txt fourni avec ce logiciel).
*/

/*   LICENCE SUM UP
	Copyright CEA, contributors : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)

	E-mail address:
	BILLARD, not reachable any more ;
	CALISTE, damien P caliste AT cea P fr.

	This software is a computer program whose purpose is to visualize atomic
	configurations in 3D.

	This software is governed by the CeCILL  license under French law and
	abiding by the rules of distribution of free software.  You can  use, 
	modify and/ or redistribute the software under the terms of the CeCILL
	license as circulated by CEA, CNRS and INRIA at the following URL
	"http://www.cecill.info". 

	The fact that you are presently reading this means that you have had
	knowledge of the CeCILL license and that you accept its terms. You can
	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
*/
#include "visu_pickMesure.h"

#include "visu_tools.h"
#include "visu_data.h"
#include "visu_object.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <glib.h>


struct pickMesure
{
  int xRef1, yRef1;
  VisuNode *ref1;
  int xRef2, yRef2;
  VisuNode *ref2;
  VisuNode *selected;
};
static struct pickMesure mesureData;
GString *mesureInfo;
GString *mesureError;

void initPickMesure()
{
  mesureData.ref1 = (VisuNode*)0;
  mesureData.ref2 = (VisuNode*)0;
  mesureData.selected = (VisuNode*)0;
  mesureInfo = (GString*)0;
  mesureError = (GString*)0;
}
void unSetReferencePickMesure()
{
  if (mesureData.ref1)
    drawEraseMark(MARK_SQUARE_BIG, mesureData.xRef1, mesureData.yRef1, 0);
  if (mesureData.ref2)
    drawEraseMark(MARK_SQUARE_SMALL, mesureData.xRef2, mesureData.yRef2, 0);
  mesureData.ref1 = (VisuNode*)0;
  mesureData.ref2 = (VisuNode*)0;
  mesureData.selected = (VisuNode*)0;
}
void endPickMesure()
{
  DBG_fprintf(stderr, "Visu Pick : Freeing pick mesure strings.\n");
  unSetReferencePickMesure();
  if (mesureInfo)
    g_string_free(mesureInfo, TRUE);
  mesureInfo = (GString*)0;
  if (mesureError)
    g_string_free(mesureError, TRUE);
  mesureError = (GString*)0;
}


void marksAndMesures(const VisuPick *info)
{
  double dx, dy, dz, dr, dx1, dy1, dz1, dr1,  dx2, dy2, dz2, dr2;
  double ang;
  float posRef1[3], posRef2[3], posSelect[3];

  if (!info)
    return;

  if (!mesureInfo)
    mesureInfo = g_string_new("");
  else
    mesureInfo = g_string_erase(mesureInfo, 0, -1);
  if (!mesureError)
    mesureError = g_string_new("");
  else
    mesureError = g_string_erase(mesureError, 0, -1);

  mesureData.selected = (VisuNode*)0;
  if (info->button == 1)
    {
      if (info->node)
	mesureData.selected = info->node;
      else
	{
	  mesureError = g_string_append(mesureError, _("You picked nothing.\n"));
	  mesureData.selected = (VisuNode*)0;
	}
    }
  else if (info->button == 2)
    {
      if (!(info->shiftMod))
	{
	  if (info->node)
	    {
	      if (mesureData.ref2 == info->node)
		mesureError =
		  g_string_append(mesureError,
				  _("1st reference node should be distinct\n"
				    "from 2nd one.\n"));
	      else
		{
		  if (mesureData.ref1)
		    drawEraseMark(MARK_SQUARE_BIG, mesureData.xRef1, mesureData.yRef1, 0);
		  mesureData.xRef1 = info->x;
		  mesureData.yRef1 = info->y;
		  mesureData.ref1 = info->node;
		  drawEraseMark(MARK_SQUARE_BIG, info->x, info->y, 1);
		}
	    }
	  else
	    {
	      mesureError = g_string_append(mesureError, _("Unset reference 1.\n"));
	      drawEraseMark(MARK_SQUARE_BIG, mesureData.xRef1, mesureData.yRef1, 0);
	      mesureData.ref1 = (VisuNode*)0;
	      drawEraseMark(MARK_SQUARE_SMALL, mesureData.xRef2, mesureData.yRef2, 0);
	      mesureData.ref2 = (VisuNode*)0;
	    }
	}
      else
	{
	  if (mesureData.ref2)
	    drawEraseMark(MARK_SQUARE_SMALL, mesureData.xRef2, mesureData.yRef2, 0);
	  mesureData.ref2 = (VisuNode*)0;
	  if (info->node)
	    {
	      if (!mesureData.ref1)
		mesureError =
		  g_string_append(mesureError,
				  _("Unable to set reference 2\n"
				    "without existing reference 1, use\n"
				    "middle button alone to set reference 1.\n"));
	      else if (mesureData.ref1 == info->node)
		mesureError =
		  g_string_append(mesureError,
				  _("2nd reference node should be distinct\n"
				    "from 1st one.\n"));
	      else
		{
		  mesureData.xRef2 = info->x;
		  mesureData.yRef2 = info->y;
		  mesureData.ref2 = info->node;
		  drawEraseMark(MARK_SQUARE_SMALL, info->x, info->y, 1);
		}
	    }
	  else
	    mesureError =
	      g_string_append(mesureError,
			      _("Unset reference 2.\n"));
	}
    }
  else
    {
      DBG_fprintf(stderr, "Mesure pick : unknown clic %d %d .\n",
		  info->button, info->shiftMod);
      return;
    }

  /* Create a string with informations. */
  if (mesureData.ref1)
    {
      g_string_append_printf(mesureInfo,
			     _("Reference node #%d\n"
			       "  x = %7.3f\n"
			       "  y = %7.3f\n"
			       "  z = %7.3f\n"),
			     mesureData.ref1->number, mesureData.ref1->x,
			     mesureData.ref1->y, mesureData.ref1->z);
    }
  if (mesureData.ref2)
    {
      visuDataGet_nodePosition(currentVisuData, mesureData.ref1, posRef1);
      visuDataGet_nodePosition(currentVisuData, mesureData.ref2, posRef2);
      dx = posRef2[0] - posRef1[0];
      dy = posRef2[1] - posRef1[1];
      dz = posRef2[2] - posRef1[2];
      g_string_append_printf(mesureInfo,
			     _("2nd Reference node #%d\n"
			       "  x = %7.3f   dx = %7.3f\n"
			       "  y = %7.3f   dy = %7.3f\n"
			       "  z = %7.3f   dz = %7.3f\n"),
			     mesureData.ref2->number,
			     mesureData.ref2->x, dx,
			     mesureData.ref2->y, dy,
			     mesureData.ref2->z, dz);
      dr = sqrt(dx*dx + dy*dy + dz*dz);
      if (mesureData.selected)
	g_string_append_printf(mesureInfo, _("           i.e. dr = %7.3f\n"), dr);
      else
	g_string_append_printf(mesureInfo, _("           i.e. dr = %7.3f"), dr);
    }
  if (mesureData.selected)
    {
      if (!mesureData.ref2)
        {
	  if (mesureData.ref1)
	    {
	      visuDataGet_nodePosition(currentVisuData, mesureData.selected, posSelect);
	      visuDataGet_nodePosition(currentVisuData, mesureData.ref1, posRef1);
	      dx = posSelect[0] - posRef1[0];
	      dy = posSelect[1] - posRef1[1];
	      dz = posSelect[2] - posRef1[2];
	      g_string_append_printf(mesureInfo,
				     _("Newly picked node #%d\n"
				       "  x = %7.3f   dx = %7.3f\n"
				       "  y = %7.3f   dy = %7.3f\n"
				       "  z = %7.3f   dz = %7.3f\n"),
				     mesureData.selected->number,
				     mesureData.selected->x, dx,
				     mesureData.selected->y, dy,
				     mesureData.selected->z, dz);
	      dr = sqrt(dx*dx + dy*dy + dz*dz);
	      g_string_append_printf(mesureInfo, _("           i.e. dr = %7.3f"), dr);
	    }
	  else
	    g_string_append_printf(mesureInfo,
				   _("Newly picked node #%d\n"
				     "  x = %7.3f   theta = %7.3f\n"
				     "  y = %7.3f   phi   = %7.3f\n"
				     "  z = %7.3f\n"),
				   mesureData.selected->number, mesureData.selected->x,
				   mesureData.selected->theta, mesureData.selected->y,
				   mesureData.selected->phi, mesureData.selected->z
				   );
	}
      else
        {
	  visuDataGet_nodePosition(currentVisuData, mesureData.selected, posSelect);
	  visuDataGet_nodePosition(currentVisuData, mesureData.ref1, posRef1);
	  visuDataGet_nodePosition(currentVisuData, mesureData.ref2, posRef2);
	  dx1 = posSelect[0] - posRef1[0];
	  dy1 = posSelect[1] - posRef1[1];
	  dz1 = posSelect[2] - posRef1[2];
	  dx2 = posRef2[0] - posRef1[0];
	  dy2 = posRef2[1] - posRef1[1];
	  dz2 = posRef2[2] - posRef1[2];
	  g_string_append_printf(mesureInfo,
				 _("Newly picked node #%d\n"
				   "  x = %7.3f   dx1 = %7.3f  dx2 = %7.3f\n"
				   "  y = %7.3f   dy1 = %7.3f  dx2 = %7.3f\n"
				   "  z = %7.3f   dz1 = %7.3f  dx2 = %7.3f\n"),
				 mesureData.selected->number,
				 mesureData.selected->x, dx1, dx2,
				 mesureData.selected->y, dy1, dy2,
				 mesureData.selected->z, dz1, dz2);
	  dr1 = sqrt(dx1*dx1 + dy1*dy1 + dz1*dz1);
	  dr2 = sqrt(dx2*dx2 + dy2*dy2 + dz2*dz2);
	  ang = acos((dx2*dx1+dy2*dy1+dz2*dz1)/(dr2*dr1))/PI180;
	    g_string_append_printf(mesureInfo,
				   _("           i.e. dr = %7.3f  dr2 = %7.3f  \n"
				     "  (Ref-Ref2, Ref-New) = %5.2f degrees"),
				   dr1, dr2, ang);
        }
    }

  DBG_fprintf(stderr, "Pick mesure info : %s\n", mesureInfo->str);
  DBG_fprintf(stderr, "Pick mesure error : %s\n", mesureError->str);

  g_signal_emit (visu, VISU_GET_CLASS (visu)->pickMesureAvailable_signal_id,
		 0 , NULL);
}

gchar* getPickMesureInfos()
{
  if (!mesureInfo)
    return (gchar*)0;
  return mesureInfo->str;
}
gchar* getPickMesureErrors()
{
  if (!mesureError)
    return (gchar*)0;
  return mesureError->str;
}
